%% Function for filtering certain kinds of data in the files that are open in Sunmap

function data_type_filter(sunmapHandles, target, action)

mainHandles = guidata(sunmapHandles);

% target: What data type is targeted
% action: What is done to the target data type

% Remove partial maps from file list:
if (strcmp(target, 'partial') == 1 && strcmp(action, 'remove') == 1)
    
    if(isempty(get(mainHandles.listbox1, 'String')) == 0 && isempty(get(mainHandles.listbox1, 'Value')) == 0)
        plottingList = get(mainHandles.listbox1,'String');
    end
    
    %size(plottingList,1)
    plottingList_new = {};
    num_removed = 0; % Number of removed files
    % Loop through listbox items and remove partial solar maps
    for idx=1:size(plottingList,1)
        
        infofilename = plottingList{idx};
        
        % Open info file, recognize type
        finfo = fopen(infofilename,'rt');
        if (finfo < 0) % Can't open file --> skip to next
            %fprintf(1, ['Could not open file ' infofilename '\n']);
            continue
        end
        % If .suninfo1, call info_reader()
        if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8)
            try
                infoStruct = info_reader(infofilename);
                % Data height in arc-min
                mapHeight = 60 * (infoStruct.enddec - infoStruct.startdec);
                % NON-declination- compensated width in arc-min:
                mapWidth = 60 * abs(infoStruct.endra - infoStruct.startra);
                if(mapHeight >= 30 && mapWidth >= 30) % Not partial map
                    plottingList_new = vertcat(plottingList_new, {infofilename}); % Add current to end of new list
                else
                    num_removed = num_removed + 1;
                end
            catch
                continue
            end
        elseif(strfind(infofilename, '.map')) % map-measure data:
            try
                map_meta = get_sunmap_metadata(infofilename);
                if(str2num(map_meta.mapxradius) >= 0.25 && str2num(map_meta.mapyradius) >= 0.25) % Not partial map
                    plottingList_new = vertcat(plottingList_new, {infofilename}); % Add current to end of new list
                else
                    num_removed = num_removed + 1;
                end
            catch err
                disp(['Problem with map-measure files: ', err]);
                continue
            end
        else % If .SMART, call info_reader_smart():
            try
                info_name = textscan(finfo, 'OBSERVER NAME    = %s', 'delimiter','\n');
                info_rx   = textscan(finfo, 'RECEIVER USED    = %s', 'delimiter','\n');
                info_df   = textscan(finfo, 'NAME OF DATAFILE = %s', 'delimiter','\n');
                info_date = textscan(finfo, 'DATE             = %s', 'delimiter','\n');
                info_cent = textscan(finfo, 'CENTER OF SUN AT = %s', 'delimiter','\n');
                info_sps  = textscan(finfo, 'SAMPLES PER SWEEP= %n', 'delimiter','\n');
                info_Tswp = textscan(finfo, 'DURATION OF SWEEP= %n', 'delimiter','\n');
                info_Nswp = textscan(finfo, 'PERFORMED SWEEPS = %d', 'delimiter','\n');
                % Determine from which row the data starts:
                info_all = textread(infofilename, '%s', 'delimiter', '\n');
                info_numrows = length(info_all);
                info_startrow = info_numrows - info_Nswp{1};
                info_time = dlmread(infofilename, '', info_startrow, 4);
                % Write variables into info_fields struct for use with
                % info_reader_smart():
                info_fields.info_df = info_df;
                info_fields.info_Tswp = info_Tswp;
                info_fields.info_time = info_time;
                info_fields.info_Nswp = info_Nswp;
                info_fields.infofilename = infofilename;
                info_fields.info_date = info_date;
                info_fields.info_rx = info_rx;
                info_fields.info_cent = info_cent;
                % Call info_reader_smart():
                infoStructSmart = info_reader_smart(info_fields);
                %infoStructSmart.mapWidth; % Non declination compensated width of the map in decimal arc minutes as read from the SMART info file, scalar
                %infoStructSmart.mapHeight; % Height of the map in decimal arc minutes as read from the SMART info file, scalar
                if(infoStructSmart.mapWidth >= 30 && infoStructSmart.mapHeight >= 30) % Not partial map, probably
                    plottingList_new = vertcat(plottingList_new, {infofilename});
                else
                    num_removed = num_removed + 1;
                end
            catch
                fprintf(1, ['Sunmap: Remove partial maps: problem reading file: ' infofilename '\n']);
                continue
            end
            
            % If something else, don't do anything
        end
        % Update user feedback
        user_feedback = ['Processing files: ', num2str(idx), ' / ', num2str(size(plottingList,1))];
        set(mainHandles.text39, 'String', user_feedback);
        set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);
        drawnow update;
        fclose(finfo);
    end
    set(mainHandles.listbox1,'String',plottingList_new);
    set(mainHandles.listbox1, 'Value', 1);
    %plottingList_new;
    user_feedback = [num2str(num_removed), ' partial maps removed from the list'];
end

set(mainHandles.text39, 'String', user_feedback);
set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

end