%% Function for loading sun-measure "2.0" metadata and returning it in a struct
function struct = get_sunmap_metadata(filename)

    fid = fopen(strrep(filename, '.map', '.metadata'));
   
    struct.cosCorrect = 1; % Cos-corrected data is used
    struct.observer_name = 'N/A';
    struct.data_integrated = 0;    
    
    tline = fgetl(fid);
    while ischar(tline)
        %disp(tline)

        % Split line by "=":
        line = strsplit(tline, '=');

        if(strcmp(line(1,1), 'maptype'))
            line_temp = line(1,2);
            struct.mapType = line_temp{1};
        elseif(strcmp(line(1,1), 'freq'))
            line_temp = line(1,2);
            struct.freq = line_temp{1};
        elseif(strcmp(line(1,1), 'id'))
            line_temp = line(1,2);
            struct.obstime = line_temp{1};
        elseif(strcmp(line(1,1), 'mapsweepwidth'))
            line_temp_name = line(1,1);
            var_name_str = line_temp_name{1};
            line_temp_val = line(1,2);
            val_str = line_temp_val{1};
            struct.(var_name_str) = val_str;
            sweepwidth = line(1,2);
            struct.sweepwidth = str2num(sweepwidth{1});
        elseif(strcmp(line(1,1), 'mapyradius'))
            line_temp_name = line(1,1);
            var_name_str = line_temp_name{1};
            line_temp_val = line(1,2);
            val_str = line_temp_val{1};
            struct.(var_name_str) = val_str;
            line_temp = line(1,2);
            struct.map_height = 2*str2num(line_temp{1});
        elseif(strcmp(line(1,1), 'mapxradius'))
            line_temp_name = line(1,1);
            var_name_str = line_temp_name{1};
            line_temp_val = line(1,2);
            val_str = line_temp_val{1};
            struct.(var_name_str) = val_str;
            line_temp = line(1,2);
            struct.map_width = 2*str2num(line_temp{1});
        elseif(strcmp(line(1,1), 'observer'))
            line_temp = line(1,2);
            struct.observer_name = line_temp{1};
        else % Read all other variables:
            line_temp_name = line(1,1);
            var_name_str = line_temp_name{1};
            line_temp_val = line(1,2);
            val_str = line_temp_val{1};
            struct.(var_name_str) = val_str;
        end

        tline = fgetl(fid);
    end
       
    struct.n_sweeps = struct.map_height / struct.sweepwidth;
    %struct.sps = struct.map_width / struct.sweepwidth;
    fclose(fid);

    % data file (start and end times)
    data_file = dlmread(filename, ' ');
    struct.start_time_unix = data_file(1,1); % start time
    struct.end_time_unix = data_file(end,1); % end time
    
    % Get minimum and maximum map dimensions in dec, ra, cosine corrected ra, el, az and
    % cosine corrected az:
    % Format: t dAz dEl dAzCC dRA dDec dRACC LinV LogV
    struct.d_az_min = min(data_file(:,2));
    struct.d_az_max = max(data_file(:,2));
    struct.d_el_min = min(data_file(:,3));
    struct.d_el_max = max(data_file(:,3));
    struct.d_az_cc_min = min(data_file(:,4));
    struct.d_az_cc_max = min(data_file(:,4));
    struct.d_ra_min = min(data_file(:,5));
    struct.d_ra_max = max(data_file(:,5));
    struct.d_dec_min = min(data_file(:,6));
    struct.d_dec_max = max(data_file(:,6));
    struct.d_ra_cc_min = min(data_file(:,7));
    struct.d_ra_cc_max = max(data_file(:,7));
    
    start_time_val = struct.start_time_unix / 86400 + datenum('Jan-01-1970 0:0:0');
    start_clock_str = datestr(start_time_val, 13);
    start_date_str = datestr(start_time_val, 29);
    end_time_val = struct.end_time_unix / 86400 + datenum('Jan-01-1970 0:0:0');
    end_clock_str = datestr(end_time_val, 13);
    struct.time_str = [start_date_str, ', ', start_clock_str, ' - ', end_clock_str];

end