%% Function  gui_map_dimensions()
% Creates 2 decimal rounded and cosine corrected solar map data matrix strings
% with angular width/height to be used for printing to the main Sunmap GUI

% Input:
% mapWidth: Non declination compensated (= non cosine corrected) angular solar map data matrix width, arc min, scalar
% mapHeight: Angluar solar map data matrix height, arc min, scalar
% cosCorrect: Boolean about doing or not doing cosine correction for the width (cos correction can be done beforehand by sun-measure)
% time_str: Time of the observation, string (e.g. '2011-02-17, 09:53:32 - 10:02:18')

% Output:
% mapWidth_print: Cosine corrected angular solar map data matrix width, arc min, rounded to 2 decimals, string
% mapHeight_print: Angular height of solar map data matrix, arc min, rounded to 2 decimals, string

function [mapWidth_print, mapHeight_print] = gui_map_dimensions(mapWidth, mapHeight, cosCorrect, time_str)

    % Calculate declination compensated data width for viewing in main
    % GUI:
    sunData = sun_data(time_str);
    if(cosCorrect == 0)
        mapWidth_print = mapWidth * cosd(sunData.dec);
        mapWidth_print = num2str(round(100 * mapWidth_print) / 100); % Round to 2 decimal accuracy, convert to string
    else
        mapWidth_print = num2str(round(100 * mapWidth) / 100);
    end
    mapHeight_print = num2str(round(100 * mapHeight) / 100);

end