%% info_reader_smart() function:
% Reads parameters from a SMART .info file and returns them in a
% struct
% Recognizes the map type (sweep direction) of the SMART map by comparing
% the coordinates in the info file
% Input:
% - A struct (info_fields) containing:
    % - infofilename: Name of the info file, string
    % - info_df: Data file name, string
    % - info_time: The values in the time column in the SMART info file, vector
    % - info_Nswp: Number of sweeps as read from the SMART info file, cell array
    % - info_Tswp: Duration of a single sweep, cell array
    % - info_date: Start time of the observation, including date, (e.g. '2009-08-06+  6:31:53.660'), string
    % - info_rx: The frequency of the receiver used, (e.g. '37 GHz'), string
    % - info_cent: The ra-dec coordinates of the center of the Sun, (e.g. '+  9: 5:37.922  + 16:38:35.630'), cell array
% Output: parameters read from the .info file in a struct
% See details about output from the end of this file

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function infoStructSmart = info_reader_smart(info_fields)

infofilename = info_fields.infofilename;
info_df = info_fields.info_df;
info_time = info_fields.info_time;
info_Nswp = info_fields.info_Nswp;
info_Tswp = info_fields.info_Tswp;
info_date = info_fields.info_date;
info_rx = info_fields.info_rx;
info_cent = info_fields.info_cent;

% End time for observation:
if info_Nswp{1} == 43

    %mapHeight = 'foo'; % Not needed, this is just to keep matlab happy
    %mapWidth = 'foo';
    sec_time = info_time(43); % The last value of the time-column of
                                             % the info-file
end


%if info_Nswp{1} == 29
 %   mapHeight = 'foo'; % Not needed, this is just to keep matlab happy :)
 %   mapWidth = 'foo';
 %   sec_time = info_time(29); % The last value of the time-column of
                                             % the info-file
%else
% ARBITRARY SWEEEP NUMBER AND SAMPLES PER SWEEP:

% Finding out the starting row of the declination and other data:
info_all = textread(infofilename, '%s', 'delimiter', '\n');
info_numrows = length(info_all);
%aux_number2 = length(info_time) - 18 % aux_number was 10 before
last = info_numrows - 1;
info_startrow = info_numrows - info_Nswp{1};


% -------------------------------------------------------------------------------------------------
% Testing RA/DE-map indetification:

% Get all columns for RA & DEC:
info_column1 = dlmread(infofilename, '', [info_startrow 0 last 0]);
info_column2 = dlmread(infofilename, '', [info_startrow 1 last 1]);
info_column3 = dlmread(infofilename, '', [info_startrow 2 last 2]);
info_column4 = dlmread(infofilename, '', [info_startrow 3 last 3]);

% Set values to width and heigh variables as 0:
mapHeight = 0;
mapWidth = 0;
mapType = 'foo';

% Test whether sweeps are made in RA or DEC direction, determine data width & height:
if (abs(info_column1(1) - info_column3(1)) < 0.000002) % 0.000002 hours is 0.1080 arc seconds. The condition is there to dismiss minor noise in the values.

    mapType = 'demap';
    % Width:
    mapWidth = abs(15*60*(info_column1(1) - info_column1(end)));
    % Height:
    for k = 1 : length(info_column1)
        info_heights(k) = info_column2(k) - info_column4(k);
    end
    mapHeight = abs(60*( mean(info_heights) ));

elseif(abs(info_column2(1) - info_column4(1)) < 0.00002) % 0.00002 degrees is 0.072 arc seconds, in practise nothing. This is here to dismiss minor noise in the values.

    mapType = 'ramap';
    % Height:
    mapHeight = abs((60*(info_column2(1) - info_column2(end))));
    % Width:
    for k = 1 : length(info_column1)
        info_widths(k) = info_column1(k) - info_column3(k);
    end
    mapWidth = abs(15*60*( mean(info_widths) ));

end

%         % Test columns to see which changes between sweeps, RA or DEC:
%         
%         if( ((info_column1(1) - info_column1(end)) ~= 0) && (info_column3(1) - info_column3(end) ~= 0) )
%             % --> Right ascension has changed between sweeps --> de-map
%             
%             mapType = 'demap'
%             
%             % Width:
%             mapWidth = abs(15*60*(info_column1(1) - info_column1(end)))
%             
%             % Height:
%             for k = 1 : length(info_column1)
%                 info_heights(k) = info_column2(k) - info_column4(k);
%             end
%             mapHeight = abs(60*( mean(info_heights) ))
%             
%         end
% 
%             
%         if( ((info_column2(1) - info_column2(end)) ~= 0) && (info_column4(1) - info_column4(end) ~= 0) )
%             % --> Declination has changed between sweeps --> ra-map
%             
%             mapType = 'ramap'
%             
%             % Height:
%             mapHeight = abs((60*(info_column2(1) - info_column2(end))))
%             
%             % Width:
%             for k = 1 : length(info_column1)
%                 info_widths(k) = info_column1(k) - info_column3(k);
%             end
%             mapWidth = abs(15*60*( mean(info_widths) ))
%                     
%         end



% / Testing
% -------------------------------------------------------------------------------------------------



info_RA11 = dlmread(infofilename, '', [info_startrow 0 info_startrow 0]);
info_RA21 = dlmread(infofilename, '', [info_startrow 2 info_startrow 2]);
info_DEC = dlmread(infofilename, '', [info_startrow 1 last 1]);

% Data matrix height in arc min:
%mapHeight = 60 * (info_DEC(info_Nswp{1}) - info_DEC(1));
% NON-declination-compensated data matrix width in arc min:
%mapWidth = 15 * 60 * (info_RA21 - info_RA11);
sec_time = info_time(info_Nswp{1}); % The last value of the time-column of
                                                          % the info-file  

%end

dataname = info_df{1};
dataname1 = dataname{1};
if(strcmp(dataname1(1), 't') == 1 || strcmp(dataname1(1), 'T') == 1) % data is a track
    track_start = info_time(info_Nswp{1});
    track_dur = info_Tswp{1};
    sec_time = track_start + track_dur;         
end

end_time_sec = sec_time + info_Tswp{1};
hour   = fix(end_time_sec/3600);
minute = fix( 60 * (end_time_sec/3600 - hour) );
second = end_time_sec - (minute*60) - (hour*3600);

hour = num2str(hour);
minute = num2str(minute);
second = num2str(second);

if str2double(hour) < 10
    hour = ['0', hour];
end

if str2double(minute) < 10
    minute = ['0', minute];   
end

if str2double(second) < 10
    second = ['0', minute];  
end

end_time = [hour, ':', minute, ':', second];

% / End time for observation


%% MODIFYING TITLE AND TIME INTO THEIR FINAL FORM

obstitle = char(info_df{1});
obstitle = regexprep(obstitle, '.bin', '');
obstime  = char(info_date{1});
obstime  = strrep(obstime, '+', ',');
obstime  = regexprep(obstime, '\:\s', '\:0');
obstime  = [obstime, ' - ', end_time];
freq = char(info_rx{1});

%% Changing sun center declination to decimal degrees:

% '+  7:39:36.034  + 21:27:59.988'

cent = cell2mat(info_cent{1});

deg = [cent(19) cent(20)];
min = [cent(22) cent(23)];
sec = [cent(25) cent(26) cent(27) cent(28) cent(29) cent(30)];

dec = (str2double(deg)) + (str2double(min) / 60) + (str2double(sec) / 3600);

%% Write variables into output struct:

infoStructSmart.mapWidth = mapWidth; % Non declination compensated width of the map in decimal arc minutes as read from the SMART info file, scalar
infoStructSmart.mapHeight = mapHeight; % Height of the map in decimal arc minutes as read from the SMART info file, scalar
infoStructSmart.mapType = mapType; % The type of the map; right ascension map ('ramap') or declination map ('demap'), string
infoStructSmart.obstitle = obstitle; % The name of the map file without the file name extension (e.g. 'sun09080603'), string
infoStructSmart.obstime = obstime; % The full observation time of the map, start - end, (e.g. '2009-08-06,  6:31:53.660 - 06:40:31.66'), string
infoStructSmart.freq = freq;  % The frequency of the receiver used, (e.g. '37 GHz', '22 GHz'), string
infoStructSmart.dec = dec; % The declination of the Sun's center at the time of the observation in decimal degrees, scalar

end