%% Function map_edges_cleaner()   
% Sets unusually high data values that are near the edges of
% a solar map data matrix into the minimum value of the whole matrix. Its
% purpose being the deletion of "off sun" data values that are as high or
% almost as high as the values on the solar disk, causing errors in data
% analysis

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function map = map_edges_cleaner(map, mapType, dataSource, nswp, sps)

% Input:
% - map: The solar map data matrix, matrix
% - mapType: The sweep direction of the map, 'ramap' or 'demap', string
% - dataSource, The source program of the data, 'SMART' or 'sm', string
% - nswp, The number of sweeps in the data, scalar
% - sps, The number of samples per sweep in the data, scalar

% Output:
% - map: The "cleaned" map data matrix

% Getting rid of "off-sun data" (reflections and what not):
    
    if(strcmp(mapType, 'ramap') == 1)
        map_rows = nswp;
        map_cols = sps;
    elseif(strcmp(mapType, 'demap') == 1)
        map_rows = sps;
        map_cols = nswp;
    end
    
    % 29 sweep SMART RA-map:
    if(strcmp(dataSource, 'SMART') == 1 && strcmp(mapType, 'ramap') == 1 && sps == 91)
        for j=1:map_rows % going through rows
             for g=1:map_cols % going through columns
                if(g < 12 || g > 81) 
                     map(j,g) = min(min(map));
                end
             end
        end
    end

    % 43 sweep SMART DE-map:
    if(strcmp(dataSource, 'SMART') == 1 && strcmp(mapType, 'demap') == 1 && nswp == 43)
         for j=1:map_cols % going through columns (samples)
             for g=1:map_rows % going through rows (sweeps)
                if(g < 5 || g > 79) 
                     map(j,g) = min(min(map));
                end
             end
         end
    end
   