% This file contains functions that are used for saving average temperatures of
% solar map polar areas into .txt files
% This whole file is a modified copy of spot_saver.m which is a modified copy of sunmap_contourplot.m

% This function goes through a list of solar map (and track-, but does
% nothing) info files, digs the basic info out of the maps, finds the
% center of the sun in the data, calls a funtion (polar_sniffer.m) to find an
% average reading of polar areas on the map and saves the info into a text file.

% (((((spot_saver() calls polarFinder() in a for-loop so that all files on a
% given list are handled. polarFinder() finds the polar average brightness of a single
% map at a time and writes the data into a file.)))))

function polar_saver(opt, sunmap_handles)

mainHandles = guidata(sunmap_handles);
if(opt.img_save == 1 || opt.fig_save == 1)
    savingHandles = guidata(opt.savingHandles);
end

% Ask target directory for files:
opt.polarFileDir = uigetdir(pwd, 'Select destination folder for polar average data files (1 file per year)');
% If user presses Cancel, nothing happens:
if(opt.polarFileDir == 0)
    return;
end

% Read data unit selection from the GUI
if(get(mainHandles.radiobutton12, 'Value') == 1)
    opt.dataUnits = 0; % % of QSL (default)
elseif(get(mainHandles.radiobutton13, 'Value') == 1 && saveToWeb == 0)
    opt.dataUnits = 1; % Original units
else
    opt.dataUnits = 0; % % of QSL (default)
end

%% File path(s) from which the .info and .bin raw data are read

% 'dbase': The starting part of the filepath. You can leave this empty if
% the whole path is in the opt.plottingList-list part of the path.
dbase = '';

%%  Reading plot info from the individual .info-files:

for fnum=1:size(opt.plottingList,1)
    
    infofilename = [dbase opt.plottingList{fnum}];
    
    finfo = fopen(infofilename,'rt');
    if (finfo < 0)
        fprintf(1, ['Could not open file ' infofilename '\n']);
        continue
    end
    
 % SUN-MEASURE .suninfo1-file:
        % ------------------------------------------------
    if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8) % strfind returns the index of the start of the string found within the string from which we're searching 
   
        % Call info_reader() that returns a struct containing the
        % neccessary variables in a struct:
        infoStruct = info_reader(infofilename);

        % Write stuff from struct to individual variables:
        n_sweeps = infoStruct.n_sweeps;
        startdec = infoStruct.startdec;
        enddec = infoStruct.enddec;
        startra= infoStruct.startra;
        endra = infoStruct.endra;
        obstime = infoStruct.obstime;
        opt.sweepDir = infoStruct.sweepDir;
        opt.sampleDir = infoStruct.sampleDir;
        opt.cosCorrect = infoStruct.cosCorrect;
        obs_mode = infoStruct.obs_mode;
        mapType = infoStruct.mapType;
        observer_name = infoStruct.observer_name;
        freq = infoStruct.freq;
        data_integrated = infoStruct.data_integrated;

        dataSource = 'sm';
        
        %data_integrated = 0; % This variable tells the difference between a a file with data incorporated in the same file with the info-stuff and a "standalone" suninfo-file
                                         % The default value is set here.
                                         % If an integrated file is
                                         % encountered later, the value is
                                         % set to 1
        
        


        % Data height in arc-min
        mapHeight = 60 * (enddec - startdec);

        % NON-declination- compensated width in arc-min:
        mapWidth = 60 * abs(endra - startra);    

        time_str = infoStruct.time_str; % Start and end time for data collection


        obstitle = regexprep(infofilename, '.suninfo1', ''); % /home/foo/bar/plaa/plaa/1284623590
        for i = 0:length(obstitle)

            if(strcmp(obstitle(end - i), '/') == 1 || strcmp(obstitle(end - i),'\') == 1)
                break;
            end
            obstitle1(i + 1) = obstitle(end - i);

        end
        obstitle = '';
         for i = 0:(length(obstitle1) - 1)
          obstitle(i + 1) = obstitle1(end - i);     
         end

        % / sun-measure .suninfo file
        % ------------------------------------------------
    
    % SMART .info-file:
    % ------------------------------------------------
    else
        
        dataSource = 'SMART';
        
        info_struct = get_sunmap_metadata_smart(finfo);
        
%         info_name = textscan(finfo, 'OBSERVER NAME    = %s', 'delimiter','\n');
%         info_rx   = textscan(finfo, 'RECEIVER USED    = %s', 'delimiter','\n');
%         info_df   = textscan(finfo, 'NAME OF DATAFILE = %s', 'delimiter','\n');
%         info_date = textscan(finfo, 'DATE             = %s', 'delimiter','\n');
%         info_cent = textscan(finfo, 'CENTER OF SUN AT = %s', 'delimiter','\n');
%         info_sps  = textscan(finfo, 'SAMPLES PER SWEEP= %n', 'delimiter','\n');
%         info_Tswp = textscan(finfo, 'DURATION OF SWEEP= %n', 'delimiter','\n');
%         info_Nswp = textscan(finfo, 'PERFORMED SWEEPS = %d', 'delimiter','\n');
        % Determine from which row the data starts:
        info_all = textread(infofilename, '%s', 'delimiter', '\n');
        info_numrows = length(info_all);
        info_startrow = info_numrows - info_struct.info_Nswp{1};

        % Read time info from info file sweep data:
        try
            info_time = dlmread(infofilename, '', info_startrow, 4);
        catch err
            
            % Recongnise an error related to missing sweep data in info
            % file and jump to next file in list:
            if (strcmp(err.identifier, 'MATLAB:textscan:BuildFormatString'))
                continue;
            else % With other errors just print the error as usual:
                rethrow(err);
            end
        end
        
    %     if length(info_time) < 29
    %         info_time = dlmread(infofilename, '', 10, 4);    
    %     end
        opt.sweepDir = 'default'; % This has to be set here, because "sweepDir" is used with sun-measure data
        opt.sampleDir = 'default'; % This has to be set here, because "sampleDir" is used with sun-measure data
        opt.cosCorrect = 0; % Default value, cosine correction not done in data --> has to be done in Sunmap
    
    end
    % ------------------------------------------------
    
    fclose(finfo);
    %%
    
    % SUN-MEASURE DATA:
    if(length(infofilename) - strfind(infofilename, '.suninfo1')  == 8)
        
        % Checking whether to look for linear or log scale data:
        
        % Linear scale (Linear checked, Log unchecked):
        if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
        
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end
            
        % Log scale (Linear unchecked, Log checked):
        elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
        
            % We're looking for a _log.txt file or a _log.matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '_log.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with _log.matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '_log.matrix0');
            end            
            
        else % If something funny is going on, i.e. both the menu items in the GUI are checked or unchecked at the same time, resort to linear data:
            
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end            
            
        end
        
        %dec = sun_declination(time_str);
        sunData = sun_data(time_str);
        dec = sunData.dec;
        
        freq = '37 GHz'; % NOTE: This is fixed and not based on anything but a guess!!
        obstitle = 'foobar'; % To be determined
        
    % If .matrix0 datafile is not found, in the case that we need a separate data file (data_integrated == 0)):
    s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
    if((exist(dataFileName, 'file') == 0 || s.bytes < 60) && data_integrated == 0)
        
        continue; % Data file problem --> jump to next map on list
        
    else % Data file exists and there is something in it:        
        
        % Data is from separate .txt-file:
        if(data_integrated == 0)
            mapdata = dlmread(dataFileName);
        end
        % If the data is from an integrated file with info and data
        % together, the data has already been read earlier
        
        map_dimensions = size(mapdata);
        sps = map_dimensions(2); % Samples per sweep.
        
    end
       
        
    % SMART DATA:
    else % This goes on for about 300 lines...

        % Write variables into info_fields struct for use with
        % info_reader_smart():
        info_fields.info_df = info_struct.info_df;
        info_fields.info_Tswp = info_struct.info_Tswp;
        info_fields.info_time = info_time;
        info_fields.info_Nswp = info_struct.info_Nswp;
        info_fields.infofilename = infofilename;
        info_fields.info_date = info_struct.info_date;
        info_fields.info_rx = info_struct.info_rx;
        info_fields.info_cent = info_struct.info_cent;
        % Call info_reader_smart():
        infoStructSmart = info_reader_smart(info_fields);
        % Write output from info_reader_smart() to variables:
        mapWidth = infoStructSmart.mapWidth; % Non declination compensated width of the map in decimal arc minutes as read from the SMART info file, scalar
        mapHeight = infoStructSmart.mapHeight; % Height of the map in decimal arc minutes as read from the SMART info file, scalar
        mapType = infoStructSmart.mapType; % The type of the map; right ascension map ('ramap') or declination map ('ramap'), string
        obstitle = infoStructSmart.obstitle; % The name of the map file without the file name extension (e.g. 'sun09080603'), string
        obstime = infoStructSmart.obstime; % The full observation time of the map, start - end, (e.g. '2009-08-06,  6:31:53.660 - 06:40:31.66'), string
        freq= infoStructSmart.freq;  % The frequency of the receiver used, (e.g. '37 GHz', '22 GHz'), string
        dec = infoStructSmart.dec; % The declination of the Sun's center at the time of the observation in decimal degrees, scalar

    %% READING THE SUNMAP RAW-DATA (.BIN-FILE) INTO A MATRIX.
    
    % OLD WAY OF READING .BIN-FILES: (NOT SUPPORTED BY THIS PROGRAM)
    % cat SUN03071620.BIN.1 | od -v -t d2 -w$(expr 91 \* 2) |cut --bytes=8-|less -S
    % run this script under Unix/Linux to convert binary form to acsii form
    
    % Creating the .bin -files name from .info-files name:
    if(strcmp(infofilename(end-3), 'i') == 1)
        binfilename = strrep(infofilename, '.info', '.bin');       
    elseif((strcmp(infofilename(end-3), 'I') == 1))
        binfilename = strrep(infofilename, '.INFO', '.BIN');
    end
    
    % If .bin-file is not found:
    if(exist(binfilename, 'file') == 0)
        set(mainHandles.text39, 'ForegroundColor', 'r');
        set(mainHandles.text39, 'String', '.bin-file not found');
        if(ispc)
            set(mainHandles.text39, 'FontSize', 8);
        end
        set(mainHandles.plot_single_button, 'Enable', 'on');
        set(mainHandles.plot_selected, 'Enable', 'on');
        set(mainHandles.plot_all_button, 'Enable', 'on');
        set(mainHandles.listbox1, 'Enable', 'on');
        set(mainHandles.pushbutton21, 'Visible', 'off');
        
        continue;
    end
    
    % Calling read_sun_data_ultimate() that reads the .bin-file.
    % (Can be found in read_sun_data_ultimate.m. 'ultimate' comes from the
    % ability to read any kind of an array from a .bin-file)
    [mapdata] = read_sun_data_ultimate(binfilename, info_struct.info_Nswp{1}, info_struct.info_sps{1});
    %mapType = 'foobar'; % With SMART data maptype (ramap, demap) is derived from the number of sweeps and samples/sweep
    
    end % END OF ELSE-CLAUSE FOR SMART-DATA
       
   %% Calling polarFinder()
   
    if(fnum == 1)
        opt.fig_count = length(findobj('Type','figure')) - 1;
    end

    % SUN-MEASURE DATA:
    if(strcmp(dataSource, 'sm') == 1)
    
        if(strcmp(obs_mode, 'track') == 1 || n_sweeps == 1)
            % Data is a track --> do nothing
        elseif(strcmp(obs_mode, 'map') == 1)
            % Data is a map:
            polarFinder(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, opt, sunmap_handles, mapType, dataSource);
        end
        
        
    % SMART DATA:
    elseif(strcmp(dataSource, 'SMART') == 1)
        
        if(strcmp(obstitle(1), 't') == 1 || strcmp(obstitle(1), 'T') == 1)
            % Data is a track --> do nothing
        else
            % Data is a map:
            polarFinder(dec, freq, info_struct.info_Nswp{1}, info_struct.info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, opt, sunmap_handles, mapType, dataSource);
        end
    
    end
    
end

end % End of spotSaver() function



%% !!!!----------- polarFinder() -function starts -----------!!!!

function polarFinder(dec, freq, nswp, sps, map, num, time, obstitle, h, w, opt, sunmap_handles, mapType, dataSource)

destFolder = opt.destFolder;
beam_size = opt.beam_size;
solar_north = opt.solar_north;
grid_toggle = opt.grid_toggle;
%cmap = opt.cmap;
abs_scale = opt.abs_scale;
c_min = opt.min;
c_max = opt.max;
mapPlottingType = opt.mapType; % opt.mapType (that tells how the map is plotted: filled contours, lines, etc.) IS NOT THE SAME AS the variable mapType (ra-map or de-map)!!!!!
customNumOfLevelsToggle = opt.customNumOfLevelsToggle ;
customNumOfLevels = opt.customNumOfLevels;
sweepDir = opt.sweepDir;
sampleDir = opt.sampleDir;
polarFileDir = opt.polarFileDir;

%     sunPlot() function info:
%
%       PARAMETERS:
%
%           - dec: declination of the Suns center (for calculating round
%             shape for Sun). E.g: 21,73563 (scalar)
%
%           - opt.destFolder: destination for the output .png-images and .fig
%             files. E.g: '/home/user/sun/plots/' (string)
%
%           - freq: The frequency of the observation. E.g: '37 GHz'
%             (string)
% 
%           - nswp: Number of sweeps in data. E.g: 29 (scalar)
%
%           - sps: Samples per sweep in data. E.g: 91 (scalar)
%
%           - map: The data matrix. E.g: of size nswp x sps samples
%
%           - num: A running number that is used to determine how many
%             figures have been printed (scalar)
% 
%           - time: The observation date (start and stop).
%             '2009-07-14,  7:25:27.620 - 07:33:51.62' (string)
% 
%           - obstitle: The plain name of the observation without any
%             filename extention. E.g: 'sun09071506' (string)
%
%           - h: The angular height of the data (in arc min), (scalar)
%            
%           - w: The NON DECLINATION-CORRECTED angular width of the data
%             (in arc opt.min) 
%           
%           - beam_size: The option to enable or disable the printing of
%             beam size on the map
%
%           - solar_north: The option to enable or disable the printing of
%             an arrow illustrating the direction of solar north on the map
%
%           - grid_toggle: The option to enable or disable the printing of
%             a grid on the plot (scalar, 1 = 'on' or 0 = 'off')
%
%           - cmap: The string from the GUI that defines the colormap
%             (string)
%
%           - abs_scale: The option to enable or disable user given
%             absolute levels-values (scalar, 1 = 'on' or 0 = 'off')
%
%           - c_min: User given minimum value from which plotting is
%             started
%
%           - c_max: User given maximum value up to which the plotting is
%             done
%
%           - sunmap_handles: The handles-structure of the sunmap -gui
%             window. The status messages on the gui can be set with the
%             help of this handles
%

% Recognizing the frequency-string:
    if(isempty(regexpi(freq,'37')) == 0)
        freq = '37 GHz';
    end
    
    if(isempty(regexpi(freq,'87')) == 0)
        freq = '87 GHz';
    end
    
    if(isempty(regexpi(freq,'22')) == 0)
        freq = '22 GHz';
    end



    %fprintf('sunPlot being called. Plotting the map...\n')
    mainHandles = guidata(sunmap_handles);
    if(isfield(opt, 'savingHandles') == 0) % savingHandles does not exist ==> sunmap_contourplot was not called by saving.m ==> "save to web" not in use
        saveToWeb = 0;
    else
        savingHandles = guidata(opt.savingHandles);
        saveToWeb = get(savingHandles.checkbox5, 'Value');
    end
    sunmap_status_text = mainHandles.text39;
    set(mainHandles.text39, 'ForegroundColor', 'blue');
    
    if(opt.img_save == 1 || opt.fig_save == 1)
        %set(sunmap_status_text, 'String', 'Saving...');
    else
        size_of_list= size(get(mainHandles.listbox1, 'Value'));
        progress_str = ['Saving polar average data: ', num2str(num), '/', num2str(size_of_list(2)),' ...'];
        set(sunmap_status_text, 'String', progress_str);
        drawnow update;
    end
    
    set(mainHandles.plot_single_button, 'Enable', 'off');
    set(mainHandles.plot_selected, 'Enable', 'off');
    set(mainHandles.plot_all_button, 'Enable', 'off');
    set(mainHandles.listbox1, 'Enable', 'off');
    set(mainHandles.pushbutton21, 'Visible', 'on');
    % For checking the value of nswp (number of sweeps in data):
    %fprintf('The number of sweeps given to sunPlot is: %d\n', nswp)
    
     %% Skipping partial maps:
    if(w < 30 || h < 30)
        return;
    end
    
    %% Map operations:
    
    % Temporary:
    % Set "overbright" values to map max value if sun-measure data:
    if(strcmp(dataSource, 'sm') == 1)
        map_size_temp = size(map);
        for ind_j = 1 : map_size_temp(1)
            for ind_g = 1 : map_size_temp(2)
                if map(ind_j,ind_g) < -30000
                     map(ind_j,ind_g) = max(max(map));
                end
            end
        end        
    end
    % / Temporary
    
    % Normalizing data so that minimum is at zero:
    map = map - min(min(map));
    
    % Calling qsl_finder() to find Quiet Sun Level of the map:
    dataUnits = 0; % "% of QSL" units in use
    quiet_sun = qsl_finder(map, dataUnits);
          
    % Calling qsl_units_transformer() to transform map units from
    % original to '% of QSL' units:
    map = qsl_units_transformer(map, quiet_sun,freq);
   
    max_temp = max(max(map));
    min_temp = min(min(map)); 
    
    %% Map levels-operations: 
    
    % protection from invalid custom levels values:
    if(get(mainHandles.checkbox31, 'Value') == 1)
        if(str2num(get(mainHandles.edit10, 'String'))) >= str2num(get(mainHandles.edit11, 'String'))
            message = ['The minimum custom levels value (', get(mainHandles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(mainHandles.edit11, 'String'), ').'];
            msgbox(message, 'Warning', 'warn', 'modal');
        return;
        end
        if(c_min >= max_temp)
            message = ['The minumun custom levels value (', num2str(c_min), ') has to be lower than the maximum value (', num2str(max_temp), ') of the selected map.'];
            msgbox(message, 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit10, 'String')))
            msgbox('Please insert a number value to the minimum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit11, 'String')))
            msgbox('Please insert a number value to the maximum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
    end
    
    %% Getting rid of "off-sun data" (reflections and what not):

    map = map_edges_cleaner(map, mapType, dataSource, nswp, sps);
    
    %% Data size vs. grid, map rotation:
    
   % Make sure variables are double:
    nswp = double(nswp);
    sps = double(sps);
    w = double(w); 
    h = double(h);
    
    % Write variables into struct for calling map_handler():
    
    % Common variables:
    mhStructIn.map = map;
    mhStructIn.dataSource = dataSource;
    mhStructIn.mapType = mapType;
    mhStructIn.nswp = nswp;
    mhStructIn.sps= sps;
    mhStructIn.w = w;
    mhStructIn.h = h;
    mhStructIn.dec = dec;

    % sun-measure specific variables:
    if(strcmp(dataSource, 'sm') == 1)
        mhStructIn.sweepDir =  sweepDir;
        mhStructIn.sampleDir = sampleDir;
        mhStructIn.cosCorrect = opt.cosCorrect;
    end

    % Call map_handler():
    mhStructOut = map_handler(mhStructIn);

    % Write variables from map_handler() output struct:
    map = mhStructOut.map;
    x = mhStructOut.x;
    y = mhStructOut.y;
    x_fov =mhStructOut.x_fov;
    y_fov = mhStructOut.y_fov;
    mapWidth = mhStructOut.mapWidth;
    mapHeight = mhStructOut.mapHeight;
    nswp = mhStructOut.nswp;

    %% Convert numbers to strings for printing on the contour map
    max_temp_str = num2str(round(max_temp * 100) / 100);
    
    %% (Plotting the figure), tweaking the settings:
    
    sunData = sun_data(time);
    rotDeg = sunData.rot;
    B0 = sunData.B0;
    L0 = sunData.L0;
    %solar_radius = sunData.radius;
    
    % Making a copy of the original map data and size vectors:
    map_orig = map;
    x_orig = x;
    y_orig = y;
        
    [foo, xSize] = size(x);
    [foo, ySize] = size(y);
    [dataHeight, dataWidth] = size(map);
    % Resizing the data matrix
    % By choosing 9 as the scaling factor the original data points
    % can be preserved:
    % The resized data matrix is only used in finding the solar disk's
    % center
    % Active areas and their parameters are searched from the original
    % data matrix
    if(dataHeight >= dataWidth) % DE-map
        map = imresize(map, [dataHeight 9*dataWidth], 'bilinear');
        x = imresize(x, [1 9*xSize], 'bilinear');               
    else % RA-map
        map = imresize(map, [9*dataHeight dataWidth], 'bilinear');
        y = imresize(y, [1 9*ySize], 'bilinear');
    end
    %end
    
    fig_number = num;
    while(isempty( find( get(0, 'children') == fig_number) ) == 0)
        fig_number  = fig_number + 1;
    end
    
    %% FINDING THE SUN'S CENTER:

    c_opt.x = x;
    c_opt.y = y;
    c_opt.rotate_disable = opt.rot_disable;
    c_opt.rotDeg = rotDeg;

    % Calling center_finder():
    center_info = center_finder(map, c_opt);

    % Save non-rotation compensated center coordinates:
    hCenterCoord_orig = center_info.hCenterCoord_orig;
    vCenterCoord_orig = center_info.vCenterCoord_orig;

    % Compensate center coordinates for map rotation:
    if(opt.rot_disable == 0) % map rotated => compensation needed. These were used for plotting
       hCenterCoord = center_info.hCenterCoord_rot;
       vCenterCoord = center_info.vCenterCoord_rot;
    end
    
    
     %% Finding average temperatures of polar areas and saving info into file(s):
    
    % Calculate Az/El for map:
    azelTime = regexprep(time, '-', '/'); % Az / El texts:
    azelTime = regexprep(azelTime, ',', '');
    [Az, El] = SolarAzEl(azelTime, 60.217809153, 24.39311053, 60.28); % Time: YYYY/MM/DD hh:mm:ss, Lat, Long, Height
    az_round = round(Az*100)/100; % 2 decimal accuracy
    el_round = round(El*100)/100; % 2 decimal accuracy
    
    spotCheck = 1; % To be replaced with a checkbox in main GUI
    %tempTreshold = 104;
    opt.mapWidth = mapWidth;
    opt.mapHeight = mapHeight;
    opt.max_temp = max_temp;
    opt.map_orig = map_orig;
    opt.rotDeg = rotDeg;
    opt.hCenterCoord = hCenterCoord_orig; % !!
    opt.vCenterCoord = vCenterCoord_orig; % !!
    opt.radius = sunData.radius;
    opt.x_orig = x_orig;
    opt.y_orig = y_orig;
    opt.B0 = B0;
    opt.L0 = L0;
    opt.az = az_round;
    opt.el = el_round;
       
    % Calculate the average temperatures of both polar regions separetly and combined:
    [polar_average, north_average, south_average] = polar_average_temperature(opt);
    
    year1 = [time(1), time(2), time(3), time(4)];

    % Create a decimal year time:
    time_year_str = [time(1), time(2), time(3), time(4)];
    time_month_str = [time(6), time(7)];
    time_day_str = [time(9), time(10)];
    %time_hour_str  = [time(13), time(14)];
    %time_minute_str =  [time(16), time(17)];
    %time_second_str = [time(19), time(20)];

    % decyear(YYYY,MM,DD,hh,mm,ss) = YYYY.NNNNN...
    time_decimal_years = decyear(str2double(time_year_str), str2double(time_month_str), str2double(time_day_str)); % , str2double(time_hour_str), str2double(time_minute_str), str2double(time_second_str)
    % Set number format:
    time_decimal_years = num2str(time_decimal_years, '%.11f'); % 11 decimals

    % polarFileDir is without a trailing '/'
    polarFileName = [polarFileDir, '/MRO_Sun_polar_average.txt'];
    % spotFileName = '/home/nlavonen/aurinkodata/AR_data_2011_2013-04-26/num_of_regions/plaa.csv';
    polar_file = fopen(polarFileName, 'a');
    tab = '\t';
    s = dir(polarFileName); % s.bytes tells the number of bytes in the data file
    if(s.bytes < 5) % If file is empty, write header row:
        fprintf(polar_file, [sprintf('%-40s','Time') tab sprintf('%-25s','Polar avg. (percent QSL)') tab sprintf('%-25s','North avg. (percent QSL)') tab sprintf('%-25s','South avg. (percent QSL)') tab sprintf('%-20s','Elevation (degrees)') tab sprintf('%-20s','B0 angle (degrees)') '\n']);
    end
    time_print = sprintf('%-40s', time_decimal_years);
    id_print = sprintf('%-15s', obstitle);
    freq_print = sprintf('%-15s', freq);
    avg_print = sprintf('%-25s', num2str(polar_average));
    north_print = sprintf('%-25s', num2str(north_average));
    south_print = sprintf('%-25s', num2str(south_average));
    elev_print = sprintf('%-20s', num2str(el_round));
    B0_print = sprintf('%-20s', num2str(B0));
    %TEMP_num_spots_print = sprintf(num2str(length(areaCoords)));

    polarFileContents = textread(polarFileName, '%s', 'delimiter', '\n');
    printString = sprintf([time_print, tab, avg_print, tab, north_print, tab, south_print, tab, elev_print, tab, B0_print, '\n']);
    %printString = sprintf([time_print, ',', TEMP_num_spots_print, ',', maxTemp_print, '\n']);
    % Testing if current string already exists in the file:
    stringChecker = 0;
    for idx4 = 1 : length(polarFileContents)
        if (strcmp(strtrim(polarFileContents{idx4}), strtrim(printString)) == 1) % strtrim removes leading and trailing white spaces
            stringChecker = 1; % The exact same line is already in the file --> no saving at this round
        end
    end

    if(stringChecker == 0 && isnan(polar_average) == 0) % This line does not exist in the file and polar_average is not NaN --> write line in file
        fprintf(polar_file, printString);
    end

    fclose(polar_file);
        
    %% Finishing up with GUI:
    
    mainHandles = guidata(sunmap_handles);
    set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);
    
    if(opt.img_save == 1 || opt.fig_save == 1)
        set(mainHandles.text39, 'String', 'Saving done');
    else
        set(mainHandles.text39, 'String', 'Polar average saving done');
    end
    
    if(ispc)
        set(mainHandles.text39, 'FontSize', 8);
    end
    set(mainHandles.plot_single_button, 'Enable', 'on');
    set(mainHandles.plot_selected, 'Enable', 'on');
    set(mainHandles.plot_all_button, 'Enable', 'on');
    set(mainHandles.listbox1, 'Enable', 'on');
    set(mainHandles.pushbutton21, 'Visible', 'off');

end
    
    
    
  