% Function qsl_units_transformer

% Input:
% - map: solar map matrix in original data units (matrix) 
% - quiet_sun: Quiet Sun Level of the map in normalized (minimum at 0) original units (scalar)
% - freq: observing frequency (string)

% Output:
% - Solar map matrix (map (matrix)) in '% of QSL' units

function map = qsl_units_transformer(map, quiet_sun, freq)

    % TRANSFORMING MAP UNITS FROM mV to Percent (%)
    % Median = 100%
    % minimum = 0%
    % maximum > 100%
    map = (map * 100) / quiet_sun;

    % READJUSTING THE MAP SO THAT THE MINIMUM LEVEL (= SKY TEMP LEVEL) IS
    % SET AT 300 KELVIN, WHICH IS ~3,85 % OF QSL (300 K / 7800 K):
    % 22 GHz: 9000 K (300 / 9000 ~0,033333333)
    % 37 GHz: 7800 K (300 / 7800 ~0,038461538)
    % 86 GHz: 7200 K (300 / 7200 ~0,041666667)

    % Notice: Isn't QSL _without_ athmosphere 7800 K? --> Thus the reading
    % with athomosphere should be 7800 K + 300 K? (2012-03-16)
    % Values then:
    % 22 GHz: 9000 K (300 / 9300 = 0.032258065)
    % 37 GHz: 7800 K (300 / 8100 = 0.037037037)
    % 86 GHz: 7200 K (300 / 7500 = 0,04)
    
    if(strcmp(freq, '37 GHz') == 1)
        map = map + (3.8461538*(100-map) / 100); 
    elseif(strcmp(freq, '87 GHz') == 1)
        map = map + (4.1666667*(100-map) / 100);
    elseif(strcmp(freq, '22 GHz') == 1)
        map = map + (3.333333*(100-map) / 100);
    end

end