% This file contains functions that are used for saving solar radius data
% from solar maps
% This file is a modified copy of spot_saver.m which is a modified copy of sunmap_contourplot.m

function radius_saver(opt, sunmap_handles)


mainHandles = guidata(sunmap_handles);
if(opt.img_save == 1 || opt.fig_save == 1)
    savingHandles = guidata(opt.savingHandles);
end

drawnow update;
pause(0.25); % Wait for 0.25 seconds to give time for the GUI to update

% Create an empty struct which is used for saving radius data:
radiusData = struct('time', [], 'id', [], 'time_obj', [], 'freq', [], 'sky_qsl_diff', [], 'radius_calc', [], 'radius_meas', []);
opt.radiusData = radiusData;

%% File path(s) from which the .info and .bin raw data are read

% 'dbase': The starting part of the filepath. You can leave this empty if
% the whole path is in the opt.plottingList-list part of the path.
dbase = '';

%%  Reading plot info from the individual .info-files:

for fnum=1:size(opt.plottingList,1)
    
    infofilename = [dbase opt.plottingList{fnum}];
    
    finfo = fopen(infofilename,'rt');
    if (finfo < 0)
        fprintf(1, ['Could not open file ' infofilename '\n']);
        continue
    end
    
    sunmap_status_text = mainHandles.text39;
    set(mainHandles.text39, 'ForegroundColor', 'blue');
    size_of_list= size(get(mainHandles.listbox1, 'Value'));
    progress_str = ['Radius analysis (1/2), Processing map: ', num2str(fnum), '/', num2str(size_of_list(2)),' ...'];
    set(sunmap_status_text, 'String', progress_str);
    drawnow update;
    
 % SUN-MEASURE .suninfo1-file:
        % ------------------------------------------------
    if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8) % strfind returns the index of the start of the string found within the string from which we're searching 
           
        % Call info_reader() that returns a struct containing the
        % neccessary variables in a struct:
        infoStruct = info_reader(infofilename);

        % Write stuff from struct to individual variables:
        n_sweeps = infoStruct.n_sweeps;
        startdec = infoStruct.startdec;
        opt.startdec = startdec;
        enddec = infoStruct.enddec;
        opt.enddec = enddec;
        startra= infoStruct.startra;
        opt.startra = startra;
        endra = infoStruct.endra;
        opt.endra = endra;
        obstime = infoStruct.obstime;
        opt.sweepDir = infoStruct.sweepDir;
        opt.sampleDir = infoStruct.sampleDir;
        opt.cosCorrect = infoStruct.cosCorrect;
        obs_mode = infoStruct.obs_mode;
        mapType = infoStruct.mapType;
        observer_name = infoStruct.observer_name;
        freq = infoStruct.freq;
        data_integrated = infoStruct.data_integrated;

        dataSource = 'sm';
        opt.startra = startra;
        opt.endra = endra;
        opt.startdec = startdec;
        opt.enddec = enddec;

        % Data height in arc-min
        mapHeight = 60 * (enddec - startdec);

        % NON-declination- compensated width in arc-min:
        mapWidth = 60 * abs(endra - startra);    

        time_str = infoStruct.time_str; % Start and end time for data collection


        obstitle = regexprep(infofilename, '.suninfo1', ''); % /home/foo/bar/plaa/plaa/1284623590
        for i = 0:length(obstitle)

            if(strcmp(obstitle(end - i), '/') == 1 || strcmp(obstitle(end - i),'\') == 1)
                break;
            end
            obstitle1(i + 1) = obstitle(end - i);

        end
        obstitle = '';
        for i = 0:(length(obstitle1) - 1)
            obstitle(i + 1) = obstitle1(end - i);     
        end

        % / sun-measure .suninfo file
        % ------------------------------------------------
    
    % map-measure .map/.metadata file
    elseif(strfind(infofilename, '.map'))
        map_meta = get_sunmap_metadata(infofilename);
        
    % SMART .info-file:
    % ------------------------------------------------
    else
        
        dataSource = 'SMART';
        
        info_struct = get_sunmap_metadata_smart(finfo);
        
%         info_name = textscan(finfo, 'OBSERVER NAME    = %s', 'delimiter','\n');
%         info_rx   = textscan(finfo, 'RECEIVER USED    = %s', 'delimiter','\n');
%         info_df   = textscan(finfo, 'NAME OF DATAFILE = %s', 'delimiter','\n');
%         info_date = textscan(finfo, 'DATE             = %s', 'delimiter','\n');
%         info_cent = textscan(finfo, 'CENTER OF SUN AT = %s', 'delimiter','\n');
%         info_sps  = textscan(finfo, 'SAMPLES PER SWEEP= %n', 'delimiter','\n');
%         info_Tswp = textscan(finfo, 'DURATION OF SWEEP= %n', 'delimiter','\n');
%         info_Nswp = textscan(finfo, 'PERFORMED SWEEPS = %d', 'delimiter','\n');
        % Determine from which row the data starts:
        info_all = textread(infofilename, '%s', 'delimiter', '\n');
        info_numrows = length(info_all);
        info_startrow = info_numrows - info_struct.info_Nswp{1};

        % Read time info from info file sweep data:
        try
            info_time = dlmread(infofilename, '', info_startrow, 4);
        catch err
            
            % Recongnise an error related to missing sweep data in info
            % file and skip the current map:
            if (strcmp(err.identifier, 'MATLAB:textscan:BuildFormatString'))
                continue;
            else % With other errors just print the error as usual:
                continue;
            end

        end

    %     if length(info_time) < 29
    %         info_time = dlmread(infofilename, '', 10, 4);    
    %     end
        opt.sweepDir = 'default'; % This has to be set here, because "sweepDir" is used with sun-measure data
        opt.cosCorrect = 0; % Default value, cosine correction not done in data --> has to be done in Sunmap

    end
    % ------------------------------------------------
    
    fclose(finfo);
    %%
    
    % SUN-MEASURE DATA:
    if(length(infofilename) - strfind(infofilename, '.suninfo1')  == 8)
        
        % (Checking whether to look for linear or log scale data:)
        % When dealing with the solar radius, only linear scale data is
        % handled, because finding the 3dB / 50 % QSL can be found only
        % from linear data:

        % We're looking for a .txt file or a .matrix0 file:
        dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
        % Check if .txt datafile exists:
        s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
        if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
            dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
        end
                    
        %dec = sun_declination(time_str);
        sunData = sun_data(time_str);
        dec = sunData.dec;
        
        if(exist('freq', 'var') == 0)
             % If the frequency of the observation hasn't been written into
             % the info-file and thus the variable 'freq' doesn't exist, we make a wild guess...
            freq = '37 GHz';
            % Otherwise the requency information is found in the "freq" variable in the form: "NN GHz".
        end
        obstitle = 'foobar'; % To be determined
        
    % If .matrix0 datafile is not found, in the case that we do need a separate data file (data_integrated == 0)):
    s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
    if((exist(dataFileName, 'file') == 0 || s.bytes < 60) && data_integrated == 0)
        %set(mainHandles.text39, 'ForegroundColor', 'r');
        %if(exist(dataFileName, 'file') == 0)
          %  set(mainHandles.text39, 'String', 'data file not found');
            %return;
            
            
            plottinglist_size = size(get(mainHandles.listbox1, 'Value'));
            if(fnum == plottinglist_size(2)) % Current map is the last one on the list and the current map is "bad"
                % --> Plot whatever we have up until now and return:
                track_plotter(sunmap_handles, opt);
                return;
            else
                continue;
            end
        %   else
        %            set(mainHandles.text39, 'String', 'data file is empty');
        %            return;
        %        end
        %        set(mainHandles.text39, 'FontSize', 10);
        %        set(mainHandles.plot_single_button, 'Enable', 'on');
        %        set(mainHandles.plot_all_button, 'Enable', 'on');
        %        set(mainHandles.listbox1, 'Enable', 'on');
        %        set(mainHandles.pushbutton21, 'Visible', 'off');
        
    else
         % Reading the actual map data:
        
        % Data is from separate .txt-file:
        if(data_integrated == 0)
            mapdata = dlmread(dataFileName);
        end
        % If the data is from an integrated file with info and data
        % together, the data has already been read earlier
        
        map_dimensions = size(mapdata);
        sps = map_dimensions(2); % Samples per sweep.
    end
       
    % map-measure data:
    elseif(strfind(infofilename, '.map'))
        
        % mapHeight, mapWidth
        mapHeight = 60*abs(map_meta.d_dec_min - map_meta.d_dec_max);
        mapWidth = 60*abs(map_meta.d_ra_cc_min - map_meta.d_ra_cc_max);
        
    % SMART DATA:
    else
        
        % Write variables into info_fields struct for use with
        % info_reader_smart():
        info_fields.info_df = info_struct.info_df;
        info_fields.info_Tswp = info_struct.info_Tswp;
        info_fields.info_time = info_time;
        info_fields.info_Nswp = info_struct.info_Nswp;
        info_fields.infofilename = infofilename;
        info_fields.info_date = info_struct.info_date;
        info_fields.info_rx = info_struct.info_rx;
        info_fields.info_cent = info_struct.info_cent;
        % Call info_reader_smart():
        infoStructSmart = info_reader_smart(info_fields);
        % Write output from info_reader_smart() to variables:
        mapWidth = infoStructSmart.mapWidth; % Non declination compensated width of the map in decimal arc minutes as read from the SMART info file, scalar
        mapHeight = infoStructSmart.mapHeight; % Height of the map in decimal arc minutes as read from the SMART info file, scalar
        mapType = infoStructSmart.mapType; % The type of the map; right ascension map ('ramap') or declination map ('ramap'), string
        obstitle = infoStructSmart.obstitle; % The name of the map file without the file name extension (e.g. 'sun09080603'), string
        obstime = infoStructSmart.obstime; % The full observation time of the map, start - end, (e.g. '2009-08-06,  6:31:53.660 - 06:40:31.66'), string
        freq= infoStructSmart.freq;  % The frequency of the receiver used, (e.g. '37 GHz', '22 GHz'), string
        dec = infoStructSmart.dec; % The declination of the Sun's center at the time of the observation in decimal degrees, scalar  
        
    %% READING THE SUNMAP RAW-DATA (.BIN-FILE) INTO A MATRIX.
    
        % Creating the .bin -files name from .info-files name:
        if(strcmp(infofilename(end-3), 'i') == 1)
            binfilename = strrep(infofilename, '.info', '.bin');       
        elseif((strcmp(infofilename(end-3), 'I') == 1))
            binfilename = strrep(infofilename, '.INFO', '.BIN');
        end

        % If .bin-file is not found:
        if(exist(binfilename, 'file') == 0)
            set(mainHandles.text39, 'ForegroundColor', 'r');
            set(mainHandles.text39, 'String', '.bin-file not found');
            if(ispc)
                set(mainHandles.text39, 'FontSize', 8);
            else
                set(mainHandles.text39, 'FontSize', 10);
            end
            set(mainHandles.plot_single_button, 'Enable', 'on');
            set(mainHandles.plot_selected, 'Enable', 'on');
            set(mainHandles.plot_all_button, 'Enable', 'on');
            set(mainHandles.listbox1, 'Enable', 'on');
            set(mainHandles.pushbutton21, 'Visible', 'off');

            continue
        end

        % Calling read_sun_data_ultimate() that reads the .bin-file.
        % (Can be found in read_sun_data_ultimate.m. 'ultimate' comes from the
        % ability to read any kind of an array from a .bin-file)
        [mapdata] = read_sun_data_ultimate(binfilename, info_struct.info_Nswp{1}, info_struct.info_sps{1});
    
    end % END OF ELSE-CLAUSE FOR SMART-DATA
    
    
   %% Setting the text in "saving"-window to display the progress of the saving process:
   if(opt.img_save == 1 || opt.fig_save == 1)
       progress_str = ['Saving figure ', num2str(fnum), '/', num2str(size(opt.plottingList,1)), '...'];
       set(savingHandles.text5, 'String', progress_str);
   end
   %% Calling radius_mapHandler

    if(fnum == 1)
        opt.fig_count = length(findobj('Type','figure')) - 1;
    end   

    if(mapHeight >= 30 && mapWidth >= 30)
    
        % SUN-MEASURE DATA:
        if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8)

            if(strcmp(obs_mode, 'track') == 1 || n_sweeps == 1)
                % Data is a track:
                %trackPlot(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), height, width, opt, sunmap_handles);
            elseif(strcmp(obs_mode, 'map') == 1)
                % Data is a map:
                dataDim.foo = 'foo';
                radiusData = radius_mapHandler(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, opt, dataDim, sunmap_handles, mapType, dataSource);
                opt.radiusData = radiusData;
            end


        % map-measure data:
        elseif(strfind(infofilename, '.map'))

            % Checking whether to look for linear or log scale data:

            % Linear scale (Linear checked, Log unchecked):
            if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
                data_col_num = 8;
            % Log scale (Linear unchecked, Log checked):
            elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
                data_col_num = 9;
            else
                data_col_num = 8;
            end

            sunData = sun_data(map_meta.time_str);
            dec = sunData.dec;

            % Create radec matrix from arbitrary sweep data
            gen_map_height = 60*abs(map_meta.d_dec_min - map_meta.d_dec_max);
            gen_map_width = 60*abs(map_meta.d_ra_cc_min - map_meta.d_ra_cc_max);
            gen_sweeps = round(gen_map_height / (60*map_meta.sweepwidth)); % generated sweeps
            gen_sps = round(gen_map_width / 0.24); % generated samples per sweep (0.24' ~ 200 sps for 0.8 degree map)
            mapType = 'ramap';
            opt.sweepDir = 'forward'; % fix this if something breaks
            opt.sampleDir = 'forward'; % fix this if something breaks
            opt.cosCorrect = 1;
            dataSource = 'mm'; % map-measure
    % 
            % Data height in arc-min
            opt.startra = map_meta.d_ra_cc_min; % Check if cc or non cc is needed
            opt.endra = map_meta.d_ra_cc_max;
            opt.startdec = map_meta.d_dec_min;
            opt.enddec = map_meta.d_dec_max;
            opt.mapxradius = map_meta.mapxradius;
            opt.mapyradius = map_meta.mapyradius;
            opt.az_center = map_meta.az_center;
            opt.el_center = map_meta.el_center;
            opt.ra_center = map_meta.ra_center;
            opt.dec_center = map_meta.dec_center;
            
            % Data dimensions:
            dataDim.startra = map_meta.d_ra_cc_min; % Check if cc or non cc is needed
            dataDim.endra = map_meta.d_ra_cc_max;
            dataDim.startdec = map_meta.d_dec_min;
            dataDim.enddec = map_meta.d_dec_max;
    
    %         data_file = dlmread(infofilename, ' ');
    % 
    %         [xi, yi] = meshgrid(linspace(15*map_meta.d_ra_cc_min,15*map_meta.d_ra_cc_max,gen_sps), linspace(map_meta.d_dec_min,map_meta.d_dec_max, gen_sweeps));
    %         zi = griddata(15*data_file(:,7), data_file(:,6), data_file(:,data_col_num), xi,yi); % fit map data to xi,yi grid. (ra,dec,data_lin)
    %         zi(isnan(zi)) = min(min(zi)); % replace nan's with map minimum
              
            radec_matrix = generate_radec_matrix(infofilename, dataDim.startra, dataDim.endra, dataDim.startdec, dataDim.enddec, gen_sps, gen_sweeps, data_col_num);
            
            radiusData = radius_mapHandler(dec, map_meta.freq, gen_sweeps, gen_sps, radec_matrix, fnum, map_meta.time_str, map_meta.obstime, gen_map_height, gen_map_width, opt, dataDim, sunmap_handles, map_meta.mapType, dataSource);
            opt.radiusData = radiusData;
            
        % SMART DATA:
        else

            if(strcmp(obstitle(1), 't') == 1 || strcmp(obstitle(1), 'T') == 1)
                % Data is a track: Nothing is done, because we're looking for
                % maps and radiuses:
                %trackPlot(dec, freq, info_struct.info_Nswp{1}, info_struct.info_sps{1}, mapdata, fnum, obstime, obstitle, height, width, opt, sunmap_handles);
            else
                % Data is a map:
                dataDim.foo = 'foo';
                radiusData = radius_mapHandler(dec, freq, info_struct.info_Nswp{1}, info_struct.info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, opt, dataDim, sunmap_handles, mapType, dataSource);
                opt.radiusData = radiusData;
            end

        end
    
    end
        
end % End of for loop going through the list of maps

%% Create time series for plotting radius data

% All maps have been gone through --> plot (and save) results:

% Write two time series objects. One for the calculated radius and one for
% the measured radius:
%opt.trackData.sun(idx).max_ts = addsample(opt.trackData.sun(idx).max_ts, 'Data', opt.trackData.sun(idx).max(idx2), 'Time', str2double(opt.trackData.sun(idx).id{idx2}));

% Create time series objects:

radius_calc_ts = timeseries; % Linearly interpolated almanac radius value
radius_meas_ts = timeseries; % Measured radius value from map
radius_diff_ts = timeseries;
radius_sky_qsl_diff = timeseries; % Difference of sky level and QSL in the original units

% Creating variables for later use:
temp_time_sum = [];
temp_radius_calc_sum = [];
temp_radius_meas_sum = [];
time_avg = [];
radius_calc_avg = [];
radius_meas_avg = [];

% Create check variable for the times.
% If a time is in the check variable, then it has been saved into the time
% series objects:
hTimes = [];

% Take an average reading for each day:
avg_opt = 0;
% 1: Calculate average, SLOW!
% 0: Don't calculate average, faster

if(avg_opt == 1)
    
    % Go through all the radiusData elements:
    for idx2 = 1 : length(radiusData)

        % Write progress to Program status string:
        sunmap_status_text = mainHandles.text39;
        set(mainHandles.text39, 'ForegroundColor', 'blue');
        progress_str = ['Radius analysis (2/2), Averaging data, writing time series: ', num2str(idx2), '/', num2str(length(radiusData)),' ...'];
        set(sunmap_status_text, 'String', progress_str);
        drawnow update;
        
        % Variables for the date and time of the current element of radiusData:
        currentTime = radiusData(idx2).time_obj; 
        currentDateStr = datestr(currentTime, 'yyyy-mm-dd'); % eg. '2011-05-25'
        % Create an empty struct for the temporary saving of data from one day
        temp = struct;

        % Check if the current element has been handled already:
        % 1 if element hasn't been handled, 0 if element has been handled:
        if(isempty(find(hTimes == currentTime, 1)))

           % Debug:
           %fprintf('Outside, adding: %d\n', idx2);
           %fprintf('I claim that this: %s isn''t in here: %s\n', num2str(currentTime), num2str(hTimes));

            % Hasn't been handled --> add to hTimes and to temp:
            if(isempty(hTimes))
                hTimes(1) = currentTime;
            else
                hTimes(end + 1) = currentTime;
            end

            % Check if the 'temp' struct is empty:
            if(isfield(temp, 'time_obj'))
                % Struct is not empty --> we can use 'end' as an index
                temp(end + 1).date = currentDateStr;
                temp(end).time_obj = currentTime;
                temp(end).radius_calc = radiusData(idx2).radius_calc;
                temp(end).radius_meas = radiusData(idx2).radius_meas;
            else
                % Struct is empty --> use '1' as index
                temp(1).date = currentDateStr;
                temp(1).time_obj = currentTime;
                temp(1).radius_calc = radiusData(idx2).radius_calc;
                temp(1).radius_meas = radiusData(idx2).radius_meas;
            end

        else
            % Element has been handled already --> jump to next element
            %fprintf('Outside, skipping: %d\n', idx2);
            continue;
        end


        % Go through all the radiusData elements (yes, again):
        for idx4 = 1 : length(radiusData)

            checkTime = radiusData(idx4).time_obj;
            checkDateStr = datestr(checkTime, 'yyyy-mm-dd');

            % If the element at idx4 is from the same day as the element at idx2, and it hasn't yet been handled:
            % --> Save values from radiusData(idx4) into the temporary struct 'temp' and save its time into hTimes:
            % In short: Same date, current element not yet handled
            if(strcmp(checkDateStr, currentDateStr) == 1 && isempty(find(hTimes == checkTime, 1)))

                % Debug:
                %fprintf('Inside, adding: %d\n', idx4);

                % Hasn't been handled --> add to hTimes and to temp:
                if(isempty(hTimes))
                    hTimes(1) = checkTime;
                else
                    hTimes(end + 1) = checkTime;
                end

                % Check if the 'temp' struct is empty:
                if(isfield(temp, 'time_obj'))
                    % Struct is not empty --> we can use 'end' as an index
                    temp(end + 1).date = checkDateStr;
                    temp(end).time_obj = checkTime;
                    temp(end).radius_calc = radiusData(idx4).radius_calc;
                    temp(end).radius_meas = radiusData(idx4).radius_meas;                
                else
                    % Struct is empty --> use '1' as index
                    temp(1).date = checkDateStr;
                    temp(1).time_obj = checkTime;
                    temp(1).radius_calc = radiusData(idx4).radius_calc;
                    temp(1).radius_meas = radiusData(idx4).radius_meas;
                end

                % If we are at the last element of radiusData, we can write the data from the current day into the corresponding time series objects
                if(idx4 == length(radiusData))

                    % Debug:
                    %fprintf('Inside: %d, Outside: %d, Adding stuff (current added)\n', idx4, idx2);

                    % Go through the temporary struct, calculate averages for
                    % time and values, add time and values into time series objects:
                    for idx5 = 1 : length(temp)

                        % Calculate cumulative sum of all elements in temp:
                        if(isempty(temp_time_sum) == 0)
                            temp_time_sum = temp_time_sum + temp(idx5).time_obj;
                            temp_radius_calc_sum = temp_radius_calc_sum + temp(idx5).radius_calc;
                            temp_radius_meas_sum = temp_radius_meas_sum + temp(idx5).radius_meas;
                        else
                            temp_time_sum = temp(idx5).time_obj;
                            temp_radius_calc_sum = temp(idx5).radius_calc;
                            temp_radius_meas_sum = temp(idx5).radius_meas;
                        end

                    end

                    % Calucate the final averages and write data into time
                    % series objects:
                    time_avg = temp_time_sum / length(temp);
                    radius_calc_avg = temp_radius_calc_sum / length(temp);
                    radius_meas_avg = temp_radius_meas_sum / length(temp);

                    % Debug:
                    %fprintf('Adding: Calc: %s, Meas: %s, temp: %d\nAvg. time: %s\n', num2str(radius_calc_avg), num2str(radius_meas_avg), length(temp), datestr(time_avg, 'yyyy-mm-dd, HH:MM:SS'));

                    radius_calc_ts = addsample(radius_calc_ts, 'Data', radius_calc_avg, 'Time', time_avg, 'OverwriteFlag', true);
                    radius_meas_ts = addsample(radius_meas_ts, 'Data', radius_meas_avg, 'Time', time_avg, 'OverwriteFlag', true);
                    diff = radius_meas_avg / radius_calc_avg;
                    radius_diff_ts = addsample(radius_diff_ts, 'Data', diff, 'Time', time_avg, 'OverwriteFlag', true);

                    % Empty the temp struct:
                    temp = struct;
                    temp_time_sum = [];
                    temp_radius_calc_sum = [];
                    temp_radius_meas_sum = [];
                    time_avg = [];
                    radius_calc_avg = [];
                    radius_meas_avg = [];

                end % / If clause: last element of radiusData

            else % Different date of element already handled

                % If we are at the last element of radiusData, we can write the data from the current day into the corresponding time series objects
                if(idx4 == length(radiusData))

                    % Debug:
                    %fprintf('Inside %d, Outside: %d, Adding stuff (current skipped)\n', idx4, idx2);

                    % Go through the temporary struct, calculate averages for
                    % time and values, add time and values into time series objects:
                    for idx5 = 1 : length(temp)

                        % Calculate cumulative sum of all elements in temp:
                        if(isempty(temp_time_sum) == 0)
                            temp_time_sum = temp_time_sum + temp(idx5).time_obj;
                            temp_radius_calc_sum = temp_radius_calc_sum + temp(idx5).radius_calc;
                            temp_radius_meas_sum = temp_radius_meas_sum + temp(idx5).radius_meas;
                        else
                            temp_time_sum = temp(idx5).time_obj;
                            temp_radius_calc_sum = temp(idx5).radius_calc;
                            temp_radius_meas_sum = temp(idx5).radius_meas;
                        end

                    end

                    % Calucate the final averages and write data into time
                    % series objects:
                    time_avg = temp_time_sum / length(temp);
                    radius_calc_avg = temp_radius_calc_sum / length(temp);
                    radius_meas_avg = temp_radius_meas_sum / length(temp);

                    % Debug:
                    %fprintf('Adding: Calc: %s, Meas: %s, temp: %d\nAvg. time: %s\n', num2str(radius_calc_avg), num2str(radius_meas_avg), length(temp), datestr(time_avg, 'yyyy-mm-dd, HH:MM:SS'));

                    radius_calc_ts = addsample(radius_calc_ts, 'Data', radius_calc_avg, 'Time', time_avg, 'OverwriteFlag', true);
                    radius_meas_ts = addsample(radius_meas_ts, 'Data', radius_meas_avg, 'Time', time_avg, 'OverwriteFlag', true);
                    diff = radius_meas_avg / radius_calc_avg;
                    radius_diff_ts = addsample(radius_diff_ts, 'Data', diff, 'Time', time_avg, 'OverwriteFlag', true);

                    % Empty the temp struct:
                    temp = struct;
                    temp_time_sum = [];
                    temp_radius_calc_sum = [];
                    temp_radius_meas_sum = [];
                    time_avg = [];
                    radius_calc_avg = [];
                    radius_meas_avg = [];


                end % / If clause: last element of radiusData

                % The inner loop element isn't from the same date as the outer
                % loop element or the inner loop element has already been
                % handled:
                %fprintf('Inside, skipping: %d\n', idx4);
                continue;

            end % / If clause for: different time but same date

            temp_time_sum = [];
            temp_radius_calc_sum = [];
            temp_radius_meas_sum = [];
            time_avg = [];
            radius_calc_avg = [];
            radius_meas_avg = [];

        end % / for loop: inner walkthrough of all the radiusData elements

    end % / for loop: outer walkthrough of all the radiusData elements

else % No averaging, a data point is created for every map (a lot faster)
       
    if(1==1) % Use whole date as time

        % Write smooth diff:
        % Not working properly with time series
%         for idx = 1 : length(radiusData)
%             diff_smooth(idx) = (radiusData(idx).radius_meas) / (radiusData(idx).radius_calc);
%         end
%         diff_smooth = smooth(diff_smooth, 1001);
%         radius_diff_smooth_ts = timeseries;
                       
        % Write data into time series objects (When data is not averaged):
        for idx = 1 : length(radiusData)
            
            
            % Drop value from time series if diff of qsl and sky in
            % original units > 7000 and year is before 1998
%             dtv = datevec(radiusData(idx).time_obj);
%             if(dtv(1) <= 1998)
%                 if(radiusData(idx).sky_qsl_diff > 7000)
%                     continue;
%                 end
%             end
            
            % Write progress to Program status string:
            sunmap_status_text = mainHandles.text39;
            set(mainHandles.text39, 'ForegroundColor', 'blue');
            progress_str = ['Radius analysis (2/2), Writing time series: ', num2str(idx), '/', num2str(length(radiusData)),' ...'];
            set(sunmap_status_text, 'String', progress_str);
            drawnow update;
            
            radius_calc_ts = addsample(radius_calc_ts, 'Data', (radiusData(idx).radius_calc * radiusData(idx).dist), 'Time', radiusData(idx).time_obj, 'OverwriteFlag', true);
            radius_meas_ts = addsample(radius_meas_ts, 'Data', (radiusData(idx).radius_meas * radiusData(idx).dist), 'Time', radiusData(idx).time_obj, 'OverwriteFlag', true);
            % Added * 60 to convert units to arc sec:
            %radius_meas_ts = addsample(radius_meas_ts, 'Data', 60*(radiusData(idx).radius_meas * radiusData(idx).dist), 'Time', radiusData(idx).time_obj, 'OverwriteFlag', true);
            diff = (radiusData(idx).radius_meas * radiusData(idx).dist) / (radiusData(idx).radius_calc * radiusData(idx).dist);
            radius_diff_ts = addsample(radius_diff_ts, 'Data', diff, 'Time', radiusData(idx).time_obj, 'OverwriteFlag', true);
            %radius_diff_smooth_ts = addsample(radius_diff_smooth_ts, 'Data', diff_smooth(idx), 'Time', radiusData(idx).time_obj, 'OverwriteFlag', true);
            radius_sky_qsl_diff = addsample(radius_sky_qsl_diff, 'Data', radiusData(idx).sky_qsl_diff, 'Time', radiusData(idx).time_obj, 'OverwriteFlag', true);
        end
        
        
        
    else % Use only time of day as time
                
        % Create ts object for decimal hour:
        dec_hour_ts = timeseries;
        
        % Write data into time series objects (When data is not averaged):
        for idx = 1 : length(radiusData)
            
            % Write progress to Program status string:
            sunmap_status_text = mainHandles.text39;
            set(mainHandles.text39, 'ForegroundColor', 'blue');
            progress_str = ['Radius analysis (2/2), Writing time series: ', num2str(idx), '/', num2str(length(radiusData)),' ...'];
            set(sunmap_status_text, 'String', progress_str);
            drawnow update;
            
            % Time vector:
            time_hour_str  = [radiusData(idx).time(13), radiusData(idx).time(14)];
            time_minute_str =  [radiusData(idx).time(16), radiusData(idx).time(17)];
            time_second_str = [radiusData(idx).time(19), radiusData(idx).time(20)];
            % Debug:
            %fprintf('%s:%s:%s\n', time_hour_str, time_minute_str, time_second_str);
            time_vector = datenum(str2double(time_hour_str), str2double(time_minute_str), str2double(time_second_str));
            
            % Write decimal hour time series
            dec_hour = str2double(time_hour_str) + (str2double(time_minute_str)/60) + (str2double(time_second_str)/3600);
            dec_hour_ts = addsample(dec_hour_ts, 'Data', dec_hour, 'Time', radiusData(idx).time_obj, 'OverwriteFlag', true);
            
            radius_calc_ts = addsample(radius_calc_ts, 'Data', radiusData(idx).radius_calc, 'Time', time_vector, 'OverwriteFlag', true);
            radius_meas_ts = addsample(radius_meas_ts, 'Data', radiusData(idx).radius_meas, 'Time', time_vector, 'OverwriteFlag', true);
            diff = (radiusData(idx).radius_meas) / (radiusData(idx).radius_calc);
            radius_diff_ts = addsample(radius_diff_ts, 'Data', diff, 'Time', time_vector, 'OverwriteFlag', true);
            radius_sky_qsl_diff = addsample(radius_sky_qsl_diff, 'Data', radiusData(idx).sky_qsl_diff, 'Time', time_vector, 'OverwriteFlag', true);
        end
        
    end
    
end % End of if clause for averaging the data

% figure, plot(get(radius_diff_ts, 'Data'))
% hold on, plot(get(radius_diff_smooth_ts, 'Data'))

% Exporting ts objects to workspace:
% assignin('base', 'radius_meas_ts', radius_meas_ts);
% assignin('base', 'radius_calc_ts', radius_calc_ts);
% assignin('base', 'radius_diff_ts', radius_diff_ts);

%% Plot the time series

figure,
subplot(2, 1, 1),
%plot(radius_calc_ts, 'o', 'MarkerEdgeColor', [0 0.5 0]),
%hold on,
plot(radius_meas_ts, '.'),
title(['MRO 37 GHz solar disk radius and Astronomical Almanac solar semidiameter,', 10, 'normalized for 1 AU Sun-Earth distance'], 'FontSize',10);
xlabel('Time [YYYY-MM]'),
%ylabel('Solar disk radius ["]');
ylabel('Radius [arc min]');
grid on,
NumTicks = 15;
L = get(gca,'XLim');
set(gca,'XTick',linspace(L(1),L(2),NumTicks))
datetick('x','yyyy-mm', 'keepticks')
%legend('Almanac value', '37 GHz value', 'Location', 'Best');
subplot(2, 1, 2),
plot(radius_sky_qsl_diff, '+', 'MarkerEdgeColor', 'r'),
%hold on,
%plot(radius_diff_smooth_ts);
title('Difference of solar map sky level and quiet sun level');
xlabel('Time [YYYY-MM]'),
ylabel('Difference [Arbitrary units]');
%set(gcf, 'Color', 'w')  % figure "outside" backgound color
grid on,
NumTicks = 15;
L = get(gca,'XLim');
set(gca,'XTick',linspace(L(1),L(2),NumTicks))
datetick('x','yyyy-mm', 'keepticks')
%subplot(2, 1, 2)
%plot(radius_sky_qsl_diff),
grid on,

% % Plot decimal hour vs. date:
% figure,
% plot(dec_hour_ts, '.'),
% grid on,
% NumTicks = 15;
% L = get(gca,'XLim');
% set(gca,'XTick',linspace(L(1),L(2),NumTicks))
% datetick('x','yyyy-mm', 'keepticks')

plot_succeed = 1;

%get(radius_diff_ts, 'Data')

%% Make sure the main GUI's handles can be accessed:

mainHandles = guidata(sunmap_handles);

%% Saving radius data

radius_file_saver(sunmap_handles, radiusData);

%% Finishing up with GUI:

set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);

if(opt.img_save == 1 || opt.fig_save == 1)
    set(mainHandles.text39, 'String', 'Saving done');
elseif(plot_succeed == 1)
    set(mainHandles.text39, 'String', 'Radius processing done');
else
    set(mainHandles.text39, 'ForegroundColor', 'r');
    set(mainHandles.text39, 'String', 'Radius processing failed');
end

if(ispc)
    set(mainHandles.text39, 'FontSize', 8);
end
set(mainHandles.plot_single_button, 'Enable', 'on');
set(mainHandles.plot_selected, 'Enable', 'on');
set(mainHandles.plot_all_button, 'Enable', 'on');
set(mainHandles.listbox1, 'Enable', 'on');
set(mainHandles.pushbutton21, 'Visible', 'off');


end % radius_saver ENDS


%% !!!!----------- radius_mapHandler() -function starts -----------!!!!

function radiusData = radius_mapHandler(dec, freq, nswp, sps, map, num, time, obstitle, h, w, opt, dataDim, sunmap_handles, mapType, dataSource)

destFolder = opt.destFolder;
beam_size = opt.beam_size;
solar_north = opt.solar_north;
grid_toggle = opt.grid_toggle;
abs_scale = opt.abs_scale;
c_min = opt.min;
c_max = opt.max;
mapPlottingType = opt.mapType; % opt.mapType (that tells how the map is plotted: filled contours, lines, etc.) IS NOT THE SAME AS the variable mapType (ra-map or de-map)!!!!!
customNumOfLevelsToggle = opt.customNumOfLevelsToggle ;
customNumOfLevels = opt.customNumOfLevels;

mainHandles = guidata(sunmap_handles);

% Saving original map matrix and data from it:
map_raw = map;

azelTime = regexprep(time, '-', '/');
azelTime = regexprep(azelTime, ',', '');
[Az El] = SolarAzEl(azelTime, 60.217809153, 24.39311053, 60.28);

% Get trackData struct from opt struct:
radiusData = opt.radiusData;


%     sunPlot() function info:
%
%       PARAMETERS:
%
%           - dec: declination of the Suns center (for calculating round
%             shape for Sun). E.g: 21,73563 (scalar)
%
%           - opt.destFolder: destination for the output .png-images and .fig
%             files. E.g: '/home/user/sun/plots/' (string)
%
%           - freq: The frequency of the observation. E.g: '37 GHz'
%             (string)
% 
%           - nswp: Number of sweeps in data. E.g: 29 (scalar)
%
%           - sps: Samples per sweep in data. E.g: 91 (scalar)
%
%           - map: The data matrix. E.g: of size nswp x sps samples
%
%           - num: A running number that is used to determine how many
%             figures have been printed (scalar)
% 
%           - time: The observation date (start and stop).
%             '2009-07-14,  7:25:27.620 - 07:33:51.62' (string)
% 
%           - obstitle: The plain name of the observation without any
%             filename extention. E.g: 'sun09071506' (string)
%
%           - h: The angular height of the data (in arc min), (scalar)
%            
%           - w: The NON DECLINATION-CORRECTED angular width of the data
%             (in arc min)
%           
%           - beam_size: The option to enable or disable the printing of
%             beam size on the map
%
%           - solar_north: The option to enable or disable the printing of
%             an arrow illustrating the direction of solar north on the map
%
%           - grid_toggle: The option to enable or disable the printing of
%             a grid on the plot (scalar, 1 = 'on' or 0 = 'off')
%
%           - cmap: The string from the GUI that defines the colormap
%             (string)
%
%           - abs_scale: The option to enable or disable user given
%             absolute levels-values (scalar, 1 = 'on' or 0 = 'off')
%
%           - c_min: User given minimum value from which plotting is
%             started
%
%           - c_max: User given maximum value up to which the plotting is
%             done
%
%           - sunmap_handles: The handles-structure of the sunmap -gui
%             window. The status messages on the gui can be set with the
%             help of this handles
%

% Recognizing the frequency-string:
    if(isempty(regexpi(freq,'37')) == 0)
        freq = '37 GHz';
    end
    
    if(isempty(regexpi(freq,'87')) == 0)
        freq = '87 GHz';
    end
    
    if(isempty(regexpi(freq,'22')) == 0)
        freq = '22 GHz';
    end



    %fprintf('sunPlot being called. Plotting the map...\n')
    mainHandles = guidata(sunmap_handles);
    if(isfield(opt, 'savingHandles') == 0) % savingHandles does not exist ==> sunmap_contourplot was not called by saving.m ==> "save to web" not in use
        saveToWeb = 0;
    else
        savingHandles = guidata(opt.savingHandles);
        saveToWeb = get(savingHandles.checkbox5, 'Value');
    end
    sunmap_status_text = mainHandles.text39;
    set(mainHandles.text39, 'ForegroundColor', 'blue');
        
    set(mainHandles.plot_single_button, 'Enable', 'off');
    set(mainHandles.plot_selected, 'Enable', 'off');
    set(mainHandles.plot_all_button, 'Enable', 'off');
    set(mainHandles.listbox1, 'Enable', 'off');
    set(mainHandles.pushbutton21, 'Visible', 'on');
    % For checking the value of nswp (number of sweeps in data):
    %fprintf('The number of sweeps given to sunPlot is: %d\n', nswp)
    
    %% Map operations:
    
    % Temporarily dealing with data that has only one value in the
    % whole data matrix/vector
    if(length(unique(map)) == 1)
        return;
    end
      
    % Set "overbright" values to map max value if sun-measure data:
    if(strcmp(dataSource, 'sm') == 1)
        map_size_temp = size(map);
        for ind_j = 1 : map_size_temp(1)
            for ind_g = 1 : map_size_temp(2)
                if map(ind_j,ind_g) < -30000
                     map(ind_j,ind_g) = max(max(map));
                end
            end
        end        
    end
    
   % Convert log units to lin
   if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1) 
        map = log_to_lin(map);
   end
    
    % Normalizing data so that minimum is at zero:
    map = map - min(min(map));

    map_abs_min = min(min(map)); % This is used in calculating the difference of minimum level and qsl for the radius plot in the original units
    
    % Calling qsl_finder() to get Quiet Sun Level:
    quiet_sun = qsl_finder(map, 0); % dataUnits = 0 in this case
    
    % Calling qsl_units_transformer() to transform map units from
    % original to '% of QSL' units:
    map = qsl_units_transformer(map, quiet_sun, freq);

    max_temp = max(max(map));
    min_temp = min(min(map));

    quiet_sun_level = 100;
       
    
    %% Map levels-operations:  
    
    % protection from invalid custom levels values:
    if(get(mainHandles.checkbox31, 'Value') == 1)
        if(str2num(get(mainHandles.edit10, 'String'))) >= str2num(get(mainHandles.edit11, 'String'))
            message = ['The minimum custom levels value (', get(mainHandles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(mainHandles.edit11, 'String'), ').'];
            msgbox(message, 'Warning', 'warn', 'modal');
        return;
        end
        if(c_min >= max_temp)
            message = ['The minumun custom levels value (', num2str(c_min), ') has to be lower than the maximum value (', num2str(max_temp), ') of the selected map.'];
            msgbox(message, 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit10, 'String')))
            msgbox('Please insert a number value to the minimum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit11, 'String')))
            msgbox('Please insert a number value to the maximum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
    end
    

    %% Getting rid of "off-sun data" (reflections and what not):
    
    map = map_edges_cleaner(map, mapType, dataSource, nswp, sps);

    %% Data size vs. grid, map rotation:
    
    % Make sure variables are double:
    nswp = double(nswp);
    sps = double(sps);
    w = double(w); 
    h = double(h);
    
    % Write variables into struct for calling map_handler():
    
    % Common variables:
    mhStructIn.map = map;
    mhStructIn.dataSource = dataSource;
    mhStructIn.mapType = mapType;
    mhStructIn.nswp = nswp;
    mhStructIn.sps= sps;
    mhStructIn.w = w;
    mhStructIn.h = h;
    mhStructIn.dec = dec;

    % sun-measure specific variables:
    if(strcmp(dataSource, 'sm') == 1)
        mhStructIn.sweepDir =  opt.sweepDir;
        mhStructIn.sampleDir = opt.sampleDir;
        mhStructIn.cosCorrect = opt.cosCorrect;
        mhStructIn.startra = opt.startra;
        mhStructIn.endra = opt.endra;
        mhStructIn.startdec = opt.startdec;
        mhStructIn.enddec = opt.enddec;
    elseif(strcmp(dataSource, 'mm') == 1)
        mhStructIn.sweepDir =  opt.sweepDir;
        mhStructIn.sampleDir = opt.sampleDir;
        mhStructIn.cosCorrect = opt.cosCorrect;
        % Temp
        mhStructIn.startra = dataDim.startra;
        mhStructIn.endra = dataDim.endra;
        mhStructIn.startdec = dataDim.startdec;
        mhStructIn.enddec = dataDim.enddec;
        % / Temp
    end

    % Call map_handler():
    mhStructOut = map_handler(mhStructIn);

    % Write variables from map_handler() output struct:
    map = mhStructOut.map;
    x = mhStructOut.x;
    y = mhStructOut.y;
    %x_fov =mhStructOut.x_fov;
    %y_fov = mhStructOut.y_fov;
    mapWidth = mhStructOut.mapWidth;
    mapHeight = mhStructOut.mapHeight;
    nswp = mhStructOut.nswp;
   
    %% Convert numbers to strings for printing on the contour map
    max_temp_str = num2str(round(max_temp * 100) / 100);
    
    %% Calling sunData() to get solar info
    sunData = sun_data(time);
    rotDeg = sunData.rot;
    B0 = sunData.B0;
    L0 = sunData.L0;
    radius_calc = sunData.radius;
    dist = sunData.dist;
        
    %%  Generating radius data
          
    % A boolean check variable that will be 1 if something is plotted and 0
    % if nothing is plotted
    plot_succeed = 0;  

    % Variables read only if the current map is a full disk map:        
    if(mapWidth >= 30 && mapHeight >= 30)
    %if(mapWidth >= 30 && mapHeight >= 30 && strcmp(mapType, 'ramap') == 1) % Temporary dropping of DE-maps from the analysis

        % Making a copy of the original map data and size vectors:
        map_orig = map;
        x_orig = x;
        y_orig = y;

        [foo, xSize] = size(x);
        [foo, ySize] = size(y);
        [dataHeight, dataWidth] = size(map);
        % By choosing 9 as the scaling factor the original data points
        % can be preserved:
        if(dataHeight >= dataWidth) % DE-map
            map = imresize(map, [dataHeight 9*dataWidth], 'bilinear');
            x = imresize(x, [1 9*xSize], 'bilinear');               
        else % RA-map
            map = imresize(map, [9*dataHeight dataWidth], 'bilinear');
            y = imresize(y, [1 9*ySize], 'bilinear');
        end
        
        % Write neccessary variables in opt-struct:
        opt.dataWidth = dataWidth;
        opt.dataHeight = dataHeight;
        opt.rotDeg = rotDeg;
        opt.rotate_disable = 0;

        % Map matrix is resized when solar grid is selected --> use
        % original matrix map_orig (and x_orig, y_orig) in that case
        % Otherwise map is the original data:
        %if(get(mainHandles.checkbox38, 'Value') == 1.0) % Solar grid selected
        % Call center_finder() function:
        opt.x = x;
        opt.y = y;
        center_info = center_finder(map, opt);
        radius_meas = center_info.radius;
        
        % SMART data:
        if(strcmp(mapType, 'foobar') == 1)
            %Date Vector:           [2003  10  24  12  45  07]
        end
        
        % Date vector:
        time_year_str = [time(1), time(2), time(3), time(4)];
        time_month_str = [time(6), time(7)];
        time_day_str = [time(9), time(10)];
        time_hour_str  = [time(13), time(14)];
        time_minute_str =  [time(16), time(17)];
        time_second_str = [time(19), time(20)];
        
        time_vector = datenum(str2double(time_year_str), str2double(time_month_str), str2double(time_day_str), str2double(time_hour_str), str2double(time_minute_str), str2double(time_second_str));
        
        sky_qsl_diff = abs(quiet_sun - map_abs_min);
        
        if(isempty(radiusData(1).time))
            
            radiusData(end).time = time;
            radiusData(end).id = obstitle;
            radiusData(end).freq = freq;
            radiusData(end).time_obj = time_vector;
            radiusData(end).sky_qsl_diff = sky_qsl_diff;
            radiusData(end).radius_calc = radius_calc;
            radiusData(end).radius_meas = radius_meas;
            radiusData(end).dist = dist;
            
        else
            
            radiusData(end + 1).time = time;
            radiusData(end).id = obstitle;
            radiusData(end).freq = freq;
            radiusData(end).time_obj = time_vector;
            radiusData(end).sky_qsl_diff = sky_qsl_diff;
            radiusData(end).radius_calc = radius_calc;
            radiusData(end).radius_meas = radius_meas;
            radiusData(end).dist = dist;
            
        end

        %opt.radiusData = radiusData; 
        
    end % End of if-clause for including only full disk maps
   
    

end % End of mapHandler() function
    
    
    
  