% This function reads sun plot data from a .bin file into a matrix.
% This version ('ultimate') can read any kind of data array, when it's
% given the number of sweeps, ('sweeps'), the number of samples per sweep
% ('samples') and the file name ('fname')

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function d = read_sun_data_ultimate(fname, sweeps, samples)

    %fprintf('\nReading new data...\n')

    sweeps = double(sweeps);
 
    rows = 0;
    columns = 0;
    
    
    % data is from fast map [samples x 29]:
    if sweeps == 29
        rows = 29;
        columns = samples;
    end
    
    % data is from fast de-map [43 x 84]:
    if sweeps == 43
        rows = 43;
        columns = 84;
    end
       
    if sweeps ~= 29 && sweeps ~= 43
        
        rows = sweeps;
        columns = samples; % partial maps are always RA-maps (assumption).
                                      % This works for tracks as well
        
    end
    
    fd = fopen(fname, 'rb', 'l'); % binary, little-endian
    
    %fprintf('Reading raw data from .bin-file...\n')
    %fprintf('Reading %d columns and %d rows of raw data from .bin-file\n', columns, rows)
    
    d = fread(fd, [columns rows], 'int16', 0); % signed int16
    % fprintf('Comaprison of the planned and actual data size:\n')
    %size(d)
    %fprintf('This concludes our data size check!\n')
    d = d';
    fclose(fd);
end
