% This file contains functions that are used for making figures of solar map and
% track observations made in Metsähovi Radio Observatory

% sunmap_contourplot() function:
% Input: A struct (opt) that contains a bunch of variables (mostly info on the state of the main GUI (sunmap.m/.fig) that calls this function)
% Output: No output as variables. Calls a plotting function within a for loop.
% Goes through a list of observation info files that contain metadata of
% the observation. Based on the info file name, costructs the file name of
% the actual data file, from which the raw observation data is read from.
function sunmap_contourplot(opt, sunmap_handles)


mainHandles = guidata(sunmap_handles);
if(opt.img_save == 1 || opt.fig_save == 1 || opt.anim_gif == 1)
    savingHandles = guidata(opt.savingHandles);
    % Reading info from saving window's "Save to web" checkbox:
    saveToWeb = get(savingHandles.checkbox5, 'Value');
else
    saveToWeb = 0;
end

% Read data unit selection from the GUI
if(get(mainHandles.radiobutton12, 'Value') == 1)
    opt.dataUnits = 0; % % of QSL (default)
elseif(get(mainHandles.radiobutton13, 'Value') == 1 && saveToWeb == 0)
    opt.dataUnits = 1; % Original units
else
    opt.dataUnits = 0; % "% of QSL" units (default)
end

%% File path(s) from which the .info and .bin raw data are read

% 'dbase': The starting part of the filepath. You can leave this empty if
% the whole path is in the opt.plottingList-list part of the path.
dbase = '';

%%  Reading plot info from the individual .info-files:

for fnum=1:size(opt.plottingList,1)
    
    infofilename = [dbase opt.plottingList{fnum}];
    
    finfo = fopen(infofilename,'rt');
    if (finfo < 0)
        %fprintf(1, ['Could not open file ' infofilename '\n']);
        cla(mainHandles.axes2, 'reset');
        axis off,
        set(mainHandles.text65, 'Visible', 'off');
        text(0.23, 0.5, 'Can''t open info file', 'Color', 'r', 'FontSize', 12);
        drawnow update;
        continue
    end
    
 % SUN-MEASURE .suninfo1-file:
        % ------------------------------------------------
    if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8) % strfind returns the index of the start of the string found within the string from which we're searching 
   
        % Call info_reader() that returns a struct containing the
        % neccessary variables in a struct:
        infoStruct = info_reader(infofilename);

        % Write stuff from struct to individual variables:
        n_sweeps = infoStruct.n_sweeps;
        startdec = infoStruct.startdec;
        enddec = infoStruct.enddec;
        startra= infoStruct.startra;
        endra = infoStruct.endra;
        obstime = infoStruct.obstime;
        opt.sweepDir = infoStruct.sweepDir;
        opt.sampleDir = infoStruct.sampleDir;
        opt.cosCorrect = infoStruct.cosCorrect;
        obs_mode = infoStruct.obs_mode;
        mapType = infoStruct.mapType;
        observer_name = infoStruct.observer_name;
        freq = infoStruct.freq;
        data_integrated = infoStruct.data_integrated;
        
        dataSource = 'sm';
        
        %data_integrated = 0; % This variable tells the difference between a a file with data incorporated in the same file with the info-stuff and a "standalone" suninfo-file
                                         % The default value is set here.
                                         % If an integrated file is
                                         % encountered later, the value is
                                         % set to 1
                                         
        % Data height in arc-min
        mapHeight = 60 * (enddec - startdec);

        % NON-declination- compensated width in arc-min:
        mapWidth = 60 * abs(endra - startra);    

        dataDim.startra = startra;
        dataDim.startdec = startdec;
        dataDim.endra = endra;
        dataDim.enddec = enddec;
   
        time_str = infoStruct.time_str; % Time string for observation: YYYY-MM-DD, HH:MM:SS - HH:MM:SS

        obstitle = regexprep(infofilename, '.suninfo1', ''); % /home/foo/bar/plaa/plaa/1284623590
        for i = 0:length(obstitle)

            if(strcmp(obstitle(end - i), '/') == 1 || strcmp(obstitle(end - i),'\') == 1)
                break;
            end
            obstitle1(i + 1) = obstitle(end - i);

        end
        obstitle = '';
         for i = 0:(length(obstitle1) - 1)
          obstitle(i + 1) = obstitle1(end - i);     
         end

        % / sun-measure .suninfo file
        % ------------------------------------------------
    
    % ------------------------------------------------
    % sun-measure "2.0" .map/.metadata file
    % Testing
    elseif(strfind(infofilename, '.map'))
        map_meta = get_sunmap_metadata(infofilename);
        
    % SMART .info-file:
    % ------------------------------------------------
    else
        dataSource = 'SMART';
        dataDim = 'foo';
        % Get map metadata in struct:
        info_struct = get_sunmap_metadata_smart(finfo);
        
        % Determine from which row the data starts:
        info_all = textread(infofilename, '%s', 'delimiter', '\n');
        info_numrows = length(info_all);
        info_startrow = info_numrows - info_struct.info_Nswp{1};
        info_time = dlmread(infofilename, '', info_startrow, 4);

        opt.sweepDir = 'default'; % This has to be set here, because "sweepDir" is used with sun-measure data
        opt.sampleDir = 'default'; % This has to be set here, because "sampleDir" is used with sun-measure data
        opt.cosCorrect = 0; % Default value, cosine correction not done in data --> has to be done in Sunmap
    end
    % ------------------------------------------------
    
    fclose(finfo);
    %%
    
    % SUN-MEASURE DATA:
    if(length(infofilename) - strfind(infofilename, '.suninfo1')  == 8)
        
        % Checking whether to look for linear or log scale data:
        
        % Linear scale (Linear checked, Log unchecked):
        if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
        
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end
            
        % Log scale (Linear unchecked, Log checked and saveToWeb is 0):
        elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1 && saveToWeb == 0)
            % (Only sunmap_contourplot.m needs checking of saveToWeb, because saving.m only calls sunmap_contourplot())
            
            % We're looking for a _log.txt file or a _log.matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '_log.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with _log.matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '_log.matrix0');
            end            
            
        else % If something funny is going on, i.e. both the menu items in the GUI are checked or unchecked at the same time, resort to linear data:
            
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end            
            
        end
        
        %dec = sun_declination(time_str);
        sunData = sun_data(time_str);
        dec = sunData.dec;
        
        %freq = '37 GHz'; % NOTE: This is fixed and not based on anything but a guess!!
        obstitle = 'foobar'; % To be determined
        
    % If .matrix0 datafile is not found, in the case that we need a separate data file (data_integrated == 0)):
    s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
    if((exist(dataFileName, 'file') == 0 || s.bytes < 60) && data_integrated == 0)
        %set(mainHandles.text39, 'ForegroundColor', 'r');
        %if(exist(dataFileName, 'file') == 0)
          %  set(mainHandles.text39, 'String', 'data file not found');
            %return;
        continue;
        %   else
        %            set(mainHandles.text39, 'String', 'data file is empty');
        %            return;
        %        end
        %        set(mainHandles.text39, 'FontSize', 10);
        %        set(mainHandles.plot_single_button, 'Enable', 'on');
        %        set(mainHandles.plot_all_button, 'Enable', 'on');
        %        set(mainHandles.listbox1, 'Enable', 'on');
        %        set(mainHandles.pushbutton21, 'Visible', 'off');
        
    else
         % Reading the actual map data:
        
        % Data is from separate .txt-file:
        if(data_integrated == 0)
            mapdata = dlmread(dataFileName);
        end
        % If the data is from an integrated file with info and data
        % together, the data has already been read earlier
        
        map_dimensions = size(mapdata);
        sps = map_dimensions(2); % Samples per sweep.
    end
       
    
    
     % ----------------------------
    % map-measure data
    elseif(strfind(infofilename, '.map'))

        % Checking whether to look for linear or log scale data:

        % Linear scale (Linear checked, Log unchecked):
        if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
            data_col_num = 8;
        % Log scale (Linear unchecked, Log checked):
        elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
            data_col_num = 9;
        else
            data_col_num = 8;
        end

        sunData = sun_data(map_meta.time_str);
        dec = sunData.dec;

        % Create radec matrix from arbitrary sweep data
        gen_map_height = 60*abs(map_meta.d_dec_min - map_meta.d_dec_max);
        gen_map_width = 60*abs(map_meta.d_ra_cc_min - map_meta.d_ra_cc_max);
        gen_sweeps = round(gen_map_height / (60*map_meta.sweepwidth)); % generated sweeps
        gen_sps = round(gen_map_width / 0.24); % generated samples per sweep (0.24' ~ 200 sps for 0.8 degree map)
        mapType = map_meta.mapType;
        opt.sweepDir = 'forward'; % fix this if something breaks
        opt.sampleDir = 'forward'; % fix this if something breaks
        opt.cosCorrect = 1;
        dataSource = 'sm'; % sun-measure (for now at least)

        % Data height in arc-min
        dataDim.startra = map_meta.d_ra_cc_min; % Check if cc or non cc is needed
        dataDim.endra = map_meta.d_ra_cc_max;
        dataDim.startdec = map_meta.d_dec_min;
        dataDim.enddec = map_meta.d_dec_max;

%         data_file = dlmread(infofilename, ' ');
% 
%         [xi, yi] = meshgrid(linspace(15*map_meta.d_ra_cc_min,15*map_meta.d_ra_cc_max,gen_sps), linspace(map_meta.d_dec_min,map_meta.d_dec_max, gen_sweeps));
%         zi = griddata(15*data_file(:,7), data_file(:,6), data_file(:,data_col_num), xi,yi); % fit map data to xi,yi grid. (ra,dec,data_lin)
%         zi(isnan(zi)) = min(min(zi)); % replace nan's with map minimum
          
        radec_matrix = generate_radec_matrix(infofilename, dataDim.startra, dataDim.endra, dataDim.startdec, dataDim.enddec, gen_sps, gen_sweeps, data_col_num);
        %disp(xi);

        %figure,
        %contourf(xi,yi,zi,linspace(15000,23000,200));
    
        
    % SMART DATA:
    else % This goes on for about 300 lines...
        
        % Write variables into info_fields struct for use with
        % info_reader_smart():
        info_fields.info_df = info_struct.info_df;
        info_fields.info_Tswp = info_struct.info_Tswp;
        info_fields.info_time = info_time;
        info_fields.info_Nswp = info_struct.info_Nswp;
        info_fields.infofilename = infofilename;
        info_fields.info_date = info_struct.info_date;
        info_fields.info_rx = info_struct.info_rx;
        info_fields.info_cent = info_struct.info_cent;
        % Call info_reader_smart():
        infoStructSmart = info_reader_smart(info_fields);
        % Write output from info_reader_smart() to variables:
        mapWidth = infoStructSmart.mapWidth; % Non declination compensated width of the map in decimal arc minutes as read from the SMART info file, scalar
        mapHeight = infoStructSmart.mapHeight; % Height of the map in decimal arc minutes as read from the SMART info file, scalar
        mapType = infoStructSmart.mapType; % The type of the map; right ascension map ('ramap') or declination map ('ramap'), string
        obstitle = infoStructSmart.obstitle; % The name of the map file without the file name extension (e.g. 'sun09080603'), string
        obstime = infoStructSmart.obstime; % The full observation time of the map, start - end, (e.g. '2009-08-06,  6:31:53.660 - 06:40:31.66'), string
        freq= infoStructSmart.freq;  % The frequency of the receiver used, (e.g. '37 GHz', '22 GHz'), string
        dec = infoStructSmart.dec; % The declination of the Sun's center at the time of the observation in decimal degrees, scalar  
        
    %% READING THE SUNMAP RAW-DATA (.BIN-FILE) INTO A MATRIX.
   
    % Creating the .bin -files name from .info-files name:
    if(strcmp(infofilename(end-3), 'i') == 1)
        binfilename = strrep(infofilename, '.info', '.bin');       
    elseif((strcmp(infofilename(end-3), 'I') == 1))
        binfilename = strrep(infofilename, '.INFO', '.BIN');
    end
    
    % If .bin-file is not found:
    if(exist(binfilename, 'file') == 0)
        set(mainHandles.text39, 'ForegroundColor', 'r');
        set(mainHandles.text39, 'String', '.bin-file not found');
        set(mainHandles.text39, 'FontSize', 8);
        set(mainHandles.plot_single_button, 'Enable', 'on');
        set(mainHandles.plot_selected, 'Enable', 'on');
        set(mainHandles.plot_all_button, 'Enable', 'on');
        set(mainHandles.listbox1, 'Enable', 'on');
        set(mainHandles.pushbutton21, 'Visible', 'off');
        
        continue
    end
    
    % Calling read_sun_data_ultimate() that reads the .bin-file.
    [mapdata] = read_sun_data_ultimate(binfilename, info_struct.info_Nswp{1}, info_struct.info_sps{1});
    %mapType = 'foobar'; % With SMART data maptype (ramap, demap) is derived from the number of sweeps and samples/sweep
    
    end % END OF ELSE-CLAUSE FOR SMART-DATA
    
    
   %% Setting the text in "saving"-window to display the progress of the saving process:
   if(opt.img_save == 1 || opt.fig_save == 1)
        progress_str = ['Saving figure ', num2str(fnum), '/', num2str(size(opt.plottingList,1)), '...'];
        set(savingHandles.text5, 'String', progress_str);
   elseif(opt.anim_gif == 1)
        progress_str = ['Processing figure ', num2str(fnum), '/', num2str(size(opt.plottingList,1)), '...'];
        set(savingHandles.text5, 'String', progress_str);
   end
   %% Calling sunPlot or trackPlot.

    if(fnum == 1)
        opt.fig_count = length(findobj('Type','figure')) - 1;
    end   
   
    % Plots a figure and saves it _as a screenshot_.
    % (So don't disable the visibility of the figure)
    % It's done like this because it seems impossible to duplicate the
    % same visual appearance settings for the visible figure and a
    % direct-to-file export.
    
    % SUN-MEASURE DATA:
    if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8)
    
        if(strcmp(obs_mode, 'track') == 1 || n_sweeps == 1)
            % Data is a track:
            trackPlot(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, opt, sunmap_handles, dataSource);
        elseif(strcmp(obs_mode, 'map') == 1)
            % Data is a map:
            sunPlot(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, opt, sunmap_handles, mapType, dataDim, dataSource);
        end
        
    % Sun-measure "2.0" data
    elseif(strfind(infofilename, '.map'))
        sunPlot(dec, map_meta.freq, gen_sweeps, gen_sps, radec_matrix, fnum, map_meta.time_str, map_meta.obstime, gen_map_height, gen_map_width, opt, sunmap_handles, mapType, dataDim, dataSource);
        
        
    % SMART DATA:
    else
        
        if(strcmp(obstitle(1), 't') == 1 || strcmp(obstitle(1), 'T') == 1)
            % Data is a track:
            trackPlot(dec, freq, info_struct.info_Nswp{1}, info_struct.info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, opt, sunmap_handles, dataSource);
        else
            % Data is a map:
            sunPlot(dec, freq, info_struct.info_Nswp{1}, info_struct.info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, opt, sunmap_handles, mapType, dataDim, dataSource);
        end
    
    end
    
end

end % sunmap_contourplot ENDS

%% FUNCTION FOR MODIFYING DATA CURSOR / DATATIP OUTPUT:

function dataTipString = data_cursor_mod(foo, cursor_object)

pos = get(cursor_object,'Position');
dataTipString = {['MOI: ',num2str(-pos(1))],...
       ['JEE: ',num2str(pos(2))],...
       ['Value: ', '-']};

end

%% !!!!----------- trackPlot -function starts -----------!!!!
% This function plots track-files
function trackPlot(dec, freq, nswp, sps, map, num, time, obstitle, h, w, opt, sunmap_handles, dataSource)
    
    mainHandles = guidata(sunmap_handles);
    sunmap_status_text = mainHandles.text39;
    set(mainHandles.text39, 'ForegroundColor', 'blue');
    
    if(opt.img_save == 1 || opt.fig_save == 1)
        set(sunmap_status_text, 'String', 'Saving...');
    else
        size_of_list= size(get(mainHandles.listbox1, 'Value'));
        progress_str = ['Plotting: ', num2str(num), '/', num2str(size_of_list(2)), ' ...'];
        set(sunmap_status_text, 'String', progress_str);
    end
    
    set(mainHandles.plot_single_button, 'Enable', 'off');
    set(mainHandles.plot_selected, 'Enable', 'off');
    set(mainHandles.plot_all_button, 'Enable', 'off');
    set(mainHandles.listbox1, 'Enable', 'off');
    set(mainHandles.pushbutton21, 'Visible', 'on');
    
    fig_number = num;
        while(isempty( find( get(0, 'children') == fig_number) ) == 0)
            fig_number  = fig_number + 1;
        end
   
    figure(fig_number),
    plot(map, 'k')
    grid on,
    % Main title and axes titles:
    title([10, 'Solar track ' freq, ' / Metsähovi Radio Observatory', 10, 'Time: ',time, ' UT,', 10, 'Filename: ',obstitle], 'FontSize',10, 'Color', 'k'),
    xlabel('Time / seconds'),
    ylabel('Voltage / mV'),
    h_xlabel = get(gca,'XLabel');
    set(h_xlabel,'FontSize',10);
    set(h_xlabel,'Color','k');
    h_ylabel = get(gca,'YLabel');
    set(h_ylabel,'FontSize',10);    
    set(h_ylabel,'Color','k');
    set(gcf, 'Color',[1,1,1]) % figure "outside" backgound color
    %colordef white;
    set(gca, 'Color', [1,1,1]) % Color of gird background white
    set(gca, 'Xcolor', 'k', 'Ycolor', 'k'); % Color of the axes to black
    
    set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);
    
    if(opt.img_save == 1 || opt.fig_save == 1)
        set(mainHandles.text39, 'String', 'Saving done');
    else
        set(mainHandles.text39, 'String', 'Plotting done');
    end
   
    % Setting the size of the figure:
    %  Position from lower left corned of screen 300x200 pixels
    % width =  800 pix, height = 250 pix
    %if(get(mainHandles.popupmenu7, 'Value') ~= 2) % 2nd item on popupmenu7 is 560x420 which is matlab default => no action taken then
    position = [300, 200, 800, 250];
    set(figure(fig_number),'Position', position);
    %end
    
    
    if(ispc)
        set(mainHandles.text39, 'FontSize', 8);
    end
    set(mainHandles.plot_single_button, 'Enable', 'on');
    set(mainHandles.plot_selected, 'Enable', 'on');
    set(mainHandles.plot_all_button, 'Enable', 'on');
    set(mainHandles.listbox1, 'Enable', 'on');
    set(mainHandles.pushbutton21, 'Visible', 'off');    
    
    % Exporting figure as a "screenshot"
    if(opt.img_save == 1.0)   
        figure(fig_number);
        X=getframe(gcf);
        % refresh(gcf);
        imwrite(X.cdata, [opt.destFolder obstitle opt.img_type]);
    end
    
    % Saving figure in figure-form (same destination as with .png's)
    if(opt.fig_save == 1.0)
        dest = strcat(opt.destFolder, obstitle);
        saveas(figure(fig_number), dest,'fig');
    end
    
    if(opt.close_prev == 1.0)       
        close(figure(fig_number));
    end
    
    if(opt.img_save == 1 || opt.fig_save == 1)
        
        savingHandles = guidata(opt.savingHandles);
    
        plottinglist_size = size(get(mainHandles.listbox1, 'Value'));
        if(num == plottinglist_size(2)) % If the current figure is the last one of the ones being saved:
            if(num == 1)
                fig_str = 'figure';
            else
                fig_str = 'figures';
            end
            string = ['Saving done! ', num2str(num), ' ', fig_str, ' saved to ', opt.destFolder];
            set(savingHandles.text5, 'String', string);
            set(savingHandles.text5, 'ForegroundColor', [0 0.5 0]);
            set(savingHandles.pushbutton3, 'String', 'Close');
            set(savingHandles.pushbutton3, 'Enable', 'on');
            set(savingHandles.text5, 'Enable', 'on');
            set(mainHandles.text39, 'String', 'Saving done');
            set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);
        end
        
    end
    
    
end


%% !!!!----------- sunPlot() -function starts -----------!!!!

% This function plots and optionally saves a single solar map as a separate figure with the
% specified settings
function sunPlot(dec, freq, nswp, sps, map, num, time, obstitle, h, w, opt, sunmap_handles, mapType, dataDim, dataSource)

destFolder = opt.destFolder;
beam_size = opt.beam_size;
solar_north = opt.solar_north;
grid_toggle = opt.grid_toggle;
cmap = opt.cmap;
abs_scale = opt.abs_scale;
c_min = opt.min;
c_max = opt.max;
mapPlottingType = opt.mapType; % opt.mapType (that tells how the map is plotted: filled contours, lines, etc.) IS NOT THE SAME AS the variable mapType (ra-map or de-map)!!!!!
customNumOfLevelsToggle = opt.customNumOfLevelsToggle ;
customNumOfLevels = opt.customNumOfLevels;
sweepDir = opt.sweepDir;
sampleDir = opt.sampleDir;

%     sunPlot() function info:
%
%       PARAMETERS:
%
%           - dec: declination of the Suns center (for calculating round
%             shape for Sun). E.g: 21,73563 (scalar)
%
%           - opt.destFolder: destination for the output .png-images and .fig
%             files. E.g: '/home/user/sun/plots/' (string)
%
%           - freq: The frequency of the observation. E.g: '37 GHz'
%             (string)
% 
%           - nswp: Number of sweeps in data. E.g: 29 (scalar)
%
%           - sps: Samples per sweep in data. E.g: 91 (scalar)
%
%           - map: The data matrix. E.g: of size nswp x sps samples
%
%           - num: A running number that is used to determine how many
%             figures have been printed (scalar)
% 
%           - time: The observation date (start and stop).
%             '2009-07-14,  7:25:27.620 - 07:33:51.62' (string)
% 
%           - obstitle: The plain name of the observation without any
%             filename extention. E.g: 'sun09071506' (string)
%
%           - h: The angular height of the data (in arc min), (scalar)
%            
%           - w: The NON DECLINATION-CORRECTED angular width of the data
%             (in arc opt.min) 
%           
%           - beam_size: The option to enable or disable the printing of
%             beam size on the map
%
%           - solar_north: The option to enable or disable the printing of
%             an arrow illustrating the direction of solar north on the map
%
%           - grid_toggle: The option to enable or disable the printing of
%             a grid on the plot (scalar, 1 = 'on' or 0 = 'off')
%
%           - cmap: The string from the GUI that defines the colormap
%             (string)
%
%           - abs_scale: The option to enable or disable user given
%             absolute levels-values (scalar, 1 = 'on' or 0 = 'off')
%
%           - c_min: User given minimum value from which plotting is
%             started
%
%           - c_max: User given maximum value up to which the plotting is
%             done
%
%           - sunmap_handles: The handles-structure of the sunmap -gui
%             window. The status messages on the gui can be set with the
%             help of this handles
%

% Recognizing the frequency-string:
    if(isempty(regexpi(freq,'37')) == 0)
        freq = '37 GHz';
    end
    
    if(isempty(regexpi(freq,'87')) == 0)
        freq = '87 GHz';
    end
    
    if(isempty(regexpi(freq,'22')) == 0)
        freq = '22 GHz';
    end



    %fprintf('sunPlot being called. Plotting the map...\n')
    mainHandles = guidata(sunmap_handles);
    if(isfield(opt, 'savingHandles') == 0) % savingHandles does not exist ==> sunmap_contourplot was not called by saving.m ==> "save to web" not in use
        saveToWeb = 0;
    else
        savingHandles = guidata(opt.savingHandles);
        saveToWeb = get(savingHandles.checkbox5, 'Value');
    end
    sunmap_status_text = mainHandles.text39;
    set(mainHandles.text39, 'ForegroundColor', 'blue');
    
    if(opt.img_save == 1 || opt.fig_save == 1 || opt.anim_gif == 1)
        set(sunmap_status_text, 'String', 'Saving...');
    else
        size_of_list= size(get(mainHandles.listbox1, 'Value'));
        progress_str = ['Plotting: ', num2str(num), '/', num2str(size_of_list(2)),' ...'];
        set(sunmap_status_text, 'String', progress_str);
    end
    
    set(mainHandles.plot_single_button, 'Enable', 'off');
    set(mainHandles.plot_selected, 'Enable', 'off');
    set(mainHandles.plot_all_button, 'Enable', 'off');
    set(mainHandles.listbox1, 'Enable', 'off');
    set(mainHandles.pushbutton21, 'Visible', 'on');
    drawnow update;
    % For checking the value of nswp (number of sweeps in data):
    %fprintf('The number of sweeps given to sunPlot is: %d\n', nswp)
    
    %% Map operations:
    
    % Set "overbright" values to map max value if sun-measure data:
    if(strcmp(dataSource, 'sm') == 1)
        map_size_temp = size(map);
        for ind_j = 1 : map_size_temp(1)
            for ind_g = 1 : map_size_temp(2)
                if map(ind_j,ind_g) < -30000
                     map(ind_j,ind_g) = max(max(map));
                end
            end
        end        
    end
    
    % Convert log units to lin
   if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1 && saveToWeb == 0)
        map = log_to_lin(map);
   end
    
    % User has selected % of QSL units (default)
    % ---------------------------------
    if(opt.dataUnits == 0 || saveToWeb == 1)
           
        % Normalizing data so that minimum is at zero:
        map = map - min(min(map));
        
        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun = qsl_finder(map, opt.dataUnits);

        % Calling qsl_units_transformer() to transform map units from
        % original to '% of QSL' units:
        map = qsl_units_transformer(map, quiet_sun,freq);

        max_temp = max(max(map));
        min_temp = min(min(map));

        quiet_sun_level = 100;

    end % End of % of QSL data units
    % ---------------------------------

    % User has selected original map data units:
    % ---------------------------------
    if(opt.dataUnits == 1 && saveToWeb == 0)
        
        max_temp = max(max(map));
        min_temp = min(min(map));

        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun = qsl_finder(map, opt.dataUnits);
        quiet_sun_level = quiet_sun;
       
       % Calculate "low" value for use with automatic levels
       % For this the % of QSL units are needed:
       % ---------------------------------------------------------------
       
       % Normalizing data so that minimum is at zero:
        map_tmp = map - min(min(map));
        
        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun_tmp = qsl_finder(map_tmp, opt.dataUnits);

       % TRANSFORMING MAP UNITS FROM mV to Percent (%)
       % Median = 100%
       % minimum = 0%
       % maximum > 100%
       map_tmp = (map_tmp * 100) / quiet_sun_tmp;
              
       % READJUSTING THE MAP SO THAT THE MINIMUM LEVEL (= SKY TEMP LEVEL) IS
       % SET AT 300 KELVIN, WHICH IS ~3,85 % OF QSL (300 K / 7800 K):
       % 22 GHz: 9000 K (300 / 9000 ~0,033333333)
       % 37 GHz: 7800 K (300 / 7800 ~0,038461538)
       % 86 GHz: 7200 K (300 / 7200 ~0,041666667)

       if(strcmp(freq, '37 GHz') == 1)
           map_tmp = map_tmp + (3.8461538*(100-map_tmp) / 100);
           sky_tmp = 3.8461538;
       elseif(strcmp(freq, '87 GHz') == 1)
           map_tmp = map_tmp + (4.1666667*(100-map_tmp) / 100);
           sky_tmp = 4.1666667;
       elseif(strcmp(freq, '22 GHz') == 1)
           map_tmp = map_tmp + (3.333333*(100-map_tmp) / 100);
           sky_tmp = 3.333333;
       end
       max_temp_tmp = max(max(map_tmp));
       low_tmp = (-0.0287*(max_temp_tmp)^2 + 5.3249*max_temp_tmp -148.5441); % 'low_tmp' is the value form which upwards the map is plotted
       
       % Experimental:
       low_tmp = (((100-sky_tmp)*min(min(map))*quiet_sun_tmp) / ((100+sky_tmp)*quiet_sun_tmp)) - ((sky_tmp*quiet_sun_tmp) / (100+sky_tmp)) + ((low_tmp*quiet_sun_tmp) / 100 + sky_tmp);
       % / Experimental
       
       %low_tmp = min_temp + ((low_tmp - 5) / 100) * abs(max_temp - min_temp);
       % ---------------------------------------------------------------

       
    end % End of original data units
    % ---------------------------------
            
    %% Map levels-operations:
    
    % Protection from invalid custom levels values:
    if(get(mainHandles.checkbox31, 'Value') == 1)
        if(str2num(get(mainHandles.edit10, 'String'))) >= str2num(get(mainHandles.edit11, 'String'))
            message = ['The minimum custom levels value (', get(mainHandles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(mainHandles.edit11, 'String'), ').'];
            msgbox(message, 'Warning', 'warn', 'modal');
        return;
        end
        if(c_min >= max_temp)
            message = ['The minumun custom levels value (', num2str(c_min), ') has to be lower than the maximum value (', num2str(max_temp), ') of the selected map.'];
            msgbox(message, 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit10, 'String')))
            msgbox('Please insert a number value to the minimum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit11, 'String')))
            msgbox('Please insert a number value to the maximum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
    end
    
    % NUMBER OF LEVELS DEPENDS ON USER SELECTED VALUE OR THE USER SELECTED COLORMAP:
    if(customNumOfLevelsToggle == 1.0) % Custom number of levels on
        
        lvls = str2num(customNumOfLevels);
             
    elseif(customNumOfLevelsToggle == 0.0 && (mapPlottingType == 3.0 || mapPlottingType == 2.0))
        
        lvls = 32; % Default number of levels value for "Filled contours with contour lines" and "Contour lines only" maps
        
    elseif(strcmp(cmap, 'default') == 1)
        lvls = 512;
        
    elseif(strcmp(cmap, 'jet16') == 1)
        lvls = 16;
    
    elseif(strcmp(cmap, 'jet32') == 1)
       lvls = 32;    
        
    elseif(strcmp(cmap, 'jet64') == 1)
        lvls = 64;

    elseif(strcmp(cmap, 'jet128') == 1)
        lvls = 128;      
        
    elseif(strcmp(cmap, 'jet256') == 1)
       lvls =  256;       
           
    elseif(strcmp(cmap, 'green16') == 1)
       lvls = 16;
        
    elseif(strcmp(cmap, 'green32') == 1)
      lvls = 32;
            
    elseif(strcmp(cmap, 'green512') == 1)
        lvls = 512;    
        
    elseif(strcmp(cmap, 'violetjet1024') == 1)
        lvls = 1024;
        
    elseif(strcmp(cmap, 'gray16') == 1)
        lvls = 16;
 
    elseif(strcmp(cmap, 'gray32') == 1)
        lvls = 32;
    
    elseif(strcmp(cmap, 'gray512') == 1)
        lvls = 512;
    
    elseif(strcmp(cmap, 'summer16') == 1)
        lvls = 16;       
        
    elseif(strcmp(cmap, 'summer32') == 1)
        lvls = 32;
        
    elseif(strcmp(cmap, 'summer512') == 1)
        lvls = 512;
    
    elseif(strcmp(cmap, 'autumn16') == 1)
        lvls = 16;    
    
    elseif(strcmp(cmap, 'autumn32') == 1)
        lvls = 32;    

    elseif(strcmp(cmap, 'autumn512') == 1)
        lvls = 512;   
 
    elseif(strcmp(cmap, 'winter16') == 1)
        lvls = 16;           
        
    elseif(strcmp(cmap, 'winter32') == 1)
        lvls = 32;          
        
    elseif(strcmp(cmap, 'winter512') == 1)
        lvls = 512;          
        
    elseif(strcmp(cmap, 'hot16') == 1)
        lvls = 16;
        
    elseif(strcmp(cmap, 'hot32') == 1)
        lvls = 32;
        
    elseif(strcmp(cmap, 'hot50') == 1)
        lvls = 50;
        
    elseif(strcmp(cmap, 'hot512') == 1)
        lvls = 512;
 
        
    elseif(strcmp(cmap, 'mro_red_16') == 1)
        lvls = 16;
        
        
    elseif(strcmp(cmap, 'mro_red_32') == 1)
        lvls = 32;
        
                
    elseif(strcmp(cmap, 'mro_red_50') == 1)
        lvls = 51; % this has been changed to 51 to avoid a weird error in the map's colors with 90-112%
        
                
    elseif(strcmp(cmap, 'mro_red_512') == 1)
        lvls = 512;

    elseif(strcmp(cmap, 'bone16') == 1)
        lvls = 16;

    elseif(strcmp(cmap, 'bone32') == 1)
        lvls = 32;

    elseif(strcmp(cmap, 'bone512') == 1)
        lvls = 512;

    elseif(strcmp(cmap, 'colorcube16') == 1)
        lvls = 16;    

    elseif(strcmp(cmap, 'colorcube32') == 1)
        lvls = 32;

    elseif(strcmp(cmap, 'colorcube512') == 1)
       lvls = 512;     
        
    elseif(strcmp(cmap, 'cool16') == 1)
        lvls = 16;          

    elseif(strcmp(cmap, 'cool32') == 1)
        lvls = 32;   

    elseif(strcmp(cmap, 'cool512') == 1)
        lvls = 512;     

    elseif(strcmp(cmap, 'copper16') == 1)
        lvls = 16; 

    elseif(strcmp(cmap, 'copper32') == 1)
        lvls = 32;    

    elseif(strcmp(cmap, 'copper512') == 1)
        lvls = 512;    

    elseif(strcmp(cmap, 'flag16') == 1)
        lvls = 16;

    elseif(strcmp(cmap, 'flag32') == 1)
        lvls = 32; 

    elseif(strcmp(cmap, 'flag512') == 1)
        lvls = 512;
        
    elseif(strcmp(cmap, 'hsv16') == 1)
        lvls = 16;

    elseif(strcmp(cmap, 'hsv32') == 1)
        lvls = 32;

    elseif(strcmp(cmap, 'hsv512') == 1)
        lvls = 512;
        
    elseif(strcmp(cmap, 'lines16') == 1)
        lvls = 16; 

    elseif(strcmp(cmap, 'lines32') == 1)
        lvls = 32;

    elseif(strcmp(cmap, 'lines512') == 1)
        lvls = 512;
        
    elseif(strcmp(cmap, 'pink16') == 1)
        lvls = 16;

    elseif(strcmp(cmap, 'pink32') == 1)
        lvls = 32;

    elseif(strcmp(cmap, 'pink512') == 1)
        lvls = 512;    
        
    elseif(strcmp(cmap, 'prism16') == 1)
        lvls = 16;

    elseif(strcmp(cmap, 'prism32') == 1)
        lvls = 32;

    elseif(strcmp(cmap, 'prism512') == 1)
        lvls = 512;          
        
    elseif(strcmp(cmap, 'spring16') == 1)
        lvls = 16;

    elseif(strcmp(cmap, 'spring32') == 1)
        lvls = 32;

    elseif(strcmp(cmap, 'spring512') == 1)
        lvls = 512;
        
    elseif(strcmp(cmap, 'white') == 1)
        lvls = 32; % Makes no difference
      
    elseif(strcmp(cmap, 'MRO_rainbow_16') == 1)
        lvls = 16;
        
    elseif(strcmp(cmap, 'MRO_rainbow_32') == 1)
        lvls = 32;
        
    elseif(strcmp(cmap, 'MRO_rainbow_512') == 1)
        lvls = 512;        
        
    else
        lvls = 512;
    end
    
    % Defining the max and min values of the map    
    if(abs_scale == 1)
        if(get(mainHandles.checkbox37, 'Value') == 1 && saveToWeb == 0) % Custom level separation is checked and saveToWeb is 0
            jump = str2num(get(mainHandles.edit13, 'String')); % Separation in % of QSL defined by user           
            levels = c_min: jump :c_max;
        else
            low = c_min;
            jump = (c_max - low) / lvls;
            levels = low: jump :c_max;    
        end
    else
        if(opt.dataUnits == 0)
            low = (-0.0287*(max_temp)^2 + 5.3249*max_temp -148.5441); % 'low' is the value form which upwards the map is plotted
            % the coefficents have been calculated using some known good
            % low-values for different max_temp-values and by fitting a 2nd
            % degree function to those.
        else
            low = low_tmp; % Original units selected --> low has been calculated earlier
        end
        
        if(get(mainHandles.checkbox37, 'Value') == 1) % Custom level separation is checked
            jump = str2num(get(mainHandles.edit13, 'String')); % Separation in % of QSL defined by user           
        else
            jump = (max_temp - low) / lvls;
        end
        
        levels = low: jump :max_temp;   
    end
    
    size(levels);

    %% Getting rid of "off-sun data" (reflections and what not):
    
    map = map_edges_cleaner(map, mapType, dataSource, nswp, sps);
    
    %% Data size vs. grid, map rotation:
    
    % Make sure variables are double:
    nswp = double(nswp);
    sps = double(sps);
    w = double(w);
    h = double(h);
    
    % Write variables into struct for calling map_handler():
    
    % Common variables:
    mhStructIn.map = map;
    mhStructIn.dataSource = dataSource;
    mhStructIn.mapType = mapType;
    mhStructIn.nswp = nswp;
    mhStructIn.sps= sps;
    mhStructIn.w = w;
    mhStructIn.h = h;
    mhStructIn.dec = dec;
    
    % temp
    if(strcmp(dataSource, 'sm') == 1)
        mhStructIn.startra = dataDim.startra;
        mhStructIn.endra = dataDim.endra;
        mhStructIn.startdec = dataDim.startdec;
        mhStructIn.enddec = dataDim.enddec;
    end
    % /temp

    % sun-measure specific variables:
    if(strcmp(dataSource, 'sm') == 1)
        mhStructIn.sweepDir =  sweepDir;
        mhStructIn.sampleDir = sampleDir;
        mhStructIn.cosCorrect = opt.cosCorrect;
    end

    % Call map_handler():
    mhStructOut = map_handler(mhStructIn);

    % Write variables from map_handler() output struct:
    map = mhStructOut.map;
    if(strcmp(dataSource, 'sm') == 1)
        x = mhStructOut.x;
        y = mhStructOut.y;
    else
        x = mhStructOut.x;
        y = mhStructOut.y;
    end
    x_fov = mhStructOut.x_fov;
    y_fov = mhStructOut.y_fov;
    mapWidth = mhStructOut.mapWidth;
    mapHeight = mhStructOut.mapHeight;
    nswp = mhStructOut.nswp;
       
    %% Convert numbers to strings for printing on the contour map
    max_temp_str = num2str(round(max_temp * 100) / 100);
    
    %% Plotting the figure, tweaking the settings:

    % Dealing temporarily with a single SMART sweep or multiple sweeps from the same coordinates (x or y size is zero/empty):
    % This part has become bloated, which is not nice. The saving part has
    % to be made into a separate function
    if(nswp == 1 && strcmp(dataSource, 'SMART') == 1 || isempty(y) || isempty(x))
        fig_number = num;
        while(isempty( find(get(0, 'children') == fig_number)) == 0)
            fig_number  = fig_number + 1;
        end
        current_fig = figure;
        %set(mainHandles.text65, 'Visible', 'off');
        plot(map'),

        % Saving:
        
        % Saving figure as a "screenshot"
        if(opt.img_save == 1.0)   
            current_fig = figure;
            X=getframe(current_fig);
            % refresh(gcf);

            % "Save to web" is on => Thumbnail saved, filenames in YYYYMMDDNN.png-notation, 
            if(saveToWeb == 1.0)
                thumbnail = imresize(X.cdata, [61 82]);
                %smallMap = imresize(X.cdata, [340 458]);
                thumbFolder = [destFolder, 'thumbs/'];
                %smallMapFolder = [destFolder, 'small/'];
                origFolder = [destFolder, 'orig/'];

                if(strcmp(obstitle(1), 's') == 1) % file is SMART map

                    year = [obstitle(4), obstitle(5)];

                    if(str2num(year) > 74)
                        newyear = ['19', year];
                    else
                        newyear = ['20', year];
                    end

                    obstitle = [obstitle(1), obstitle(2), obstitle(3), newyear, obstitle(6), obstitle(7), obstitle(8), obstitle(9), obstitle(10), obstitle(11)];

        %         elseif(strcmp(obstitle(1), 't') == 1) % file is a track
        %             
        %             year = [obstitle(6), obstitle(7)];
        %             
        %             if(str2num(year) > 74)
        %                 newyear = ['19', year];
        %             else
        %                 newyear = ['20', year];
        %             end
        %             
        %             obstitle = [obstitle(1), obstitle(2), obstitle(3), obstitle(4), obstitle(5), newyear, obstitle(10), obstitle(11), obstitle(12), obstitle(13), obstitle(13), obstitle(14), obstitle(15), obstitle(16), obstitle(17), obstitle(18)];

                elseif(strcmp(obstitle(1), 't') ~= 1) % Map is sun-measure map or track (File is not a SMART track or a SMART map):

                    % time
                    % Sun-measure files are named: sunYYYYMMDD_HHMM, where HHMM is
                    % the map starting time
                    obsYear = [time(1), time(2), time(3), time(4)]; % YYYY
                    obsMonth = [time(6), time(7)];
                    obsDay = [time(9), time(10)];
                    obsHour = [time(13), time(14)];
                    obsMinute = [time(16), time(17)];
                    obstitle = ['sun', obsYear, obsMonth, obsDay, '_', obsHour, obsMinute];

                end

                if exist(thumbFolder, 'dir') == 0
                    mkdir(thumbFolder);
                end
                %if exist(smallMapFolder, 'dir') == 0
                %    mkdir(smallMapFolder);
                %end
                imwrite(thumbnail, [thumbFolder, obstitle, opt.img_type]); % thumb
                %imwrite(smallMap, [smallMapFolder, obstitle, opt.img_type]); % small map
                imwrite(X.cdata, [origFolder obstitle opt.img_type]); % large image
            else
                % "Save to web" is off => No thumbmnail, filename in sunYYMMDDNN.png / NNNNNNNNNN.png-notation
                imwrite(X.cdata, [destFolder obstitle opt.img_type]);

            end % End of Save to web -part


        end

        % Saving selected figures as animated gif:

        if(opt.anim_gif == 1 && saveToWeb == 0.0)
            %set(gca,'nextplot','replacechildren','visible','off')
            %global animGif;
            current_fig = figure;
            X=getframe(current_fig);
            global gifIm;
            global gifMap;
            global Sunmap_gifStartName;
            global Sunmap_gifStopName;
            plottinglist_size = size(get(mainHandles.listbox1, 'Value'));

            if(num == 1)
                [gifIm,gifMap] = rgb2ind(X.cdata,65536,'dither');
                gifIm(1,1,1,plottinglist_size(2)) = 0;
                Sunmap_gifStartName = obstitle;
            end

            gifIm(:, :, 1, num) = rgb2ind(X.cdata, gifMap, 'dither');

            if(num == plottinglist_size(2))
                Sunmap_gifStopName = obstitle;
                anim_name = [Sunmap_gifStartName, '-', Sunmap_gifStopName];

                savingHandles = guidata(opt.savingHandles);
                secPerFrame = str2num(get(savingHandles.edit2, 'String'));

                cmp_gif = strcmp(num2str(str2num(get(savingHandles.edit2, 'String'))), '');
                %cmp2 = strcmp(num2str(str2num(get(handles.edit11, 'String'))), '');
                if(cmp_gif == 1)
                    msgbox('You have entered something else than numbers to the seconds per frame field. Please use only numbers.', 'Warning', 'warn', 'modal');
                    return;
                else

                    secPerFrame = str2num(get(savingHandles.edit2, 'String'));

                    secPerFrame = round(100 * secPerFrame) / 100; % Round to 0.01 second accuracy

                    if(secPerFrame < 0.01)
                        secPerFrame = 0.01;
                    end

                    %fps = 1 / secPerFrame;

                end

                imwrite(gifIm,gifMap,[destFolder, anim_name, '_animation.gif'], 'DelayTime', secPerFrame, 'LoopCount', inf); % 0.1 seconds ==> 10 fps
            end

        end
        % / Animated gif

        % saving figure in figure-form (same destination as with .png's)
        if(opt.fig_save == 1.0)
            dest = strcat(destFolder, obstitle);
            saveas(current_fig, dest,'fig');
        end
        
        % / Saving
        
        % Finishing up with GUI:
        mainHandles = guidata(sunmap_handles);
        set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);

        if(opt.img_save == 1 || opt.fig_save == 1 || opt.anim_gif == 1)
            set(mainHandles.text39, 'String', 'Saving done');
        else
            set(mainHandles.text39, 'String', 'Plotting done');
        end

        if(ispc)
            set(mainHandles.text39, 'FontSize', 8);
        end
        set(mainHandles.plot_single_button, 'Enable', 'on');
        set(mainHandles.plot_selected, 'Enable', 'on');
        set(mainHandles.plot_all_button, 'Enable', 'on');
        set(mainHandles.listbox1, 'Enable', 'on');
        set(mainHandles.pushbutton21, 'Visible', 'off');

        if(opt.close_prev == 1.0)       
            close(current_fig);
        end


        if(opt.img_save == 1 || opt.fig_save == 1 || opt.anim_gif == 1)

            savingHandles = guidata(opt.savingHandles);

            plottinglist_size = size(get(mainHandles.listbox1, 'Value'));
            if(num == plottinglist_size(2)) % If the current figure is the last one of the ones being saved:
                if(num == 1)
                    fig_str = 'figure';
                else
                    fig_str = 'figures';
                end
                string = ['Saving done! ', num2str(num), ' ', fig_str, ' saved to ', opt.destFolder];

                if(opt.anim_gif == 1 && opt.img_save == 0 && opt.fig_save == 0)
                    string = ['Saving done! Gif animation with ', num2str(num), ' ', 'frames saved to ', opt.destFolder];
                end

                set(savingHandles.text5, 'String', string);
                set(savingHandles.text5, 'ForegroundColor', [0 0.5 0]);
                set(savingHandles.pushbutton3, 'String', 'Close');
                set(savingHandles.pushbutton3, 'Enable', 'on');
                set(savingHandles.text5, 'Enable', 'on');
            end

        end
        % / Finishing up with GUI
        
        
        return;
    end
    % / Dealing temporarily with a single SMART sweep or multiple sweeps from the same coordinates (x or y size is zero/empty):
     
    
    % MAIN PLOTTING:
    sunData = sun_data(time);
    rotDeg = sunData.rot;
    B0 = sunData.B0;
    L0 = sunData.L0;
    %solar_radius = sunData.radius;
    
    % Resize map matrix to add fineness to data grid
    if(get(mainHandles.checkbox38, 'Value') == 1.0 && saveToWeb == 0)

        % Making a copy of the original map data and size vectors:
        map_orig = map;
        x_orig = x;
        y_orig = y;
         
%         [foo, xSize] = size(x);
%         [foo, ySize] = size(y);
%         [dataHeight, dataWidth] = size(map);
%         % By choosing 9 as the scaling factor the original data points
%         % can be preserved:
%         if(dataHeight >= dataWidth) % DE-map
%             map = imresize(map, [dataHeight 9*dataWidth], 'bilinear');
%             x = imresize(x, [1 9*xSize], 'bilinear');
%         else % RA-map
%             map = imresize(map, [9*dataHeight dataWidth], 'bilinear');
%             y = imresize(y, [1 9*ySize], 'bilinear');
%         end
       
    else
        
        % Making a copy of the original map data and size vectors anyways:
        map_orig = map;
        x_orig = x;
        y_orig = y;
        
    end
    
    % Create a version of the map that is non-resized and that has QSL
    % units regardless of the user's selection in the GUI
    % This version is used when looking for and marking active areas on the
    % map:    
    
    % User has selected "% of QSL" units for plotting --> the map used for plotting is already in the units needed by the center_finder() function
    if(opt.dataUnits == 0)
        
        map_orig_qsl = map_orig; % This version is needed when printing active area contour on the plot
   
    % User has selected original map data matrix units for plotting --> map variable is on the wrong units for center_finder() --> create map variable with "% of QSL" units
    else

        % Normalizing data so that minimum is at zero:
        map_orig_qsl = map_orig - min(min(map_orig));

        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun_orig_qsl = qsl_finder(map_orig_qsl, opt.dataUnits);

        % Calling qsl_units_transformer() to transform map units from
        % original to '% of QSL' units:
        map_orig_qsl = qsl_units_transformer(map_orig_qsl, quiet_sun_orig_qsl,freq);
        
    end
    
    
    fig_number = num;
%     while(isempty( find( get(0, 'children') == fig_number) ) == 0)
%         fig_number  = fig_number + 1;
%     end
    % Create a new figure:
    current_fig = figure('Visible', 'off'); % The figure is created as invisible, but turned visible after resized
   
    %if(ispc)
    set(current_fig, 'Renderer', 'zbuffer'); % forcing renderer to zbuffer in win mahcines to keep maps similar in win & linux
    %end
    tr = hgtransform;
    if(opt.rot_disable == 0) % Map rotation is not disabled:
        
        % Map type is set to "Contour lines only":
        if(mapPlottingType == 3.0)
            warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
            [plaa1, cont2] = contour(x, y, map, levels, '-k', 'Parent', tr);
            
            % Contour line annotations:
            if(get(mainHandles.checkbox36, 'Value') == 1.0)
                %set(cont2,'ShowText','on','TextStep', 5),
                text_handle = clabel(plaa1, cont2, 'FontSize',9, 'Labelspacing',1000, 'Parent', tr); % "Warning: Text handle output is not supported for managed labels."
                set(text_handle,'BackgroundColor',[1 1 .6],...
                'Edgecolor',[.7 .7 .7], 'Rotation', 0);
                %rotate(text_handle,[0 0 1],-rotDeg)
            end
            
        % Other map types:
        else
            [plaa1, cont2] = contourf(x,y,map,levels, 'Parent', tr);
        end

    else % Map rotation is disabled:
               
        % Map type is set to "Contour lines only":
        if(mapPlottingType == 3.0)
            %warning('off', 'MATLAB:contour:DeprecatedV6Argument');
            warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
            [plaa1, cont2] = contour(x, y, map, levels, '-k', 'Parent', tr);
             
            % Contour line annotations:
            if(get(mainHandles.checkbox36, 'Value') == 1.0)
                %set(cont2,'ShowText','on','TextStep', 5),
                text_handle = clabel(plaa1, cont2, 'FontSize',9, 'Labelspacing',1000, 'Parent', tr);
                set(text_handle,'BackgroundColor',[1 1 .6],...
                'Edgecolor',[.7 .7 .7], 'Rotation',0)
            end
            
        % Map type is set to "Filled contours, no contour lines (Default)" or "Filled contours with contour lines":
        else
            [plaa1, cont2] = contourf(x,y,map,levels, 'Parent', tr);
        end
        
        if(get(mainHandles.checkbox28, 'Value') == 1.0)
            datacursormode on,
            cursor = datacursormode(current_fig);
            set(cursor, 'SnapToDataVertex','off');
        end
        % set(cursor,'UpdateFcn',@data_cursor_mod); % calling the function that modifies the output of the data cursor
    end
    
    % Contour line annotations:
    if(mapPlottingType == 3.0 && get(mainHandles.checkbox36, 'Value') == 1.0)
        % [C, h] = contour(x, y, map, levels, '-k'),
        %set(cont2,'ShowText','on','TextStep', 5),
        %text_handle = clabel(plaa1, cont2, 'FontSize',9, 'Labelspacing',1000);
        %size(cont2)
    end
    
    % When rotDeg == -300, the rot angle hasn't been found
    % Rotating solar north straight up:
    if(rotDeg ~= -300 && opt.rot_disable == 0)
        % Rotation done with hgtransform
        set(tr,'Matrix',makehgtform('zrotate',-rotDeg*pi/180))
        %rotate(get(cont2, 'Children'),[0 0 1],-rotDeg);
    end
    
     % Map type is set to "Filled contours, no contour levels (Default)":
    if(mapPlottingType == 1.0)
        set(cont2,'LineStyle','none');
        %shading flat,
    end
    
    % QSL contour drawn on map:
    %opt.qsl_visual = 1;
    if(opt.qsl_cont == 1)
%          x1 = -15 - 0.5;
%          y1 = -15 - 1;
%          rectangle('Curvature',[1 1], 'Position', [x1 y1 31 31], 'Linewidth', 3, 'edgecolor', 'w');
        %warning('off', 'MATLAB:contour:DeprecatedV6Argument');
        warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
        hold on
        if(get(mainHandles.popupmenu9, 'Value') == 1.0) % "Filled contours, no contour lines"
            [plaa3, qslCont2] = contour(x, y, map, [quiet_sun_level quiet_sun_level], '-k', 'Parent', tr);
        elseif(get(mainHandles.popupmenu9, 'Value') == 2.0) % "Filled contours with contour lines"
            [plaa3, qslCont2] = contour(x, y, map, [quiet_sun_level quiet_sun_level], '-m', 'Parent', tr);
        else % "Contour lines only"
            [plaa3, qslCont2] = contour(x, y, map, [quiet_sun_level quiet_sun_level], '-m', 'Parent', tr);
        end
        %if(rotDeg ~= -300 && opt.rot_disable == 0)
            % Rotation done with hgtransform
            %set(qslCont2,'Parent', tr);
            %rotate(get(qslCont2, 'Children'), [0 0 1], -rotDeg);
        %end

    end
    
    
    % ------------    
    
    % COLORMAP:
    set_colormap(cmap);

    % simpleMap determines a mode where the maps are plotted
    % with minimal info, without a data axis and without a colorbar:
    if(saveToWeb == 0) % simpleMap selection only read when saveToWeb isn't on
        simpleMap = mainHandles.fig_simple;
    else % If saveToWeb is on, then simpleMap is set to 0
        simpleMap = 0;
    end
    
    if(grid_toggle == 1 && simpleMap == 0)
        grid on,
    end
    
    set(gca,'Layer','top'),

    if (abs_scale == 1)
        caxis([c_min, c_max]);
    end
    
    axis equal,
    if(simpleMap == 0 && get(mainHandles.checkbox56, 'Value') == 0)
        xlim([-21 21]) % -21 21
        ylim([-y_fov y_fov])
    elseif(get(mainHandles.checkbox56, 'Value') == 1) % Auto axis limits
        axis tight,
    end
    
    % Determening colors for the plot:
    % Contour lines only:
    if(mapPlottingType == 3.0)
        plotFontColor = 'k';
        plotBgColor = 'w';
    % Other map types
    else
        plotFontColor = 'w';
        plotBgColor = 'k';
    end
    
    % ADDING COLORBAR FOR CONTOUR LEVELS:
    
    if(mapPlottingType ~= 3.0 && simpleMap == 0)    
        h = colorbar;
        if(ispc)
            colorBarFontSize = 11;
        else
            colorBarFontSize = 11;
        end
        if(opt.dataUnits == 0) % [% of QSL] levels (DEFAULT)
            set(get(h,'ylabel'),'string','Percentage of Quiet Sun Level','Color',plotFontColor, 'FontSize', colorBarFontSize, 'FontWeight', 'normal');
            h.Color = plotFontColor;
        elseif(saveToWeb == 0) % Original levels (only allowed when saveToWeb isn't in use)
            set(get(h,'ylabel'),'string','Signal strength','Color',plotFontColor, 'FontSize', colorBarFontSize, 'FontWeight', 'normal');
        else % [% of QSL] levels (DEFAULT). ... and this if-elseif-else way of doing this is stupid...
            set(get(h,'ylabel'),'string','Percentage of Quiet Sun Level','Color',plotFontColor, 'FontSize', colorBarFontSize, 'FontWeight', 'normal');
        end
    end
    
    sweep = num2str(nswp);
    
    % Direction string:
    if(strcmp(mapType, 'ramap') == 1)
        direction = 'Right Ascension';
    elseif(strcmp(mapType, 'demap') == 1)
        direction = 'Declination';
    elseif(strcmp(mapType, 'azel') == 1)
        direction = 'Azimuth';
    end
    
    % QSL string:
    if(opt.dataUnits == 0) % [% of QSL] levels
        qslString = ', Quiet Sun Level: 100%,';
        unitString = '%';
        simpleMapMaxString = ['Max: ',  num2str(max_temp_str), '% of QSL'];
    elseif(saveToWeb == 0) % Original units:
        qslString = [', Quiet Sun Level: ', num2str(round(quiet_sun_level*100) /100), ','];
        unitString = '';
        simpleMapMaxString = ['Max level: ',  num2str(max_temp_str)];
    else % [% of QSL] levels
        qslString = ', Quiet Sun Level: 100%,';
        unitString = '%';
        simpleMapMaxString = ['Max: ',  num2str(max_temp_str), '% of QSL'];
    end

    % ADDING THE TITLES AND LABELS FOR X- AND Y-AXIS:    
    if (simpleMap == 0) % Normal map:
        
        if(strcmp(freq, 'N/A') == 1)
            freq_fig = '37 GHz';
        else
            freq_fig = freq;
        end
        
        if(ispc)
            title([10, 'Solarmap ' freq_fig, ' / Metsähovi Radio Observatory', 10, 'Time: ',time, ' UT,', 10, 'Filename: ',obstitle, qslString, ' max: ',  num2str(max_temp_str), unitString,  10, 'Sweep direction: ' direction, ', Number of sweeps: ' sweep], 'FontSize',10, 'Color', plotFontColor, 'FontWeight', 'normal'),
            xlabel('Relative Arc minutes'),
            ylabel('Relative Arc minutes');
        else
            title([10, 10, 'Solarmap ' freq_fig, ' / Metsähovi Radio Observatory', 10, 'Time: ',time, ' UT,', 10, 'Filename: ',obstitle, qslString, ' max: ',  num2str(max_temp_str), unitString,  10, 'Sweep direction: ' direction, ', Number of sweeps: ' sweep], 'FontSize',10, 'Color', plotFontColor, 'FontWeight', 'normal'),
            xlabel('Relative Arc minutes'),
            ylabel('Relative Arc minutes');
        end
    
        % TICK SPACING IN INTERVALS OF 5

        yspace=5; xspace=5;
        set(gca, 'ytick', -yspace*ceil(y_fov/yspace):yspace:yspace*ceil(y_fov/yspace) );
        set(gca, 'xtick', -xspace*ceil(x_fov/xspace):xspace:xspace*ceil(x_fov/xspace) );
        box on, % Draw axis borders

        % CHANGING THE FONTSIZE OF THE X AND Y-AXIS TITLES:

        h_xlabel = get(gca,'XLabel');
        if(ispc)
            set(h_xlabel,'FontSize',11, 'FontWeight', 'normal');
        else
            set(h_xlabel,'FontSize',11, 'FontWeight', 'normal');
        end
        set(h_xlabel,'Color',plotFontColor);
        h_ylabel = get(gca,'YLabel');
        if(ispc)
           set(h_ylabel,'FontSize',11, 'FontWeight', 'normal');
        else
            set(h_ylabel,'FontSize',11, 'FontWeight', 'normal');
        end
        set(h_ylabel,'Color',plotFontColor);

    else % Simple map:
        
        if(strcmp(freq, 'N/A') == 1)
            freq_fig = '37 GHz';
        else
            freq_fig = freq;
        end
        
        if(ispc)
            %mapTitle = title([10, 'MRO ', freq_fig, ', ',  time, ' UT, ', simpleMapMaxString], 'FontSize',8, 'Color', plotFontColor, 'FontWeight', 'normal');
            font_size = 9;
        else
            %mapTitle = title([10, 'MRO ', freq_fig, ', ',  time, ' UT, ', simpleMapMaxString], 'FontSize',8, 'Color', plotFontColor, 'FontWeight', 'normal');
            font_size = 9;
        end
        title_text = text(0.5,0.08, ['Metsähovi Radio Observatory solar map @',freq_fig,  char(10), time, ' UT'], 'FontSize',font_size, 'Color', plotFontColor, 'FontWeight', 'normal', 'Units', 'normalized', 'HorizontalAlignment', 'Center');
       
        axis off;
        if(get(mainHandles.checkbox56, 'Value') == 0) % manual limits only when no dynamic fov
            xlim([-21 21]); % Default
            ylim([-22 21]); % Default
            %xlim([-32 32]); % Eclipse maps
            %ylim([-27 29]); % Eclipse maps
            %xlim([-50 50]); % Eclipse full maps
            %ylim([-27 29]); % Eclipse full maps
            %set(mapTitle, 'Position', [0 -18 0]);
        else
            %set(mapTitle, 'Position', [0 -40 0]);
        end

        set(gca, 'Position', [-0.35 -0.03 1.7 1.1]),
        set(gca, 'XTick', []),
        set(gca, 'YTick', []),
        set(gca, 'box', 'off'),
        set(gca, 'XColor', [0 0 0]), % not working?
        set(gca, 'YColor', [0 0 0]),
        
        
    end % End of simpleMap condition
           
    % CHANGING FIGURE COLORS:

    set(current_fig, 'Color',plotBgColor) % figure "outside" backgound color
    set(gca, 'Color', plotBgColor) % Color of gird background
    if(simpleMap == 0)
        %set(gca, 'Xcolor', plotFontColor, 'Ycolor', plotFontColor); % Color of the axes to white
        ax = gca;
        ax.XColor = plotFontColor;
        ax.YColor = plotFontColor;
    end
    
    %% CREATING A CIRCLE THAT ILLUSTRATES BEAM SIZE ONTO THE FIGURE:
   
    if (beam_size == 1)
    
        if(strcmp(freq, '37 GHz') == 1 || strcmp(freq, 'N/A') == 1)
            freq = '37 GHz';
            str(1) = {'Beam size'};
            str(2) = {'2,4 arc min'};
            beamSize = 2.4;
            beam_circle = rectangle('Curvature',[1 1], 'Position', [18 10.5 beamSize beamSize], 'Linewidth', 1, 'edgecolor', plotFontColor);
        end

        if(strcmp(freq, '87 GHz') == 1)
            freq = '87 GHz';
            str(1) = {'Beam size'};
            str(2) = {'1,05 arc min'};
            beamSize = 1.05;
            beam_circle = rectangle('Curvature',[1 1], 'Position', [18 10.5 beamSize beamSize], 'Linewidth', 1, 'edgecolor', plotFontColor);
        end

        if(strcmp(freq, '22 GHz') == 1)
            freq = '22 GHz';
            str(1) = {'Beam size'};
            str(2) = {'4,0 arc min'};
            beamSize = 4;
            beam_circle = rectangle('Curvature',[1 1], 'Position', [16 9 beamSize beamSize], 'Linewidth', 1, 'edgecolor', plotFontColor);
        end

         % TEXT PROPERTIES:
         % Location is bound to the data grid, which causes the text to be in a
         % different place compared to the circle on almost every figure.
         textLocationH = 20;
         textLocationV = 14.5;

        % CREATING TEXT ANNOTATION FOR CIRCLE:
        beam_str = text(textLocationH, textLocationV, str, 'HorizontalAlignment','right', 'Color', plotFontColor, 'FontSize', 8);
    
    end
    
    
    %% Setting the size of the figure:

    % When map type has colors in it --> colorbar --> map aspect ratio ~4:3
    if(mapPlottingType ~= 3.0)        
        % 2nd item on popupmenu1 in separate figure settings is 560x420 which is matlab default => no action taken then:
        if(mainHandles.fig_size ~= 2)
            position = [300, 200, opt.img_w, opt.img_h];
            set(current_fig,'Position', position);
        end
        
    % Map type is "Contour lines only": no colors --> no colorbar -->
    % map aspect ratio narrower than 4:3:
    else
        position = [300, 200, 572, 500]; % Position from lower left corned of screen 300x200 pixels
        set(current_fig,'Position', position);
    end
    
    if(simpleMap == 1)
        position = [300 200 466 358]; % normal
        %position = [300 200 600 550]; % eclipse
        %position = [300 200 600 350]; % eclipse full
        set(current_fig,'Position', position);
    end
    
    % Set figure visible only after setting the size and position
    % TEST: don't display figure if display figures checkbox is unchecked
    if((opt.img_save == 1 || opt.fig_save == 1 || opt.anim_gif == 1) && saveToWeb == 0.0)
        savingHandles = guidata(opt.savingHandles);
        get(savingHandles.checkbox16, 'Value')
        if(get(savingHandles.checkbox16, 'Value') == 0)
        else
            set(current_fig, 'Visible', 'On');
        end
    else
        set(current_fig, 'Visible', 'On');
    end
    
    %% Adding an arrow illustrating solar north:
        
    if(solar_north == 1) % User has selected to draw marker of solar north on figure
    
        if(rotDeg ~= -300) % A valid rot angle is found
            
            % Map is rotated:
            if(opt.rot_disable == 0)

            %     xLocation = [xStart, xEnd];
                  xLocation = [0.16, 0.16];
            %     yLocation = [yStart, yEnd];
                  yLocation = [0.17, 0.25];

                % Creating arrow, modifying settings
                 arrow = annotation('arrow', xLocation, yLocation);
                 set(arrow, 'HeadLength', 12);
                 set(arrow, 'HeadWidth', 12);
                 set(arrow, 'HeadStyle', 'cback2');
                 set(arrow, 'Color', 'green'); % To "cycle" the arrow color: Since 2011-03-21 the arrow's head was black. This fixed the problem
                 set(arrow, 'Color', plotFontColor);
                 set(arrow, 'LineWidth', 1);

                % ADDING TEXT ANNOTATION FOR SOLAR NORTH UP ARROW:
                arrowAnnHorizontal = -14;
                arrowAnnVertical = -15.5;

                dirStr = 'Solar north';

                rot_arrow_annotation = text(arrowAnnHorizontal, arrowAnnVertical, dirStr, 'HorizontalAlignment','right', 'Color', plotFontColor, 'FontSize', 8, 'FontWeight', 'normal');               

                % Rot angle text:
                rotAnn = (round(rotDeg * 100)) / 100;
                string = ['Rot angle: ' num2str(rotAnn), char(176)];
                rot_angle_string = text(-20, 15, string, 'Color', plotFontColor, 'FontSize', 8, 'FontWeight', 'normal');
                % Location is arc min in the axis    
                
            % The map is not rotated:
            elseif(opt.rot_disable == 1)
                
                rotAnn = (round(rotDeg * 100)) / 100;
                noteString(1) = {['Rot angle: ' num2str(rotAnn), char(176)]};
                %rot_angle_string = text(-20, 15, string, 'Color', plotFontColor, 'FontSize', 9);
                %noteString(1) = {'Map not rotated,'}; % Replace with "Rot angle xx.xx*"
                noteString(2) = {'solar north not up'};
                rot_notification_string = text(-19.8, 14.6, noteString, 'HorizontalAlignment','left', 'Color', plotFontColor, 'FontSize', 8, 'FontWeight', 'normal');
                
            end
            
        elseif(rotDeg == -300) % A valid rot angle is not found
            noteString(1) = {'Rot angle not found,'};
            noteString(2) = {'solar north not up'};
            rot_notification_string = text(-19.8, 14.6, noteString, 'HorizontalAlignment','left', 'Color', plotFontColor, 'FontSize', 8, 'FontWeight', 'normal');
        end

        
    end
    
    %% Finding the center of the solar disk, drawing solar coordinate grid on the map, printing radec offset errors in GUI, resizing map matrix 
       
    % NOTICE: It's assumed that solar maps are made with no ra or dec
    % offsets.
    % How to distinguish between intentional offsets for achieving proper
    % pointing and just a map that's been intentionally aimed off-center of
    % the Sun?
    if(strcmp(dataSource, 'SMART') == 1 && (mapWidth < 30 || mapHeight < 30))
        % Do nothing, if data is from SMART and the map is partial
    elseif((get(mainHandles.checkbox38, 'Value') == 1.0 || strcmp(get(mainHandles.Untitled_121, 'Checked'), 'on') == 1) && rotDeg ~= -300 && saveToWeb == 0) % saveToWeb -option added 2011-06-30

        % Map has already been flipped so that RA-maps and DE-maps are
        % similar at this stage (radec direction-wise)
        % Making three vectors for each direction => average center
        % location from those used as true center:
        
        
        if(mapWidth >= 30 && mapHeight >= 30) % The data's size is larger or about the same size as the disk of the sun, data has to be from sun-measure
                                                            %==> assume the map is _meant_ as a full disk map and that it _contains_ the full disk

            % Write neccessary variables in opt-struct:
            center_opt.x = x;
            center_opt.y = y;
            center_opt.mapWidth = mapWidth;
            center_opt.mapHeight = mapHeight;
            center_opt.rotDeg = rotDeg;
            center_opt.rotate_disable = opt.rot_disable;

            
            % Calling center_finder() function:
            % User has selected "% of QSL" units for plotting --> the map used for plotting is already in the units needed by the center_finder() function
            if(opt.dataUnits == 0)
            
                opt.x = x;
                opt.y = y;
                center_info = center_finder(map, center_opt);
            
            % User has selected original map data matrix units for plotting --> map variable is on the wrong units for center_finder() --> create map variable with "% of QSL" units
            else
                
                % Normalizing data so that minimum is at zero:
                map_qsl = map - min(min(map));
                
                % Calling qsl_finder() to get Quiet Sun Level:
                quiet_sun_qsl = qsl_finder(map_qsl, opt.dataUnits);

                % Calling qsl_units_transformer() to transform map units from
                % original to '% of QSL' units:
                map_qsl = qsl_units_transformer(map_qsl, quiet_sun_qsl,freq);

                opt.x = x;
                opt.y = y;
                center_info = center_finder(map_qsl, center_opt);
                
            end
            
            hCenterCoord = center_info.hCenterCoord_orig;
            vCenterCoord = center_info.vCenterCoord_orig;        

        else % Map smaller than 30 arc min in either direction ==> partial map ==> use map coordinates to illustrate solar center      
                
        hCenterCoord = 60 * (dataDim.startra + dataDim.endra) / 2; % Minus sign because RA grows from right to left
        vCenterCoord = -60 * (dataDim.startdec + dataDim.enddec) / 2;
                
        end

        % Save non-rotation compensated center coordinates:
        hCenterCoord_orig = hCenterCoord;
        vCenterCoord_orig = vCenterCoord;
        
        % Compensate center coordinates for map rotation:
        if(opt.rot_disable == 0) % map rotated => compensation needed
           hCenterCoord = (hCenterCoord_orig * cosd(rotDeg)) + (vCenterCoord_orig * sind(rotDeg));
           vCenterCoord = (vCenterCoord_orig * cosd(rotDeg)) - (hCenterCoord_orig * sind(rotDeg));
        end

        % Drawing coordinate grid on map:
        if(get(mainHandles.checkbox38, 'Value') == 1.0)
        % The solar grid is only printed if the user has selected Solar
        % grid & sphere fitting analysis                    
        
            if(opt.rot_disable == 0) % Map rotated
                ax = axesm('ortho','Origin',[B0 L0 0], 'FalseEasting', hCenterCoord, 'FalseNorthing', vCenterCoord, 'ScaleFactor', sunData.radius, 'MLineLocation', 20, 'PLineLocation', 20, 'GColor', plotFontColor);
            else % Map not rotated => grid must be rotated instead:
                ax = axesm('ortho','Origin',[B0 L0 -rotDeg], 'FalseEasting', hCenterCoord, 'FalseNorthing', vCenterCoord, 'ScaleFactor', sunData.radius, 'MLineLocation', 20, 'PLineLocation', 20, 'GColor', plotFontColor);
            end

            if(L0 <= 180)
                PLabel_meridian = L0 + 20;
            else
                PLabel_meridian = L0 + 20 - 360;
            end

            setm(ax,'MLabelParallel',0, 'FontColor', plotFontColor, 'GAltitude', 0, 'MLabelLocation', 40, 'PLabelLocation', 40, 'PLabelMeridian', PLabel_meridian); % GAltitude determines the z-axis location of the grid. Seeting it to 0 lets the data cursor be above or "in front of" the grid  % ,'PLabelMeridian',0
            mlabel on,
            plabel on,
            gridm on,
            mlabelzero22pi; % Display longitude labels in the range of 0 to 360 degrees
            
            % Redraw the tick labels that are overwritten by axesm():
            yspace=5; xspace=5;
            set(gca, 'ytick', -yspace*ceil(y_fov/yspace):yspace:yspace*ceil(y_fov/yspace) );
            set(gca, 'xtick', -xspace*ceil(x_fov/xspace):xspace:xspace*ceil(x_fov/xspace) );

            % Drawing circle on figure:
            rectangle('Curvature',[1 1], 'Position', [hCenterCoord-sunData.radius vCenterCoord-sunData.radius sunData.radius*2 sunData.radius*2], 'Linewidth', 0.5, 'edgecolor',plotFontColor, 'Tag', 'gridCircle', 'LineStyle', ':'); % Tag given so that this rect can be recognized later

        end
        
        % Trying out centering plot on solar disk center:
        if(strcmp(get(mainHandles.Untitled_121, 'Checked'), 'on') == 1 && saveToWeb == 0)

            %axis([-x_fov+hCenterCoord x_fov+hCenterCoord -y_fov+vCenterCoord y_fov+vCenterCoord]),

            figure_xfov = xlim;
            figure_yfov = ylim;
            xlim([figure_xfov(1)+hCenterCoord figure_xfov(2)+hCenterCoord]);
            ylim([figure_yfov(1)+vCenterCoord figure_yfov(2)+vCenterCoord]);

            % Fixing the locations of annotations and shapes printed on the
            % axis:
            % Rot angle string:
            if(exist('rot_angle_string', 'var'))
                rot_angle_string_location = get(rot_angle_string, 'Position');
                set(rot_angle_string, 'Position', [rot_angle_string_location(1)+hCenterCoord rot_angle_string_location(2)+vCenterCoord 0]);
            end
            % Solar north arrow annotation:
            if(exist('rot_arrow_annotation', 'var'))
                rot_arrow_annotation_location = get(rot_arrow_annotation, 'Position');
                set(rot_arrow_annotation, 'Position', [rot_arrow_annotation_location(1)+hCenterCoord rot_arrow_annotation_location(2)+vCenterCoord 0]);
            end
            % Beam size annotation:
            if(exist('beam_str', 'var'))
                beam_str_location = get(beam_str, 'Position');
                set(beam_str, 'Position', [beam_str_location(1)+hCenterCoord beam_str_location(2)+vCenterCoord 0]);
            end
            % Beam circle:
            if(exist('beam_circle', 'var'))
                beam_circle_location = get(beam_circle, 'Position');
                set(beam_circle, 'Position', [beam_circle_location(1)+hCenterCoord beam_circle_location(2)+vCenterCoord beam_circle_location(3) beam_circle_location(4)]);
            end
            % Simple figure title:
            if(exist('mapTitle', 'var'))
                mapTitle_location = get(mapTitle, 'Position');
                set(mapTitle, 'Position', [mapTitle_location(1)+hCenterCoord mapTitle_location(2)+vCenterCoord mapTitle_location(3)]);
            end
            % Rotation notification string:
            if(exist('rot_notification_string', 'var'))
                rot_notification_location = get(rot_notification_string, 'Position');
                set(rot_notification_string, 'Position', [rot_notification_location(1)+hCenterCoord rot_notification_location(2)+vCenterCoord rot_notification_location(3)]);
            end
        end
        % / Trying out
        
    else
        
%         % Clear the offset strings:
%         set(mainHandles.text63, 'String', '');
%         set(mainHandles.text64, 'String', '');
    end
    
    
    
    %% Finding coordinates of active areas (if there are any):
    
    % Reading Custom Active Region detection treshold from GUI:
    % (Protection from invalid values done in sunmap.m):
    
    if(get(mainHandles.checkbox51, 'Value') == 1)   
        tempTreshold = str2num(get(mainHandles.edit17, 'String'));         
    else
        % Set default value:
        tempTreshold = 104; % "% of QSL"
    end
    spotCheck = get(mainHandles.checkbox41, 'Value'); % Marking of active areas on map
   
    % Define a custom max_temp variable to be used with QSL and original
    % units in the condition for executing the active area search/marking:
    if(opt.dataUnits == 0) % QSL units
        max_temp_cond = max_temp;
    else
        max_temp_cond = max(max(map_orig_qsl));
    end
    
    % Conditions for execution:
    % tempTreshold = 104;
    % Solar center finding is on,
    % Rot angle is found,
    % Active area marking is on, 
    % Map max temp is higher than the active area treshold
    % Map size is larger than 30 arc min in both directions
    if(get(mainHandles.checkbox38, 'Value') == 1.0 && rotDeg ~= -300 && spotCheck == 1 && max_temp_cond >= tempTreshold && mapWidth > 30 && mapHeight > 30 && saveToWeb == 0)

        %[spotRows, spotCols] = find(map_orig > 106); % The original version of the map data and "size vectors" are used
        
        %figure(1);
        
        % Setting up variables needed for calling area_sniffer function:
        %tempTreshold = 104; % "% of QSL"
        opt.mapWidth = mapWidth;
        opt.mapHeight = mapHeight;
        opt.max_temp = max_temp;
        opt.map_orig = map_orig_qsl; % Non-resized QSL units version of the map matrix (calcualted earlier in the code even in the case when the user has selected original units)
        opt.rotDeg = rotDeg;
        opt.hCenterCoord = hCenterCoord_orig; % !!
        opt.vCenterCoord = vCenterCoord_orig; % !!
        opt.radius = sunData.radius;
        opt.x_orig = x_orig;
        opt.y_orig = y_orig;
        opt.B0 = B0;
        opt.L0 = L0;
        %opt.dataUnits = opt.dataUnits;

        % Calling area_sniffer function. areaCoords is a struct that contains
        % data on all the active areas of the map
        areaCoords = area_sniffer(tempTreshold, opt);

        % If areaCoords struct is empty (no spots were found or something went wrong)
%         if(isempty(areaCoords))
%             set(mainHandles.text65, 'Visible', 'off');
%             drawnow update;
%             return;
%         end
% -------------------------------------------------------------------------------------------
        %for i = 1 : length(areaCoords) % This for loop was for showing the
        % actual levels from which upwards certain areas are regarded as
        % separate areas. When this is in use, the tempTreshold has to be
        % taken from the individual areas: areaCoords(i).tempTreshold
        % The problem is that the contours are drawn everywhere, not only
        % around the areas that use that exact treshold
        
        % TESTING
%         for i = 1 : length(areaCoords)
%             
%             % spotElements is a list of all the indexes of the map matrix elements in
%             % the area in question (the elements in the listed indexes are in the area)
%             spotElements = areaCoords(i).spotElements;
%             map_temp = map_orig;
%             map_temp(:,:) = 0; % Set everything to zero, but keep the original size
%             for j = 1 : length(spotElements)
%                 map_temp(spotElements(j)) = map_orig(spotElements(j)); % Set the elements in the current area into their own values --> only wanted area has values --> contour can be drawn
%                 % Bummer, the contours are very "pointy" and rectangular
%                 % The contour drawing needs values under and over the
%                 % specific level (for interpolation)
%             end
%             
%             % --->
%             % The contour plotting part here
%         axes(mainHandles.axes2),
%         hold on,
%         warning('off', 'MATLAB:contour:DeprecatedV6Argument');
%         warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
%         [plaa3, areaCont] = contour('v6',x, y, map, [areaCoords(i).tempTreshold areaCoords(i).tempTreshold], '--k');
%         if(rotDeg ~= -300 && get(mainHandles.checkbox40, 'Value') == 0)
%             rotate(areaCont, [0 0 1], -rotDeg);
%         end
%             % <---
%             
%         end
%         % / TESTING
        
%
%-->
% TEMPTRESHOLD-CONTOURIN PLOTTAUSKOODI OLI TÄSSÄ
%<--
%
        % TESTING
        %plaa3
        %size(plaa3);
        %cont_x = plaa3(1,:);
        %cont_y = plaa3(2,:);
        %polyarea(cont_x, cont_y)
        % areaCont is an array of line handle objects.
        % Every separate contour line has its own handle --> can be deleted if there is a way to know which is which
        % plaa3 seems to have info on the vertices of the contour lines.
        % 2xN matrix
        % All the data is in plaa3. --> How to separate individual
        % lines/areas?
        %areaCont
        %delete(areaCont(1))

        %end
        % / TESTING
% -------------------------------------------------------------------------------------------
   
        % Plot a single contour line to tempTreshold (set at 104 % of QSL 2012-01):
        hold on,
        %warning('off', 'MATLAB:contour:DeprecatedV6Argument');
        warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
        [plaa3, areaCont] = contour(x_orig, y_orig, map_orig_qsl, [tempTreshold tempTreshold], '--k', 'Parent', tr);
        if(rotDeg ~= -300 && get(mainHandles.checkbox40, 'Value') == 0) % hgtransform replaces this
            %rotate(get(areaCont, 'Children'), [0 0 1], -rotDeg, [0 0 0]);
        end

        % Reading active area locations and numbers from web:
        % Setting proxy settings:
        if(ispc == 0) % Use only with non-win computers (= in Metsähovi most likely)
            %jobj = com.mathworks.mlwidgets.html.HTMLPrefs; % This doesn't work
            %setUseProxy(jobj,1);
            %setProxyHost(jobj,'proxy');
            %setProxyPort(jobj,'3128');
        end
        %map_grad = grad(map_orig)
        %datevector = datevec(date);
        year1 = [time(1), time(2), time(3), time(4)];
        dateVector = datevec(date);
        
        if(str2num(year1) >= 1996) % If the map being processed is from the year 1996 or later, data from NOAA can be used
            month1 = [time(6), time(7)];
            day1 = [time(9), time(10)];
            srs_filename = [year1, month1, day1, 'SRS.txt'];
            
            % Calling srs_reader()
            [NOAAspotNumber, NOAAspotLat, NOAAspotLon, NOAAspotAbsLong] = srs_reader(time, sunmap_handles);            
            
%             %clc
%             if(str2num(year1) == dateVector(1)) % If the map is from the current year --> read .txt file from NOAA web server
%                 spotFileUrl = ['http://www.swpc.noaa.gov/ftpdir/warehouse/', num2str(year1), '/SRS/', srs_filename];
%                 try % Try reading the srs file for the current day
%                 spotFile = urlread(spotFileUrl);
%                 % If the file opening fails, notify the user and reset the
%                 % GUI:
%                 catch err
%                     if (strcmp(err.identifier, 'MATLAB:urlread:ConnectionFailed'))
%                         %message = 'Unable to reach NOAA web server. Recognising active areas with NOAA area numbers can''t be done.';
%                         %msgbox(message, 'Error', 'error', 'non-modal');
%                         
%                     else % With other errors just print the error as usual:
%                         rethrow(err);
%                     end
%                     %set(mainHandles.text65, 'Visible', 'off');
%                     %drawnow update;
%                     %return;
%                 end
%                 if(exist('spotFile', 'var'))
%                     spotFileRows = regexp(spotFile,'\n','split');
%                 end
%             else % Map from last year or earlier --> read .txt file from Sunmap's aux_data folder
%                 spotFilePath = [pwd, '/aux_data/NOAA SWPC/', num2str(year1), '_SRS/', srs_filename];
%                 if(exist(spotFilePath, 'file'))
%                     spotFileRows = textread(spotFilePath, '%s', 'delimiter', '\n');
%                 else
% %                     message = ['The NOAA SRS active area data file ', spotFilePath, ' doesn''t seem to exist. If the year has just changed, the data files for last year probably haven''t yet been downloaded to sunmap''s folder: /aux_data/NOAA SWPC/YYYY_SRS/ The data files should be found in NOAA''s web page. E.g. for 2011: http://www.swpc.noaa.gov/ftpdir/warehouse/2011/2011_SRS.tar.gz'];
% %                     msgbox(message, 'Missing data', 'warn', 'modal');
% %                     axes(get(gcf,'CurrentAxes')), % Set the plotting axes to current axes to prevent stuff from being drawn on the error message window
%                 end
%             end
%            
%             % Going through the SRS file (if one has been opened)
%             if(exist('spotFileRows', 'var'))
%                 for idx3 = 1 : length(spotFileRows)
%                     if(strcmp(spotFileRows{idx3}, 'Nmbr Location  Lo  Area  Z   LL   NN Mag Type')  == 1 || strcmp(spotFileRows{idx3}, 'NMBR LOCATION  LO  AREA  Z   LL   NN MAG TYPE')  == 1)
% 
%                         % Write stuff into the NOAA variables:
%                         % Take next row, split it into separate 'words'. If there
%                         % are 8 words in the row, we're dealing with a sunspot row,
%                         % when there are 10 "words" we've reached the next "header"
%                         ind = 0;
%                         thisRow = regexp(spotFileRows{idx3+1 + ind}, '\s+', 'split'); % split tabs & spaces
%                         while (length(thisRow) == 8)
% 
%                             % From the SRS file we read:
%                             % spot number,
%                             % spot latitude,
%                             % spot realtive longitude (not in helicentric coords, but from central meridian seen from Earth)
%                             % These are enough for comparison with the coords from the map
% 
%                             ind = ind + 1;
%                             NOAAspotNumber{ind} = thisRow(1);
%                             %thisRow(1)
%                             NOAAspotLocation{ind} = thisRow(2);
% 
%                             % Extract lat number from lat/long string (SXXWYY / NXXEYY etc.)
%                             NOAAcurrentLatStr = thisRow(2);
%                             NOAAcurrentLatStr = NOAAcurrentLatStr{1};
%                             NOAAspotLatStr = [NOAAcurrentLatStr(1), NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]; % S25, N05 etc.
%                             if(strcmp(NOAAcurrentLatStr(1), 'S') == 1) % South
%                                 NOAAspotLat(ind) = str2num(['-', NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
%                             elseif(strcmp(NOAAcurrentLatStr(1), 'N') == 1) % North
%                                 NOAAspotLat(ind) = str2num([NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
%                             end
% 
%                             %thisRow(2)
%                             NOAAlong_temp = thisRow(3);
%                             NOAAspotAbsLong(ind) = str2num(NOAAlong_temp{1});
%                             %thisRow(3)
%                             NOAAspotArea{ind} = thisRow(4); % This is not used for now
%                             %thisRow(4)
%                             % Read the next row for use on the next iteration
%                             thisRow = regexp(spotFileRows{idx3+1 + ind}, '\s+', 'split'); % split by tabs & spaces
% 
% 
%                             % Starting to read the "IA. H-alpha Plages without Spots" section:
%                             % Checking if we've reached the next header yet
%                             % The length of the row that is read above is checked.
%                             % This: "IA. H-alpha Plages without Spots.  Locations Valid at 19/2400Z Jun"
%                             % ..means that we've reached the next header (length (num of separate "words") is 10 for the row above)
%                             % Checking that the row after that is 'Nmbr Location  Lo' confirms this
%                             % If there is actual spot data available, the length
%                             % of the field after 'Nmbr  Location  Lo' is 3 words
%                             if(length(thisRow) == 10 && strcmp(spotFileRows{idx3+2 + ind}, 'Nmbr  Location  Lo') == 1 && length(regexp(spotFileRows{idx3+3 + ind}, '\s+', 'split')) == 3)
%                                 % Reading a new thisRow:
%                                 thisRow = regexp(spotFileRows{idx3+3 + ind}, '\s+', 'split'); % split by tabs & spaces
% 
%                                 while(length(thisRow) == 3)
% 
%                                     ind = ind + 1; % Jump ind to the next index
%                                     NOAAspotNumber{ind} = thisRow(1);
%                                     %thisRow(1)
%                                     NOAAspotLocation{ind} = thisRow(2);
% 
%                                     % Extract lat number from lat/long string (SXXWYY / NXXEYY etc.)
%                                     NOAAcurrentLatStr = thisRow(2);
%                                     NOAAcurrentLatStr = NOAAcurrentLatStr{1};
%                                     NOAAspotLatStr = [NOAAcurrentLatStr(1), NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]; % S25, N05 etc.
%                                     if(strcmp(NOAAcurrentLatStr(1), 'S') == 1) % South
%                                         NOAAspotLat(ind) = str2num(['-', NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
%                                     elseif(strcmp(NOAAcurrentLatStr(1), 'N') == 1) % North
%                                         NOAAspotLat(ind) = str2num([NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
%                                     end
% 
%                                     NOAAlong_temp = thisRow(3);
%                                     NOAAspotAbsLong(ind) = str2num(NOAAlong_temp{1});
%                                     thisRow = regexp(spotFileRows{idx3+3 + ind}, '\s+', 'split'); % split by tabs & spaces
% 
%                                 end % While loop that goes through rows with 3 "words"
% 
%                                 break;
% 
%                             end % If clause for recognizing the section starting with this kind of a row: "IA. H-alpha Plages without Spots.  Locations Valid at 19/2400Z Jun". The second section in the file that has info on active areas
% 
%                         end % While loop that goes through rows with 8 "words". The first section of the file containing info on active areas
% 
% 
%                     else
%                         % No active areas in the file, or the SRS files aren't as stardardised
%                         % as I've hoped...
%                     end
%                 end
%             end
                                   
            % Compare NOAA lat/long coords with map lat/long coords:
            % Go through areaCoords struct containing info on active areas on the current map
            for i = 1 : length(areaCoords) % 

                NOAAspotNumberStr = []; % Initializing NOAA spot number string as empty
                
                % Go through NOAA spot coords (if it exists)
                if(exist('NOAAspotLat', 'var'))
                    for j = 1 : length(NOAAspotLat)

                        if(NOAAspotLat(j) < areaCoords(i).Lat + 6 && NOAAspotLat(j) > areaCoords(i).Lat - 6 && NOAAspotAbsLong(j) < areaCoords(i).Long + 10 && NOAAspotAbsLong(j) > areaCoords(i).Long - 10)
                            % Match!
                            NOAAspotNumberStr = NOAAspotNumber{j};
                            NOAAspotNumberStr = NOAAspotNumberStr{1};
                            if(str2num(NOAAspotNumberStr) <= 5000 && str2num(year1) > 2002) % Area number 10000 was passed in June 2002 (http://www.cloudynights.com/ubbthreads/showflat.php/Cat/0/Number/3619261/Main/3618592)
                                NOAAspotNumberStr = ['AR1', NOAAspotNumberStr];
                            else
                                NOAAspotNumberStr = ['AR', NOAAspotNumberStr];                         
                            end

                        elseif(isempty(NOAAspotNumberStr)) % NOAA spot number string is empty ==> no match has been found on this "round"
                            NOAAspotNumberStr = 'No match';
                        end
                    end
                else
                    NOAAspotNumberStr = 'N/A';
                end

                % Text of the info box:
                infoStr = sprintf(['ID: ', NOAAspotNumberStr, '\nLat: ', num2str(areaCoords(i).Lat, 4), char(176), '\nLon: ', num2str(areaCoords(i).Long, 4), char(176), '\nRel Lon: ', num2str(areaCoords(i).Long_rel, 4), char(176), '\nMax: ', num2str(areaCoords(i).Max_temp, 5), ' %%']);
                % X that marks the active area:
                text1 = text(areaCoords(i).X, areaCoords(i).Y, 'X', 'HorizontalAlignment','center', 'Color', 'k', 'FontSize', 12, 'Parent', tr);
                % Calculating position for info box in pixels (units in text-function are in units of the map (arc min in this case). We want a constant position in pixels instead --> Conversion needed):
                axes2pos = get(mainHandles.axes2, 'Position'); % size and position of plotting window in pixels
                amin_px_ratio =  (2 * y_fov) / axes2pos(4); % ratio of arc min per pixels in plotting window (arc min / px) (arc min = px * (arc_min / px))
                x_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                y_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                % Info box associated with the X and the active area:
                infoText = text(areaCoords(i).X + x_correct, areaCoords(i).Y + y_correct, infoStr, 'HorizontalAlignment','left', 'Color', 'k', 'FontSize', 6, 'BackgroundColor', [0.99 0.99 0.99], 'EdgeColor', 'k', 'Parent', tr);
                
                % Set info box's z value higher so that the box is shown on
                % top of the active area contour lines
                uistack(infoText, 'top'); % uistack(h) raises object h one unit upwards in the z direction
                % ...aand nothing happens, because the contour lines are
                % plotted after this
                
                % Rotate the X's and labels if the map is rotated:
                if(opt.rot_disable == 0) % map rotated => compensation needed
                    % Rotation done with hgtransform
                    %rotate(text1, [0 0 1], -rotDeg, [0 0 0]);
                    %rotate(infoText, [0 0 1], -rotDeg, [0 0 0]);   
                end
               
            end % End of for loop for going through areaCoords struct containing info active areas on the current map
            
            
        else % Map is from before 1996 --> No NOAA SRS file reading & spot id number
            
            
            for i = 1 : length(areaCoords) % Assume that variables spotLat & spotLong are the same size
                                                         
                NOAAspotNumberStr = 'N/A';
                % Text of the info box:
                infoStr = sprintf(['ID: ', NOAAspotNumberStr, '\nLat: ', num2str(areaCoords(i).Lat, 4), char(176), '\nLon: ', num2str(areaCoords(i).Long, 4), char(176), '\nRel Lon: ', num2str(areaCoords(i).Long_rel, 4), char(176), '\nMax: ', num2str(areaCoords(i).Max_temp, 5), ' %%']);
                %longStr= ['Long: ', num2str(long,3)];
                %maxStr = ['Max: ', num2str(spot_max, 4)];
                % X that marks the active area:
                text1 = text(areaCoords(i).X, areaCoords(i).Y, 'X', 'HorizontalAlignment','center', 'Color', 'k', 'FontSize', 10, 'Parent', tr);

                % Calculating position for info box in pixels (units in text-function are in units of the map (arc min in this case). We want a constant position in pixels instead --> Conversion needed):
                % NOTICE: Using properties of the main gui to determine
                % stuff in a separate figure is really screwed, but it
                % still gives a better result than no compensation at all
                % This all ()active area marking has been copied from preview.m, hence the
                % weirdness
                axes2pos = get(mainHandles.axes2, 'Position'); % size and position of plotting window in pixels
                amin_px_ratio =  (2 * y_fov) / axes2pos(4); % ratio of arc min per pixels in plotting window (arc min / px) (arc min = px * (arc_min / px))
                x_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                y_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                % Info box associated with the X and the active area:
                infoText = text(areaCoords(i).X + x_correct, areaCoords(i).Y + y_correct, infoStr, 'HorizontalAlignment','left', 'Color', 'k', 'FontSize', 6, 'BackgroundColor', [0.99 0.99 0.99], 'EdgeColor', 'k', 'Parent', tr);

                % Rotate the X's and labels if the map is rotated:
                if(opt.rot_disable == 0) % map rotated => compensation needed
                    % Rotation done with hgtransform
                    %rotate(text1, [0 0 1], -rotDeg, [0 0 0]);
                    %rotate(infoText, [0 0 1], -rotDeg, [0 0 0]);
                end
               
            end
            
        end
               
        %[spotLocationY, spotLocationX] = find(max_locations==max(max_locations(:)));
        %figure, imshow(imresize(max_locations, [200 200]))
        %contourf(max_locations)
        %figure(2), plot(hGrad5),
        %for i = 1 : length(spotRows)
            %fprintf('\nSample: %d\nSweep: %d\n', spotCols(i), spotRows(i))
        %end
        
    end
    %tempTreshold = tempTreshold + 0.5;
    %end
        
%% Drawing all NOAA Active Regions on map:

% TESTING:
% Calling srs_reader()
if(get(mainHandles.checkbox38, 'Value') == 1.0 && get(mainHandles.checkbox50, 'Value') == 1)
    [NOAAspotNumber, NOAAspotLat, NOAAspotLon, NOAAspotAbsLong] = srs_reader(time, sunmap_handles);

    year1 = [time(1), time(2), time(3), time(4)];
end
% TESTING:
% Plotting all NOAA Active Regions on map:
%  Go through NOAA spot coords (if it exists),


% Conditions for execution:
% Solar center finding is on,
% Rot angle is found,
% Map size is larger than 30 arc min in both directions
% NOAAspotLat exists (i.e. srs_reader() has returned non empty variables)
% NOAA Active area marking is on
if(get(mainHandles.checkbox38, 'Value') == 1.0 && rotDeg ~= -300 && mapWidth > 30 && mapHeight > 30 && exist('NOAAspotLat', 'var') && get(mainHandles.checkbox50, 'Value') == 1)
    for j = 1 : length(NOAAspotLat)
        % Text of the info box:
        NOAAspotNumberStr = NOAAspotNumber{j};
        NOAAspotNumberStr = NOAAspotNumberStr{1};
        if(str2num(NOAAspotNumberStr) <= 5000 && str2num(year1) > 2002) % Area number 10000 was passed in June 2002 (http://www.cloudynights.com/ubbthreads/showflat.php/Cat/0/Number/3619261/Main/3618592)
            NOAAspotNumberStr = ['AR1', NOAAspotNumberStr];
        else
            NOAAspotNumberStr = ['AR', NOAAspotNumberStr];
        end

        if(L0 >= 90 && L0 <= 270)
            L_rel = NOAAspotAbsLong(j) - L0;
        elseif(NOAAspotAbsLong(j) > 180 && L0 < 90)
            L_rel = NOAAspotAbsLong(j) - 360 - L0;
        elseif(NOAAspotAbsLong(j) > 180 && L0 > 270)
            L_rel = NOAAspotAbsLong(j) - L0;      
        elseif(NOAAspotAbsLong(j) <= 180 && L0 < 90)
            L_rel = NOAAspotAbsLong(j) - L0;
        elseif(NOAAspotAbsLong(j) <= 180 && L0 > 270)
            L_rel = NOAAspotAbsLong(j) + 360 -L0;
        end
        
        infoStr = sprintf(['ID: ', NOAAspotNumberStr, '\nLat: ', num2str(NOAAspotLat(j), 4), char(176),  '\nLon: ', num2str(NOAAspotAbsLong(j), 4), char(176), '\nRel Lon: ', num2str(L_rel, 4), ' / ', num2str(NOAAspotLon(j)), char(176)]);

        if(L0 <= 180) % L0: 0...180
            L0_mod = L0; % The right half of the solar sphere starting from the zero meridian as seen from Earth 0...180 degrees
        else % L0: 180...360
            L0_mod = L0 - 360; % The left half of the solar sphere starting from the zero meridian as seen from Earth 0...-180 degrees
        end

        % X that marks the active area:
        % Calculate x and y from spot lat & rel long with orthographic projection:
        % Compensate center coordinates for map rotation:
        hCenterCoord_mod = (hCenterCoord_orig * cosd(rotDeg)) + (vCenterCoord_orig * sind(rotDeg));
        vCenterCoord_mod = (vCenterCoord_orig * cosd(rotDeg)) - (hCenterCoord_orig * sind(rotDeg));               
        x_loc = sunData.radius * cosd(NOAAspotLat(j)) * sind(L_rel) + hCenterCoord_mod;
        y_loc = sunData.radius * (((cosd(B0)) * sind(NOAAspotLat(j))) - (sind(B0) * cosd(NOAAspotLat(j)) * cosd(L_rel))) + vCenterCoord_mod;
        tr2 = hgtransform;
        text2 = text(x_loc, y_loc, 'X', 'HorizontalAlignment','center', 'Color', 'w', 'FontSize', 12, 'Parent', tr2);
        % Calculating position for info box in pixels (units in text-function are in units of the map (arc min in this case). We want a constant position in pixels instead --> Conversion needed):
        axes2pos = get(mainHandles.axes2, 'Position'); % size and position of plotting window in pixels
        amin_px_ratio =  (2 * y_fov) / axes2pos(4); % ratio of arc min per pixels in plotting window (arc min / px) (arc min = px * (arc_min / px))
        x_correct = 15 * amin_px_ratio; % pixels converted into plotting window arc minutes
        y_correct = 22 * amin_px_ratio; % pixels converted into plotting window arc minutes        
        % Info box associated with the X and the active area:        
        infoText2 = text(x_loc + x_correct, y_loc + y_correct, infoStr, 'HorizontalAlignment','left', 'Color', 'k', 'FontSize', 6, 'BackgroundColor', [0.99 0.80 0.70], 'EdgeColor', 'k', 'Parent', tr2);
        % Rotate the X's and labels if the map is not rotated:
        if(opt.rot_disable == 1) % map NOT rotated => compensation needed in the opposite direction than normally (rotDeg vs. -rotDeg)
            set(tr2,'Matrix',makehgtform('zrotate',rotDeg*pi/180))   
        end
    end
end
% / TESTING
    
        
    %% Saving figure as a "screenshot"

    if(isequal(get(current_fig,'Position'), position))
    else
        disp('incorrect position');
        set(current_fig,'Position', position);
    end

    % Image save and figure display on:
    if(opt.img_save == 1.0 && get(savingHandles.checkbox16, 'Value') == 1)
        %figure(fig_number); % Why is this here?
        X = getframe(current_fig);
        % refresh(gcf);

        % "Save to web" is on => Thumbnail saved, filenames in YYYYMMDDNN.png-notation, 
        if(saveToWeb == 1.0)
            thumbnail = imresize(X.cdata, [61 82]);
            %smallMap = imresize(X.cdata, [340 458]);
            % Create directories by year and month
            dirYear = [time(1), time(2), time(3), time(4)]; % YYYY
            dirMonth = [time(6), time(7)]; % MM
            thumbFolder = [destFolder, 'thumbs/', dirYear, '/', dirMonth, '/'];
            %smallMapFolder = [destFolder, 'small/', dirYear, '/', dirMonth, '/'];
            origFolder = [destFolder, 'orig/', dirYear, '/', dirMonth, '/'];

            if(strcmp(obstitle(1), 's') == 1) % file is SMART map

                year = [obstitle(4), obstitle(5)];

                if(str2num(year) > 74)
                    newyear = ['19', year];
                else
                    newyear = ['20', year];
                end

                obstitle = [obstitle(1), obstitle(2), obstitle(3), newyear, obstitle(6), obstitle(7), obstitle(8), obstitle(9), obstitle(10), obstitle(11)];

            elseif(strcmp(obstitle(1), 't') ~= 1) % Map is sun-measure map or track (File is not a SMART track or a SMART map):

                % time
                % Sun-measure files are named: sunYYYYMMDD_HHMM, where HHMM is
                % the map starting time
                obsYear = [time(1), time(2), time(3), time(4)]; % YYYY
                obsMonth = [time(6), time(7)];
                obsDay = [time(9), time(10)];
                obsHour = [time(13), time(14)];
                obsMinute = [time(16), time(17)];
                obstitle = ['sun', obsYear, obsMonth, obsDay, '_', obsHour, obsMinute];

            end

            if exist(origFolder, 'dir') == 0
                mkdir(origFolder);
            end
            if exist(thumbFolder, 'dir') == 0
                mkdir(thumbFolder);
            end
            %if exist(smallMapFolder, 'dir') == 0
            %    mkdir(smallMapFolder);
            %end
            imwrite(thumbnail, [thumbFolder, obstitle, opt.img_type]); % thumb
            %imwrite(smallMap, [smallMapFolder, obstitle, opt.img_type]); % small map
            imwrite(X.cdata, [origFolder obstitle opt.img_type]); % large image
            % Overwrite image to be the latest solar map image so it can be
            % read by automap:
            %imwrite(X.cdata, '/home/www-data/localweb/documentroot/sun_aux/solar_map/latest_solar_map.png'); % Original size latest image
            %[origFolder obstitle opt.img_type]
        else
            % "Save to web" is off => No thumbmnail, filename in sunYYMMDDNN.png / NNNNNNNNNN.png-notation
            imwrite(X.cdata, [destFolder obstitle opt.img_type]);
        end % End of Save to web -part

    % Image saving without displaying figure
    elseif(opt.img_save == 1.0 && get(savingHandles.checkbox16, 'Value') == 0)
        if(1==1)
            r = get(0,'ScreenPixelsPerInch');
            set(current_fig, 'PaperUnits', 'inches', 'PaperPosition', get(current_fig,'Position') / r);
            set(current_fig, 'InvertHardcopy', 'off'); % Prevents matlab from inverting black & white
            file_format_str = '-dpng'; % Determine file format:
            if(strcmp(opt.img_type, '.png') == 1)
                file_format_str = '-dpng';
            elseif(strcmp(opt.img_type, '.bmp') == 1)
                file_format_str = '-dbmp';
            elseif(strcmp(opt.img_type, '.tif') == 1)
                file_format_str = '-dtiffn';
            elseif(strcmp(opt.img_type, '.jpg') == 1)
                file_format_str = '-djpeg';
            end
            print(current_fig, file_format_str, '-opengl', sprintf('-r%d',r), strcat(destFolder, obstitle));
            
            % Save as latest.png if image is latest
            
%             contents = dir('/home/www/www.metsahovi.fi/documentroot/sun/eclipse_2015/solar_maps/');
%             newest = 0;
%             %disp(strcat('Contents size:', num2str(size(contents,1))));
%             for indx=1:size(contents,1)
%                 %disp(strcat('current file:"', contents(indx).name, '"'))
%                 if(length(contents(indx).name) > 4)
%                     current_str = contents(indx).name;
%                     current_str = current_str(1:end-4);
%                     %disp(strcat('file name only:',num2str(current_str)));
%                     if(isempty(str2num(current_str)) == 0) % File name can be converted into a number
%                         %disp(strcat(current_str, ', ', newest));
%                         if(str2num(current_str) > newest) % Current name is larger than current latest name
%                             newest = str2num(current_str); % Save current name as latest
%                         end
%                     end
%                 end
%             end
%             disp(strcat('Newest after for loop:', num2str(newest)))
%             disp(strcat('Larger or equal?:', num2str((str2num(obstitle) >= newest))));
%             if(newest ~= 0) % A valid newest file name exists
%                 if(str2num(obstitle) >= newest)
%                     disp('Saving latest');
%                     print(current_fig,'-dpng', '-opengl', sprintf('-r%d',r), '/home/www/www.metsahovi.fi/documentroot/sun/eclipse_2015/latest_map.png');
%                 end
%             elseif(1==0)
%             	disp('No existing found; saving latest:');
%                 print(current_fig,'-dpng', '-opengl', sprintf('-r%d',r), '/home/www/www.metsahovi.fi/documentroot/sun/eclipse_2015/latest_map.png');
%             end
            %disp(obstitle);
        end
    end
    
%% Saving selected figures as animated gif:

if(opt.anim_gif == 1 && saveToWeb == 0.0)
    %set(gca,'nextplot','replacechildren','visible','off')
    %global animGif;
    %figure(fig_number); % Why is this here?
    set(current_fig, 'Visible', 'On'); % Visibility has to be on to allow getframe to work
    X=getframe(current_fig);
    global gifIm;
    global gifMap;
    global Sunmap_gifStartName;
    global Sunmap_gifStopName;
    plottinglist_size = size(get(mainHandles.listbox1, 'Value'));

    if(num == 1)
        [gifIm,gifMap] = rgb2ind(X.cdata,65536,'dither');
        gifIm(1,1,1,plottinglist_size(2)) = 0;
        Sunmap_gifStartName = obstitle;
    end

    gifIm(:, :, 1, num) = rgb2ind(X.cdata, gifMap, 'dither');

    if(num == plottinglist_size(2))
        Sunmap_gifStopName = obstitle;
        anim_name = [Sunmap_gifStartName, '-', Sunmap_gifStopName];

        savingHandles = guidata(opt.savingHandles);
        secPerFrame = str2num(get(savingHandles.edit2, 'String'));

        cmp_gif = strcmp(num2str(str2num(get(savingHandles.edit2, 'String'))), '');
        %cmp2 = strcmp(num2str(str2num(get(handles.edit11, 'String'))), '');
        if(cmp_gif == 1)
            msgbox('You have entered something else than numbers to the seconds per frame field. Please use only numbers.', 'Warning', 'warn', 'modal');
            return;
        else

            secPerFrame = str2num(get(savingHandles.edit2, 'String'));

            secPerFrame = round(100 * secPerFrame) / 100; % Round to 0.01 second accuracy

            if(secPerFrame < 0.01)
                secPerFrame = 0.01;
            end

            %fps = 1 / secPerFrame;

        end

        imwrite(gifIm,gifMap,[destFolder, anim_name, '_animation.gif'], 'DelayTime', secPerFrame, 'LoopCount', inf); % 0.1 seconds ==> 10 fps
    end

end
% / Animated gif

    %% saving figure in figure-form (same destination as with .png's)
    if(opt.fig_save == 1.0)
        dest = strcat(destFolder, obstitle);
        saveas(current_fig, dest,'fig');
        % Testing export_fig 2014-10-15
        %addpath('aux_data/export_fig');
        %export_fig '/home/nlavonen/Desktop/testing.pdf' -nocrop;
        % / Testing
    end
    %% EXPERIMENTAL/: saving figure directly in png-form (same destination as with screenshot .png's)
%      dest = strcat(destFolder, obstitle, 'testi');
%      set(gcf, 'Renderer', 'OpenGL') % tottelee, muttei vaikutusta (vrt, gui:n kautta)
%      get(gcf, 'Renderer')
%      set(gca, 'shading', 'flat'),
%      print('-dpng', '-opengl', dest);

%-----
    % (This get very close already:)
    % Update: This seems to work 1:1 with 2014b's opengl
    % get(current_fig, 'Renderer')
    if(1==0)
        disp('manual save');
        r = get(0,'ScreenPixelsPerInch');
        %set(gcf, 'PaperPositionMode', 'auto');
        %set(current_fig, 'Interpreter', 'none');
        %axes_pos = get(gca, 'OuterPosition');
        %set(gca, 'OuterPosition');
        %set(current_fig, 'PaperUnits', 'inches', 'PaperPosition', [0 0 674 500] / r);
        set(current_fig, 'InvertHardcopy', 'off'); % Prevents matlab from inverting black & white
        save_folder = '/home/nlavonen/Aurinko/eclipse_2015/1/';
        print(current_fig,'-dpng', '-opengl', sprintf('-r%d',r), strcat(save_folder, obstitle));
        %saveas(current_fig, '/home/nlavonen/Desktop/saveas_test2.png');
        % This only has a problem with slightly different text from the
        % original (Larger spacing between lines in the title than in the original)
        % This seems to affect all text in the figure
    end
%-----

    % /EXPERIMENTAL
    
    %% Finishing with GUI
    
    mainHandles = guidata(sunmap_handles);
    set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);
    
    if(opt.img_save == 1 || opt.fig_save == 1 || opt.anim_gif == 1)
        set(mainHandles.text39, 'String', 'Saving done');
    else
        set(mainHandles.text39, 'String', 'Plotting done');
    end
    
    if(ispc)
        set(mainHandles.text39, 'FontSize', 8);
    end
    set(mainHandles.plot_single_button, 'Enable', 'on');
    set(mainHandles.plot_selected, 'Enable', 'on');
    set(mainHandles.plot_all_button, 'Enable', 'on');
    set(mainHandles.listbox1, 'Enable', 'on');
    set(mainHandles.pushbutton21, 'Visible', 'off');
    
    %fprintf('\nPlotting and saving done for %s\n----------------------------------------------\n', obstitle)
    
    if(opt.close_prev == 1.0)       
        close(current_fig);
    end
    
    
    if(opt.img_save == 1 || opt.fig_save == 1 || opt.anim_gif == 1)
        
        savingHandles = guidata(opt.savingHandles);
    
        plottinglist_size = size(get(mainHandles.listbox1, 'Value'));
        if(num == plottinglist_size(2)) % If the current figure is the last one of the ones being saved:
            if(num == 1)
                fig_str = 'figure';
            else
                fig_str = 'figures';
            end
            string = ['Saving done! ', num2str(num), ' ', fig_str, ' saved to ', opt.destFolder];
            
            if(opt.anim_gif == 1 && opt.img_save == 0 && opt.fig_save == 0)
                string = ['Saving done! Gif animation with ', num2str(num), ' ', 'frames saved to ', opt.destFolder];
            end
            
            set(savingHandles.text5, 'String', string);
            set(savingHandles.text5, 'ForegroundColor', [0 0.5 0]);
            set(savingHandles.pushbutton3, 'String', 'Close');
            set(savingHandles.pushbutton3, 'Enable', 'on');
            set(savingHandles.text5, 'Enable', 'on');
        end
        
    end

end
    
    
    
