% Functions for saving values of partial solar
% maps into track-files 
% This whole file is a modified copy of spot_saver.m which is a modified copy of sunmap_contourplot.m

function track_maker(opt, sunmap_handles)


mainHandles = guidata(sunmap_handles);
if(opt.img_save == 1 || opt.fig_save == 1)
    savingHandles = guidata(opt.savingHandles);
end

drawnow update;
pause(0.25); % Wait for 0.25 seconds to give time for the GUI to update

% Create an empty struct which is used for saving tracking data:
trackData = struct('sun', [], 'sky', [], 'first', [], 'last', []);
% first and last are fields for times of the first and last map in the whole track,
% including sun and sky maps

opt.trackData = trackData;

%% File path(s) from which the .info and .bin raw data are read

% 'dbase': The starting part of the filepath. You can leave this empty if
% the whole path is in the opt.plottingList-list part of the path.
dbase = '';

%%  Reading plot info from the individual .info-files:

for fnum=1:size(opt.plottingList,1)

    infofilename = [dbase opt.plottingList{fnum}];
    
    finfo = fopen(infofilename,'rt');
    if (finfo < 0)
        fprintf(1, ['Could not open file ' infofilename '\n']);
        continue
    end 
    
 % SUN-MEASURE .suninfo1-file:
        % ------------------------------------------------
    if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8) % strfind returns the index of the start of the string found within the string from which we're searching 
           
        % Call info_reader() that returns a struct containing the
        % neccessary variables in a struct:
        infoStruct = info_reader(infofilename);

        % Write stuff from struct to individual variables:
        n_sweeps = infoStruct.n_sweeps;
        startdec = infoStruct.startdec;
        opt.startdec = startdec;
        enddec = infoStruct.enddec;
        opt.enddec = enddec;
        startra= infoStruct.startra;
        opt.startra = startra;
        endra = infoStruct.endra;
        opt.endra = endra;
        obstime = infoStruct.obstime;
        opt.sweepDir = infoStruct.sweepDir;
        opt.sampleDir = infoStruct.sampleDir;
        opt.cosCorrect = infoStruct.cosCorrect;
        obs_mode = infoStruct.obs_mode;
        mapType = infoStruct.mapType;
        observer_name = infoStruct.observer_name;
        freq = infoStruct.freq;
        data_integrated = infoStruct.data_integrated;

        dataSource = 'sm';
        
        % Data height in arc-min
        mapHeight = 60 * (enddec - startdec);

        % NON-declination- compensated width in arc-min:
        mapWidth = 60 * abs(endra - startra);    

        time_str = infoStruct.time_str; % Start and end time for data collection


        obstitle = regexprep(infofilename, '.suninfo1', ''); % /home/foo/bar/plaa/plaa/1284623590
        for i = 0:length(obstitle)

            if(strcmp(obstitle(end - i), '/') == 1 || strcmp(obstitle(end - i),'\') == 1)
                break;
            end
            obstitle1(i + 1) = obstitle(end - i);

        end
        obstitle = '';
        for i = 0:(length(obstitle1) - 1)
            obstitle(i + 1) = obstitle1(end - i);     
        end

        % / sun-measure .suninfo file
        % ------------------------------------------------
    
            % ------------------------------------------------
    % map-measure .map/.metadata file
    elseif(strfind(infofilename, '.map'))
        map_meta = get_sunmap_metadata(infofilename);
        
    % SMART .info-file:
    % ------------------------------------------------
    else
        
        % If the maps are SMART data, an error pop up window is given and
        % the for loop is made to jump to the next iteration>:
        dataSource = 'SMART';
        message = 'Generating tracking data from SMART data isn''t supported';
        msgbox(message, 'Warning', 'warn', 'modal');
        fclose(finfo);
        plottinglist_size = size(get(mainHandles.listbox1, 'Value'));
        if(fnum == plottinglist_size(2)) % Current map is the last one on the list and the current map is "bad"
            track_plotter(sunmap_handles, opt);
            return;
        else
            continue;
        end

    end
    % ------------------------------------------------
    
    fclose(finfo);
    %%
    
    % SUN-MEASURE DATA:
    if(length(infofilename) - strfind(infofilename, '.suninfo1')  == 8)
        
        % Checking whether to look for linear or log scale data:
        
        % Linear scale (Linear checked, Log unchecked):
        if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
        
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end
            
        % Log scale (Linear unchecked, Log checked):
        elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
        
            % We're looking for a "_log.txt" file or a "_log.matrix0" file:
            dataFileName = regexprep(infofilename, '.suninfo1', '_log.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with _log.matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '_log.matrix0');
            end
            
        else % If something funny is going on, i.e. both the menu items in the GUI are checked or unchecked at the same time, resort to linear data:
            
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end            
            
        end
        
        %dec = sun_declination(time_str);
        sunData = sun_data(time_str);
        dec = sunData.dec;
        
        if(exist('freq', 'var') == 0)
             % If the frequency of the observation hasn't been written into
             % the info-file and thus the variable 'freq' doesn't exist, we make a wild guess...
            freq = '37 GHz';
            % Otherwise the requency information is found in the "freq" variable in the form: "NN GHz".
        end
        obstitle = 'foobar'; % To be determined
        
    % If .matrix0 datafile is not found, in the case that we do need a separate data file (data_integrated == 0)):
    s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
    if((exist(dataFileName, 'file') == 0 || s.bytes < 60) && data_integrated == 0)
        %set(mainHandles.text39, 'ForegroundColor', 'r');
        %if(exist(dataFileName, 'file') == 0)
          %  set(mainHandles.text39, 'String', 'data file not found');
            %return;
            
            
            plottinglist_size = size(get(mainHandles.listbox1, 'Value'));
            if(fnum == plottinglist_size(2)) % Current map is the last one on the list and the current map is "bad"
                % --> Plot whatever we have up until now and return:
                plot_succeed = track_plotter(sunmap_handles, opt);
                
                if(plot_succeed == 1)
                    set(mainHandles.text39, 'String', 'Track generation done');
                else
                    set(mainHandles.text39, 'ForegroundColor', 'r');
                    set(mainHandles.text39, 'String', 'Track generation failed');
                end
                
                % Reset the GUI:
                if(ispc)
                    set(mainHandles.text39, 'FontSize', 8);
                end
                set(mainHandles.plot_single_button, 'Enable', 'on');
                set(mainHandles.plot_selected, 'Enable', 'on');
                set(mainHandles.plot_all_button, 'Enable', 'on');
                set(mainHandles.listbox1, 'Enable', 'on');
                set(mainHandles.pushbutton21, 'Visible', 'off');
                
                return;
            else
                continue;
            end
        %   else
        %            set(mainHandles.text39, 'String', 'data file is empty');
        %            return;
        %        end
        %        set(mainHandles.text39, 'FontSize', 10);
        %        set(mainHandles.plot_single_button, 'Enable', 'on');
        %        set(mainHandles.plot_all_button, 'Enable', 'on');
        %        set(mainHandles.listbox1, 'Enable', 'on');
        %        set(mainHandles.pushbutton21, 'Visible', 'off');
        
    else
         % Reading the actual map data:
        
        % Data is from separate .txt-file:
        if(data_integrated == 0)
            mapdata = dlmread(dataFileName);
        end
        % If the data is from an integrated file with info and data
        % together, the data has already been read earlier
        
        map_dimensions = size(mapdata);
        sps = map_dimensions(2); % Samples per sweep.
    end
    % ----------------------------
    
    % map-measure data:
    elseif(strfind(infofilename, '.map'))

        % Checking whether to look for linear or log scale data:

        % Linear scale (Linear checked, Log unchecked):
        if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
            data_col_num = 8;
        % Log scale (Linear unchecked, Log checked):
        elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
            data_col_num = 9;
        else
            data_col_num = 8;
        end

        sunData = sun_data(map_meta.time_str);
        dec = sunData.dec;

%         % Create radec matrix from arbitrary sweep data
%         gen_map_height = 60*abs(map_meta.d_dec_min - map_meta.d_dec_max);
%         gen_map_width = 60*abs(map_meta.d_ra_cc_min - map_meta.d_ra_cc_max);
%         gen_sweeps = round(gen_map_height / (60*map_meta.sweepwidth)); % generated sweeps
%         gen_sps = round(gen_map_width / 0.24); % generated samples per sweep (0.24' ~ 200 sps for 0.8 degree map)
%         mapType = 'ramap';
%         opt.sweepDir = 'forward'; % fix this if something breaks
%         opt.sampleDir = 'forward'; % fix this if something breaks
         opt.cosCorrect = 1;
         dataSource = 'mm'; % map-measure (for now at least)
% 
         % Data height in arc-min
         opt.startra = map_meta.d_ra_cc_min; % Check if cc or non cc is needed
         opt.endra = map_meta.d_ra_cc_max;
         opt.startdec = map_meta.d_dec_min;
         opt.enddec = map_meta.d_dec_max;
         opt.mapxradius = map_meta.mapxradius;
         opt.mapyradius = map_meta.mapyradius;
         opt.az_center = map_meta.az_center;
         opt.el_center = map_meta.el_center;
         opt.ra_center = map_meta.ra_center;
         opt.dec_center = map_meta.dec_center;
% 
% %         data_file = dlmread(infofilename, ' ');
% % 
% %         [xi, yi] = meshgrid(linspace(15*map_meta.d_ra_cc_min,15*map_meta.d_ra_cc_max,gen_sps), linspace(map_meta.d_dec_min,map_meta.d_dec_max, gen_sweeps));
% %         zi = griddata(15*data_file(:,7), data_file(:,6), data_file(:,data_col_num), xi,yi); % fit map data to xi,yi grid. (ra,dec,data_lin)
% %         zi(isnan(zi)) = min(min(zi)); % replace nan's with map minimum
%           
%         radec_matrix = generate_radec_matrix(infofilename, dataDim.startra, dataDim.endra, dataDim.startdec, dataDim.enddec, gen_sps, gen_sweeps, data_col_num);
        
        
    % SMART DATA:
    else

        % Write variables into info_fields struct for use with
        % info_reader_smart():
        info_fields.info_df = info_df;
        info_fields.info_Tswp = info_Tswp;
        info_fields.info_time = info_time;
        info_fields.info_Nswp = info_Nswp;
        info_fields.infofilename = infofilename;
        info_fields.info_date = info_date;
        info_fields.info_rx = info_rx;
        info_fields.info_cent = info_cent;
        % Call info_reader_smart():
        infoStructSmart = info_reader_smart(info_fields);
        % Write output from info_reader_smart() to variables:
        mapWidth = infoStructSmart.mapWidth; % Non declination compensated width of the map in decimal arc minutes as read from the SMART info file, scalar
        mapHeight = infoStructSmart.mapHeight; % Height of the map in decimal arc minutes as read from the SMART info file, scalar
        mapType = infoStructSmart.mapType; % The type of the map; right ascension map ('ramap') or declination map ('ramap'), string
        obstitle = infoStructSmart.obstitle; % The name of the map file without the file name extension (e.g. 'sun09080603'), string
        obstime = infoStructSmart.obstime; % The full observation time of the map, start - end, (e.g. '2009-08-06,  6:31:53.660 - 06:40:31.66'), string
        freq= infoStructSmart.freq;  % The frequency of the receiver used, (e.g. '37 GHz', '22 GHz'), string
        dec = infoStructSmart.dec; % The declination of the Sun's center at the time of the observation in decimal degrees, scalar  
        
    %% READING THE SUNMAP RAW-DATA (.BIN-FILE) INTO A MATRIX.
       
    % Creating the .bin -files name from .info-files name:
    if(strcmp(infofilename(end-3), 'i') == 1)
        binfilename = strrep(infofilename, '.info', '.bin');       
    elseif((strcmp(infofilename(end-3), 'I') == 1))
        binfilename = strrep(infofilename, '.INFO', '.BIN');
    end
    
    % If .bin-file is not found:
    if(exist(binfilename, 'file') == 0)
        set(mainHandles.text39, 'ForegroundColor', 'r');
        set(mainHandles.text39, 'String', '.bin-file not found');
        set(mainHandles.text39, 'FontSize', 8);
        set(mainHandles.plot_single_button, 'Enable', 'on');
        set(mainHandles.plot_selected, 'Enable', 'on');
        set(mainHandles.plot_all_button, 'Enable', 'on');
        set(mainHandles.listbox1, 'Enable', 'on');
        set(mainHandles.pushbutton21, 'Visible', 'off');
        
        continue
    end
    
    % Calling read_sun_data_ultimate() that reads the .bin-file.
    % (Can be found in read_sun_data_ultimate.m. 'ultimate' comes from the
    % ability to read any kind of an array from a .bin-file)
    [mapdata] = read_sun_data_ultimate(binfilename, info_Nswp{1}, info_sps{1});
    %mapType = 'foobar'; % With SMART data maptype (ramap, demap) is derived from the number of sweeps and samples/sweep
    
    end % END OF ELSE-CLAUSE FOR SMART-DATA
    
    %% Calling mapHandler

    if(fnum == 1)
        opt.fig_count = length(findobj('Type','figure')) - 1;
    end   
   
    % SUN-MEASURE DATA:
    if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8)
    
        if(strcmp(obs_mode, 'track') == 1 || n_sweeps == 1)
            % Data is a track:
            %trackPlot(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, opt, sunmap_handles);
        elseif(strcmp(obs_mode, 'map') == 1)
            % Data is a map:
            trackData = mapHandler(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, opt, sunmap_handles, mapType, dataSource);
            opt.trackData = trackData;
        end
        
    % Sun-measure "2.0" data
    elseif(strfind(infofilename, '.map'))
        sps = 200; % TEMP!
        data_file = dlmread(infofilename, ' ');
        mapdata = data_file(:,data_col_num);
        trackData = mapHandler(dec, map_meta.freq, map_meta.n_sweeps, sps, mapdata, fnum, map_meta.time_str, map_meta.obstime, '', '', opt, sunmap_handles, map_meta.mapType, dataSource);
        opt.trackData = trackData;
        
    % SMART DATA:
    else
        
        if(strcmp(obstitle(1), 't') == 1 || strcmp(obstitle(1), 'T') == 1)
            % Data is a track: Nothing is done, because we're looking for
            % active area coordinates
            %trackPlot(dec, freq, info_Nswp{1}, info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, opt, sunmap_handles);
        else
            % Data is a map:
            trackData = mapHandler(dec, freq, info_Nswp{1}, info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, opt, sunmap_handles, mapType, dataSource);
            opt.trackData = trackData;
        end
    
    end
    
end % End of for loop going through the list of maps:

% Call track_plotter function that plots the data and calls the track_saver
% function that in turn saves the results in a file:
opt.trackData = trackData;
%trackData
%trackData.sun.coord
plot_succeed = track_plotter(sunmap_handles, opt);

% Finishing up with GUI:

mainHandles = guidata(sunmap_handles);
set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);

if(opt.img_save == 1 || opt.fig_save == 1)
    set(mainHandles.text39, 'String', 'Saving done');
elseif(plot_succeed == 1)
    set(mainHandles.text39, 'String', 'Track generation done');
else
    set(mainHandles.text39, 'ForegroundColor', 'r');
    set(mainHandles.text39, 'String', 'Track generation failed');
end

if(ispc)
    set(mainHandles.text39, 'FontSize', 8);
end
set(mainHandles.plot_single_button, 'Enable', 'on');
set(mainHandles.plot_selected, 'Enable', 'on');
set(mainHandles.plot_all_button, 'Enable', 'on');
set(mainHandles.listbox1, 'Enable', 'on');
set(mainHandles.pushbutton21, 'Visible', 'off');


end % track_maker ENDS


%% !!!!----------- mapHandler() -function starts -----------!!!!
% Not to be confused with map_handler.m
function trackData = mapHandler(dec, freq, nswp, sps, map, num, time, obstitle, h, w, opt, sunmap_handles, mapType, dataSource)

% destFolder = opt.destFolder;
% beam_size = opt.beam_size;
% solar_north = opt.solar_north;
% grid_toggle = opt.grid_toggle;
% %cmap = opt.cmap;
% abs_scale = opt.abs_scale;
% c_min = opt.min;
% c_max = opt.max;
% mapPlottingType = opt.mapType; % opt.mapType (that tells how the map is plotted: filled contours, lines, etc.) IS NOT THE SAME AS the variable mapType (ra-map or de-map)!!!!!
% customNumOfLevelsToggle = opt.customNumOfLevelsToggle ;
% customNumOfLevels = opt.customNumOfLevels;
% sweepDir = opt.sweepDir;
% sampleDir = opt.sampleDir;

mainHandles = guidata(sunmap_handles);

% Saving original map matrix and data from it:
map_raw = map;

azelTime = regexprep(time, '-', '/');
azelTime = regexprep(azelTime, ',', '');
[Az El] = SolarAzEl(azelTime, 60.217809153, 24.39311053, 60.28);

% Get trackData struct from opt struct:
trackData = opt.trackData;

% Recognizing the frequency-string:
if(isempty(regexpi(freq,'37')) == 0)
    freq = '37 GHz';
end

if(isempty(regexpi(freq,'87')) == 0)
    freq = '87 GHz';
end

if(isempty(regexpi(freq,'22')) == 0)
    freq = '22 GHz';
end



%fprintf('sunPlot being called. Plotting the map...\n')
mainHandles = guidata(sunmap_handles);
if(isfield(opt, 'savingHandles') == 0) % savingHandles does not exist ==> sunmap_contourplot was not called by saving.m ==> "save to web" not in use
    saveToWeb = 0;
else
    savingHandles = guidata(opt.savingHandles);
    saveToWeb = get(savingHandles.checkbox5, 'Value');
end
sunmap_status_text = mainHandles.text39;
set(mainHandles.text39, 'ForegroundColor', 'blue');

if(opt.img_save == 1 || opt.fig_save == 1)
    %set(sunmap_status_text, 'String', 'Saving...');
else
    size_of_list= size(get(mainHandles.listbox1, 'Value'));
    progress_str = ['Generating track: ', num2str(num), '/', num2str(size_of_list(2)),' ...'];
    set(sunmap_status_text, 'String', progress_str);
    drawnow update;
end

set(mainHandles.plot_single_button, 'Enable', 'off');
set(mainHandles.plot_selected, 'Enable', 'off');
set(mainHandles.plot_all_button, 'Enable', 'off');
set(mainHandles.listbox1, 'Enable', 'off');
set(mainHandles.pushbutton21, 'Visible', 'on');
% For checking the value of nswp (number of sweeps in data):
%fprintf('The number of sweeps given to sunPlot is: %d\n', nswp)
    
    %% Map operations:
    
%     global track_sky_mean; %
%     if(isempty(track_sky_mean)) % No sky maps have been read yet --> use a predefined approximation:
%         %map = map + 2300; %min(min(map)), -(-2300) = +2300
%         %map
%     elseif(length(track_sky_mean) >= 1)
%         % Use the previous mean reading from a sky map as the minimum:
%         %map = map - track_sky_mean(end);
%     end
    
    % Temporary:
    % Set "overbright" values to map max value if sun-measure or map-measure data:
    if(strcmp(dataSource, 'sm') == 1 || strcmp(dataSource, 'mm') == 1)
        map_size_temp = size(map);
        for ind_j = 1 : map_size_temp(1)
            for ind_g = 1 : map_size_temp(2)
                if map(ind_j,ind_g) < -30000
                     map(ind_j,ind_g) = max(max(map));
                     map_raw(ind_j,ind_g) = max(max(map_raw));
                end
            end
        end        
    end
    % / Temporary
   
    % Normalizing data so that minimum is at zero:
    map = map - min(min(map));
     
    % Finding median level (quiet sun level):
    dataUnits = 1; % 1 --> original data units (not % QSL)
    quiet_sun = qsl_finder(map, dataUnits);

    map = qsl_units_transformer(map, quiet_sun, freq);
    max_temp = max(max(map));
    
     
    mapWidth = 60*abs(opt.startra - opt.endra);
    if(opt.cosCorrect == 1)
        mapWidth = mapWidth * cosd(dec);
    end
    mapHeight = 60*abs(opt.startdec - opt.enddec);
       
    fig_number = num;
    while(isempty( find( get(0, 'children') == fig_number) ) == 0)
        fig_number  = fig_number + 1;
    end

    
    %%  Generating tracking data
       
    % Read max, min and mean of current map
    % Later the value is classified as a sun or sky map depending on
    % the mean value (mean > 0 --> sun, mean <= 0 --> sky)
    raw_max = max(max(map_raw));
    raw_min = min(min(map_raw));
    raw_mean = mean(mean(map_raw));
    raw_comp_max = cosd(El) * raw_max; % This is an experimental elevation compensated max temp. Hasn't worked as well as hoped, not in use anymore.
    
    % A boolean check variable that will be 1 if something is plotted and 0
    % if nothing is plotted
    plot_succeed = 0;

    % The map coordinates are available in these variables:
    %     opt.startra
    %     opt.endra
    %     opt.startdec
    %     opt.enddec
    
    % Variables read only if the current map is a partial map:      
    %mapWidth
    %mapHeight
    if(mapWidth < 30 && mapHeight < 30)      
        
        if(raw_mean > 0) % "On-sun" map
            
            if(isempty(trackData.sun)) % No "on sun" data yet --> make first entrys into trackData.sun
                
                trackData.sun(1).max = raw_max;
                trackData.sun(1).QSL_max = max_temp;
                trackData.sun(1).el = El; % El (Sun's elevation) gotten at function's beginning
                trackData.sun(1).comp_max = raw_comp_max;
                trackData.sun(1).min = raw_min;
                trackData.sun(1).mean = raw_mean;
                trackData.sun(1).QSL_norm = quiet_sun;
                %track_sun_time = opt.time_total;
                trackData.sun(1).time{1} = time; % Has to be cell array because time is a string
                trackData.sun(1).max_min = raw_max - raw_min;
                trackData.sun(1).id{1} = obstitle;
                if(strcmp(dataSource, 'sm') == 1) % sun-measure
                    trackData.sun(1).coords = [opt.startra opt.endra opt.startdec opt.enddec];
                elseif(strcmp(dataSource, 'mm') == 1) % map-measure
                    trackData.sun(1).coords = [opt.mapxradius opt.mapyradius opt.az_center opt.el_center opt.ra_center opt.dec_center];
                end
                trackData.sun(1).freq{1} = freq; % Has to be cell array because freq is a string (e.g. "37 GHz")
                
            elseif(length(trackData.sun) >= 1) % The sun part of the struct is not empty:
                
                % Check the map coordinates against all the current
                % coordinates in the trackData.sun structs.
                % The first trackData.sun(c_idx) whose coords array matches the current map's coords, is
                % the one where the current map's data is written
                % If no match is found, create new sun(c_idx+1)
                for c_idx = 1 : length(trackData.sun)
                    if(strcmp(dataSource, 'sm') == 1) % sun-measure
                        if(isequal(trackData.sun(c_idx).coords, [opt.startra opt.endra opt.startdec opt.enddec]))
                            match_idx = c_idx;
                            break;
                        else
                            match_idx = c_idx + 1;
                        end
                    elseif(strcmp(dataSource, 'mm') == 1) % map-measure
                        if(isequal(trackData.sun(c_idx).coords, [opt.mapxradius opt.mapyradius opt.az_center opt.el_center opt.ra_center opt.dec_center]))
                            match_idx = c_idx;
                            break;
                        else
                            match_idx = c_idx + 1;
                        end
                    end
                end
                
                if(length(trackData.sun) < match_idx) % We have skipped to a new sun() --> new fields have to be written: 
                    trackData.sun(match_idx).max = raw_max;
                    trackData.sun(match_idx).QSL_max = max_temp;
                    trackData.sun(match_idx).el = El; % El gotten at function's beginning
                    trackData.sun(match_idx).comp_max = raw_comp_max;
                    trackData.sun(match_idx).min = raw_min;
                    trackData.sun(match_idx).mean = raw_mean;
                    trackData.sun(match_idx).QSL_norm = quiet_sun;
                    %track_sun_time = opt.time_total;
                    trackData.sun(match_idx).time{1} = time; % Has to be cell array because time is a string
                    trackData.sun(match_idx).max_min = raw_max - raw_min;
                    trackData.sun(match_idx).id{1} = obstitle;
                    if(strcmp(dataSource, 'sm') == 1) % sun-measure
                        trackData.sun(match_idx).coords = [opt.startra opt.endra opt.startdec opt.enddec];
                    elseif(strcmp(dataSource, 'mm') == 1) % map-measure
                        trackData.sun(match_idx).coords = [opt.mapxradius opt.mapyradius opt.az_center opt.el_center opt.ra_center opt.dec_center];
                    end
                    trackData.sun(match_idx).freq{1} = freq; % Has to be cell array because freq is a string (e.g. "37 GHz")
                else % The coord(match_idx) already exists --> write values to the end of existing fields:
                    trackData.sun(match_idx).max(end + 1) = raw_max;
                    trackData.sun(match_idx).QSL_max(end + 1) = max_temp;
                    trackData.sun(match_idx).comp_max(end + 1) = raw_comp_max;
                    trackData.sun(match_idx).el(end + 1) = El;
                    trackData.sun(match_idx).min(end + 1) = raw_min;
                    trackData.sun(match_idx).mean(end + 1) = raw_mean;
                    trackData.sun(match_idx).QSL_norm(end + 1) = quiet_sun;
                    %track_sun_time(end + 1) = opt.time_total;
                    trackData.sun(match_idx).time{end + 1} = time; % Has to be cell array because time is a string
                    trackData.sun(match_idx).max_min(end + 1) = raw_max - raw_min;
                    trackData.sun(match_idx).id{end + 1} = obstitle;
                    trackData.sun(match_idx).freq{end + 1} = freq; % Has to be cell array because freq is a string (e.g. "37 GHz")
                end
                


            end

        else % "Off-sun" map (=sky) (mean < 0)
            
            % trackData.sky is empty:
            if(isempty(trackData.sky))
                
                % Write values in the first index of .sky and of the subfields:
                trackData.sky(1).mean = raw_mean;
                trackData.sky(1).max = raw_max;
                trackData.sky(1).min = raw_min;
                trackData.sky(1).time{1} = time; % Has to be cell array because time is a string
                trackData.sky(1).id{1} = obstitle;
                trackData.sky(1).freq{1} = freq; % Has to be cell array because freq is a string (e.g. "37 GHz")
                if(strcmp(dataSource, 'sm') == 1) % sun-measure
                    trackData.sky(1).coords = [opt.startra opt.endra opt.startdec opt.enddec];
                elseif(strcmp(dataSource, 'mm') == 1) % map-measure
                    trackData.sky(1).coords = [opt.mapxradius opt.mapyradius opt.az_center opt.el_center opt.ra_center opt.dec_center];
                end
                    
                
            % trackData.sky is not empty:
            elseif(length(trackData.sky) >= 1)
                          
                % Check the date of the current sky map (the first 10 chars in the "time" string)
                % against the date in the other trackData.sky structs.
                % The first trackData.sky(c_idx) whose date (any date within sky(x_idx) will do) matches the current map's date, is
                % the one where the current map's data is written
                % If no match is found, create new sky(c_idx+1)
                for c_idx = 1 : length(trackData.sky)
                    if(strncmpi([trackData.sky(c_idx).time{1}], time, 10) == 1)
                        match_idx = c_idx;
                        break;
                    else
                        match_idx = c_idx + 1;
                    end
                end
                
                if(length(trackData.sky) < match_idx) % We have skipped to a new sky() --> new fields have to be written: 
                    trackData.sky(match_idx).mean = raw_mean;
                    trackData.sky(match_idx).max = raw_max;
                    trackData.sky(match_idx).min = raw_min;
                    trackData.sky(match_idx).time{1} = time; % Has to be cell array because time is a string
                    trackData.sky(match_idx).id{1} = obstitle;
                    trackData.sky(match_idx).freq{1} = freq; % Has to be cell array because freq is a string (e.g. "37 GHz")
                    if(strcmp(dataSource, 'sm') == 1) % sun-measure
                        trackData.sky(match_idx).coords = [opt.startra opt.endra opt.startdec opt.enddec];
                    elseif(strcmp(dataSource, 'mm') == 1) % map-measure
                        trackData.sky(match_idx).coords = [opt.mapxradius opt.mapyradius opt.az_center opt.el_center opt.ra_center opt.dec_center];
                    end
                else % The sky(match_idx) already exists --> write values to the end of existing fields:
                    trackData.sky(match_idx).mean(end + 1) = raw_mean;
                    trackData.sky(match_idx).max(end + 1) = raw_max;
                    trackData.sky(match_idx).min(end + 1) = raw_min;
                    trackData.sky(match_idx).time{end + 1} = time; % Has to be cell array because time is a string
                    trackData.sky(match_idx).id{end + 1} = obstitle;
                    trackData.sky(match_idx).freq{end + 1} = freq;
                end

            end % End of if-clause for separating empty and non-empty trackData.sky structs
            
        end % End of if-clause for separating "sun" and "sky" maps
    
    end % End of if-clause for skipping full disk maps
    
    opt.trackData = trackData;
    

end % End of mapHandler() function
    
    
    
  