% This function is used when plotting figures out of tracking data that has
% been taken from multiple small partial solar maps
% Output:
%   - plot_succeed: 1: plotting succeeded, 0 plotting failed

function plot_succeed = track_plotter(sunmap_handles, opt)

mainHandles = guidata(sunmap_handles);

plot_succeed = 0;

% Variable in which names of items for a legend will be written:
legendStr = {};

first = [];
last = [];

% If both the sun and sky fields are empty, return with a plotting failure:
if(isempty(opt.trackData.sun) && isempty(opt.trackData.sky))
    plot_succeed = 0;
    return;
end

% Find times for first and last map of whole track:
if(isempty(opt.trackData.sun) == 0) % sun maps
    for trc_idx = 1 : length(opt.trackData.sun)
        if(isempty(first))
            first = str2double(opt.trackData.sun(trc_idx).id{1});
            first_time = opt.trackData.sun(trc_idx).time{1};
        end
        if(isempty(last))
            last = str2double(opt.trackData.sun(trc_idx).id{1});
            last_time = opt.trackData.sun(trc_idx).time{1};
        end
        for trc_idx2 = 1 : length(opt.trackData.sun(trc_idx).id)
            if(last < str2double(opt.trackData.sun(trc_idx).id{trc_idx2}))
                last = str2double(opt.trackData.sun(trc_idx).id{trc_idx2});
                last_time = opt.trackData.sun(trc_idx).time{trc_idx2};
            elseif(first > str2double(opt.trackData.sun(trc_idx).id{trc_idx2}))
                first = str2double(opt.trackData.sun(trc_idx).id{trc_idx2});
                first_time = opt.trackData.sun(trc_idx).time{trc_idx2};
            end
        end
    end
end

if(isempty(opt.trackData.sky) == 0) % sky maps
    for trc_idx = 1 : length(opt.trackData.sky)
        if(isempty(first))
            first = str2double(opt.trackData.sky(trc_idx).id{1});
            first_time = opt.trackData.sky(trc_idx).time{1};
        end
        if(isempty(last))
            last = str2double(opt.trackData.sky(trc_idx).id{1});
            last_time = opt.trackData.sky(trc_idx).time{1};
        end
        for trc_idx2 = 1 : length(opt.trackData.sky(trc_idx).id)
            if(last < str2double(opt.trackData.sky(trc_idx).id{trc_idx2}))
                last = str2double(opt.trackData.sky(trc_idx).id{trc_idx2});
                last_time = opt.trackData.sky(trc_idx).time{trc_idx2};
            elseif(first > str2double(opt.trackData.sky(trc_idx).id{trc_idx2}))
                first = str2double(opt.trackData.sky(trc_idx).id{trc_idx2});
                first_time = opt.trackData.sky(trc_idx).time{trc_idx2};
            end
        end
    end
end

% Use only first 20 characters of first_time and last_time:
opt.trackData.first = first; % These are used in track_saver.m
opt.trackData.last = last; % 
first_time = first_time(1:20);
last_time = last_time(1:20);

% Create time series objects for all the "on sun" tracks
% Write stuff in the objects, plot the objects
for idx = 1 : length(opt.trackData.sun)
    
    % Create time series objects in opt.trackData.sun(i):
    opt.trackData.sun(idx).max_ts = timeseries;
    opt.trackData.sun(idx).min_ts = timeseries;
    opt.trackData.sun(idx).mean_ts = timeseries;
          
    % Write data points and times into time series variables:
    for idx2 = 1 : length(opt.trackData.sun(idx).max)
        
        % Create date-time time vector from the time string:
        % Debug:
        time_str = char(opt.trackData.sun(idx).time(idx2));
        time_year_str = time_str(1:4);
        time_month_str = time_str(6:7); 
        time_day_str = time_str(9:10); 
        time_hour_str  = time_str(13:14);
        time_minute_str = time_str(16:17);
        time_second_str = time_str(19:20);
        % Debug:
        %fprintf('%s:%s:%s\n', time_hour_str, time_minute_str, time_second_str);
        time_vector = datenum(str2double(time_year_str), str2double(time_month_str), str2double(time_day_str), str2double(time_hour_str), str2double(time_minute_str), str2double(time_second_str));
        
        opt.trackData.sun(idx).max_ts = addsample(opt.trackData.sun(idx).max_ts, 'Data', opt.trackData.sun(idx).max(idx2), 'Time', time_vector);
        opt.trackData.sun(idx).mean_ts = addsample(opt.trackData.sun(idx).mean_ts, 'Data', opt.trackData.sun(idx).mean(idx2), 'Time', time_vector);
        opt.trackData.sun(idx).min_ts = addsample(opt.trackData.sun(idx).min_ts, 'Data', opt.trackData.sun(idx).min(idx2), 'Time', time_vector);
    end  
    
    % Plotting:
    if(idx == 1)
        figure,
    else
        hold on,
    end
    plot(opt.trackData.sun(idx).max_ts, 'Color', [0.7 0 0]), % Sun Max
    legendStr{1} = 'Sun max';
    grid on,
    hold on,
    plot(opt.trackData.sun(idx).min_ts, 'Color', [0 0.7 0]), % Sun Min
    legendStr{2} = 'Sun min';
    datetick('x','HH:MM', 'keepticks')
    
    plot_succeed = 1;
    
end

% Create time series objects for all the sky tracks
% Write stuff in the objects, plot the objects
for idx = 1 : length(opt.trackData.sky)
    
    % Create timeseries object in opt.trackData.sky(idx):
    opt.trackData.sky(idx).ts = timeseries;
        
    % Write data and times in opt.trackData.sky(idx).ts:
    for idx2 = 1 : length(opt.trackData.sky(idx).mean)
        
        % Create date-time time vector from the time string:
        % Debug:
        time_str = char(opt.trackData.sky(idx).time(idx2));
        time_year_str = time_str(1:4);
        time_month_str = time_str(6:7); 
        time_day_str = time_str(9:10); 
        time_hour_str  = time_str(13:14);
        time_minute_str = time_str(16:17);
        time_second_str = time_str(19:20);
        % Debug:
        %fprintf('%s:%s:%s\n', time_hour_str, time_minute_str, time_second_str);
        time_vector = datenum(str2double(time_year_str), str2double(time_month_str), str2double(time_day_str), str2double(time_hour_str), str2double(time_minute_str), str2double(time_second_str));
        
        opt.trackData.sky(idx).ts = addsample(opt.trackData.sky(idx).ts, 'Data', opt.trackData.sky(idx).mean(idx2), 'Time', time_vector);
    end

    if(isempty(opt.trackData.sun))
        figure,
        legendStr{1} = 'Sky mean';
    else
        hold on,
        legendStr{3} = 'Sky mean';
    end
    plot(opt.trackData.sky(idx).ts, 'Color', [0 0 0.7]), % Sky Mean
    
    NumTicks = 6;
    L = get(gca,'XLim');
    set(gca,'XTick',linspace(L(1),L(2), NumTicks));
    datetick('x','HH:MM', 'keepticks')
    
    plot_succeed = 1;

end

% Draw legend on figure (if at least something has been plotted):
if(isempty(opt.trackData.sun) == 0 || isempty(opt.trackData.sky) == 0)
    grid on,
    %char(opt.trackData.sun(1).freq{1});
    % Recognizing linear / log scales:
    % Linear scale:
    if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
        scale_str = 'linear scale';
        opt.trackData.scale = 'linear';
    % Log scale:
    elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
        scale_str = 'log scale';
        opt.trackData.scale = 'log';
    % Weird case:
    else
        scale_str = 'linear scale';
        opt.trackData.scale = 'linear';
    end
    title(['MRO solar track, ', char(opt.trackData.sun(1).freq{1}), ', ' scale_str, 10, 'Time: ', first_time, ' - ', last_time], 'FontSize', 10),
    xlabel('Time'),
    ylabel('Backend signal level (pseudo units)');
    set(gcf,'Color', 'w');
    legend(legendStr, 'Location', 'Best');
end

%opt.trackData
%opt.trackData.sun(1);
track_saver(opt.trackData);

return;
%--------------------------------
% Depreceated code from here on:





















if(isempty(opt.trackData.sun_max) == 0)
    plot_succeed = 1;
    figure,
    plot(opt.trackData.sun_max, 'k'); % Black line for "on sun" max
end

if(isempty(opt.trackData.sun_mean) == 0)
    plot_succeed = 1;
    hold on,
    plot(opt.trackData.sun_mean, 'Color', [0.5 0.1 0.1]), % Brown line for "on sun" mean
end

if(isempty(opt.trackData.sun_min) == 0)
    plot_succeed = 1;
    hold on,
    plot(opt.trackData.sun_min, 'Color', [0 0.8 0]), % Green line for "on sun" minimum
    grid on,
    % Recognizing linear / log scales:
    % Linear scale:
    if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
        title_str = 'Solar track data (linear)';
    % Log scale:
    elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
        title_str = 'Solar track data (logarithmic)';
    % Weird case:
    else
        title_str = 'Solar track data (linear)';
    end
    title(title_str, 'FontSize',12),
    set(gcf,'Color',[0.97254902 0.97254902 0.97254902]);
    xlabel('Map number'),
    ylabel('Temp, backend pseudo units');
end

if(isempty(opt.trackData.sky_mean) == 0)
    plot_succeed = 1;
    %figure,
    hold on,
    plot(opt.trackData.sky_mean); % Blue line for sky mean
    grid on,
    legend('Sun: max', 'Sun: mean', 'Sun: min', 'Sky: mean', 'Location','Best');
    %title('Sky temp, mean', 'FontSize',12),
    %xlabel('Map number'),
    %ylabel('Receiver temp, mV');
end

if(isempty(opt.trackData.sun_el) == 0)
    plot_succeed = 1;
    figure,
    plot(opt.trackData.sun_el);
    grid on,
    title('Sun elevation', 'FontSize',12),
    xlabel('Map number'),
    ylabel('Sun elevation, degrees');
end

if(isempty(opt.trackData.sun_QSL_max) == 0)
    plot_succeed = 1;
    figure,
    plot(opt.trackData.sun_QSL_max);
    grid on,
    % Recognizing linear / log scales:
    % Linear scale:
    if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
        title_str = 'Sun max temp, QSL %';
    % Log scale:
    elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
        title_str = 'Sun max temp, QSL % (logarithmic data scale, values not valid)';
    % Weird case:
    else
        title_str = 'Sun max temp, QSL %';
    end
    title(title_str, 'FontSize',12),
    xlabel('Map number'),
    ylabel('Sun max temp, % of QSL (unreliable if no background sky on the map)');
end


%         if(isempty(opt.trackData.sun_max) == 0 && isempty(opt.trackData.sun_el) == 0)
%             R = corrcoef(opt.trackData.sun_max, opt.trackData.sun_el);
%         end
%         if(isempty(R) == 0)
%             figure,
%             plot(R);
%         end

% If all the needed variables are non-empty, the data is saved
% into a file whose name is asked from the user

if(isempty(opt.trackData.sun_max) == 0 && isempty(opt.trackData.sun_mean) == 0 && isempty(opt.trackData.sun_min) == 0 && isempty(opt.trackData.sky_mean) == 0 && isempty(opt.trackData.sky_time) == 0 && isempty(opt.trackData.sun_time) == 0)
    %opt.time = time;
    %opt.obstitle = obstitle;
    track_saver(opt.trackData);
end

set(mainHandles.text39, 'ForegroundColor', [0 0.5 0]);

if(opt.img_save == 1 || opt.fig_save == 1)
    set(mainHandles.text39, 'String', 'Saving done');
elseif(plot_succeed == 1)
    set(mainHandles.text39, 'String', 'Track generation done');
else
    set(mainHandles.text39, 'ForegroundColor', 'r');
    set(mainHandles.text39, 'String', 'Track generation failed');
end

if(ispc)
    set(mainHandles.text39, 'FontSize', 8);
end
set(mainHandles.plot_single_button, 'Enable', 'on');
set(mainHandles.plot_selected, 'Enable', 'on');
set(mainHandles.plot_all_button, 'Enable', 'on');
set(mainHandles.listbox1, 'Enable', 'on');
set(mainHandles.pushbutton21, 'Visible', 'off');


end
