% This function takes a struct created by track_maker.m and saves the data in it into a
% .txt file. The target folder and name for the file are asked from the user

function track_saver(trackData)

% Check that the at least the other of the sun and sky fields is not empty:
if(isempty(trackData.sun) == 0 || isempty(trackData.sky) == 0)
    
    % trackData.first: id of first map in track (scalar)
    % trackData.last: id of last map in track (scalar)
    % trackData.scale: "linear" or "log" tells if data scale is linear or logarithmic
    trackFileName_default = [pwd,'/MRO_solar_track_', num2str(trackData.first), '-', num2str(trackData.last), '_', trackData.scale, '.txt'];

    % Ask user for file name;
    [trackFileName, trackFileDir] = uiputfile(trackFileName_default);
    
    % If user presses 'Cancel' nothing happens
    if(trackFileDir == 0)
        return;
    end

    % trackFileDir is without a trailing '/'
    trackFileName = [trackFileDir, trackFileName];

    track_file = fopen(trackFileName, 'w'); % Open file, w for overwriting

    % If track_file can't be opened, error dialog and return:
    if(track_file == 0)
        message = ['Can''t open target file (', trackFileName, ')'];
        errordlg(message, 'Error', 'modal');
        return;
    end

    tab = '\t';
    s = dir(trackFileName); % s.bytes tells the number of bytes in the data file
    if(s.bytes < 5)  % under 5 bytes --> file empty --> write header line into file:
        fprintf(track_file, [sprintf('%-40s','Time') sprintf('%-15s', 'Map ID') sprintf('%-15s','Sun or Sky?') sprintf('%-15s','Max') sprintf('%-15s', 'Mean') sprintf('%-20s', 'Min') sprintf('%-17s', 'startra') sprintf('%-15s', 'endra') sprintf('%-18s', 'startdec') sprintf('%-18s', 'enddec')  '\n']);
    end
    

    %for n_idx = 1 : (length(trackData.sun_max) + length(trackData.sky_mean))
     
    % Go through tracking vectors, save data into file:
    
    % Sun fields:
    if(isempty(trackData.sun) == 0)
        for idx1 = 1 : length(trackData.sun) % for loop going through all .sun fields

            for idx2 = 1 : length(trackData.sun(idx1).max) % for loop going through fields inside a single .sun(idx1) field

                sun_time_print = sprintf('%-40s', trackData.sun(idx1).time{idx2});
                sun_id_print = sprintf('%-15s', trackData.sun(idx1).id{idx2});
                map_type = sprintf('%-15s', 'Sun');
                sun_max = sprintf('%-15s', num2str(trackData.sun(idx1).max(idx2)));
                sun_mean = sprintf('%-15s', num2str(trackData.sun(idx1).mean(idx2)));
                sun_min = sprintf('%-20s', num2str(trackData.sun(idx1).min(idx2)));
                sun_startra = sprintf('%-17s', num2str(trackData.sun(idx1).coords(1)));
                sun_endra = sprintf('%-15s', num2str(trackData.sun(idx1).coords(2)));
                sun_startdec = sprintf('%-18s', num2str(trackData.sun(idx1).coords(3)));
                sun_enddec = sprintf('%-16s', num2str(trackData.sun(idx1).coords(4)));

                trackFileContents = textread(trackFileName, '%s', 'delimiter', '\n');
                printString = sprintf([sun_time_print, sun_id_print, map_type, sun_max, sun_mean, sun_min, sun_startra, sun_endra, sun_startdec, sun_enddec]);

                % Testing if current string already exists in the file:
                stringChecker = 0;
                for idx4 = 1 : length(trackFileContents)
                    if (strcmp(strtrim(trackFileContents{idx4}), strtrim(printString)) == 1) % strtrim removes leading and trailing white spaces
                        stringChecker = 1;
                    end
                end

                if(stringChecker == 0)
                    fprintf(track_file, [printString, '\n']);
                end

            end % end of for loop for stuff inside a single .sun(idx1)

        end % End of for-loop for all .sun fields
    end
        
    % Sky fields:
    if(isempty(trackData.sky) == 0)
        for idx1 = 1 : length(trackData.sky) % for loop going through all .sky fields

            for idx2 = 1 : length(trackData.sky(idx1).mean) % for loop going through fields inside a single .sky(idx1) field
        
                sky_time_print = sprintf('%-40s', trackData.sky(idx1).time{idx2});
                map_type = sprintf('%-15s', 'Sky');
                sky_id_print = sprintf('%-15s', trackData.sky(idx1).id{idx2});
                sky_max = sprintf('%-15s', num2str(trackData.sky(idx1).max(idx2)));
                sky_mean_print = sprintf('%-15s', num2str(trackData.sky(idx1).mean(idx2)));
                sky_min = sprintf('%-20s', num2str(trackData.sky(idx1).min(idx2)));
                sky_startra = sprintf('%-17s', num2str(trackData.sky(idx1).coords(1)));
                sky_endra = sprintf('%-15s', num2str(trackData.sky(idx1).coords(2)));
                sky_startdec = sprintf('%-18s', num2str(trackData.sky(idx1).coords(3)));
                sky_enddec = sprintf('%-16s', num2str(trackData.sky(idx1).coords(4)));

                trackFileContents = textread(trackFileName, '%s', 'delimiter', '\n');
                printString = sprintf([sky_time_print, sky_id_print, map_type, sky_max, sky_mean_print, sky_min, sky_startra, sky_endra, sky_startdec, sky_enddec]);

                % Testing if current string already exists in the file:
                stringChecker = 0;
                for idx4 = 1 : length(trackFileContents)
                    if (strcmp(strtrim(trackFileContents{idx4}), strtrim(printString)) == 1) % strtrim removes leading and trailing white spaces
                        stringChecker = 1;
                    end
                end

                if(stringChecker == 0)
                    fprintf(track_file, [printString, '\n']);
                end
          
            end % end of for loop for stuff inside a single .sky(idx1)

        end % End of for-loop for all .sky fields
            
    end
        
    %end
    
    fclose(track_file); % Close file

end

end