% This is the main GUI function of the "Sunmap" solar plotting tool. This file
% generates and controls the main GUI of the program.

function varargout = sunmap(varargin)
% SUNMAP M-file for sunmap.fig
%      SUNMAP, by itself, creates a new SUNMAP or raises the existing
%      singleton*.
%
%      H = SUNMAP returns the handle to a new SUNMAP or the handle to
%      the existing singleton*.
%
%      SUNMAP('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in SUNMAP.M with the given input arguments.
%
%      SUNMAP('Property','Value',...) creates a new SUNMAP or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before sunmap_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to sunmap_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help sunmap

% Last Modified by GUIDE v2.5 25-May-2015 13:20:04

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @sunmap_OpeningFcn, ...
                   'gui_OutputFcn',  @sunmap_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT
end




% --- Executes just before sunmap is made visible.
function sunmap_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to sunmap (see VARARGIN)

% Choose default command line output for sunmap
handles.output = hObject;

% UIWAIT makes sunmap wait for user response (see UIRESUME)
% uiwait(handles.figure1);

set(handles.listbox1,'Max',2); % Multiselection is enabled in the listbox for the observation files
set(handles.listbox6,'Max',2); % Multiselection is enabled in the listbox for the sun-measure commands
handles.rot_temp = 1;
handles.pushbutton36_enable = 'off';

handles.list = -999; % a variable, into which the last selections of the observation file listbox are saved

% Create two variables for saving the custom level separation values in the
% edit13 box, one for % of QSL units, one for original units:
handles.customSepQSL = '0.5';
handles.customSepOrig = '200';

% Create variables for the custom min/max values:
handles.customMinQSL = '90';
handles.customMinOrig = '-2500';
handles.customMaxQSL = '112';
handles.customMaxOrig = '35000';

% Create variables with default values for the separate figure settings:
handles.fig_size = 1.0;
handles.fig_beam_size = 1.0;
handles.fig_rot_angle = 1.0;
handles.fig_grid = 0.0;
handles.fig_simple = 0.0;

% Make live update from slider to min custom level:
% addlistener(handles.slider4,'Action',@updateMinLevel, handles);

if(ispc == 0)
    %set(handles.checkbox20, 'FontSize', 8);
    %set(handles.checkbox26, 'FontSize', 8);
    set(handles.checkbox27, 'FontSize', 7);
    set(handles.checkbox28, 'FontSize', 7);
    %set(handles.checkbox30, 'FontSize', 8);
    set(handles.checkbox36, 'FontSize', 7);
else
    set(handles.listbox1, 'FontSize', 8);
end

% Set a custom image as an icon for the pan tool button:
pan_tool_str = [pwd, '/img/pan_tool_icon.png'];
pan_tool_button_img = imread(pan_tool_str);
set(handles.togglebutton5, 'cdata', pan_tool_button_img);

% Set a custom image as an icon for the solar grid & circle fitting button:
solar_lines_tool_str = [pwd, '/img/solar_area_lines_icon.png']; % 
solar_lines_tool_button_img = imread(solar_lines_tool_str);
set(handles.togglebutton7, 'cdata', solar_lines_tool_button_img);

% Set a custom image as an icon for the solar disk centering toggle button:
solar_center_tool_str = [pwd, '/img/center_tool_icon.png'];
solar_center_tool_button_img = imread(solar_center_tool_str);
set(handles.togglebutton15, 'cdata', solar_center_tool_button_img);

% Make a multiline tooltip for togglebutton9 and togglebutton11 (sum-measure coordinate/command getting tools)
mapCommandString = sprintf('Draw (by dragging with mouse) a rectangular area on the map preview image\nto generate a sun-measure command in the listbox.\nSingle click on map results in error.');
set(handles.togglebutton9, 'TooltipString', mapCommandString);
trackCommandString = sprintf('Right-click or double click a point on the map to get a\ntracking command for sun-measure on that point.\nSingle left click draws a marker on the map but doesn''t get the coordinates.');
set(handles.togglebutton11, 'TooltipString', trackCommandString);

% A file containing plotting options in a struct is created:
fileOpenDir = '/data/sun';
%fileOpenDir = '/home/nlavonen';
% Write fileOpenDir string to an invisible text field in the main GUI:
set(handles.text66, 'String', fileOpenDir);
%options = struct('b_size', 1.0, 'grid_on', 0.0, 'rot_north', 1.0, 'colormap_opt', 1.0, 'min_val', 50, 'max_val', 130, 'custom_lvls', 0.0, 'close_previous', 1.0, 'image_size', 1.0, 'b_size_changed', 0.0, 'rot_disable', 0.0, 'qsl_cont', 0.0); % , 'open_files_dir', fileOpenDir
%save('sunmap_options.mat', '-struct', 'options');

set(handles.figure1, 'Name', 'Sunmap');

% Update handles structure
guidata(hObject, handles);

end

% function updateMinLevel(handles)
% 
% sliderValue = get(handles.slider4, 'Value');
% set(handles.edit10, 'String', num2str(sliderValue));
%     
% end

% --- Outputs from this function are returned to the command line.
function varargout = sunmap_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;
end


% --- Executes on button press in add_files_button.  -OPEN FILE(S)-
function add_files_button_Callback(hObject, eventdata, handles)
% hObject    handle to add_files_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%options = open('sunmap_options.mat');
fileOpenDir = get(handles.text66, 'String'); % text66 is an invisible text field in the main GUI that stores the pathname for future use

%gets input file(s) from user
[input_file,pathname] = uigetfile( ...
       {'*.info;*.INFO;*.suninfo1;*.map;*.mat;', 'MRO solar map info/data file (*.info, *.INFO, *.suninfo1, *map), Sunmap file list (*.mat)'; ...
        '*.*', 'All Files (*.*)'}, ...
        'Select files', ... 
        'MultiSelect', 'on', fileOpenDir);
 
%if file selection is cancelled, pathname should be zero
%and nothing should happen
if pathname == 0
    return
else
    set(handles.text66, 'String', pathname); % text66 is an invisible text field in the main GUI that stores the pathname for future use
end
 
% Get the current data file names inside the listbox
inputFileNames = get(handles.listbox1,'String');
 
% % Number of files in listbox, before:
if(isempty(get(handles.listbox1, 'String')) == 0)
    list_length_start = length(inputFileNames);
else
    list_length_start = 0;
end

% Update the file list:
inputFileNames = update_filelist(inputFileNames, input_file, pathname);

% % Number of files in listbox, after:
if(isempty(inputFileNames) == 0)
    list_length_end = length(inputFileNames); % Number of files in listbox after adding files
else
    list_length_end = 0;
end

% Set status bar text:
if(list_length_end - list_length_start >=2)
    set(handles.text39, 'String', [num2str(list_length_end-list_length_start), ' new files opened'])
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
elseif(list_length_end - list_length_start >= 1)
    set(handles.text39, 'String', [num2str(list_length_end-list_length_start), ' new file opened'])
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
else
    set(handles.text39, 'String', 'Selected file(s) already open')
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
end

% Update the listbox:
set(handles.listbox1,'String',inputFileNames);
 
% Make sure first file is always selected, so it doesn't go out of range.
% The GUI will break if this value is out of range
set(handles.listbox1,'Value',1);
 
% If the listbox isn't empty, the plotting and list-emptying buttons are made active:
if(isempty(get(handles.listbox1, 'String')) == 0)
    set(handles.plot_all_button, 'Enable', 'on');
    set(handles.Save, 'Enable', 'on');
    set(handles.save_file_list, 'Enable', 'on');
    set(handles.track_save_button, 'Enable', 'on');
    set(handles.plot_single_button, 'Enable', 'on');
    set(handles.plot_selected, 'Enable', 'on');
    set(handles.spot_save_button, 'Enable', 'on');
    set(handles.polar_avg_button, 'Enable', 'on');
    set(handles.radius_save_button, 'Enable', 'on');
    set(handles.clear_list_button, 'Enable', 'on');
    set(handles.Clear_list, 'Enable', 'on');
    set(handles.remove_partial_maps, 'Enable', 'on');
    set(handles.remove_file_button, 'Enable', 'on');
    set(handles.remove_selected, 'Enable', 'on');
    set(handles.Untitled_100, 'Enable', 'on'); % lb1 context menu items
    set(handles.Untitled_101, 'Enable', 'on');
    set(handles.Untitled_102, 'Enable', 'on');
    set(handles.Untitled_114, 'Enable', 'on');
    set(handles.Untitled_115, 'Enable', 'on');
    set(handles.Untitled_116, 'Enable', 'on');
    set(handles.Untitled_117, 'Enable', 'on');
    set(handles.Untitled_118, 'Enable', 'on');
    set(handles.Untitled_134, 'Enable', 'on');
end

set(handles.clear_list_button, 'String', 'Clear list');
set(handles.Clear_list, 'Label', 'Clear list');
set(handles.clear_list_button, 'Enable', 'on');
set(handles.Clear_list, 'Enable', 'on');
set(handles.remove_partial_maps, 'Enable', 'on');

% Call preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    preview(plot_map, handles.figure1);
end

% Update handles structure
guidata(hObject, handles);

end

% Function for adding files to Sunmap's listbox:
% Input:
% - inputFileNames: original cell array of listbox items (full paths)
% - input_file: string or cell array of new file(s) to be added
% - pathname: path to new files
function inputFileNames = update_filelist(inputFileNames, input_file, pathname)

    % Single file, not in cell format:
    if iscell(input_file) == 0

        file_listFile_path = [pathname, input_file];

        % Check if the opened file is a file list file:
        if(ischar(input_file) && isempty(regexpi(input_file,'.mat')) == 0)
            % Filename contains '.mat' --> file list
            % Open the file:
            load_data = load(file_listFile_path);
            if(isfield(load_data, 'plottingList'))
                % Combine the current contents of the listbox with the contents
                % of the plotting list:
                inputFileNames = vertcat(inputFileNames, load_data.plottingList);
            else
                mat_file_error(file_listFile_path);
            end
        else % File is not a file list --> it's (or it should be) an info file
            % Add the most recent data file selected to the cell containing
            % all the data file names
            inputFileNames{end+1} = file_listFile_path;
        end


    % Multiple files, in cell format:
    else
        for n = 1:length(input_file)
            % Check if the currently handled file name is that of a .mat file
            % (file name contains '.mat')
            if(isempty(regexpi(input_file{n}, '.mat')) == 0)

                % Open the file:
                load_data = load(fullfile(pathname,input_file{n}));
                if(isfield(load_data, 'plottingList'))
                    % (:) is there to make the contents of inputFileNames
                    % compatible with vertcat:
                    inputFileNames = vertcat(inputFileNames(:), load_data.plottingList);
                else
                    mat_file_error(fullfile(pathname,input_file{n}));
                end

            % File is not *.mat file --> it's added to file list:
            else
                inputFileNames{end+1} = fullfile(pathname,input_file{n});
            end   
        end
    end

    % Removing duplicate paths from the list:
    inputFileNames = unique(inputFileNames);
    
end

% Function for adding files to Sunmap's listbox:
% Input:
% - inputFileNames: original cell array of listbox items (full paths)
% - input_file: Cell array of new file(s) to be added
function inputFileNames = update_filelist_multi(inputFileNames, input_file)

    % Multiple files, in cell format:
    for n = 1:length(input_file)
        % Check if the currently handled file name is that of a .mat file
        % (file name contains '.mat')
        if(isempty(regexpi(input_file{n}, '.mat')) == 0)

            % Open the file:
            load_data = load(input_file{n});
            if(isfield(load_data, 'plottingList'))
                % (:) is there to make the contents of inputFileNames
                % compatible with vertcat:
                inputFileNames = vertcat(inputFileNames(:), load_data.plottingList);
            else
                mat_file_error(input_file{n});
            end

        % File is not *.mat file --> it's added to file list:
        else
            inputFileNames{end+1} = input_file{n};
        end   
    end

    % Removing duplicate paths from the list:
    inputFileNames = unique(inputFileNames);

end

% Function for giving error dialog on unsuccessful opening of a .mat file:
function mat_file_error(file_path)

    message = ['The .mat file being opened (', file_path, ') does not contain a Sunmap file list and it will be dismissed.'];
    errordlg(message, 'Error', 'modal');

end

% --------------------------------------------------------------------
% Callback function for "Open folder(s)" menu item in the File -menu:
function open_folders_Callback(hObject, eventdata, handles)
% hObject    handle to open_folders (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get the handles object:
sunmap_handles = guidata(handles.figure1);
%sunmap_handles.last_directory

% Read stored previous directory:
%if(isfield(sunmap_handles, 'last_directory'))
%    last_directory = sunmap_handles.last_directory;
%else
%    last_directory = '/data/sun/';
%    sunmap_handles.last_directory = last_directory;
%end

% Open dialog:
list = uipickfiles('FilterSpec', '/data/sun/', 'Type', {
    '*.suninfo1', '.suninfo1 - MRO solar map info file'; 
    '*.map', '.map - Map-measure data file';
    '*.mat', '*.mat - Sunmap file list';
    '*.info', '*.info - MRO solar map info file';
    '*.INFO', '*.INFO - MRO solar map info file';
    '*', '* - All files'; }, 'Output', 'struct');

% File selection canceled:
if(isstruct(list) == 0)
    if(list == 0)
        return;
    end
end

% Create empty filelist cell array:
sunmap_handles.filelist = {};

guidata(handles.figure1, sunmap_handles);

file_listing(list, '', handles.figure1);

% Get the current data file names inside the listbox
inputFileNames = get(handles.listbox1,'String');

sunmap_handles = guidata(handles.figure1);

% Number of files in listbox, before:
if(isempty(get(handles.listbox1, 'String')) == 0)
    list_length_start = length(inputFileNames);
else
    list_length_start = 0;
end

% Call to update the list:
inputFileNames = update_filelist_multi(inputFileNames, sunmap_handles.filelist);

% Number of files in listbox, after:
if(isempty(inputFileNames) == 0)
    list_length_end = length(inputFileNames); % Number of files in listbox after adding files
else
    list_length_end = 0;
end

% Set status bar text:
if(list_length_end - list_length_start >=2)
    set(handles.text39, 'String', [num2str(list_length_end-list_length_start), ' new files opened'])
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
elseif(list_length_end - list_length_start >= 1)
    set(handles.text39, 'String', [num2str(list_length_end-list_length_start), ' new file opened'])
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
else
    set(handles.text39, 'String', 'Selected file(s) already open')
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
end

% Update the listbox:
set(handles.listbox1,'String',inputFileNames);
 
% Make sure first file is always selected, so it doesn't go out of range.
% The GUI will break if this value is out of range
set(handles.listbox1,'Value',1);
 
% If the listbox isn't empty, the plotting and list-emptying buttons are made active:
if(isempty(get(handles.listbox1, 'String')) == 0)
    set(handles.plot_all_button, 'Enable', 'on');
    set(handles.Save, 'Enable', 'on');
    set(handles.save_file_list, 'Enable', 'on');
    set(handles.track_save_button, 'Enable', 'on');
    set(handles.plot_single_button, 'Enable', 'on');
    set(handles.plot_selected, 'Enable', 'on');
    set(handles.spot_save_button, 'Enable', 'on');
    set(handles.polar_avg_button, 'Enable', 'on');
    set(handles.radius_save_button, 'Enable', 'on');
    set(handles.clear_list_button, 'Enable', 'on');
    set(handles.Clear_list, 'Enable', 'on');
    set(handles.remove_partial_maps, 'Enable', 'on');
    set(handles.remove_file_button, 'Enable', 'on');
    set(handles.remove_selected, 'Enable', 'on');
    set(handles.Untitled_100, 'Enable', 'on'); % lb1 context menu items
    set(handles.Untitled_101, 'Enable', 'on');
    set(handles.Untitled_102, 'Enable', 'on');
    set(handles.Untitled_114, 'Enable', 'on');
    set(handles.Untitled_115, 'Enable', 'on');
    set(handles.Untitled_116, 'Enable', 'on');
    set(handles.Untitled_117, 'Enable', 'on');
    set(handles.Untitled_118, 'Enable', 'on');
    set(handles.Untitled_134, 'Enable', 'on');
end

set(handles.clear_list_button, 'String', 'Clear list');
set(handles.Clear_list, 'Label', 'Clear list');
set(handles.clear_list_button, 'Enable', 'on');
set(handles.Clear_list, 'Enable', 'on');
set(handles.remove_partial_maps, 'Enable', 'on');

% Call preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    preview(plot_map, handles.figure1);
end



% Update handles structure
guidata(hObject, handles);

end % END OF open_folders_Callback()


% Function for creating a list of directory contents recursively:
% Input:
% - list: 
% - parent
% - handles
function file_listing(list, parent, handles)

    sunmap_handles = guidata(handles);
    
    list_length = length(list);
   
    % Create a list of plain files
    for n = 1:list_length
        
        % Item is '.' or '..' --> skip
        if(strcmp(list(n).name, '.') == 1 || strcmp(list(n).name, '..') == 1)
            continue;
        end    
        
        % Item is a directory
        if (list(n).isdir == 1)
            subparent = fullfile(parent, list(n).name);
            sublist = dir(subparent);
            file_listing(sublist, subparent, handles);

        % Item is a file
        else
            [pathstr, name, extension] = fileparts(list(n).name);
            if(strcmp(extension, '.suninfo1') == 1 || strcmp(extension, '.INFO') == 1 || strcmp(extension, '.info') == 1 || strcmp(extension, '.mat') == 1)
                %fprintf(fullfile(parent, list(n).name));
                sunmap_handles.filelist{end + 1} = fullfile(parent, list(n).name);
                guidata(handles, sunmap_handles);
                %fprintf('\n');
                
            end
        end 

    end

end


% --- Executes on button press in remove_file_button. -REMOVE FILE-
function remove_file_button_Callback(hObject, eventdata, handles)
% hObject    handle to remove_file_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


%get the current list of file names from the listbox
inputFileNames = get(handles.listbox1,'String');
 
%get the values for the selected file names
option = get(handles.listbox1,'Value');
 
%is there is nothing to delete, nothing happens
if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
    return;
end
 
%erases the contents of highlighted item in data array
inputFileNames(option) = [];

% Save the number of selected items on the list:
selection_size = size(option); % 1x2 matrix. The number of selected items is in the second element:
num_selected = selection_size(2);
 
%updates the gui, erasing the selected item from the listbox
set(handles.listbox1,'String',inputFileNames);
 
%moves the highlighted item to an appropiate value or else will get error
if option(end) > length(inputFileNames)
    set(handles.listbox1,'Value',length(inputFileNames));
end
 
% Set the Program status string to tell the user about the removal:
if(num_selected == 1)
    user_feedback = [num2str(num_selected), ' file removed from the list'];
else
    user_feedback = [num2str(num_selected), ' files removed from the list'];
end
set(handles.text39, 'String', user_feedback);
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

% If the list is empty after Remove file button is pressed, the
% plotting buttons are made inactive:
if(isempty(get(handles.listbox1, 'String')) == 1)
    set(handles.plot_all_button, 'Enable', 'off');
    set(handles.Save, 'Enable', 'off');
    set(handles.save_file_list, 'Enable', 'off');
    set(handles.track_save_button, 'Enable', 'off');
    set(handles.plot_single_button, 'Enable', 'off');
    set(handles.plot_selected, 'Enable', 'off');
    set(handles.spot_save_button, 'Enable', 'off');
    set(handles.polar_avg_button, 'Enable', 'off');
    set(handles.radius_save_button, 'Enable', 'off');
    set(handles.clear_list_button, 'Enable', 'off');
    set(handles.Clear_list, 'Enable', 'off');
    set(handles.remove_partial_maps, 'Enable', 'off');
    set(handles.remove_file_button, 'Enable', 'off');
    set(handles.remove_selected, 'Enable', 'off');
    set(handles.Untitled_100, 'Enable', 'off'); % lb1 context menu items
    set(handles.Untitled_101, 'Enable', 'off');
    set(handles.Untitled_102, 'Enable', 'off');
    set(handles.Untitled_114, 'Enable', 'off');
    set(handles.Untitled_115, 'Enable', 'off');
    set(handles.Untitled_116, 'Enable', 'off');
    set(handles.Untitled_117, 'Enable', 'off');
    set(handles.Untitled_118, 'Enable', 'off');
    set(handles.Untitled_134, 'Enable', 'off');

    cla(handles.axes2);
    set(handles.text40, 'String', '');
    set(handles.text41, 'String', '');
    set(handles.text42, 'String', '');
    set(handles.text43, 'String', '');
    set(handles.text44, 'String', '');
    set(handles.text45, 'String', '');
    set(handles.text49, 'String', '');
    set(handles.text64, 'String', '');
    set(handles.text63, 'String', '');
    set(handles.text79, 'String', '');
    set(handles.text80, 'String', '');
    set(handles.text81, 'String', '');
        
% If the list is not empty after removal of item(s):
elseif(isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    set(handles.listbox1, 'Value', indexes(1));
        
    preview(plot_map, handles.figure1);

end

% Update handles structure
guidata(hObject, handles);

drawnow;

end



% --- Executes on selection change in listbox1.
% The main listbox for observation files
function listbox1_Callback(hObject, eventdata, handles)
% hObject    handle to listbox1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

key = double(get(gcbf, 'CurrentCharacter'));
if gcbo == handles.listbox1;
%     Value =  get(handles.listbox1, 'Value')
%     size(Value)
end

% If the list is clicked normally with left mouse button and there are more
% than one selected items, the clicked list item is added to the list of
% selected items
% if(handles.list ~= -999)
%     if(isempty(handles.list) ~= 1)
%         if(handles.list > 1)
%             if(strcmp(get(handles.figure1, 'SelectionType'), 'Normal') == 1)
%                 new_list = get(hObject, 'Value');
%                 
%             end
%         end
%     end
% end

% Listbox is not empty:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(hObject, 'Value')) == 0)

    plottingList = get(handles.listbox1,'String');    
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    % Write info on number of selected files to status bar:
    list_value = get(handles.listbox1, 'Value');
    list_num_selected = size(list_value);
    if(list_num_selected(2) >= 2)
        set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)
        set(handles.text39, 'String', [num2str(list_num_selected(2)), ' files selected']);
    else
        set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)
        set(handles.text39, 'String', '');
    end

%     if(exist('key', 'var'))
%         if(key == 13)
%             % Don't preview if "Enter" was pressed, because it's
%             % reserved for plotting selected as separate figures
%             % Spacebar can't be handled here because apparently a mouse
%             % click also produces the spacebar ascii key == 32 :S
%             % A mouse click is _sometimes_ registered as enter -->
%             % disabled
%         else
%             preview(plot_map, handles.figure1);
%         end
%     else
%     end

    % Only update the preview plot if the currently selected map has changed:
    lb_value_old = handles.list;
    if(isempty(lb_value_old)) % There isn't yet anything in handles.list
        lb_value_old = 1;
    end
    lb_value_new = get(handles.listbox1, 'Value');
    lb_value_old_length = length(lb_value_old);
    lb_value_new_length = length(lb_value_new);
    if(lb_value_old(1) ~= lb_value_new(1) || lb_value_old_length == lb_value_new_length || lb_value_new_length == 1)
        preview(plot_map, handles.figure1);
    end

    % A double click plots the selected map:
    % This also executes on keyboard enter and space (key == 13)
    if(strcmp(get(handles.figure1, 'SelectionType'), 'open') == 1)
        plot_single_button_Callback(hObject, eventdata, handles);
    end
    %set(handles.text39, 'String', '');
    %set(handles.text39, 'ForegroundColor', [0 0.5 0]);
end

% The current selection "list" of the listbox is saved for the next
% callback:
handles.list = get(handles.listbox1, 'Value');
guidata(hObject, handles);

end

% --- Executes during object creation, after setting all properties.
function listbox1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

end



% --- Executes on button press in plot_single_button.
function plot_single_button_Callback(hObject, eventdata, handles)
% hObject    handle to plot_single_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Checking that the user has entered only numbers into the custom levels
% fields:
if(get(handles.checkbox31, 'Value') == 1.0)
    
    if(str2num(get(handles.edit10, 'String'))) >= str2num(get(handles.edit11, 'String'))
        message = ['The minimum custom levels value (', get(handles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(handles.edit11, 'String'), ').'];
        msgbox(message, 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
        return;
    end
    
    if(isempty(get(handles.edit10, 'String')))
        msgbox('Please insert a number value in the minimum custom levels field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
    end
    
    if(isempty(get(handles.edit11, 'String')))
        msgbox('Please insert a number value in the maximum custom levels field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
    end    
    
        cmp1 = strcmp(num2str(str2num(get(handles.edit10, 'String'))), '');
        cmp2 = strcmp(num2str(str2num(get(handles.edit11, 'String'))), '');
    
        if(cmp1 == 1 || cmp2 == 1)
            % Notice; % Old self made notice window
            % Using warning window as with other warnings:
            msgbox('You have entered something else than numbers to the custom levels fields. Please use only numbers as custom levels.', 'Warning', 'warn', 'modal');
            figure1_ResizeFcn(hObject, eventdata, handles);
            return;
        end
        
end

% Protection from invalid "Number of levels" values:
if(str2num(get(handles.edit12, 'String')) <= 0)
    msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

if(str2num(get(handles.edit12, 'String')) > 4096)
    message = ['You have entered a very large (> 4096) "Number of levels" value (', get(handles.edit12, 'String'), ') This can make plotting the map very slow and is not allowed.'];
    msgbox(message, 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

if(isempty(get(handles.edit12, 'String')))
    msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

cmp = strcmp(num2str(str2num(get(handles.edit12, 'String'))), '');

if(cmp == 1)
    msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

% Protection from invalid Custom level separation values:
if(get(handles.checkbox37, 'Value') == 1.0)
    cmp1 = strcmp(num2str(str2num(get(handles.edit13, 'String'))), '');
    if(cmp1 == 1)
        msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
        return;
    elseif(str2num(get(handles.edit13, 'String')) <= 0)
        msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
        return;        
    end
end

% Protection from invalid Active Area treshold values:
if(get(handles.checkbox51, 'Value') == 1)   
    % Checking that the user has entered only numbers into the edit box:
    cmp1 = strcmp(num2str(str2num(get(handles.edit17, 'String'))), '');
    if(cmp1 == 1)
        msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
        %tempTreshold = 104; % "% of QSL"
        return;
    end
    % Checking that the edit box isn't empty:     
    if(isempty(get(handles.edit17, 'String')))
        msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
        %tempTreshold = 104; % "% of QSL"
        return;
    end
end

%options = open('sunmap_options.mat');

%get the current list of file names from the listbox
inputFileNames = get(handles.listbox1,'String');
    
%get the values for the selected file names
option = get(handles.listbox1,'Value');
 
%if there is nothing to plot, nothing happens
if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
    return
end

inputFileNames = get(handles.listbox1,'String');
%get the values for the selected file names
option = get(handles.listbox1,'Value');

%only the selected file+path is written into the list:
filenames = inputFileNames(option);

%destFolder = get(handles.listbox2, 'String');
cmap = get_colormap(handles.figure1);

opt.img_type = '.png'; % not needed anymore
opt.destFolder = 'foo'; % not needed anymore
opt.anim_gif = 0;
opt.plottingList = filenames;
opt.beam_size = handles.fig_beam_size;
opt.solar_north = handles.fig_rot_angle;
opt.grid_toggle = handles.fig_grid; %get(handles.checkbox26, 'Value');
opt.cmap = cmap;
opt.abs_scale = get(handles.checkbox31, 'Value');
opt.mapType = get(handles.popupmenu9, 'Value') ;
opt.customNumOfLevelsToggle = get(handles.checkbox35, 'Value') ;
opt.customNumOfLevels = get(handles.edit12, 'String') ;

if(ischar(get(handles.edit10, 'String')) == 1 || ischar(get(handles.edit11, 'String')) == 1)
    opt.min = str2num(get(handles.edit10, 'String'));
    opt.max = str2num(get(handles.edit11, 'String'));
else
    opt.min = get(handles.edit10, 'String');
    opt.max = get(handles.edit11, 'String');
end
opt.img_save = 0;
opt.fig_save = 0;
opt.close_prev = 0;
%opt.img_size = options.image_size;
opt.rot_disable = get(handles.checkbox40, 'Value');
opt.qsl_cont = get(handles.checkbox27, 'Value');

% Image size
switch handles.fig_size;
    case 1
        opt.img_h = 500;
        opt.img_w = 674;
    case 2
        opt.img_h = 420;
        opt.img_w = 560;
    case 3
        opt.img_h = 600;
        opt.img_w = 800;
    case 4
        opt.img_h = 675;
        opt.img_w = 900;
    otherwise
        opt.img_h = 500;
        opt.img_w = 674;
end


% CALLING sunmap_contourplot:
sunmap_contourplot(opt, handles.figure1);

end


% --- Executes on button press in plot_all_button.
% "Save selected..." button
function plot_all_button_Callback(hObject, eventdata, handles)
% hObject    handle to plot_all_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Checking that the user has entered only numbers into the custom levels
% fields:
if(get(handles.checkbox31, 'Value') == 1.0)
cmp1 = strcmp(num2str(str2num(get(handles.edit10, 'String'))), '');
cmp2 = strcmp(num2str(str2num(get(handles.edit11, 'String'))), '');

if(cmp1 == 1 || cmp2 == 1)
    % Notice; % Old self made notice window
    % Using warning window as with other warnings:
    msgbox('You have entered something else than numbers to the custom levels fields. Please use only numbers as custom levels.', 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end
end

% Protection from invalid "Number of levels" values:
if(str2num(get(handles.edit12, 'String')) <= 0)
    msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

if(str2num(get(handles.edit12, 'String')) > 4096)
    message = ['You have entered a very large (> 4096) "Number of levels" value (', get(handles.edit12, 'String'), ') This can make generating the map preview very slow and is not recommended.'];
    msgbox(message, 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

if(isempty(get(handles.edit12, 'String')))
    msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

cmp = strcmp(num2str(str2num(get(handles.edit12, 'String'))), '');

if(cmp == 1)
    msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
    figure1_ResizeFcn(hObject, eventdata, handles);
    return;
end

% Protection from invalid Custom level separation values:
if(get(handles.checkbox37, 'Value') == 1.0)
    cmp1 = strcmp(num2str(str2num(get(handles.edit13, 'String'))), '');
    if(cmp1 == 1)
        msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
        return;
    elseif(str2num(get(handles.edit13, 'String')) <= 0)
        msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
        return;
    end
end

% Protection from invalid Active Area treshold values:
if(get(handles.checkbox51, 'Value') == 1)   
    % Checking that the user has entered only numbers into the edit box:
    cmp1 = strcmp(num2str(str2num(get(handles.edit17, 'String'))), '');
    if(cmp1 == 1)
        msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
        %tempTreshold = 104; % "% of QSL"
        return;
    end
    % Checking that the edit box isn't empty:     
    if(isempty(get(handles.edit17, 'String')))
        msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
        %tempTreshold = 104; % "% of QSL"
        return;
    end
end

saving(handles.figure1);

end


% --- Executes on button press in exit_button.
function exit_button_Callback(hObject, eventdata, handles)
% hObject    handle to exit_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
clear global;
close 'Sunmap';
% if(exist('options'))
% close options;
% end
% if(exist('microvax_ftp'))
% close microvax_ftp;
% end

end

% Function for resizing the Sunmap main window:
function figure1_ResizeFcn(hObject, eventdata, handles)

guiPosition = get(handles.figure1, 'position');
guiX = guiPosition(1);
guiY = guiPosition(2);
guiWidth = guiPosition(3);
guiHeight = guiPosition(4);


% Min size:
if(guiWidth < 885)
   guiWidth = 885;
end
if(guiHeight < 609)
    guiHeight = 609;
end

leftPanelWidth = guiWidth - 344;
leftPanelHeight = guiHeight - 281;

% 'Tools & options' uipanel30:
rightPanelPosition = get(handles.uipanel30, 'position');
rightPanelX = leftPanelWidth + 16;
rightPanelY = guiHeight - rightPanelPosition(4) - 4; % this achieves target of 39
%rightPanelWidth = rightPanelPosition(3);
%rightPanelHeight = rightPanelPosition(4);

% Options window:
set(handles.uipanel30, 'position', [rightPanelX, rightPanelY,  rightPanelPosition(3),  rightPanelPosition(4)]);

%set(handles.uipanel28, 'position', [8 278 leftPanelWidth leftPanelHeight]);

% Listbox for files:
lbPosition = get(handles.listbox1, 'Position');
listboxHeight = lbPosition(4); % Height is set as whatever it has been --> changes made in GUIDE apply directly
listbox_verticalPosition = lbPosition(2); % Vertical position is set as whatever it has been --> changes made in GUIDE apply directly
set(handles.listbox1, 'position', [8 listbox_verticalPosition leftPanelWidth listboxHeight]);

% Plotting axes box  'Map preview' uipanel37:
plotboxPosition = get(handles.uipanel37, 'position');
plotboxX = plotboxPosition(1);
plotboxY = plotboxPosition(2);
plotboxHeight = guiHeight - 266;
plotboxWidth = guiWidth - 626;
set(handles.uipanel37, 'position', [plotboxX, plotboxY, plotboxWidth, plotboxHeight]);

% Status bar:
statusBarPos = get(handles.uipanel42, 'Position');
set(handles.uipanel42, 'position', [statusBarPos(1) statusBarPos(2) guiWidth - 121 25]);

% Plotting axes:
%axesPosition = get(handles.axes2, 'position');
% axesX = axesPosition(1);
% axesY = axesPosition(2);
axesHeight = plotboxHeight - 24;
axesWidth = plotboxWidth - 9;

% Testing:
%axesHeight = plotboxHeight - 54;
%axesWidth = plotboxWidth - 29;
% / Testing
set(handles.axes2, 'position', [3, 5, axesWidth, axesHeight]);
% Testing
%set(handles.axes2, 'position', [20, 20, axesWidth, axesHeight]);
% / Testing

% "Generating preview..." text:
%prevtextPosition = get(handles.text65, 'position');
prevtextX = 4;
prevtextY = plotboxHeight - 32;
prevtextWidth = 200;
prevtextHeight = 16;
set(handles.text65, 'position', [prevtextX, prevtextY, prevtextWidth, prevtextHeight]);

% Preview box uipanels:
set(handles.uipanel52, 'position', [6, guiHeight - 170, 250, 166]); % Observation info
set(handles.uipanel53, 'position', [6, guiHeight - 278, 250, 101]); % Data analysis
set(handles.uipanel54, 'position', [6, guiHeight - 347, 250, 62]); % Solar data

% Preview tools/buttons box:
prevToolsPosition = get(handles.uipanel43, 'position');
prevToolsX = 577 + plotboxWidth - 315;
prevToolsY = prevToolsPosition(2);
set(handles.uipanel43, 'position', [prevToolsX, prevToolsY, prevToolsPosition(3), guiHeight - 273]);

% Exit button:
exitX = guiWidth - 106;
exitY = 5;
exitPosition = get(handles.exit_button, 'position');
set(handles.exit_button, 'position', [exitX, exitY, exitPosition(3), exitPosition(4)]);

end


% --- Executes on button press in clear_list_button.
function clear_list_button_Callback(hObject, eventdata, handles)
% hObject    handle to clear_list_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Button is used for clearing the list:
if(strcmp(get(handles.clear_list_button, 'String'), 'Clear list') == 1);

    % If the listbox is not empty, a backup is made:
    if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
        handles.lb1backup = get(handles.listbox1, 'String');
        handles.lb1_value_backup = get(handles.listbox1, 'Value');
    end
    % Changing the text into "Undo"
    set(handles.clear_list_button, 'String', 'Undo');
    set(handles.Clear_list, 'Label', 'Undo clear list');

    %get the current list of file names from the listbox
    inputFileNames = get(handles.listbox1,'String');

    %get the values for the selected file names
    option = get(handles.listbox1,'Value');

    %is there is nothing to delete, nothing happens
    if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
        return
    end

    %erases the contents of highlighted item in data array
    %updates the gui, erasing the selected item from the listbox
    set(handles.listbox1,'String', '');

    %moves the highlighted item to an appropiate value or else will get error
    if option(end) > length(inputFileNames)
        set(handles.listbox1,'Value',length(inputFileNames));
    end


    % Making the Plot and list-buttons inactive, since the list is empty and there is
    % nothing to plot and no files to remove or clear:
    set(handles.plot_all_button, 'Enable', 'off');
    set(handles.Save, 'Enable', 'off');
    set(handles.save_file_list, 'Enable', 'off');
    set(handles.track_save_button, 'Enable', 'off');
    set(handles.plot_single_button, 'Enable', 'off');
    set(handles.plot_selected, 'Enable', 'off');
    set(handles.spot_save_button, 'Enable', 'off');
    set(handles.polar_avg_button, 'Enable', 'off');
    set(handles.radius_save_button, 'Enable', 'off');
    %set(handles.clear_list_button, 'Enable', 'off');
    set(handles.remove_file_button, 'Enable', 'off');
    set(handles.remove_selected, 'Enable', 'off');
    set(handles.remove_partial_maps, 'Enable', 'off');
    set(handles.Untitled_100, 'Enable', 'off'); % lb1 context menu items
    set(handles.Untitled_101, 'Enable', 'off');
    set(handles.Untitled_102, 'Enable', 'off');
    set(handles.Untitled_114, 'Enable', 'off');
    set(handles.Untitled_115, 'Enable', 'off');
    set(handles.Untitled_116, 'Enable', 'off');
    set(handles.Untitled_117, 'Enable', 'off');
    set(handles.Untitled_118, 'Enable', 'off');
    set(handles.Untitled_134, 'Enable', 'off');

    cla(handles.axes2);
    axis off,
    set(handles.text40, 'String', '');
    set(handles.text41, 'String', '');
    set(handles.text42, 'String', '');
    set(handles.text43, 'String', '');
    set(handles.text44, 'String', '');
    set(handles.text45, 'String', '');
    set(handles.text49, 'String', '');
    set(handles.text63, 'String', '');
    set(handles.text64, 'String', '');
    set(handles.text79, 'String', '');
    set(handles.text80, 'String', '');
    set(handles.text81, 'String', '');
    set(handles.text86, 'String', '');
    
    % Set the status bar text to inform user of removal:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)
    set(handles.text39, 'String', 'File list cleared. Press "Undo" to cancel.');

else % Button is used for "Undo" of list clearing
    
    set(handles.clear_list_button, 'String', 'Clear list');
    set(handles.Clear_list, 'Label', 'Clear list');
    %global backup;
    set(handles.listbox1, 'String', handles.lb1backup);
    set(handles.listbox1, 'Value', handles.lb1_value_backup);

    if(isempty(get(handles.listbox1, 'Value')) == 0)
        plottingList = get(handles.listbox1,'String');
        indexes = get(handles.listbox1, 'Value');
        plot_map = plottingList(indexes(1));
        preview(plot_map, handles.figure1);
    end

    set(handles.plot_all_button, 'Enable', 'on');
    set(handles.Save, 'Enable', 'on');
    set(handles.save_file_list, 'Enable', 'on');
    set(handles.track_save_button, 'Enable', 'on');
    set(handles.plot_single_button, 'Enable', 'on');
    set(handles.plot_selected, 'Enable', 'on');
    set(handles.spot_save_button, 'Enable', 'on');
    set(handles.polar_avg_button, 'Enable', 'on');
    set(handles.radius_save_button, 'Enable', 'on');
    %set(handles.clear_list_button, 'Enable', 'off');
    set(handles.remove_file_button, 'Enable', 'on');
    set(handles.remove_selected, 'Enable', 'on');
    set(handles.remove_partial_maps, 'Enable', 'on');
    set(handles.Untitled_100, 'Enable', 'on'); % lb1 context menu items
    set(handles.Untitled_101, 'Enable', 'on');
    set(handles.Untitled_102, 'Enable', 'on');
    set(handles.Untitled_114, 'Enable', 'on');
    set(handles.Untitled_115, 'Enable', 'on');
    set(handles.Untitled_116, 'Enable', 'on');
    set(handles.Untitled_117, 'Enable', 'on');
    set(handles.Untitled_118, 'Enable', 'on');
    set(handles.Untitled_134, 'Enable', 'on');
    
    % Set the status bar text to inform user of undoing of list clearing:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)
    set(handles.text39, 'String', 'File list clearing cancelled.');

end

% Update handles structure
guidata(hObject, handles);

end


% --- Executes on button press in clear_default_button.
% The old incarnation onf the "Reset to default" button, that at some point
% seems to have been detached from the actual button.
% --> Completely useless and redundant
function clear_default_button_Callback(hObject, eventdata, handles)
% hObject    handle to clear_default_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(ispc == 1)
path_string = strcat(pwd, '\plots\', datestr(date, 29), '\');
else
path_string = strcat(pwd, '/plots/', datestr(date, 29), '/');    
end

set(handles.text21, 'String', 'Currently using default folder:');

set(handles.listbox2, 'String', path_string);

% Button made inactive after it is pressed:
set(handles.clear_default_button, 'Enable', 'off');

% Use default -button made active after reset to default is pressed:
set(handles.update, 'Enable', 'on');

end


% --- Executes on button press in update.
% USE CUSTOM -BUTTON
function update_Callback(hObject, eventdata, handles)
% hObject    handle to update (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

default_path = [pwd, '/plots/', datestr(date, 29), '/'];
current_path = get(handles.listbox2, 'String');
current_path(end) = '';
path = uigetdir(current_path, 'Select destination folder');

% If user presses Cancel, nothing happens:
if(path == 0)
    return
else
    
    custom_dir = [path, '/', datestr(date, 29), '/'];
    set(handles.listbox2, 'String', custom_dir);
    if(strcmp(default_path, custom_dir) == 1)
        % If user selected folder is the default folder:
        set(handles.text21, 'String', 'Currently using default folder:');
        set(handles.clear_default_button, 'Enable', 'off');
    else
        % If user selected folder is not the default folder:
        set(handles.text21, 'String', 'Currently using user selected folder:');
        set(handles.clear_default_button, 'Enable', 'on');
    end
end

set(handles.text21, 'String', 'Currently using user selected folder:');

end


% --- Executes on selection change in listbox2.
function listbox2_Callback(hObject, eventdata, handles)
% hObject    handle to listbox2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns listbox2 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox2
end

% --- Executes during object creation, after setting all properties.
function listbox2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

end


% --- Executes on key press with focus on listbox1 and none of its controls.
function listbox1_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to listbox1 (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)

%plot_single_button_Callback(hObject, eventdata, handles);
key = double(get(gcbf, 'CurrentCharacter'));
if gcbo == handles.listbox1;
    if key == 127 % for DEL key
        remove_file_button_Callback(hObject, eventdata, handles);
        return;
        %Plot selected map when keyboard 'enter' is pressed: 
        %CAUSES MATLAB CRASH @ close(figure(num)) @sunmap_contourplot:
    % elseif key == 13
    %     if(isempty(get(handles.listbox1, 'String')) == 0)
    %         plot_single_button_Callback(hObject, eventdata, handles);
    %     end
    elseif key == 13 % Enter or space --> do nothing. Enter & space are handled by listbox1 callback
        return;
    elseif key == 32
        return;
    end
end

% If listbox isn't empty:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    
    % No plotting from here if:
    % Delete pressed
    % Or:
    % Other than last or first list item is selected AND more than one list
    % item exists AND up or down arrow has been pressed:
    % (Neglecting this has resulted in double preview plotting when browsing maps with up/down arrow keys)
        
    if(isempty(key) == 0) % All keys don't register as numbers, but as empty (alt, crtl, etc)
        % If Up arrow AND the first list item not among the selected items AND there is more than
        % one list item:
        if(key == 30 && isempty(find(get(handles.listbox1, 'Value') == 1, 1)) == 1 && length(get(handles.listbox1, 'String')) > 1)
            return; 
        % Down arrow AND the last list item not among the selected items AND there is more
        % than one list item:
        elseif(key == 31 && isempty(find(get(handles.listbox1, 'Value') == length(get(handles.listbox1, 'String')), 1)) == 1 && length(get(handles.listbox1, 'String')) > 1)
            return;
        end
    else
        return;
    end
    
    % Only update the preview plot if the currently selected map has changed:
    lb_value_old = handles.list;
    if(isempty(lb_value_old)) % There isn't yet anything in handles.list
        lb_value_old = 1;
    end
    lb_value_new = get(handles.listbox1, 'Value');
    lb_value_old_length = length(lb_value_old);
    lb_value_new_length = length(lb_value_new);
    if(lb_value_old(1) ~= lb_value_new(1) || lb_value_old_length == lb_value_new_length || lb_value_new_length == 1)
        preview(plot_map, handles.figure1);
    end

end

% The current selection "list" of the listbox is saved for the next
% callback:
handles.list = get(handles.listbox1, 'Value');
guidata(hObject, handles);

end



% --- Executes during object creation, after setting all properties.
function axes1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes1
imshow('img/sun6.png');

end


% --- Executes on button press in checkbox14.
function checkbox14_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox14 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.checkbox14, 'Value') == 1.0)

set(handles.text21, 'Enable', 'on');
set(handles.listbox2, 'Enable', 'on');
set(handles.update, 'Enable', 'on');

if(strcmp(get(handles.text21, 'String'), 'Currently using user selected folder:') == 1)
    set(handles.clear_default_button, 'Enable', 'on');
end

end
    
    
if(get(handles.checkbox14, 'Value') == 0.0)
    
if(get(handles.checkbox16, 'Value') == 0.0)
    set(handles.text21, 'Enable', 'off');
    set(handles.listbox2, 'Enable', 'off');
    set(handles.update, 'Enable', 'off');
    set(handles.clear_default_button, 'Enable', 'off');
end

end


% Hint: get(hObject,'Value') returns toggle state of checkbox14
end

% --- Executes on button press in checkbox16.
function checkbox16_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox16 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if(get(handles.checkbox16, 'Value') == 1.0)

set(handles.popupmenu4, 'Enable', 'on');
set(handles.text38, 'Enable', 'on');
set(handles.text21, 'Enable', 'on');
set(handles.listbox2, 'Enable', 'on');
set(handles.update, 'Enable', 'on');

if(strcmp(get(handles.text21, 'String'), 'Currently using user selected folder:') == 1)
    set(handles.clear_default_button, 'Enable', 'on');
end

end
    
    
if(get(handles.checkbox16, 'Value') == 0.0)
    
set(handles.popupmenu4, 'Enable', 'off');
set(handles.text38, 'Enable', 'off');

if(get(handles.checkbox14, 'Value') == 0.0)
    set(handles.text21, 'Enable', 'off');
    set(handles.listbox2, 'Enable', 'off');
    set(handles.update, 'Enable', 'off');
    set(handles.clear_default_button, 'Enable', 'off');
end

end


% Hint: get(hObject,'Value') returns toggle state of checkbox16
end

% --- Executes on selection change in popupmenu4.
function popupmenu4_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu4 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu4
end

% --- Executes during object creation, after setting all properties.
function popupmenu4_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


% --- Executes during object deletion, before destroying properties.
function figure1_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
clear global;
end

% --- Executes during object creation, after setting all properties.
function axes2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes2
end


% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over listbox1.
function listbox1_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to listbox1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% Double click on list item plots selected:

list_selected = get(handles.listbox1, 'Value');
set(handles.listbox1, 'Value', list_selected);

end


% --- Executes on button press in pushbutton21.
% "Reset status" button for resetting the gui when it hasn't been reset by
% the functionality that set it in the first place
function pushbutton21_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if(isempty(get(handles.listbox1, 'String')) == 0)
    set(handles.plot_all_button, 'Enable', 'on');
    set(handles.Save, 'Enable', 'on');
    set(handles.save_file_list, 'Enable', 'on');
    set(handles.track_save_button, 'Enable', 'on');
    set(handles.plot_single_button, 'Enable', 'on');
    set(handles.plot_selected, 'Enable', 'on');
    set(handles.spot_save_button, 'Enable', 'on');
    set(handles.polar_avg_button, 'Enable', 'on');
    set(handles.radius_save_button, 'Enable', 'on');
end
set(handles.listbox1, 'Enable', 'on');
set(handles.text39, 'String', 'User interface reset');
set(handles.text39, 'ForegroundColor', [0 0.5 0]);
set(hObject, 'Visible', 'off'); % ktnxbye

end


% --- Executes on button press in pushbutton23.
function pushbutton23_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton23 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(hObject, 'Enable', 'off');
set(hObject, 'String', 'Opening...');

microvax_ftp(handles.figure1);
end


% --- Executes on button press in tab1.
function tab1_Callback(hObject, eventdata, handles)
% hObject    handle to tab1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.tab2, 'BackgroundColor', [0.906 0.906 0.906]);
set(handles.tab1, 'BackgroundColor', [0.602 0.602 0.602]);
set(handles.uipanel30, 'Visible', 'off');
set(handles.uipanel19, 'Visible', 'on');

end

% --- Executes on button press in tab2.
function tab2_Callback(hObject, eventdata, handles)
% hObject    handle to tab2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.tab1, 'BackgroundColor', [0.906 0.906 0.906]);
set(handles.tab2, 'BackgroundColor', [0.602 0.602 0.602]);
set(handles.uipanel19, 'Visible', 'off');
set(handles.uipanel30, 'Visible', 'on');

end

% --- Executes on button press in checkbox20.
% Separate figure Beam size checkbox:
function checkbox20_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox20 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Checked:
if(get(handles.checkbox20, 'Value') == 1.0)
    % Handles variable:
    handles.fig_beam_size = 1.0;
    % Menu item:
    set(handles.Untitled_109, 'Checked', 'on');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure beam size marker on');
else % Unchecked:
    % Handles variable:
    handles.fig_beam_size = 0.0;
    % Menu item:
    set(handles.Untitled_109, 'Checked', 'off');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure beam size marker off');
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Update handles structure
guidata(hObject, handles);

end

% --- Executes on button press in checkbox26.
% Separate figure grid checkbox
function checkbox26_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox26 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Checked:
if(get(handles.checkbox26, 'Value') == 1.0)
    % Handles variable:
    handles.fig_grid = 1.0;    
    % Menu item:    
    set(handles.Untitled_111, 'Checked', 'on');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure grid on');
else % Unchecked:
    % Handles variable:
    handles.fig_grid = 0.0;    
    % Menu item: 
    set(handles.Untitled_111, 'Checked', 'off');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure grid off');    
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Update handles structure
guidata(hObject, handles);

end

% --- Executes on button press in checkbox27.
function checkbox27_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox27 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox27

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.checkbox27, 'Value') == 1)
    set(handles.Untitled_122, 'Checked', 'on'); % Corresponding menu item
    set(handles.togglebutton8, 'Value', 1.0); % Corresponding toggle button
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Quiet Sun Level contour drawing on');
else
    set(handles.Untitled_122, 'Checked', 'off'); % Corresponding menu item
    set(handles.togglebutton8, 'Value', 0.0); % Corresponding toggle button
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Quiet Sun Level contour drawing off');
end

if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)

    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    preview(plot_map, handles.figure1);
   
end

end

% --- Executes on button press in checkbox28.
function checkbox28_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox28 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Hint: get(hObject,'Value') returns toggle state of checkbox28
if(get(handles.checkbox28, 'Value') == 1)
    datacursormode on,
    cursor = datacursormode(gcf);
    set(cursor, 'SnapToDataVertex','off');
    getCoordinates = 1;
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Data cursor on');
    
    if(getCoordinates == 1) % If a checkbox in GUI is checked:
        %set(cursor,'UpdateFcn',{@data_cursor_mod}); % calling the function that modifies the output of the data cursor
    end
    set(gca,'Layer','top'),
    %handles.rot_temp = get(handles.checkbox30, 'Value'); % remember cbox30 value before it's written over
    %set(handles.checkbox30, 'Value', 0.0);
    %set(handles.checkbox30, 'Enable', 'off');
    %set(handles.checkbox28, 'ForegroundColor', 'red');
    %set(handles.checkbox28, 'BackgroundColor', [1 0.8 0.8]);
    %guidata(hObject, handles);
else
    % take cbox30 value from memory, and set value
    %set(handles.checkbox30, 'Value', handles.rot_temp);
    %set(handles.checkbox30, 'Enable', 'on');
    %set(handles.checkbox28, 'ForegroundColor', 'black');
    %set(handles.checkbox28, 'BackgroundColor', [0.906 0.906 0.906]);
    datacursormode off;
    delete(findall(gca,'Type','hggroup','HandleVisibility','off'));
    if(get(handles.togglebutton5, 'Value') == 1.0)
        pan on;
    end
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Data cursor off');
    
end

% Preview is replotted only when the data cursor is selected "on".
% Otherwise deleting the datatip and turning off the data cursor mode is
% enough
% If the map is not replotted, the data tip string modifying function
% "data_cursor_mod()" in preview.m will not be called and the data cursor
% will only display the default values (x, y, level)
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0 && get(handles.checkbox28, 'Value') == 1)

    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    preview(plot_map, handles.figure1);
   
end


end


% --- Executes on button press in checkbox30.
% Rot angle & arrow checkbox
function checkbox30_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox30 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Checked:
if(get(handles.checkbox30, 'Value') == 1.0)
    % Handles variable:
    handles.fig_rot_angle = 1.0;
    % Menu item:
    set(handles.Untitled_110, 'Checked', 'on');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure rotate angle & arrow markers on');
else % Unchecked:
    % Handles variable:
    handles.fig_rot_angle = 0.0;    
    % Menu item:
    set(handles.Untitled_110, 'Checked', 'off');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure rotate angle & arrow markers off');
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Update handles structure
guidata(hObject, handles);

end

% --- Executes on selection change in popupmenu7.
function popupmenu7_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu7 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

set(hObject, 'BackgroundColor', 'w');

size_val = get(handles.popupmenu7, 'Value'); % Which size is selected

% Set related menu items:
if(size_val == 1)
    set(handles.Untitled_60, 'Checked', 'On');
    set(handles.Untitled_61, 'Checked', 'Off');
    set(handles.Untitled_62, 'Checked', 'Off');
    set(handles.Untitled_63, 'Checked', 'Off');
elseif(size_val == 2)
    set(handles.Untitled_60, 'Checked', 'Off');
    set(handles.Untitled_61, 'Checked', 'On');
    set(handles.Untitled_62, 'Checked', 'Off');
    set(handles.Untitled_63, 'Checked', 'Off');
elseif(size_val == 3)
    set(handles.Untitled_60, 'Checked', 'Off');
    set(handles.Untitled_61, 'Checked', 'Off');
    set(handles.Untitled_62, 'Checked', 'Off');
    set(handles.Untitled_63, 'Checked', 'On');
elseif(size_val == 4) % Notice: Menu items Untitled_62 and Untitled_63 are unintuitively placed: 63 is physically above 62. Thus the weird way of setting the values here (3 --> 63 on, 4 --> 62 on)
    set(handles.Untitled_60, 'Checked', 'Off');
    set(handles.Untitled_61, 'Checked', 'Off');
    set(handles.Untitled_62, 'Checked', 'On');
    set(handles.Untitled_63, 'Checked', 'Off');
else % This is here as a failsafe; the checkbox's value should always be 1, 2, 3 or 4
    set(handles.Untitled_60, 'Checked', 'On');
    set(handles.Untitled_61, 'Checked', 'Off');
    set(handles.Untitled_62, 'Checked', 'Off');
    set(handles.Untitled_63, 'Checked', 'Off');
end

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu7 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu7
end

% --- Executes during object creation, after setting all properties.
function popupmenu7_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu7 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end

% --- Executes on selection change in popupmenu6.
function popupmenu6_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns popupmenu6 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu6

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

set(hObject, 'BackgroundColor', 'w');

if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)

    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    preview(plot_map, handles.figure1);
   
end

end

% --- Executes during object creation, after setting all properties.
function popupmenu6_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


function edit10_Callback(hObject, eventdata, handles)
% hObject    handle to edit10 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit10 as text
%        str2double(get(hObject,'String')) returns contents of edit10 as a double

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

key = double(get(gcbf, 'CurrentCharacter'));
% if 'enter' is pressed in the edit box, the preview function is called
if key == 13
    if(isempty(get(handles.listbox1, 'String')) == 0)
        
        
    % Checking that the user has entered only numbers into the custom levels
    % fields:
    if(get(handles.checkbox31, 'Value') == 1.0)
        cmp1 = strcmp(num2str(str2num(get(handles.edit10, 'String'))), '');
        cmp2 = strcmp(num2str(str2num(get(handles.edit11, 'String'))), '');
        if(cmp1 == 1 || cmp2 == 1)
            % Notice; % Old self made notice window
            % Using warning window as with other warnings:
            msgbox('You have entered something else than numbers to the custom levels fields. Please use only numbers as custom levels.', 'Warning', 'warn', 'modal');
            figure1_ResizeFcn(hObject, eventdata, handles);
            return;
        elseif(str2num(get(handles.edit10, 'String'))) >= str2num(get(handles.edit11, 'String'))
            message = ['The minimum custom levels value (', get(handles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(handles.edit11, 'String'), ').'];
            msgbox(message, 'Warning', 'warn', 'modal');
            figure1_ResizeFcn(hObject, eventdata, handles);
            return;
        end
    end
    if(isempty(get(handles.listbox1, 'Value')) == 0)
        plottingList = get(handles.listbox1,'String');
        indexes = get(handles.listbox1, 'Value');
        plot_map = plottingList(indexes(1));
        preview(plot_map, handles.figure1);
    end
    end
end

end

% --- Executes during object creation, after setting all properties.
function edit10_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit10 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


function edit11_Callback(hObject, eventdata, handles)
% hObject    handle to edit11 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit11 as text
%        str2double(get(hObject,'String')) returns contents of edit11 as a double

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

key = double(get(gcbf, 'CurrentCharacter'));
% if 'enter' is pressed in the edit box, the preview function is called
if key == 13
    if(isempty(get(handles.listbox1, 'String')) == 0)
        
        % Checking that the user has entered only numbers into the custom levels
        % fields:
        if(get(handles.checkbox31, 'Value') == 1.0)
            cmp1 = strcmp(num2str(str2num(get(handles.edit10, 'String'))), '');
            cmp2 = strcmp(num2str(str2num(get(handles.edit11, 'String'))), '');
            if(cmp1 == 1 || cmp2 == 1)
                % Notice; % Old self made notice window
                % Using warning window as with other warnings:
                msgbox('You have entered something else than numbers to the custom levels fields. Please use only numbers as custom levels.', 'Warning', 'warn', 'modal');
                figure1_ResizeFcn(hObject, eventdata, handles);
                return;
            elseif(str2num(get(handles.edit10, 'String'))) >= str2num(get(handles.edit11, 'String'))
                message = ['The minimum custom levels value (', get(handles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(handles.edit11, 'String'), ').'];
                msgbox(message, 'Warning', 'warn', 'modal');
                figure1_ResizeFcn(hObject, eventdata, handles);
            return;
            end
        end
        if(isempty(get(handles.listbox1, 'Value')) == 0)
        plottingList = get(handles.listbox1,'String');
        indexes = get(handles.listbox1, 'Value');
        plot_map = plottingList(indexes(1));
        preview(plot_map, handles.figure1);
        end
    end
end

end

% --- Executes during object creation, after setting all properties.
function edit11_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit11 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end

% --- Executes on button press in checkbox31.
function checkbox31_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox31 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Hint: get(hObject,'Value') returns toggle state of checkbox31

if(get(handles.checkbox31, 'Value') == 1.0)
set(handles.edit10, 'Enable', 'on');
set(handles.edit11, 'Enable', 'on');
set(handles.text46, 'Enable', 'on');
set(handles.text47, 'Enable', 'on');
set(handles.text53, 'Enable', 'on');
set(handles.text54, 'Enable', 'on');

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'Custom min/max levels in use');

end

if((get(handles.checkbox31, 'Value') == 0.0))
set(handles.edit10, 'Enable', 'off');
set(handles.edit11, 'Enable', 'off');
set(handles.text46, 'Enable', 'off');
set(handles.text47, 'Enable', 'off');
set(handles.text53, 'Enable', 'off');
set(handles.text54, 'Enable', 'off');

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'Automatic min/max levels in use');

end

if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)

    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    preview(plot_map, handles.figure1);
   
end

end


% --- Executes on button press in pushbutton27.
% The Reset to default button:
function pushbutton27_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton27 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.checkbox20, 'Value', 1);
set(handles.checkbox26, 'Value', 0);
set(handles.checkbox27, 'Value', 0);
set(handles.togglebutton8, 'Value', 0);
set(handles.Untitled_122, 'Checked', 'off');
set(handles.checkbox28, 'Value', 0);
set(handles.checkbox30, 'Value', 1);
set(handles.Untitled_110, 'Checked', 'on'); % Corresponding menu item to checkbox30
set(handles.checkbox31, 'Value', 1.0); % Changed 2012-03-30
set(handles.checkbox39, 'Value', 0); % Simple figure
set(handles.checkbox56, 'Value', 0); % Dynamic fov
set(handles.checkbox26, 'Enable', 'on');
set(handles.Untitled_111, 'Enable', 'on'); % Corresponding menu item to checkbox26
set(handles.checkbox30, 'Enable', 'on');
set(handles.Untitled_110, 'Enable', 'on'); % Corresponding menu item to checkbox30

% Colormap: "MRO red - 512 colors":
set(handles.popupmenu6, 'Value', 30);
set(handles.popupmenu6, 'Enable', 'on');

set(handles.popupmenu6, 'BackgroundColor', 'w');
%set(handles.popupmenu7, 'Value', 1);
%set(handles.popupmenu7, 'BackgroundColor', 'w');
set(handles.checkbox28, 'ForegroundColor', 'black');
set(handles.checkbox28, 'BackgroundColor', [0.906 0.906 0.906]);

set(handles.edit10, 'Enable', 'on'); % Changed 2012-03-30
set(handles.edit11, 'Enable', 'on'); % Changed 2012-03-30
set(handles.edit10, 'String', '90'); % Changed 2012-03-30
set(handles.edit11, 'String', '112'); % Changed 2012-03-30
set(handles.edit12, 'String', '50'); % Changed 2012-03-30
set(handles.edit13, 'String', '0.5');
set(handles.edit13, 'Enable', 'off');
set(handles.edit15, 'String', '');
set(handles.edit15, 'Enable', 'off');
set(handles.edit16, 'String', '');
set(handles.edit16, 'Enable', 'off');
set(handles.text46, 'Enable', 'on'); % Changed 2012-03-30
set(handles.text47, 'Enable', 'on'); % Changed 2012-03-30
set(handles.text53, 'Enable', 'on'); % Added 2012-03-30
set(handles.text54, 'Enable', 'on'); % Added 2012-03-30

set(handles.popupmenu9, 'Value', 1.0);
set(handles.checkbox35, 'Value', 0);
set(handles.checkbox35, 'Value', 1.0); % Changed 2012-03-30
set(handles.checkbox36, 'Enable', 'off');
set(handles.Untitled_58, 'Enable', 'off');
set(handles.checkbox36, 'Value', 0);
set(handles.Untitled_58, 'Checked', 'off');
set(handles.checkbox37, 'Value', 0);
set(handles.checkbox38, 'Value', 0);
set(handles.Untitled_57, 'Checked', 'off');
set(handles.togglebutton7, 'Value', 0);
solar_area_lines_str = [pwd, '/img/solar_area_lines_icon.png'];
solar_area_lines_img = imread(solar_area_lines_str);
set(handles.togglebutton7, 'cdata', solar_area_lines_img); % Reset button icon
set(handles.checkbox40, 'Value', 0);
set(handles.checkbox41, 'Value', 0);
set(handles.checkbox41, 'Enable', 'off');
set(handles.checkbox51, 'Enable', 'off'); % AR detect. treashold checkbox, text & edit box
set(handles.checkbox51, 'Value', 0.0);
set(handles.checkbox50, 'Enable', 'off'); % Mark NOAA AR's on map
set(handles.checkbox50, 'Value', 0.0);
set(handles.text88, 'Enable', 'off');
set(handles.edit17, 'Enable', 'off');
set(handles.edit17, 'String', '104');
set(handles.Untitled_55, 'Checked', 'off');
set(handles.Untitled_55, 'Enable', 'off');
set(handles.Untitled_135, 'Checked', 'off');
set(handles.Untitled_135, 'Enable', 'off');
set(handles.checkbox40, 'ForegroundColor', 'black');
set(handles.checkbox40, 'BackgroundColor', [0.906 0.906 0.906]);
set(handles.checkbox28, 'Enable', 'off');
% Command type checkboxes:
set(handles.radiobutton20, 'Value', 1.0); % Map-measure as default
set(handles.radiobutton21, 'Value', 0.0);
set(handles.radiobutton20, 'Enable', 'off');
set(handles.radiobutton21, 'Enable', 'off');

set(handles.togglebutton9, 'Value', 0);
set(handles.togglebutton11, 'Value', 0);
set(handles.togglebutton9, 'Enable', 'off');
set(handles.togglebutton11, 'Enable', 'off');
set(handles.pushbutton37, 'Enable', 'off');
set(handles.pushbutton36, 'Enable', 'off');
set(handles.pushbutton39, 'Enable', 'off');
set(handles.listbox6, 'Enable', 'off');
set(handles.popupmenu6, 'Enable', 'on');
%set(handles.popupmenu7, 'Enable', 'on');
set(handles.Untitled_59, 'Enable', 'on'); % Figure size menu items:
% set(handles.Untitled_60, 'Enable', 'on');
% set(handles.Untitled_61, 'Enable', 'on');
% set(handles.Untitled_62, 'Enable', 'on');
% set(handles.Untitled_63, 'Enable', 'on');
set(handles.Untitled_60, 'Checked', 'on');
set(handles.Untitled_61, 'Checked', 'off');
set(handles.Untitled_62, 'Checked', 'off');
set(handles.Untitled_63, 'Checked', 'off');

set(handles.edit12, 'Enable', 'on'); % Changed 2012-03-30

% Reset data scale menu items:
set(handles.lin_scale, 'Checked', 'on');
set(handles.log_scale, 'Checked', 'off');
% Reset the data scale radio buttons on the GUI:
set(handles.radiobutton10, 'Value', 1.0);
set(handles.radiobutton11, 'Value', 0.0);
% Reset the data units radio buttons on the GUI:
set(handles.radiobutton12, 'Value', 1.0);
set(handles.radiobutton13, 'Value', 0.0);
% Reset the data units menu items:
set(handles.Untitled_106, 'Checked', 'On');
set(handles.Untitled_107, 'Checked', 'Off');

% Reset map type menu items:
set(handles.filled_contours_no_contour_lies, 'Checked', 'On');
set(handles.filled_contours_with_contour_lines, 'Checked', 'Off');
set(handles.contour_lines_only, 'Checked', 'Off');

% Reset Separate figure menu items:
set(handles.Untitled_109, 'Checked', 'on');
set(handles.Untitled_110, 'Checked', 'on');
set(handles.Untitled_111, 'Checked', 'off');
set(handles.Untitled_112, 'Checked', 'off');

% Change the text of custom min/max levels checkbox (checkbox31) to
% % of QSL units:
set(handles.checkbox31, 'String', ' Custom min/max levels (% of QSL)');

% Set the texts that describe units for min/max units after the edit boxes:
set(handles.text52, 'String', '%'); % Custom level separation
set(handles.text54, 'String', '%'); % Min
set(handles.text53, 'String', '%'); % Max

% Reset zoom:
pushbutton30_Callback(hObject, eventdata, handles);
% If pan tool is on, turn it off:
if(get(handles.togglebutton5, 'Value') == 1.0)
    pan_tool_Callback(hObject, eventdata, handles);
end

if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)

    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));

    preview(plot_map, handles.figure1);
   
end

% Reset the custom level separation values:
handles.customSepQSL = '0.5';
handles.customSepOrig = '200';

% Reset the custom min/max values:
handles.customMinQSL = '90';
handles.customMinOrig = '-2500';
handles.customMaxQSL = '112';
handles.customMaxOrig = '35000';

% Reset separate figure settings variables:
handles.fig_size = 1.0;
handles.fig_beam_size = 1.0;
handles.fig_rot_angle = 1.0;
handles.fig_grid = 0.0;
handles.fig_simple = 0.0;

% Reset solar disk centering menu items & togglebutton
set(handles.Untitled_120, 'Checked', 'on');
set(handles.Untitled_121, 'Checked', 'off');
set(handles.togglebutton15, 'Value', 0.0);
% Set a custom image as an icon for the solar disk centering toggle button:
solar_center_tool_str = [pwd, '/img/center_tool_icon.png'];
solar_center_tool_button_img = imread(solar_center_tool_str);
set(handles.togglebutton15, 'cdata', solar_center_tool_button_img);


set(handles.pushbutton27, 'Enable', 'off'); % GUI Reset button enable off
set(handles.reset_settings, 'Enable', 'off'); % Menu Reset button enable off

% Set the status bar text to notify the user of the resetting:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)
set(handles.text39, 'String', 'Settings reset');

% Update handles structure
guidata(hObject, handles);

end


% --- Executes on key press with focus on edit10 and none of its controls.
function edit10_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to edit10 (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
end


% --- Executes on key press with focus on edit11 and none of its controls.
function edit11_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to edit11 (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
end


% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over popupmenu6.
function popupmenu6_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- Executes on selection change in popupmenu9.
% GUI popupmenu for map type (still in GUI but hidden and only used for reading values in functions)
function popupmenu9_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu9 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable Reset settings button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

set(handles.popupmenu6, 'Enable', 'on');
   
if(get(handles.popupmenu9, 'Value') == 3) % Contour lines only
    set(handles.popupmenu6, 'Enable', 'off');    
    set(handles.checkbox36, 'Enable', 'on');
    set(handles.Untitled_58, 'Enable', 'on');
    %set(handles.popupmenu7, 'Enable', 'off'); % Countour lines only mode has its own figure size (572x500px)
    set(handles.Untitled_59, 'Enable', 'off'); % Figure size menu items:
%     set(handles.Untitled_60, 'Enable', 'off');
%     set(handles.Untitled_61, 'Enable', 'off');
%     set(handles.Untitled_62, 'Enable', 'off');
%     set(handles.Untitled_63, 'Enable', 'off');
    set(handles.filled_contours_no_contour_lies, 'Checked', 'Off'); % Set corresponding menu items: (notice typo in element name :S)
    set(handles.filled_contours_with_contour_lines, 'Checked', 'Off');
    set(handles.contour_lines_only, 'Checked', 'On');
else
    set(handles.checkbox36, 'Enable', 'off');
    set(handles.Untitled_58, 'Enable', 'off');
    %set(handles.popupmenu7, 'Enable', 'on');
    set(handles.Untitled_59, 'Enable', 'on'); % Figure size menu items:
%     set(handles.Untitled_60, 'Enable', 'on');
%     set(handles.Untitled_61, 'Enable', 'on');
%     set(handles.Untitled_62, 'Enable', 'on');
%     set(handles.Untitled_63, 'Enable', 'on');
    
    % Set corresponding menu items: (notice typo in element name :S)
    if(get(handles.popupmenu9, 'Value') == 1)
        set(handles.filled_contours_no_contour_lies, 'Checked', 'On');
        set(handles.filled_contours_with_contour_lines, 'Checked', 'Off');
        set(handles.contour_lines_only, 'Checked', 'Off');
    elseif(get(handles.popupmenu9, 'Value') == 2)
        set(handles.filled_contours_no_contour_lies, 'Checked', 'Off');
        set(handles.filled_contours_with_contour_lines, 'Checked', 'On');
        set(handles.contour_lines_only, 'Checked', 'Off');
    end
    
    
end

% Call Preview function:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end

% --- Executes during object creation, after setting all properties.
function popupmenu9_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu9 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end



function edit12_Callback(hObject, eventdata, handles)
% hObject    handle to edit12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

key = double(get(gcbf, 'CurrentCharacter'));
% if 'enter' is pressed in the edit box, the preview function is called
if key == 13
    if(isempty(get(handles.listbox1, 'String')) == 0)
        
        
    % Checking that the user has entered only numbers into the custom levels
    % fields:
    if(get(handles.popupmenu9, 'Value') == 3.0)
        cmp1 = strcmp(num2str(str2num(get(handles.edit12, 'String'))), '');
        if(cmp1 == 1)
            msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
            figure1_ResizeFcn(hObject, eventdata, handles);
            return;
        end
    end
    if(isempty(get(handles.listbox1, 'Value')) == 0)
        plottingList = get(handles.listbox1,'String');
        indexes = get(handles.listbox1, 'Value');
        plot_map = plottingList(indexes(1));
        preview(plot_map, handles.figure1);
    end
    end
end

end

% --- Executes during object creation, after setting all properties.
function edit12_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


% --- Executes on button press in checkbox35.
function checkbox35_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox35 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.checkbox35, 'Value') == 1.0)
    
    set(handles.edit12, 'Enable', 'on');
    
    set(handles.edit13, 'Enable', 'off');
    set(handles.text52, 'Enable', 'off');
    set(handles.checkbox37, 'Value', 0.0);
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Custom number of levels in use');
    
else
    
    set(handles.edit12, 'Enable', 'off');
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Automatic number of levels in use (default)');

end

if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end


% --- Executes on button press in checkbox36.
% Contour line annotations
function checkbox36_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox36 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.checkbox36, 'Value') == 1)
    set(handles.Untitled_58, 'Checked', 'on'); % Corresponding menu item
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Contour line annotations on');
else
    set(handles.Untitled_58, 'Checked', 'off'); % Corresponding menu item
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Contour line annotations off');
end

% Call Preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end


% --- Executes on button press in checkbox37.
function checkbox37_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox37 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Turn the cutsom numer of levels-checkbox and textbox (edit) off when
% custom level separation is enabled:
if(get(handles.checkbox37, 'Value')  == 1.0)
    
    set(handles.edit12, 'Enable', 'off');
    set(handles.checkbox35, 'Value', 0.0);
    
    set(handles.edit13, 'Enable', 'on');
    set(handles.text52, 'Enable', 'on');
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Custom level separation in use');
    
elseif(get(handles.checkbox37, 'Value')  == 0.0)
    
    set(handles.checkbox35, 'Enable', 'on');
    set(handles.edit13, 'Enable', 'off');
    set(handles.text52, 'Enable', 'off');
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Automatic level separation in use (default)');
    
end

if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

% Hint: get(hObject,'Value') returns toggle state of checkbox37
end


function edit13_Callback(hObject, eventdata, handles)
% hObject    handle to edit13 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


key = double(get(gcbf, 'CurrentCharacter'));
% if 'enter' is pressed in the edit box, the preview function is called
if key == 13
    if(isempty(get(handles.listbox1, 'String')) == 0)
        
        
    % Checking that the user has entered only numbers into the field:
    
    cmp1 = strcmp(num2str(str2num(get(handles.edit13, 'String'))), '');
    if(cmp1 == 1)
        msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
        return;
    elseif(str2num(get(handles.edit13, 'String')) <= 0)
        msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
        figure1_ResizeFcn(hObject, eventdata, handles);
        return;        
    end

    if(isempty(get(handles.listbox1, 'Value')) == 0)
        plottingList = get(handles.listbox1,'String');
        indexes = get(handles.listbox1, 'Value');
        plot_map = plottingList(indexes(1));
        preview(plot_map, handles.figure1);
    end
    end
end

end

% --- Executes during object creation, after setting all properties.
function edit13_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit13 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


% --- Executes on button press in checkbox38.
% Solar grid & circle fit analysis checkbox
function checkbox38_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox38 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.checkbox38, 'Value') == 1)
    set(handles.checkbox41, 'Enable', 'On');
    set(handles.checkbox50, 'Enable', 'On');
    if(get(handles.checkbox41, 'Value') == 1.0)
        set(handles.checkbox51, 'Enable', 'On'); % AR detect. treshold checkbox, text & edit box
        if(get(handles.checkbox51, 'Value') == 1.0)
            set(handles.text88, 'Enable', 'On');
            set(handles.edit17, 'Enable', 'On');
        end
    end
    set(handles.Untitled_55, 'Enable', 'On'); % Mark active areas on map menu item enable
    set(handles.Untitled_135, 'Enable', 'On'); % Mark NOAA AR's on map menu item enable
    set(handles.togglebutton7, 'Value', 1.0); % The togglebutton that does the same thing as checkbox38
    set(handles.Untitled_57, 'Checked', 'on'); % Corresponding menu item
    solar_area_lines_str = [pwd, '/img/solar_area_lines_icon_down.png'];
    solar_area_lines_img = imread(solar_area_lines_str);
    set(handles.togglebutton7, 'cdata', solar_area_lines_img);
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Solar grid & circle fit anaysis on');
    
else
    set(handles.checkbox41, 'Enable', 'Off');
    set(handles.checkbox51, 'Enable', 'Off'); % AR detect. treashold checkbox, text & edit box
    % Mark NOAA AR's on map: disabled, off:
    set(handles.checkbox50, 'Enable', 'Off')
    set(handles.checkbox50, 'Value', 0.0);
    set(handles.text88, 'Enable', 'Off');
    set(handles.edit17, 'Enable', 'Off');
    set(handles.Untitled_55, 'Enable', 'Off'); % Mark active areas on map menu item disable
    set(handles.Untitled_55, 'Checked', 'Off');
    set(handles.Untitled_135, 'Enable', 'Off'); % Mark NOAA AR's on map menu item disable
    set(handles.Untitled_135, 'Checked', 'Off');    
    set(handles.checkbox41, 'Value', 0);
    set(handles.togglebutton7, 'Value', 0.0); % The togglebutton that does the same thing as this checkbox
    set(handles.Untitled_57, 'Checked', 'off'); % The menu item
    solar_area_lines_str = [pwd, '/img/solar_area_lines_icon.png'];
    solar_area_lines_img = imread(solar_area_lines_str);
    set(handles.togglebutton7, 'cdata', solar_area_lines_img);
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Solar grid & circle fit anaysis off');
    
end

if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end


% --- Executes on button press in pushbutton28.
% Zoom + button
function pushbutton28_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton28 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Notice: there is also a premade zoom provided by matlab:
% zoom(0.8) zooms out
% zoom(1/0.8) zooms in

axes(handles.axes2)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.togglebutton5, 'Value') == 0)
    pan(handles.axes2);
    % Pan tool button icon change
    pan_tool_down_str = [pwd, '/img/pan_tool_icon_down.png'];
    pan_tool_button_img = imread(pan_tool_down_str);
    set(handles.togglebutton5, 'cdata', pan_tool_button_img);
    % Menu item to checked:
    set(handles.pan_tool, 'Checked', 'on');
    
    set(handles.togglebutton5, 'Value', 1);
    set(handles.togglebutton5, 'Background', [1 0.69 0.39]);
end
xlim_new = xlim * 0.8;
ylim_new = ylim * 0.8;
axis([xlim_new ylim_new]),

% Activate the "Reset zoom" button
set(handles.pushbutton30, 'Enable', 'on');
set(handles.reset_zoom, 'Enable', 'on');

% New zoom settings are written to hidden text fields in the gui:
set(handles.text57, 'String', num2str(abs((ylim_new(2) - ylim_new(1))/2)));
set(handles.text60, 'String', num2str(abs((xlim_new(2) - xlim_new(1))/2)));

end

% --- Executes on button press in pushbutton29.
% Zoom - button
function pushbutton29_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton29 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

axes(handles.axes2)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.togglebutton5, 'Value') == 0)
    pan(handles.axes2);
    % Pan tool button icon change
    pan_tool_down_str = [pwd, '/img/pan_tool_icon_down.png'];
    pan_tool_button_img = imread(pan_tool_down_str);
    set(handles.togglebutton5, 'cdata', pan_tool_button_img);
    % Menu item to checked:
    set(handles.pan_tool, 'Checked', 'on');
    
    set(handles.togglebutton5, 'Value', 1);
    set(handles.togglebutton5, 'Background', [1 0.69 0.39]);
end
xlim_new = xlim / 0.8;
ylim_new = ylim / 0.8;
axis([xlim_new ylim_new]),

% Activate the "Reset zoom" button
set(handles.pushbutton30, 'Enable', 'on');
set(handles.reset_zoom, 'Enable', 'on');

% New zoom settings are written to hidden text fields in the gui:
set(handles.text57, 'String', num2str(abs((ylim_new(2) - ylim_new(1))/2)));
set(handles.text60, 'String', num2str(abs((xlim_new(2) - xlim_new(1))/2)));

end

% --- Executes on button press in pushbutton30.
% Zoom 100 % Button
function pushbutton30_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton30 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

axes(handles.axes2)
axis([-23 23 -19 19]),

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Deactivate the "Reset zoom" button
set(handles.pushbutton30, 'Enable', 'off');
set(handles.reset_zoom, 'Enable', 'off');

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'Preview zoom reset');

% New zoom settings are written to hidden text fields in the gui:
set(handles.text57, 'String', 19);
set(handles.text60, 'String', 23);

end

% --- Executes on button press in togglebutton5.
% Pan tool toggle
function togglebutton5_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

axes(handles.axes2)
%pan(handles.axes2);

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.togglebutton5, 'Value') == 0)
   
    pan_tool_str = [pwd, '/img/pan_tool_icon.png'];
    pan_tool_button_img = imread(pan_tool_str);
    set(handles.togglebutton5, 'cdata', pan_tool_button_img);
    set(handles.pan_tool, 'Checked', 'off');
    
%     if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
%         plottingList = get(handles.listbox1,'String');
%         indexes = get(handles.listbox1, 'Value');
%         plot_map = plottingList(indexes(1));
%         preview(plot_map, handles.figure1);
%     end

    % Reading x_fov and y_fov from hidden text fields in the gui:
    %x_fov = str2num(get(handles.text60, 'String'));
    %y_fov = str2num(get(handles.text57, 'String'));
    
    %axis([-23 23 -16.8 16.8]),
    %axis([-x_fov x_fov -y_fov y_fov]),
    pan off;
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Pan tool off');    
    
    if(get(handles.checkbox40, 'Value') == 1.0) % Data cursor & rot disable on
        if(get(handles.checkbox28, 'Value') == 1.0)
            datacursormode on,
            cursor = datacursormode(gcf);
            set(cursor, 'SnapToDataVertex','off');
            set(gca,'Layer','top'),
        end
    end
    
else
    pan_tool_down_str = [pwd, '/img/pan_tool_icon_down.png'];
    pan_tool_button_img = imread(pan_tool_down_str);
    set(handles.togglebutton5, 'cdata', pan_tool_button_img);
    set(handles.pan_tool, 'Checked', 'on');
    pan_obj = pan;
    pan on;
    % Define a custom function that is run after the user has panned the axis:
    set(pan_obj,'ActionPostCallback',@pan_post_callback);
    set(pan_obj,'Enable','on');

    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Pan tool on');
end

% Function that is run after the user has panned the axis with the pan tool:
function pan_post_callback(obj,evd)

    % Write field of view into handles variables:
    handles.xlims = get(evd.Axes,'XLim');
    handles.ylims = get(evd.Axes,'YLim');
    
    %setX = handles.xlims
    %setY = handles.ylims
    
    guidata(hObject, handles); % Update the handles structure
    
    %set(handles.edit57, 'String', num2str(YLims(1)));
    %set(handles.edit58, 'String', num2str(YLims(2)));    
    %set(handles.edit59, 'String', num2str(XLims(1)));
    %set(handles.edit60, 'String', num2str(XLims(2)));

end

end

% --------------------------------------------------------------------
function File_Callback(hObject, eventdata, handles)
% hObject    handle to File (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

end
% --------------------------------------------------------------------
function open_files_Callback(hObject, eventdata, handles)
% hObject    handle to open_files (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
add_files_button_Callback(hObject, eventdata, handles);
end

% --------------------------------------------------------------------
function Save_Callback(hObject, eventdata, handles)
% hObject    handle to Save (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
plot_all_button_Callback(hObject, eventdata, handles)
end

% --------------------------------------------------------------------
function Exit_Callback(hObject, eventdata, handles)
% hObject    handle to Exit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
exit_button_Callback(hObject, eventdata, handles)
end


% --------------------------------------------------------------------
function Edit_Callback(hObject, eventdata, handles)
% hObject    handle to Edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
function remove_selected_Callback(hObject, eventdata, handles)
% hObject    handle to remove_selected (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
remove_file_button_Callback(hObject, eventdata, handles)
end

% --------------------------------------------------------------------
function Clear_list_Callback(hObject, eventdata, handles)
% hObject    handle to Clear_list (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
clear_list_button_Callback(hObject, eventdata, handles)
end


% --------------------------------------------------------------------
function plot_selected_Callback(hObject, eventdata, handles)
% hObject    handle to plot_selected (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
plot_single_button_Callback(hObject, eventdata, handles)
end


% --------------------------------------------------------------------
function help_Callback(hObject, eventdata, handles)
% hObject    handle to help (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function info_on_sunmap_Callback(hObject, eventdata, handles)
% hObject    handle to info_on_sunmap (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
function view_Callback(hObject, eventdata, handles)
% hObject    handle to view (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function tools_Callback(hObject, eventdata, handles)
% hObject    handle to tools (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_9_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_9 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function custom_nro_of_levels_Callback(hObject, eventdata, handles)
% hObject    handle to custom_nro_of_levels (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function custom_level_separation_Callback(hObject, eventdata, handles)
% hObject    handle to custom_level_separation (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function custom_min_max_levels_Callback(hObject, eventdata, handles)
% hObject    handle to custom_min_max_levels (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function colormap_Callback(hObject, eventdata, handles)
% hObject    handle to colormap (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function pan_tool_Callback(hObject, eventdata, handles)
% hObject    handle to pan_tool (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

axes(handles.axes2)
%pan(handles.axes2);

if(get(handles.togglebutton5, 'Value') == 1)
   
    set(handles.togglebutton5, 'Value', 0.0); % Button to down position
    pan_tool_str = [pwd, '/img/pan_tool_icon.png'];
    pan_tool_button_img = imread(pan_tool_str);
    set(handles.togglebutton5, 'cdata', pan_tool_button_img);
    set(handles.pan_tool, 'Checked', 'off');
    
%     if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
%         plottingList = get(handles.listbox1,'String');
%         indexes = get(handles.listbox1, 'Value');
%         plot_map = plottingList(indexes(1));
%         preview(plot_map, handles.figure1);
%     end

    % Reading x_fov and y_fov from hidden text fields in the gui:
    x_fov = str2num(get(handles.text60, 'String'));
    y_fov = str2num(get(handles.text57, 'String'));
    
    %axis([-23 23 -16.8 16.8]),
    axis([-x_fov x_fov -y_fov y_fov]),
    pan off;
    
    if(get(handles.checkbox40, 'Value') == 1.0) % Data cursor & rot disable on
        if(get(handles.checkbox28, 'Value') == 1.0)
            datacursormode on,
            cursor = datacursormode(gcf);
            set(cursor, 'SnapToDataVertex','off');
            set(gca,'Layer','top'),
        end
    end
    
else
    set(handles.togglebutton5, 'Value', 1.0); % Button to up position
    pan_tool_down_str = [pwd, '/img/pan_tool_icon_down.png'];
    pan_tool_button_img = imread(pan_tool_down_str);
    set(handles.togglebutton5, 'cdata', pan_tool_button_img);
    set(handles.pan_tool, 'Checked', 'on');
    pan on;
end
end

% --------------------------------------------------------------------
function zoom_in_Callback(hObject, eventdata, handles)
% hObject    handle to zoom_in (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
pushbutton28_Callback(hObject, eventdata, handles);
end

% --------------------------------------------------------------------
function zoom_out_Callback(hObject, eventdata, handles)
% hObject    handle to zoom_out (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
pushbutton29_Callback(hObject, eventdata, handles);
end

% --------------------------------------------------------------------
function reset_zoom_Callback(hObject, eventdata, handles)
% hObject    handle to reset_zoom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
pushbutton30_Callback(hObject, eventdata, handles);
end

% --------------------------------------------------------------------
function Untitled_18_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_18 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_19_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_19 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_20_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_20 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_21_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_21 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_22_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_22 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_23_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_23 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_24_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_24 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_25_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_25 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_26_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_26 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_27_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_27 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_28_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_28 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_29_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_29 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_30_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_30 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_31_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_31 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_32_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_32 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_33_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_33 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_34_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_34 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_35_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_35 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function filled_contours_no_contour_lies_Callback(hObject, eventdata, handles)
% hObject    handle to filled_contours_no_contour_lies (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Set the menu item check's:
set(handles.filled_contours_no_contour_lies, 'Checked', 'On');
set(handles.filled_contours_with_contour_lines, 'Checked', 'Off');
set(handles.contour_lines_only, 'Checked', 'Off');

set(handles.popupmenu9, 'Value', 1.0) % Set the corresponding popupmenu value (this is used for plotting!)

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set text color to green
set(handles.text39, 'String', 'Using: Filled contours (Default)');
if(ispc)
    set(handles.text39, 'FontSize', 8);
else
    set(handles.text39, 'FontSize', 10);
end

set(handles.popupmenu6, 'Enable', 'on');
set(handles.checkbox36, 'Enable', 'off');
set(handles.Untitled_58, 'Enable', 'off');
%set(handles.popupmenu7, 'Enable', 'on');
set(handles.Untitled_59, 'Enable', 'on'); % Figure size menu items:
% set(handles.Untitled_60, 'Enable', 'on');
% set(handles.Untitled_61, 'Enable', 'on');
% set(handles.Untitled_62, 'Enable', 'on');
% set(handles.Untitled_63, 'Enable', 'on');

% Call Preview function:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end

% --------------------------------------------------------------------
function filled_contours_with_contour_lines_Callback(hObject, eventdata, handles)
% hObject    handle to filled_contours_with_contour_lines (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Set the menu item check's:
set(handles.filled_contours_no_contour_lies, 'Checked', 'Off');
set(handles.filled_contours_with_contour_lines, 'Checked', 'On');
set(handles.contour_lines_only, 'Checked', 'Off');

set(handles.popupmenu9, 'Value', 2.0) % Set the corresponding popupmenu value

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set text color to green
set(handles.text39, 'String', 'Using: Filled contours with contour lines');

set(handles.popupmenu6, 'Enable', 'on');
set(handles.checkbox36, 'Enable', 'off');
set(handles.Untitled_58, 'Enable', 'off');
%set(handles.popupmenu7, 'Enable', 'on');
set(handles.Untitled_59, 'Enable', 'on'); % Figure size menu items:
% set(handles.Untitled_60, 'Enable', 'on');
% set(handles.Untitled_61, 'Enable', 'on');
% set(handles.Untitled_62, 'Enable', 'on');
% set(handles.Untitled_63, 'Enable', 'on');

% Call review function:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end

% --------------------------------------------------------------------
function contour_lines_only_Callback(hObject, eventdata, handles)
% hObject    handle to contour_lines_only (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Set the menu item check's:
set(handles.filled_contours_no_contour_lies, 'Checked', 'Off');
set(handles.filled_contours_with_contour_lines, 'Checked', 'Off');
set(handles.contour_lines_only, 'Checked', 'On');

set(handles.popupmenu9, 'Value', 3.0) % Set the corresponding popupmenu value

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set text color to green
set(handles.text39, 'String', 'Using: Contour lines only');

set(handles.popupmenu6, 'Enable', 'off');
set(handles.checkbox36, 'Enable', 'on');
set(handles.Untitled_58, 'Enable', 'on');
%set(handles.popupmenu7, 'Enable', 'off'); % Countour lines only mode has its own figure size (572x500px)
set(handles.Untitled_59, 'Enable', 'off'); % Figure size menu items:
% set(handles.Untitled_60, 'Enable', 'off');
% set(handles.Untitled_61, 'Enable', 'off');
% set(handles.Untitled_62, 'Enable', 'off');
% set(handles.Untitled_63, 'Enable', 'off');

% Call Preview function:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end


% --------------------------------------------------------------------
function Untitled_36_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_36 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_37_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_37 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_38_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_38 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_39_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_39 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_40_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_40 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_41_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_41 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_42_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_42 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_43_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_43 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_44_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_44 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_45_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_45 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_46_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_46 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_47_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_47 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_48_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_48 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
function Untitled_49_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_49 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
function Untitled_54_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_54 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_53_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_53 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_59_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_59 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function reset_settings_Callback(hObject, eventdata, handles)
% hObject    handle to reset_settings (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
pushbutton27_Callback(hObject, eventdata, handles);
set(handles.reset_settings, 'Enable', 'off'); % Menu Reset button enable off
%set(handles.pushbutton27, 'Enable', 'off'); % GUI Reset button enable off
end

% --------------------------------------------------------------------
% Figure size: 674x500
function Untitled_60_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_60 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

%set(hObject, 'BackgroundColor', 'w');

%set(handles.popupmenu7, 'Value', 1.0);
handles.fig_size = 1.0;
set(handles.Untitled_60, 'Checked', 'On');
set(handles.Untitled_61, 'Checked', 'Off');
set(handles.Untitled_62, 'Checked', 'Off');
set(handles.Untitled_63, 'Checked', 'Off');

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set text color to green
set(handles.text39, 'String', 'Using size 674x500 pixels for plotting figures');

guidata(hObject, handles);

end

% --------------------------------------------------------------------
% Figure size: 560x420
function Untitled_61_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_61 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

%set(hObject, 'BackgroundColor', 'w');

%set(handles.popupmenu7, 'Value', 2.0);
handles.fig_size = 2.0;
set(handles.Untitled_60, 'Checked', 'Off');
set(handles.Untitled_61, 'Checked', 'On');
set(handles.Untitled_62, 'Checked', 'Off');
set(handles.Untitled_63, 'Checked', 'Off');

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set text color to green
set(handles.text39, 'String', 'Using size 560x420 pixels for plotting figures');

guidata(hObject, handles);

end

% --------------------------------------------------------------------
% Figure size: 900x675
function Untitled_62_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_62 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

%set(hObject, 'BackgroundColor', 'w');

set(handles.popupmenu7, 'Value', 4.0);
handles.fig_size = 4.0;
set(handles.Untitled_60, 'Checked', 'Off');
set(handles.Untitled_61, 'Checked', 'Off');
set(handles.Untitled_62, 'Checked', 'On');
set(handles.Untitled_63, 'Checked', 'Off');

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set text color to green
set(handles.text39, 'String', 'Using size 900x675 pixels for plotting figures');

guidata(hObject, handles);

end

% --------------------------------------------------------------------
% Figure size: 800x600
function Untitled_63_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_63 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

%set(hObject, 'BackgroundColor', 'w');

%set(handles.popupmenu7, 'Value', 3.0);
handles.fig_size = 3.0;
set(handles.Untitled_60, 'Checked', 'Off');
set(handles.Untitled_61, 'Checked', 'Off');
set(handles.Untitled_62, 'Checked', 'Off');
set(handles.Untitled_63, 'Checked', 'On');

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set text color to green
set(handles.text39, 'String', 'Using size 800x600 pixels for plotting figures');

guidata(hObject, handles);

end


% --------------------------------------------------------------------
function Untitled_50_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_50 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_51_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_51 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_52_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_52 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
% Find & mark AR's on map menu item
function Untitled_55_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_55 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.Untitled_55, 'Checked'), 'off'))
    set(handles.Untitled_55, 'Checked', 'on');
    set(handles.checkbox41, 'Value', 1.0);
    checkbox41_Callback(hObject, eventdata, handles);
elseif(strcmp(get(handles.Untitled_55, 'Checked'), 'on'))
    set(handles.Untitled_55, 'Checked', 'off');
    set(handles.checkbox41, 'Value', 0.0);
    checkbox41_Callback(hObject, eventdata, handles);
end

end

% --------------------------------------------------------------------
function Untitled_56_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_56 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
% Solar grid & circle fit analysis
function Untitled_57_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_57 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.Untitled_57, 'Checked'), 'off'))
    set(handles.Untitled_57, 'Checked', 'on');
    set(handles.checkbox38, 'Value', 1.0);
    checkbox38_Callback(hObject, eventdata, handles);
elseif(strcmp(get(handles.Untitled_57, 'Checked'), 'on'))
    set(handles.Untitled_57, 'Checked', 'off');
    set(handles.checkbox38, 'Value', 0.0);
    checkbox38_Callback(hObject, eventdata, handles);
end

end

% --------------------------------------------------------------------
function Untitled_58_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_58 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.Untitled_58, 'Checked'), 'off'))
    set(handles.Untitled_58, 'Checked', 'on');
    set(handles.checkbox36, 'Value', 1.0);
    checkbox36_Callback(hObject, eventdata, handles);
elseif(strcmp(get(handles.Untitled_58, 'Checked'), 'on'))
    set(handles.Untitled_58, 'Checked', 'off');
    set(handles.checkbox36, 'Value', 0.0);
    checkbox36_Callback(hObject, eventdata, handles);
end

end


% % --- Executes on slider movement.
% function slider4_Callback(hObject, eventdata, handles)
% % hObject    handle to slider4 (see GCBO)
% % eventdata  reserved - to be defined in a future version of MATLAB
% % handles    structure with handles and user data (see GUIDATA)
% 
% % Hints: get(hObject,'Value') returns position of slider
% %        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
% 
%     minValueSlider = get(handles.slider4, 'Value');
%     set(handles.edit10, 'String', minValueSlider);
% 
%     if(isempty(get(handles.listbox1, 'String')) == 0)
%         
%         
%     % Checking that the user has entered only numbers into the custom levels
%     % fields:
%     if(get(handles.checkbox31, 'Value') == 1.0)
%         cmp1 = strcmp(num2str(str2num(get(handles.edit10, 'String'))), '');
%         cmp2 = strcmp(num2str(str2num(get(handles.edit11, 'String'))), '');
%         if(cmp1 == 1 || cmp2 == 1)
%             Notice;
%             return;
%         elseif(str2num(get(handles.edit10, 'String'))) >= str2num(get(handles.edit11, 'String'))
%             message = ['The minimum custom levels value (', get(handles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(handles.edit11, 'String'), ').'];
%             msgbox(message, 'Warning', 'warn', 'modal');
%             return;
%         end
%     end
%     if(isempty(get(handles.listbox1, 'Value')) == 0)
%         plottingList = get(handles.listbox1,'String');
%         indexes = get(handles.listbox1, 'Value');
%         plot_map = plottingList(indexes(1));
%         preview(plot_map, handles.figure1);
%     end
%     end
% 
% end

% function figure_WindowButtonMotionFcn(hObject, eventdata, handles)
% 1+1
% guiCurrentObject = gco;
% if(~isempty(guiCurrentObject))
% 
%     if(guiCurrentObject == handles.slider4)
%         minValueSlider = get(handles.slider4, 'Value');
%         set(handles.edit10, 'String', minValueSlider);
%     end
%     
% end
% 
% end

% % --- Executes during object creation, after setting all properties.
% function slider4_CreateFcn(hObject, eventdata, handles)
% % hObject    handle to slider4 (see GCBO)
% % eventdata  reserved - to be defined in a future version of MATLAB
% % handles    empty - handles not created until after all CreateFcns called
% 
% % Hint: slider controls usually have a light gray background.
% if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
%     set(hObject,'BackgroundColor',[.9 .9 .9]);
% end
% end


% --- Executes on button press in togglebutton7.
function togglebutton7_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton7 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.checkbox38, 'Value') == 0)
    set(handles.checkbox38, 'Value', 1.0); % Checkbox in GUI
    set(handles.Untitled_57, 'Checked', 'on'); % Menu item
    set(handles.Untitled_55, 'Enable', 'on'); % Enable on for menu item for marking active areas on map
    set(handles.Untitled_135, 'Enable', 'on'); % Enable on for menu item for marking NOAA AR's on map
    solar_area_lines_str = [pwd, '/img/solar_area_lines_icon_down.png'];
    solar_area_lines_img = imread(solar_area_lines_str);
    set(handles.togglebutton7, 'cdata', solar_area_lines_img);
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Solar grid & circle fit anaysis on');
    
else
    set(handles.checkbox38, 'Value', 0.0); % Checkbox in GUI
    set(handles.Untitled_57, 'Checked', 'off'); % Menu item
    set(handles.Untitled_55, 'Enable', 'off'); % Enable off for menu item for marking active areas on map
    set(handles.Untitled_135, 'Enable', 'off'); % Enable off for menu item for marking NOAA AR's on map
    solar_area_lines_str = [pwd, '/img/solar_area_lines_icon.png'];
    solar_area_lines_img = imread(solar_area_lines_str);
    set(handles.togglebutton7, 'cdata', solar_area_lines_img);
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Solar grid & circle fit anaysis off');
    
end

% Call the related checkbox's callback --> everything done automatically the right
% way:
checkbox38_Callback(hObject, eventdata, handles);

% set(handles.pushbutton27, 'Enable', 'on');
% set(handles.reset_settings, 'Enable', 'on');
% 
% if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
%     plottingList = get(handles.listbox1,'String');
%     indexes = get(handles.listbox1, 'Value');
%     plot_map = plottingList(indexes(1));
%     preview(plot_map, handles.figure1);
% end
end

% --- Executes on button press in togglebutton8.
% QSL contour togglebutton
function togglebutton8_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton8 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Call the corresponding checkbox:
if(get(handles.togglebutton8, 'Value') == 1.0)
    set(handles.checkbox27, 'Value', 1.0);
    checkbox27_Callback(hObject, eventdata, handles);
elseif(get(handles.togglebutton8, 'Value') == 0.0)
    set(handles.checkbox27, 'Value', 0.0);
    checkbox27_Callback(hObject, eventdata, handles);
end

end


% --- Executes on button press in checkbox39.
% Simple figure main GUI checkbox:
function checkbox39_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox39 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Checked:
if(get(handles.checkbox39, 'Value') == 1.0)
    % Handles variable:
    handles.fig_simple = 1.0;    
    % Menu item:        
    set(handles.Untitled_112, 'Checked', 'on');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure simple settings on');        
else % Unchecked:
    % Handles variable:
    handles.fig_simple = 0.0;    
    % Menu item:        
    set(handles.Untitled_112, 'Checked', 'off');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure simple settings off');        
end

if(get(handles.checkbox39, 'Value') == 1.0)
    set(handles.checkbox26, 'Enable', 'off');
    set(handles.Untitled_111, 'Enable', 'off'); % Corresponding menu item to checkbox26
    %set(handles.popupmenu7, 'Enable', 'off'); % Figure size popupmenu
    set(handles.Untitled_59, 'Enable', 'off'); % Figure size menu items:
    set(handles.Untitled_60, 'Enable', 'off');
    set(handles.Untitled_61, 'Enable', 'off');
    set(handles.Untitled_62, 'Enable', 'off');
    set(handles.Untitled_63, 'Enable', 'off');
else
    set(handles.checkbox26, 'Enable', 'on');
    set(handles.Untitled_111, 'Enable', 'on'); % Corresponding menu item to checkbox26
    set(handles.popupmenu7, 'Enable', 'on');
    set(handles.Untitled_59, 'Enable', 'on'); % Figure size menu items:
    set(handles.Untitled_60, 'Enable', 'on');
    set(handles.Untitled_61, 'Enable', 'on');
    set(handles.Untitled_62, 'Enable', 'on');
    set(handles.Untitled_63, 'Enable', 'on');
end

set(handles.pushbutton27, 'Enable', 'on'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Update handles structure
guidata(hObject, handles);

end


% --------------------------------------------------------------------
function Untitled_65_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_65 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_66_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_66 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_67_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_67 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_68_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_68 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_69_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_69 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_70_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_70 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_71_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_71 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

end
% --------------------------------------------------------------------
function Untitled_72_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_72 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_73_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_73 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_74_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_74 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_75_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_75 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_76_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_76 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_77_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_77 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_78_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_78 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_79_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_79 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_80_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_80 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_81_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_81 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_82_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_82 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_83_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_83 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_84_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_84 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_85_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_85 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_86_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_86 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_87_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_87 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_88_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_88 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_89_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_89 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_90_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_90 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_91_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_91 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_92_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_92 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_93_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_93 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_94_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_94 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_95_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_95 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_96_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_96 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over edit10.
function edit10_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to edit10 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over edit11.
function edit11_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to edit11 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over edit12.
function edit12_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to edit12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over edit13.
function edit13_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to edit13 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
function Untitled_97_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_97 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

end


% --- Executes when uipanel28 is resized.
function uipanel28_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to uipanel28 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- Executes when uipanel43 is resized.
function uipanel43_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to uipanel43 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- Executes on button press in togglebutton9.
% Observe custom map button
% Partial map drawing & command generation:
function togglebutton9_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton9 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% There are no files in the listbox --> notification to user
if(isempty(get(handles.listbox1, 'String')) == 1)
    message = 'Open a solar map in the preview first.';
    msgbox(message, 'Solar map missing', 'warn', 'modal');
    set(handles.togglebutton9, 'Value', 0.0);
    return;
end

% Remove any existing rects (except for the solar grid rect tagged 'gridCircle'):
rects = findobj(gca,'type','rectangle');
for indexi = 1 : length(rects)
    if(strcmp( get(rects(indexi), 'Tag'),  'gridCircle') == 0)
        delete(rects(indexi))
    end
end

% Guidance text for the user:
set(handles.text65, 'String', 'Draw custom map area here');
% Set guidance text size to accommodate more text:
prevtextX = 4;
guiPosition = get(handles.figure1, 'position');
guiHeight = guiPosition(4);
plotboxHeight = guiHeight - 266;
prevtextY = plotboxHeight - 32;
prevtextWidth = 200;
prevtextHeight = 16;
set(handles.text65, 'position', [prevtextX, prevtextY, prevtextWidth, prevtextHeight]);
set(handles.text65, 'Visible', 'On');

% Blink the outer rim of the preview axes to guide the user in the right direction:
set(handles.uipanel37, 'BackgroundColor', [0.6 0.6 0.6]);
pause(0.2);
set(handles.uipanel37, 'BackgroundColor', [0.906 0.906 0.906]);
pause(0.2);
set(handles.uipanel37, 'BackgroundColor', [0.6 0.6 0.6]);
pause(0.2);
set(handles.uipanel37, 'BackgroundColor', [0.906 0.906 0.906]);
% pause(0.2);
% set(handles.uipanel37, 'BackgroundColor', [0.4 0.4 0.4]);
% pause(0.2);
% set(handles.uipanel37, 'BackgroundColor', [0.906 0.906 0.906]);

% Set the string for the list clearing button to "Clear":
set(handles.pushbutton39, 'String', 'Clear');

% Clear the edit box:
set(handles.edit15, 'String', '');

% If the "Observe custom map" button is not pressed, return:
if(get(handles.togglebutton9, 'Value') == 0.0)
    return;
end
    
set(handles.text74, 'String', '');
set(handles.text75, 'String', '');
set(handles.text76, 'String', '');
set(handles.text77, 'String', '');

rect = getrect(handles.axes2);
rectangle('position', rect);
%pause(0.5);

rectW_deg = rect(3) / 60;
rectW_deg = round(rectW_deg * 10000);
rectW_deg = rectW_deg / 10000;
rectXRad_deg = num2str(rectW_deg/2, 4); % Radius in x direction
rectW_deg = num2str(rectW_deg, 4);
rectH_deg = rect(4) / 60;
rectH_deg = round(rectH_deg * 10000);
rectH_deg = rectH_deg / 10000;
rectYRad_deg = num2str(rectH_deg/2, 4); % Radius in y direction
rectH_deg  = num2str(rectH_deg, 4);


wString = ['W: ', rectW_deg];
hString = ['H: ', rectH_deg];


% Dealing with partial source maps
% If these text fields area not empty the map currently shown is a
% partial map and the center coordinates are in the text fields,
% hCenterCoord (x direction) in text58 and vCenterCoord (y direction) in text59

if(isempty(get(handles.text58, 'String')) == 0 && isempty(get(handles.text59, 'String')) == 0)

    centerX = (rect(1) + 0.5*rect(3) - str2num(get(handles.text58, 'String'))) / 60; % Compensate with hCenterCoord
    centerX = round(centerX * 10000);
    centerX = - centerX / 10000; % The minus is added because sun-measure uses ra-dec coordinate system in which ra-value increases when we go left
    centerY = (rect(2) + 0.5*rect(4) - str2num(get(handles.text59, 'String'))) / 60; % Compensate with vCenterCoord
    centerY = round(centerY * 10000);
    centerY = centerY / 10000;
    %centerXString = ['Raoff: ', num2str(centerX, 4)];
    %centerYString = ['DeOff: ', num2str(centerY, 4)];

else % Source map is a normal full disk map

    % Try to figure out what offsets were used in the full sun
    % map on which the partial map area is drawn:
    try        
        % Read the file name of the currently shown map (in the preview)
        % from the listbox's value:        
        lb_value = get(handles.listbox1, 'Value');
        lb_string = get(handles.listbox1, 'String');
        infofile_string = lb_string{lb_value(1)};
        infoStruct = info_reader(infofile_string);
        raoff = infoStruct.raoff;
        decoff = infoStruct.decoff;
    catch info_err
        % If there is an error*, write default values in the raoff and
        % decoff variables:
        raoff = 0;
        decoff = 0;
        % *Errors are to be expected because this will not work with
        % SMART data. But because this is just a small auxiliary task,
        % it can be sacrificed to be done in this quick and dirty way
    end

    centerX = (rect(1) + 0.5*rect(3)) / 60;
    centerX = round(centerX * 10000);
    centerX = - centerX / 10000; % The minus is added because sun-measure uses ra-dec coordinate system in which ra-value increases when we go left
    centerX = centerX + raoff; % Compensate center coordinates with the right ascension offset of the map onto which the partial map rectangle was drawn
    centerY = (rect(2) + 0.5*rect(4)) / 60;
    centerY = round(centerY * 10000);
    centerY = centerY / 10000;
    centerY = centerY + decoff; % Compensate center coordinates with the declination offset of the map onto which the partial map rectangle was drawn
    %centerXString = ['Raoff: ', num2str(centerX, 4)];
    %centerYString = ['DeOff: ', num2str(centerY, 4)];

end

numSweeps = round(0.5 + (rect(4) * 2) / 2.4);
% Set minimum number of sweeps:
if numSweeps < 3
    numSweeps = 3;
end

numSweepsString = [' --numsweeps ', num2str(numSweeps)];
% Command type:
if(get(handles.radiobutton20, 'Value') == 0.0 && get(handles.radiobutton21, 'Value') == 1.0) % sun-measure, legacy
    smString = 'sun-measure --freq 37 --coordinate-system radec --maptype ramap --az-off 0.007 --el-off -0.0165 --no-standby '; % --direction forward --sweep-direction forward
    coordString = [' --ra-size ', rectW_deg, ' --dec-size ', rectH_deg, ' --ra-off ', num2str(centerX, 4), ' --dec-off ', num2str(centerY, 4)]; % ' --observer Sunmap_user'
    commandString = [smString, numSweepsString, coordString];
else % map-measure, default:
    smString = 'map-measure --az-off 0.000 --el-off -0.017 ';
    coordString = [' --xradius ', rectXRad_deg, ' --yradius ', rectYRad_deg, ' --ra-center ', num2str(centerX, 4), ' --dec-center ', num2str(centerY, 4)]; % ' --observer Sunmap_user'
    aux_str = '--speed 0.1 --sweepwidth 0.01';
    commandString = [smString, aux_str, coordString];
end
clipboard('copy', commandString); % Copy commandString to clipboard

set(handles.edit15, 'String', commandString);

% Add command to listbox
listboxString = get(handles.listbox6, 'String');
if(isempty(listboxString) == 0)
    listboxString = cellstr(listboxString);
    listboxString{end + 1} = commandString;
else
    listboxString = commandString;
end

set(handles.listbox6, 'String', listboxString);
set(handles.listbox6, 'Value', 1.0);

set(handles.pushbutton36, 'Enable', 'on'); % Enable the command file saving button
set(handles.pushbutton39, 'Enable', 'on'); % Enable the list clearing button
set(handles.listbox6, 'Enable', 'on');

set(handles.togglebutton9, 'Value', 0.0);

% Reset axes guidance text:
set(handles.text65, 'Visible', 'Off');
set(handles.text65, 'String', 'Generating preview...');
% Set text size to accommodate the standard guidance text:
prevtextX = 5;
guiPosition = get(handles.figure1, 'position');
guiHeight = guiPosition(4);
leftPanelHeight = guiHeight - 281;
plotboxHeight = leftPanelHeight - 17;
prevtextY = plotboxHeight - 22;
prevtextWidth = 200;
prevtextHeight = 16;
set(handles.text65, 'position', [prevtextX, prevtextY, prevtextWidth, prevtextHeight]);

% Set the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'New custom map command added to obs. comm. list');

% Feedback to user that the value has updated (blinking the box's color)
set(handles.listbox6, 'BackgroundColor', [0.5 0.5 0.5]);
pause(0.25);
set(handles.listbox6, 'BackgroundColor', 'w');
pause(0.25);
set(handles.listbox6, 'BackgroundColor', [0.5 0.5 0.5]);
pause(0.25);
set(handles.listbox6, 'BackgroundColor', 'w');

%drawnow update;


end


% --- Executes on button press in togglebutton11.
function togglebutton11_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton11 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of togglebutton11


% Remove any existing rects (except for the solar grid rect tagged 'gridCircle'):
rects = findobj(gca,'type','rectangle');
for indexi = 1 : length(rects)
    if(strcmp( get(rects(indexi), 'Tag'),  'gridCircle') == 0)
        delete(rects(indexi))
    end
end

% Clear the edit box:
set(handles.edit16, 'String', '');

if(get(handles.togglebutton11, 'Value') == 1.0)
    
    [x, y] = getpts(handles.axes2);
    %num2str(x, 4)
    %num2str(y, 4)
    rectangle('Position', [x(end) - 1.2 y(end) - 1.2 2.4 2.4], 'Curvature', [1 1]); % The last point that was clicked on the map is used for the coords to allow user to recover from erroneous clicks
    xCoord = x(end) / 60;
    xCoord = round(xCoord * 10000); % Round to 5 significant digits
    xCoord = xCoord / 10000;
    xCoordString = num2str(xCoord, 4);
    yCoord = y(end) / 60;
    yCoord = round(yCoord * 10000); % Round to 5 significant digits
    yCoord = yCoord / 10000;
    yCoordString = num2str(yCoord, 4);
    
    coordString = ['--raoff ', xCoordString, ' --decoff ', yCoordString];
    % Save coordString to clipboard:
    clipboard('copy', coordString)
    
    set(handles.edit16, 'String', coordString);
    set(handles.togglebutton11, 'Value', 0.0);
    
    % Feedback to user that the value has updated (blinking the box's color)
    set(handles.edit16, 'BackgroundColor', [0.5 0.5 0.5]);
    pause(0.25);
    set(handles.edit16, 'BackgroundColor', 'w');
    pause(0.25);
    set(handles.edit16, 'BackgroundColor', [0.5 0.5 0.5]);
    pause(0.25);
    set(handles.edit16, 'BackgroundColor', 'w');
   
end

end

% --- Executes on button press in checkbox40.
% Rotate disable & coordinate tools enable checkbox
function checkbox40_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox40 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.checkbox40, 'Value') == 1) % The checkbox is checked by the user:
    
    set(handles.checkbox28, 'Enable', 'on');
    set(handles.togglebutton9, 'Enable', 'on');
    set(handles.togglebutton11, 'Enable', 'on');
    set(handles.pushbutton37, 'Enable', 'on');
    
    % Command type radio buttons (map-meas. sun-meas):
    set(handles.radiobutton20, 'Enable', 'on');
    set(handles.radiobutton21, 'Enable', 'on');
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Map rotation disabled, coord. & obs. tools enabled');
    
    % If needed, enable the "Continue..." button:
    if(isempty(get(handles.listbox6, 'String')) == 0 && isempty(get(handles.listbox6, 'Value')) == 0)
        set(handles.pushbutton36, 'Enable', 'On');
        set(handles.pushbutton39, 'Enable', 'on'); % Enable the list clearing button
    elseif(strcmp(get(handles.pushbutton39, 'String'), 'Undo') == 1)
        set(handles.pushbutton39, 'Enable', 'on'); % Enable the list clearing button, if it is in "Undo" "mode"
    else
        set(handles.pushbutton36, 'Enable', 'Off');
        set(handles.pushbutton39, 'Enable', 'Off'); % Disable the list clearing button
    end
    
    % Set pushbutton 36 'Enable' using saved enable status:
    %set(handles.pushbutton36, 'Enable', handles.pushbutton36_enable);
    
    set(handles.listbox6, 'Enable', 'on');    
    set(handles.edit15, 'Enable', 'on');
    set(handles.edit16, 'Enable', 'on');
    %handles.rot_temp = handles.fig_rot_angle; % remember value before it's written over
    %set(handles.checkbox30, 'Value', 0.0); % Replaced by handles.fig_rot_angle
    %handles.fig_rot_angle = 0.0; % Set sepatate figure rot angle marker toggle value
    %set(handles.Untitled_110, 'Checked', 'off'); % Corresponding menu item to checkbox30
    %set(handles.checkbox30, 'Enable', 'off');
    %set(handles.Untitled_110, 'Enable', 'off'); % Corresponding menu item to checkbox30
    set(handles.checkbox40, 'ForegroundColor', 'red');
    set(handles.checkbox40, 'BackgroundColor', [1 0.8 0.8]);
    guidata(hObject, handles);

else % The checkbox is unchecked by the user:
    
    set(handles.checkbox28, 'Enable', 'off');
    set(handles.checkbox28, 'Value', 0.0);
    set(handles.togglebutton9, 'Enable', 'off');
    set(handles.togglebutton11, 'Enable', 'off');
    set(handles.pushbutton37, 'Enable', 'off');
    
    % Command type radio buttons (map-meas. sun-meas):
    set(handles.radiobutton20, 'Enable', 'off');
    set(handles.radiobutton21, 'Enable', 'off');
    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Map rotation enabled, coord. & obs. tools disabled');   
    % Save pushbutton36 enable status into handles before setting enable off
    handles.pushbutton36_enable = get(handles.pushbutton36, 'Enable');
    set(handles.listbox6, 'Enable', 'off');
    set(handles.pushbutton36, 'Enable', 'off');
    set(handles.pushbutton39, 'Enable', 'off'); % Disable the list clearing button
    set(handles.edit15, 'Enable', 'off');
    set(handles.edit16, 'Enable', 'off');
    % take rot angle marker value from memory, and set value
    %handles.fig_rot_angle = handles.rot_temp;
    %set(handles.checkbox30, 'Value', handles.rot_temp);
%     if(handles.rot_temp == 1.0)
%         set(handles.Untitled_110, 'Checked', 'on'); % Corresponding menu item to checkbox30
%     else
%         set(handles.Untitled_110, 'Checked', 'off'); % Corresponding menu item to checkbox30
%     end
    %set(handles.checkbox30, 'Enable', 'on');
%    set(handles.Untitled_110, 'Enable', 'on'); % Corresponding menu item to checkbox30
    set(handles.checkbox40, 'ForegroundColor', 'black');
    set(handles.checkbox40, 'BackgroundColor', [0.906 0.906 0.906]);
    if(get(handles.togglebutton5, 'Value') == 1.0)
        pan on;
    end
end

% Call Preview function.
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

% Reset the guidance text:
set(handles.text65, 'Visible', 'Off');
% Set guidance text size to accommodate less text:
prevtextX = 2;
guiPosition = get(handles.figure1, 'position');
guiHeight = guiPosition(4);
leftPanelHeight = guiHeight - 281;
plotboxHeight = leftPanelHeight - 17;
prevtextY = plotboxHeight - 22;
prevtextWidth = 200;
prevtextHeight = 16;
set(handles.text65, 'position', [prevtextX, prevtextY, prevtextWidth, prevtextHeight]);
set(handles.text65, 'String', 'Generating preview...');

% Update handles structure
guidata(hObject, handles);

end


function edit15_Callback(hObject, eventdata, handles)
% hObject    handle to edit15 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit15 as text
%        str2double(get(hObject,'String')) returns contents of edit15 as a double
end

% --- Executes during object creation, after setting all properties.
function edit15_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit15 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end



function edit16_Callback(hObject, eventdata, handles)
% hObject    handle to edit16 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit16 as text
%        str2double(get(hObject,'String')) returns contents of edit16 as a double
end

% --- Executes during object creation, after setting all properties.
function edit16_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit16 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end

% --------------------------------------------------------------------
function lbContext_Callback(hObject, eventdata, handles)
% hObject    handle to lbContext (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
% Save active area data of selected maps menu item
function spot_save_button_Callback(hObject, eventdata, handles)
% hObject    handle to spot_save_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%get the current list of file names from the listbox
inputFileNames = get(handles.listbox1,'String');
    
%get the values for the selected file names
option = get(handles.listbox1,'Value');
 
%if there is nothing to plot, nothing happens
if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
    return
end

% Protection from invalid Active Area treshold values:
if(get(handles.checkbox51, 'Value') == 1)   
    % Checking that the user has entered only numbers into the edit box:
    cmp1 = strcmp(num2str(str2num(get(handles.edit17, 'String'))), '');
    if(cmp1 == 1)
        msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
        %tempTreshold = 104; % "% of QSL"
        return;
    end
    % Checking that the edit box isn't empty:     
    if(isempty(get(handles.edit17, 'String')))
        msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
        %tempTreshold = 104; % "% of QSL"
        return;
    end
    
    % Ask the user confirmation that he/she really wants to use custom AR
    % treshold for saving active area data:
    usr_ans = questdlg(['You are currently using a custom Active Region treshold (', get(handles.edit17, 'String'), ' %). The default value is 104 %. Do you really want to use the custom value in extracting and saving Active Region data?'], 'Confirm custom AR treshold', 'Yes, use custom value', 'No, use default value', 'Cancel', 'Cancel'); % Cancel is the key that is executed when the user presses enter
    if(strcmp(usr_ans, 'Yes, use custom value') == 1)
        % Do nothing, continue normally
    elseif(strcmp(usr_ans, 'No, use default value') == 1)
        % Uncheck the custom AR treshold checkbox and disable the custom AR
        % treshold edit box, then continue normally:
        set(handles.checkbox51, 'Value', 0.0);
        set(handles.edit17, 'Enable', 'Off');
        pause(0.25); % Wait for 0.25 seconds so the GUI can update
    elseif(strcmp(usr_ans, 'Cancel') == 1)
        % Return
        return;
    end
    
end

inputFileNames = get(handles.listbox1,'String');
%get the values for the selected file names
option = get(handles.listbox1,'Value');

%only the selected file+path is written into the list:
filenames = inputFileNames(option);

opt.img_type = '.png'; % not needed anymore
opt.destFolder = 'foo'; % not needed anymore
opt.plottingList = filenames;
opt.beam_size = handles.fig_beam_size;
opt.solar_north =handles.fig_rot_angle;
opt.grid_toggle = handles.fig_grid; %get(handles.checkbox26, 'Value');
%opt.cmap = cmap;
opt.abs_scale = get(handles.checkbox31, 'Value');
opt.mapType = get(handles.popupmenu9, 'Value') ;
opt.customNumOfLevelsToggle = get(handles.checkbox35, 'Value') ;
opt.customNumOfLevels = get(handles.edit12, 'String') ;

if(ischar(get(handles.edit10, 'String')) == 1 || ischar(get(handles.edit11, 'String')) == 1)
    opt.min = str2num(get(handles.edit10, 'String'));
    opt.max = str2num(get(handles.edit11, 'String'));
else
    opt.min = get(handles.edit10, 'String');
    opt.max = get(handles.edit11, 'String');
end
opt.img_save = 0;
opt.fig_save = 0;
opt.close_prev = 0;
%opt.img_size = options.image_size;
opt.rot_disable = get(handles.checkbox40, 'Value');
opt.qsl_cont = get(handles.checkbox27, 'Value');

% Image size
% Figure size shouldn't be needed in spot_saver() because spot_saver
% doesn't make solar map figures
switch handles.fig_size;
    case 1
        opt.img_h = 500;
        opt.img_w = 674;
    case 2
        opt.img_h = 420;
        opt.img_w = 560;
    case 3
        opt.img_h = 600;
        opt.img_w = 800;
    case 4
        opt.img_h = 675;
        opt.img_w = 900;
    otherwise
        opt.img_h = 500;
        opt.img_w = 674;
end


spot_saver(opt, handles.figure1);

end


% --- Executes on button press in pushbutton36.
function pushbutton36_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton36 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(isempty(get(handles.listbox6, 'String')))
%     % Feedback to user that the edit box is empty (blinking the box's color)
%     set(handles.edit15, 'BackgroundColor', [0.5 0.5 0.5]);
%     pause(0.25);
%     set(handles.edit15, 'BackgroundColor', 'w');
%     pause(0.25);
%     set(handles.edit15, 'BackgroundColor', [0.5 0.5 0.5]);
%     pause(0.25);
%     set(handles.edit15, 'BackgroundColor', 'w');
else
    sm_command(handles.figure1);
end

end


% --- Executes on selection change in listbox6.
function listbox6_Callback(hObject, eventdata, handles)
% hObject    handle to listbox6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns listbox6 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox6
end

% --- Executes during object creation, after setting all properties.
function listbox6_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


% --- Executes on key press with focus on listbox6 and none of its controls.
function listbox6_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to listbox6 (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)

key = double(get(gcbf, 'CurrentCharacter'));
if gcbo == handles.listbox6;
    if key == 127 % for DEL key


        %get the current list of file names from the listbox
        inputFileNames = get(handles.listbox6,'String');

        %get the values for the selected file names
        option = get(handles.listbox6,'Value');

        %is there is nothing to delete, nothing happens
        if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
            return
        end

        if(ischar(inputFileNames)) 
            inputFileNames = cellstr(inputFileNames);
        end
        %erases the contents of highlighted item in data array
        inputFileNames(option) = [];

        %updates the gui, erasing the selected item from the listbox
        set(handles.listbox6,'String',inputFileNames);

        %moves the highlighted item to an appropiate value or else will get error
        if option(end) > length(inputFileNames)
            set(handles.listbox6,'Value',length(inputFileNames));
        end

        if(isempty(get(handles.listbox6, 'String')))
            set(handles.pushbutton36, 'Enable', 'off');
            set(handles.pushbutton39, 'Enable', 'off'); % Disable the list clearing button
            set(handles.listbox6, 'Enable', 'off');
        end
        
        % Update handles structure
        guidata(hObject, handles);


    end
end


end


% --- Executes on button press in checkbox41.
% Mark active areas on map
function checkbox41_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox41 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'on'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.checkbox41, 'Value') == 1)
    set(handles.Untitled_55, 'Checked', 'on'); % Corresponding menu item
    % Custom AR treshold checkbox, edit box & text:
    set(handles.checkbox51, 'Enable', 'On');
    if(get(handles.checkbox51, 'Value') == 1.0)
        set(handles.edit17, 'Enable', 'On');
        set(handles.text88, 'Enable', 'On');
    end
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Active Region search & marking on');
else
    set(handles.Untitled_55, 'Checked', 'off'); % Corresponding menu item
    % Custom AR treshold checkbox, edit box & text:
    set(handles.checkbox51, 'Enable', 'Off');
    set(handles.edit17, 'Enable', 'Off');
    set(handles.text88, 'Enable', 'Off');    
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Active Region search & marking off');
end

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end


% --- Executes on button press in pushbutton37.
% Observe full disk map button
function pushbutton37_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton37 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Command type:
if(get(handles.radiobutton20, 'Value') == 0.0 && get(handles.radiobutton21, 'Value') == 1.0) % sun-measure, legacy
    commandString = 'sun-measure --freq 37 --ra-size 0.8 --dec-size 0.8 --ra-off 0.025 --dec-off 0 --numsweeps 30 --coordinate-system radec --maptype ramap --direction forward --sweep-direction forward --az-off 0.007 --el-off -0.0165 --no-standby'; %  --observer Sunmap_user    
else % map-measure, default
    commandString = 'map-measure --az-off 0.000 --el-off 0.017';
end

clipboard('copy', commandString); % Copy commandString to clipboard

% Set the string for the list clearing button to "Clear":
set(handles.pushbutton39, 'String', 'Clear');

% Add command to listbox
listboxString = get(handles.listbox6, 'String');
if(isempty(listboxString) == 0)
    listboxString = cellstr(listboxString);
    listboxString{end + 1} = commandString;
else
    listboxString = commandString;
end

set(handles.listbox6, 'String', listboxString);
set(handles.listbox6, 'Value', 1.0);

set(handles.pushbutton36, 'Enable', 'on'); % Enable the command file saving button
set(handles.pushbutton39, 'Enable', 'on'); % Enable the list clearing button
set(handles.listbox6, 'Enable', 'on');

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'New full disk map command added to obs. comm. list');

% Feedback to user that the value has updated (blinking the box's color)
set(handles.listbox6, 'BackgroundColor', [0.5 0.5 0.5]);
pause(0.25);
set(handles.listbox6, 'BackgroundColor', 'w');
pause(0.25);
set(handles.listbox6, 'BackgroundColor', [0.5 0.5 0.5]);
pause(0.25);
set(handles.listbox6, 'BackgroundColor', 'w');

end


% --------------------------------------------------------------------
% Context menu Save selected
function Untitled_100_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_100 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.plot_all_button, 'Enable'), 'on') == 1 || strcmp(get(handles.plot_all_button, 'Enable'), 'On') == 1)
    plot_all_button_Callback(hObject, eventdata, handles); % Yes, the save button is the old "Plot all" button
end

end

% --------------------------------------------------------------------
% Context menu Plot selected
function Untitled_101_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_101 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.plot_single_button, 'Enable'), 'on') == 1 || strcmp(get(handles.plot_single_button, 'Enable'), 'On') == 1)
    plot_single_button_Callback(hObject, eventdata, handles); % Yes, the "Plot selected" button is the old "Plot single" button
end

end

% --------------------------------------------------------------------
% Context menu Remove selected
function Untitled_102_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_102 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.remove_file_button, 'Enable'), 'on') == 1 || strcmp(get(handles.remove_file_button, 'Enable'), 'On') == 1)
    remove_file_button_Callback(hObject, eventdata, handles); % Yes, the save button is the old "Plot all" button
end

end


% --------------------------------------------------------------------
function lb2Context_Callback(hObject, eventdata, handles)
% hObject    handle to lb2Context (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
function Untitled_104_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_104 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
        
%get the current list of file names from the listbox
inputFileNames = get(handles.listbox6,'String');

%get the values for the selected file names
option = get(handles.listbox6,'Value');

%is there is nothing to delete, nothing happens
if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
    return
end

if(ischar(inputFileNames)) 
    inputFileNames = cellstr(inputFileNames);
end
%erases the contents of highlighted item in data array
inputFileNames(option) = [];

%updates the gui, erasing the selected item from the listbox
set(handles.listbox6,'String',inputFileNames);

%moves the highlighted item to an appropiate value or else will get error
if option(end) > length(inputFileNames)
    set(handles.listbox6,'Value',length(inputFileNames));
end

if(isempty(get(handles.listbox6, 'String')))
    set(handles.pushbutton36, 'Enable', 'off');
    set(handles.pushbutton39, 'Enable', 'off'); % Disable the list clearing button
    set(handles.listbox6, 'Enable', 'off');
end

% Set the Program status string to tell the user about the removal:
selection_size = size(option);
num_selected = selection_size(2);
if(num_selected == 1)
    user_feedback = [num2str(num_selected), ' file removed from obs. comm. list'];
else
    user_feedback = [num2str(num_selected), ' files removed from obs. comm. list'];
end
set(handles.text39, 'String', user_feedback);
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green

% Update handles structure
guidata(hObject, handles);
        
end


% --------------------------------------------------------------------
function track_save_button_Callback(hObject, eventdata, handles)
% hObject    handle to track_save_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%get the current list of file names from the listbox
inputFileNames = get(handles.listbox1,'String');
    
%get the values for the selected file names
option = get(handles.listbox1,'Value');
 
%if there is nothing to plot, nothing happens
if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
    return
end

inputFileNames = get(handles.listbox1,'String');
%get the values for the selected file names
option = get(handles.listbox1,'Value');

%only the selected file+path is written into the list:
filenames = inputFileNames(option);

opt.img_type = '.png'; % not needed anymore
opt.destFolder = 'foo'; % not needed anymore
opt.plottingList = filenames;
opt.beam_size = handles.fig_beam_size;
opt.solar_north =handles.fig_rot_angle;
opt.grid_toggle = handles.fig_grid; %get(handles.checkbox26, 'Value');
%opt.cmap = cmap;
opt.abs_scale = get(handles.checkbox31, 'Value');
opt.mapType = get(handles.popupmenu9, 'Value') ;
opt.customNumOfLevelsToggle = get(handles.checkbox35, 'Value') ;
opt.customNumOfLevels = get(handles.edit12, 'String') ;

if(ischar(get(handles.edit10, 'String')) == 1 || ischar(get(handles.edit11, 'String')) == 1)
opt.min = str2num(get(handles.edit10, 'String'));
opt.max = str2num(get(handles.edit11, 'String'));
else
opt.min = get(handles.edit10, 'String');
opt.max = get(handles.edit11, 'String');
end
opt.img_save = 0;
opt.fig_save = 0;
opt.close_prev = 0;
%opt.img_size = options.image_size;
opt.rot_disable = get(handles.checkbox40, 'Value');
opt.qsl_cont = get(handles.checkbox27, 'Value');

% Image size
% Figure size shouldn't be needed in track_saver() because track_saver
% doesn't make solar map figures
switch handles.fig_size;
    case 1
        opt.img_h = 500;
        opt.img_w = 674;
    case 2
        opt.img_h = 420;
        opt.img_w = 560;
    case 3
        opt.img_h = 600;
        opt.img_w = 800;
    case 4
        opt.img_h = 675;
        opt.img_w = 900;
    otherwise
        opt.img_h = 500;
        opt.img_w = 674;
end

drawnow update;
track_maker(opt, handles.figure1);

end


% --------------------------------------------------------------------
function save_file_list_Callback(hObject, eventdata, handles)
% hObject    handle to save_file_list (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[input_file,pathname] = uiputfile('sunmap_filelist.mat');

if(pathname ~= 0)
    if(isempty(input_file) == 0 && isempty(pathname) == 0)

        if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
            plottingList = get(handles.listbox1,'String');
        end

        filePathStr = [pathname, input_file];
        save(filePathStr, 'plottingList');
    end
end


end


% --------------------------------------------------------------------
function data_source_Callback(hObject, eventdata, handles)
% hObject    handle to data_source (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function lin_scale_Callback(hObject, eventdata, handles)
% hObject    handle to lin_scale (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Data scale is selected to be linear --> Linear checked, Logarithmic unchecked:
set(handles.lin_scale, 'Checked', 'on');
set(handles.log_scale, 'Checked', 'off');

% Set the radio buttons on the GUI:
set(handles.radiobutton10, 'Value', 1.0);
set(handles.radiobutton11, 'Value', 0.0);

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'Linear data scale selected (default)');

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end

% --------------------------------------------------------------------
function log_scale_Callback(hObject, eventdata, handles)
% hObject    handle to log_scale (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Data scale is selected to be logarithmic --> Linear unchecked, Logarithmic checked:
set(handles.log_scale, 'Checked', 'on');
set(handles.lin_scale, 'Checked', 'off');

% Set the radio buttons on the GUI:
set(handles.radiobutton10, 'Value', 0.0);
set(handles.radiobutton11, 'Value', 1.0);

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'Logarithmic data scale selected');

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end

% --- Executes on button press in radiobutton10.
function radiobutton10_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton10 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.radiobutton10, 'Value') == 1.0)
    set(handles.radiobutton11, 'Value', 0.0);
    lin_scale_Callback(hObject, eventdata, handles); % Call the funtion of the menu item for linear scale that "does all the work"
elseif(get(handles.radiobutton10, 'Value') == 0.0)
    set(handles.radiobutton10, 'Value', 1.0);
    lin_scale_Callback(hObject, eventdata, handles); % Call the funtion of the menu item for linear scale that "does all the work"
end

end

% --- Executes on button press in radiobutton11.
function radiobutton11_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton11 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.radiobutton11, 'Value') == 1.0)
    set(handles.radiobutton10, 'Value', 0.0);
    log_scale_Callback(hObject, eventdata, handles); % Call the funtion of the menu item for log scale that "does all the work"
elseif(get(handles.radiobutton11, 'Value') == 0.0)
    set(handles.radiobutton11, 'Value', 1.0);
    log_scale_Callback(hObject, eventdata, handles); % Call the funtion of the menu item for log scale that "does all the work"
end

end

% --- Executes on button press in radiobutton12.
function radiobutton12_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton12 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.radiobutton12, 'Value') == 1.0)
    set(handles.radiobutton13, 'Value', 0.0);
    % Save custom level separation reading into handles:
    handles.customSepOrig = get(handles.edit13, 'String');
    % Save custom levels readings into handles:
    handles.customMinOrig = get(handles.edit10, 'String'); % Min
    handles.customMaxOrig = get(handles.edit11, 'String'); % Max
elseif(get(handles.radiobutton12, 'Value') == 0.0)
    set(handles.radiobutton12, 'Value', 1.0);
end

% Change the text of custom min/max levels checkbox (checkbox31) to
% % of QSL units:
set(handles.checkbox31, 'String', ' Custom min/max levels (% of QSL)');

% Change the Custom level separation edit box value
set(handles.edit13, 'String', handles.customSepQSL); % Back to the value assigned for % of QSL units

% Change the Custom Min and Max edit box values:
set(handles.edit10, 'String', handles.customMinQSL);
set(handles.edit11, 'String', handles.customMaxQSL);

% Set the texts that describe units for min/max units after the edit boxes:
set(handles.text52, 'String', '%'); % Custom level separation
set(handles.text54, 'String', '%'); % Min
set(handles.text53, 'String', '%'); % Max

% Set the corresponding menu items:
set(handles.Untitled_106, 'Checked', 'On');
set(handles.Untitled_107, 'Checked', 'Off');

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'QSL data units selected (default)');

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

% Update handles structure
guidata(hObject, handles);

end

% --- Executes on button press in radiobutton13.
function radiobutton13_Callback(hObject, eventdata, handles)
% hObject    handle to radiobutton13 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.radiobutton13, 'Value') == 1.0)
    set(handles.radiobutton12, 'Value', 0.0);
    % Save custom level separation reading into handles:
    handles.customSepQSL = get(handles.edit13, 'String');
    % Save custom levels readings into handles:
    handles.customMinQSL = get(handles.edit10, 'String'); % Min
    handles.customMaxQSL = get(handles.edit11, 'String'); % Max
elseif(get(handles.radiobutton13, 'Value') == 0.0)
    set(handles.radiobutton13, 'Value', 1.0);
end

% Change the text of custom min/max levels checkbox (checkbox31) to mV or
% pseudo units:
set(handles.checkbox31, 'String', ' Custom min/max levels (mV or pseudo)');

% Change the Custom level separation edit box value
set(handles.edit13, 'String', handles.customSepOrig); % Value that has been saved when % of QSL units were selected

% Change the Custom Min and Max edit box values:
set(handles.edit10, 'String', handles.customMinOrig);
set(handles.edit11, 'String', handles.customMaxOrig);

% Set the texts that describe units for min/max units and Custom level separation after the edit boxes:
set(handles.text52, 'String', ''); % Custom level separation
set(handles.text54, 'String', ''); % Min
set(handles.text53, 'String', ''); % Max

% Set the corresponding menu items:
set(handles.Untitled_106, 'Checked', 'Off');
set(handles.Untitled_107, 'Checked', 'On');

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'Original data units selected');

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

% Update handles structure
guidata(hObject, handles);

end


% --------------------------------------------------------------------
function Untitled_105_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_105 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_106_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_106 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Data units are selected to be [% of QSL] --> % of QSL checked, Original unchecked:
set(handles.Untitled_106, 'Checked', 'on');
set(handles.Untitled_107, 'Checked', 'off');

% Set the radio buttons on the GUI:
set(handles.radiobutton12, 'Value', 1.0);
set(handles.radiobutton13, 'Value', 0.0);

% Change the text of custom min/max levels checkbox (checkbox31) to
% % of QSL units:
set(handles.checkbox31, 'String', ' Custom min/max levels (% of QSL)');

% Save custom level separation reading into handles:
handles.customSepOrig = get(handles.edit13, 'String');
% Save custom levels readings into handles:
handles.customMinOrig = get(handles.edit10, 'String'); % Min
handles.customMaxOrig = get(handles.edit11, 'String'); % Max

% Change the Custom level separation edit box value
set(handles.edit13, 'String', handles.customSepQSL); % Back to the value assigned for % of QSL units

% Change the Custom Min and Max edit box values:
set(handles.edit10, 'String', handles.customMinQSL);
set(handles.edit11, 'String', handles.customMaxQSL);

% Set the texts that describe units for min/max units after the edit boxes:
set(handles.text52, 'String', '%'); % Custom level separation
set(handles.text54, 'String', '%'); % Min
set(handles.text53, 'String', '%'); % Max

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'QSL data units selected (default)');

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

% Update handles structure
guidata(hObject, handles);

end

% --------------------------------------------------------------------
function Untitled_107_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_107 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Data units are selected to be [Original] --> % Original checked, % of QSL unchecked:
set(handles.Untitled_106, 'Checked', 'off');
set(handles.Untitled_107, 'Checked', 'on');

% Set the radio buttons on the GUI:
set(handles.radiobutton12, 'Value', 0.0);
set(handles.radiobutton13, 'Value', 1.0);

% Change the text of custom min/max levels checkbox (checkbox31) to mV or
% pseudo units:
set(handles.checkbox31, 'String', ' Custom min/max levels (mV or pseudo)');

% Save custom level separation reading into handles:
handles.customSepQSL = get(handles.edit13, 'String');
% Save custom levels readings into handles:
handles.customMinQSL = get(handles.edit10, 'String'); % Min
handles.customMaxQSL = get(handles.edit11, 'String'); % Max

% Change the Custom level separation edit box value
set(handles.edit13, 'String', handles.customSepOrig); % Value that has been saved when % of QSL units were selected

% Change the Custom Min and Max edit box values:
set(handles.edit10, 'String', handles.customMinOrig);
set(handles.edit11, 'String', handles.customMaxOrig);

% Set the texts that describe units for min/max units and Custom level separation after the edit boxes:
set(handles.text52, 'String', ''); % Custom level separation
set(handles.text54, 'String', ''); % Min
set(handles.text53, 'String', ''); % Max

% Update the status bar text:
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
set(handles.text39, 'String', 'Original data units selected');

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

% Update handles structure
guidata(hObject, handles);

end

% --- Executes on button press in pushbutton39.
function pushbutton39_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton39 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


if(strcmp(get(handles.pushbutton39, 'String'), 'Clear') == 1);

    %global backup2
    handles.lb6backup = get(handles.listbox6, 'String');
    % Changing the text into "Undo"
    set(handles.pushbutton39, 'String', 'Undo');
    %set(handles.pushbutton39, 'Label', 'Undo clear');

    % Disable the 'Continue...' button:
    set(handles.pushbutton36, 'Enable', 'off');
    
    %get the current list of commands from the listbox
    lbContents = get(handles.listbox6,'String');

    %get the values for the selected commands
    option = get(handles.listbox6,'Value');

    %is there is nothing to delete, nothing happens
    if (isempty(option) == 1 || option(1) == 0 || isempty(lbContents))
        return;
    end

    %erases the contents of highlighted item in data array
    %updates the gui, erasing the selected item from the listbox
    set(handles.listbox6,'String', '');

    %moves the highlighted item to an appropiate value or else will get error
    if option(end) > length(lbContents)
        set(handles.listbox6,'Value',length(lbContents));
    end

    % Set the status bar text to inform user of removal:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)
    set(handles.text39, 'String', 'Observation command list cleared. Press "Undo" to cancel.');
    
else
    
    set(handles.pushbutton39, 'String', 'Clear');
    % Enable the 'Continue...' button:
    set(handles.pushbutton36, 'Enable', 'On');
    %    set(handles.Cpushbutton39, 'Label', 'Clear commands');
    %global backup2;
    set(handles.listbox6, 'String', handles.lb6backup);
    set(handles.listbox6, 'Value', 1);
    
    % Set the status bar text to inform user of undoing of list clearing:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)
    set(handles.text39, 'String', 'Observation command list clearing cancelled.');    

end

% Remove any existing rects (except for the solar grid rect tagged 'gridCircle'):
% This is done because the name "Clear" suggests that everything related
% to the commands in the list is cleared, including the rectangle
rects = findobj(gca,'type','rectangle');
for indexi = 1 : length(rects)
    if(strcmp( get(rects(indexi), 'Tag'),  'gridCircle') == 0)
        delete(rects(indexi))
    end
end

% Update handles structure
guidata(hObject, handles);


end


% --------------------------------------------------------------------
function Untitled_108_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_108 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
% Beam size menu item
function Untitled_109_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_109 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set checked or unchecked
if(strcmp(get(handles.Untitled_109, 'Checked'), 'on') == 1)
    set(handles.Untitled_109, 'Checked', 'off');
    %set(handles.checkbox20, 'Value', 0.0); % Set corresponding GUI checkbox value
    handles.fig_beam_size = 0.0; % Set handles variable value
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure beam size marker off');    
else
    set(handles.Untitled_109, 'Checked', 'on');
    %set(handles.checkbox20, 'Value', 1.0); % Set corresponding GUI checkbox value
    handles.fig_beam_size = 1.0; % Set handles variable value
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure beam size marker on');    
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

guidata(hObject, handles);

end

% --------------------------------------------------------------------
% Rot. angle & arrow menu item
function Untitled_110_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_110 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set checked or unchecked
if(strcmp(get(handles.Untitled_110, 'Checked'), 'on') == 1)
    set(handles.Untitled_110, 'Checked', 'off');
    %set(handles.checkbox30, 'Value', 0.0); % Set corresponding GUI checkbox value
    handles.fig_rot_angle = 0.0; % Set handles variable value
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure rotate angle & arrow markers off');    
else
    set(handles.Untitled_110, 'Checked', 'on');
    %set(handles.checkbox30, 'Value', 1.0); % Set corresponding GUI checkbox value
    handles.fig_rot_angle = 1.0; % Set handles variable value
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure rotate angle & arrow markers on');
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

guidata(hObject, handles);

end

% --------------------------------------------------------------------
% Grid menu item
function Untitled_111_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_111 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set checked or unchecked
if(strcmp(get(handles.Untitled_111, 'Checked'), 'on') == 1)
    set(handles.Untitled_111, 'Checked', 'off');
    %set(handles.checkbox26, 'Value', 0.0); % Set corresponding GUI checkbox value
    handles.fig_grid = 0.0; % Set handles variable value
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure grid off');
else
    set(handles.Untitled_111, 'Checked', 'on');
    %set(handles.checkbox26, 'Value', 1.0); % Set corresponding GUI checkbox value
    handles.fig_grid = 1.0; % Set handles variable value
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure grid on');    
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

guidata(hObject, handles);

end

% --------------------------------------------------------------------
% Simple figure menu item
function Untitled_112_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_112 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set checked or unchecked
if(strcmp(get(handles.Untitled_112, 'Checked'), 'on') == 1)
    set(handles.Untitled_112, 'Checked', 'off');
    %set(handles.checkbox39, 'Value', 0.0); % Set corresponding GUI checkbox value
    handles.fig_simple = 0.0; % Set handles variable value
    %set(handles.checkbox26, 'Enable', 'on');
    set(handles.Untitled_111, 'Enable', 'on'); % Corresponding menu item to checkbox26
    %set(handles.popupmenu7, 'Enable', 'on');
    set(handles.Untitled_59, 'Enable', 'on'); % Figure size menu items:
%     set(handles.Untitled_60, 'Enable', 'on');
%     set(handles.Untitled_61, 'Enable', 'on');
%     set(handles.Untitled_62, 'Enable', 'on');
%     set(handles.Untitled_63, 'Enable', 'on');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure simple settings off');
else
    set(handles.Untitled_112, 'Checked', 'on');
    %set(handles.checkbox39, 'Value', 1.0); % Set corresponding GUI checkbox value
    handles.fig_simple = 1.0; % Set handles variable value
    %set(handles.checkbox26, 'Enable', 'off');
    set(handles.Untitled_111, 'Enable', 'off'); % Corresponding menu item to checkbox26
    %set(handles.popupmenu7, 'Enable', 'off'); % Figure size popupmenu
    set(handles.Untitled_59, 'Enable', 'off'); % Figure size menu items:
%     set(handles.Untitled_60, 'Enable', 'off');
%     set(handles.Untitled_61, 'Enable', 'off');
%     set(handles.Untitled_62, 'Enable', 'off');
%     set(handles.Untitled_63, 'Enable', 'off');
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'Separate figure simple settings on');    
end

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

guidata(hObject, handles);

end


% --------------------------------------------------------------------
% "Solar gallery" preset menu item
function Untitled_113_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_113 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set all the settings in the GUI and in the menus to the "solar gallery"
% mode:

% Data scale to linear:
% GUI:
set(handles.radiobutton10, 'Value', 1.0);
set(handles.radiobutton11, 'Value', 0.0);
% Menus:
set(handles.lin_scale, 'Checked', 'On');
set(handles.log_scale, 'Checked', 'Off');

% Data units to [% of QSL]:
% GUI:
set(handles.radiobutton12, 'Value', 1.0);
set(handles.radiobutton13, 'Value', 0.0);
% Menus:
set(handles.Untitled_106, 'Checked', 'On');
set(handles.Untitled_107, 'Checked', 'Off');

% Map type:
% GUI (!!!!) Remember: The popupmenu is still in the GUI and it's still
% used in getting the values even though it's been set invisible!!
set(handles.popupmenu9, 'Value', 1.0);
% Menu:
set(handles.filled_contours_no_contour_lies, 'Checked', 'on'); % SIC!!
set(handles.filled_contours_with_contour_lines, 'Checked', 'off');
set(handles.contour_lines_only, 'Checked', 'off');

% Custom number of levels to 50:
set(handles.checkbox35, 'Value', 1.0);
set(handles.edit12, 'Enable', 'On');
set(handles.edit12, 'String', '50');
set(handles.checkbox37, 'Value', 0.0); % Uncheck custom level separation
set(handles.edit13, 'Enable', 'Off'); % Disable custom level separation edit box
set(handles.text52, 'Enable', 'Off'); % Disable custom level separation unit text (%)

% Custom levels on: 90-112 % of QSL
set(handles.checkbox31, 'Value', 1.0);
set(handles.edit10, 'Enable', 'On');
set(handles.edit10, 'String', '90');
set(handles.edit11, 'Enable', 'On');
set(handles.edit11, 'String', '112');
set(handles.text46, 'Enable', 'On'); % Enable texts associated with the custom levels:
set(handles.text47, 'Enable', 'On');
set(handles.text53, 'Enable', 'On');
set(handles.text54, 'Enable', 'On');

% Colormap: "MRO red - 512 colors":
set(handles.popupmenu6, 'Value', 30);
set(handles.popupmenu6, 'Enable', 'on');

% Miscellaneous:

% Solar grid & radec offs. err: off:
% Checkbox:
set(handles.checkbox38, 'Value', 0.0);
set(handles.checkbox38, 'Enable', 'On');
% Toggle button:
set(handles.togglebutton7, 'Value', 0.0);
solar_area_lines_str = [pwd, '/img/solar_area_lines_icon.png'];
solar_area_lines_img = imread(solar_area_lines_str);
set(handles.togglebutton7, 'cdata', solar_area_lines_img); % Reset button icon
% Menu item:
set(handles.Untitled_57, 'Checked', 'off');

% Mark active areas on map: disabled, off:
set(handles.checkbox41, 'Value', 0.0);
set(handles.checkbox41, 'Enable', 'Off');
%Menu item:
set(handles.Untitled_55, 'Checked', 'off');
set(handles.Untitled_55, 'Enable', 'off');

% Mark NOAA AR's on map: disabled, off:
set(handles.checkbox50, 'Enable', 'Off');
set(handles.checkbox50, 'Value', 0.0);
% Menu item:
set(handles.Untitled_135, 'Checked', 'off');
set(handles.Untitled_135, 'Enable', 'off');

% AR detect. treashold checkbox, text & edit box disabled:
set(handles.checkbox51, 'Enable', 'off');
set(handles.text88, 'Enable', 'off');
set(handles.edit17, 'Enable', 'off');

% Quiet Sun Level contour: off
set(handles.checkbox27, 'Value', 0.0);
set(handles.checkbox27, 'Enable', 'On');
% Menu item:
set(handles.Untitled_122, 'Checked', 'off');
set(handles.Untitled_122, 'Enable', 'On');
% Toggle button
set(handles.togglebutton8, 'Value', 0.0);

% Contour line annotations: disabled, off:
set(handles.checkbox36, 'Value', 0.0);
set(handles.checkbox36, 'Enable', 'Off');
set(handles.Untitled_58, 'Enable', 'off');
set(handles.Untitled_58, 'Checked', 'off');

% Separate figure:

% Beam size: on:
%GUI
set(handles.checkbox20, 'Value', 1.0);
set(handles.checkbox20, 'Enable', 'On');
% Menu:
set(handles.Untitled_109, 'Checked', 'on');
% Handles variable:
handles.fig_beam_size = 1.0;

% Rot angl. & arrow: on:
% GUI:
set(handles.checkbox30, 'Value', 1.0);
set(handles.checkbox30, 'Enable', 'On');
% Menu:
set(handles.Untitled_110, 'Checked', 'on');
set(handles.Untitled_110, 'Enable', 'on');
% Handles variable:
handles.fig_rot_angle = 1.0;

% Grid: off:
% GUI:
set(handles.checkbox26, 'Value', 0.0);
set(handles.checkbox26, 'Enable', 'On');
% Menu:
set(handles.Untitled_111, 'Checked', 'off');
set(handles.Untitled_111, 'Enable', 'on');
% Handles variable:
handles.fig_grid = 0.0;

% Simple figure: off:
% GUI:
set(handles.checkbox39, 'Value', 0.0);
set(handles.checkbox39, 'Enable', 'On');
% Menu:
set(handles.Untitled_112, 'Checked', 'off');
% Handles variable:
handles.fig_simple = 0.0;

% Size:
% GUI (!!!)
%set(handles.popupmenu7, 'Value', 1.0);
% Menu:
set(handles.Untitled_59, 'Enable', 'on');
set(handles.Untitled_60, 'Checked', 'on');
set(handles.Untitled_61, 'Checked', 'off');
set(handles.Untitled_62, 'Checked', 'off');
set(handles.Untitled_63, 'Checked', 'off');
% Handles variable:
handles.fig_size = 1.0;

% Coordinate & observation tools:

% Enable tools & disable map rotation: off:
set(handles.checkbox40, 'Value', 0.0);
set(handles.checkbox40, 'Enable', 'On');
set(handles.checkbox40, 'BackgroundColor', [0.906 0.906 0.906]); % Background color
set(handles.checkbox40, 'ForegroundColor', [0 0 0]); % Text color

% Data cursor: disabled, off:
set(handles.checkbox28, 'Value', 0.0);
set(handles.checkbox28, 'Enable', 'Off');

% "Observe custom map" button disabled, up:
set(handles.togglebutton9, 'Enable', 'Off');
set(handles.togglebutton9, 'Value', 0.0);

% "Observe full disk map" button disabled:
set(handles.pushbutton37, 'Enable', 'Off');

% Command listbox disabled:
set(handles.listbox6, 'Enable', 'Off');

% "Clear" button disabled:
set(handles.pushbutton39, 'Enable', 'Off');

% "Continue" button disabled:
set(handles.pushbutton36, 'Enable', 'Off');

% Enable "Reset settings" button:
% GUI:
set(handles.pushbutton27, 'Enable', 'On');
% Menu:
set(handles.reset_settings, 'Enable', 'On');

% Call the "Use solar disk center" menu item:
Untitled_120_Callback(hObject, eventdata, handles);

% Notify user that the settings are in use:
set(handles.text39, 'String', '"Solar gallery" preset settings in use');
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

guidata(hObject, handles);

end


% --------------------------------------------------------------------
function Untitled_114_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_114 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

spot_save_button_Callback(hObject, eventdata, handles);

end

% --------------------------------------------------------------------
function Untitled_115_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_115 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

track_save_button_Callback(hObject, eventdata, handles);

end


% --------------------------------------------------------------------
function Untitled_116_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_116 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

save_file_list_Callback(hObject, eventdata, handles);

end


% --------------------------------------------------------------------
function radius_save_button_Callback(hObject, eventdata, handles)
% hObject    handle to radius_save_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


%get the current list of file names from the listbox
inputFileNames = get(handles.listbox1,'String');
    
%get the values for the selected file names
option = get(handles.listbox1,'Value');
 
%if there is nothing to plot, nothing happens
if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
    return
end

% Only the selected file+path is written into the list:
filenames = inputFileNames(option);

opt.img_type = '.png'; % not needed anymore
opt.destFolder = 'foo'; % not needed anymore
opt.plottingList = filenames;
opt.beam_size = handles.fig_beam_size;
opt.solar_north =handles.fig_rot_angle;
opt.grid_toggle = handles.fig_grid; %get(handles.checkbox26, 'Value');
%opt.cmap = cmap;
opt.abs_scale = get(handles.checkbox31, 'Value');
opt.mapType = get(handles.popupmenu9, 'Value') ;
opt.customNumOfLevelsToggle = get(handles.checkbox35, 'Value') ;
opt.customNumOfLevels = get(handles.edit12, 'String') ;

if(ischar(get(handles.edit10, 'String')) == 1 || ischar(get(handles.edit11, 'String')) == 1)
opt.min = str2num(get(handles.edit10, 'String'));
opt.max = str2num(get(handles.edit11, 'String'));
else
opt.min = get(handles.edit10, 'String');
opt.max = get(handles.edit11, 'String');
end
opt.img_save = 0;
opt.fig_save = 0;
opt.close_prev = 0;
%opt.img_size = options.image_size;
opt.rot_disable = get(handles.checkbox40, 'Value');
opt.qsl_cont = get(handles.checkbox27, 'Value');

% Image size
switch handles.fig_size;
    case 1
        opt.img_h = 500;
        opt.img_w = 674;
    case 2
        opt.img_h = 420;
        opt.img_w = 560;
    case 3
        opt.img_h = 600;
        opt.img_w = 800;
    case 4
        opt.img_h = 675;
        opt.img_w = 900;
    otherwise
        opt.img_h = 500;
        opt.img_w = 674;
end

drawnow update;


radius_saver(opt, handles.figure1);
end


% --------------------------------------------------------------------
function Untitled_117_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_117 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% If the listbox isn't empty, open the currently selected file in Matlab
% editor:
 if(isempty(get(handles.listbox1, 'String')) == 0)
     
    %get the current list of file names from the listbox
    inputFileNames = get(handles.listbox1,'String');
    
    %get the values for the selected file names
    option = get(handles.listbox1,'Value');
 
    %if there is nothing to plot, nothing happens
    if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
        return
    end

    % Only the selected file+path is written into the list:
    filenames = inputFileNames(option);
     
    % Call the editor with the current info file:
     eval(['edit ', filenames{1}]);
 end

end


% --------------------------------------------------------------------
function Untitled_118_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_118 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
    
Untitled_117_Callback(hObject, eventdata, handles);

end


% --------------------------------------------------------------------
function Untitled_119_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_119 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

% --------------------------------------------------------------------
function Untitled_120_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_120 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.Untitled_120, 'Checked', 'on');
set(handles.Untitled_121, 'Checked', 'off');

% Set the related toggle button:
set(handles.togglebutton15, 'Value', 0.0);
% Set a custom image as an icon for the solar disk centering toggle button:
solar_center_tool_str = [pwd, '/img/center_tool_icon.png'];
solar_center_tool_button_img = imread(solar_center_tool_str);
set(handles.togglebutton15, 'cdata', solar_center_tool_button_img);

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

% Set the status ber text:
set(handles.text39, 'String', 'Using data matrix center for plotting');
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

end

% --------------------------------------------------------------------
function Untitled_121_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_121 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.Untitled_121, 'Checked', 'on');
set(handles.Untitled_120, 'Checked', 'off');

% Set the related toggle button:
set(handles.togglebutton15, 'Value', 1.0);
% Set a custom image as an icon for the solar disk centering toggle button:
solar_center_tool_str = [pwd, '/img/center_tool_icon_down.png'];
solar_center_tool_button_img = imread(solar_center_tool_str);
set(handles.togglebutton15, 'cdata', solar_center_tool_button_img);

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

% Set the status ber text:
set(handles.text39, 'String', 'Using solar disk center for plotting when possible.');
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

set(handles.pushbutton27, 'Enable', 'On'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

end


% --------------------------------------------------------------------
% QSL contour
function Untitled_122_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_122 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.Untitled_122, 'Checked'), 'off'))
    set(handles.Untitled_122, 'Checked', 'on');
    set(handles.checkbox27, 'Value', 1.0);
    checkbox27_Callback(hObject, eventdata, handles);
elseif(strcmp(get(handles.Untitled_122, 'Checked'), 'on'))
    set(handles.Untitled_122, 'Checked', 'off');
    set(handles.checkbox27, 'Value', 0.0);
    checkbox27_Callback(hObject, eventdata, handles);
end

end


% --- Executes on button press in pushbutton40.
function pushbutton40_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton40 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Call fig_opt.m:
fig_opt(handles.figure1);

end


% --- Executes on button press in checkbox50.
function checkbox50_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox50 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'on'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

if(get(handles.checkbox50, 'Value') == 1)
    set(handles.Untitled_135, 'Checked', 'on'); % Corresponding menu item
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'NOAA Active Region marking on');
else
    set(handles.Untitled_135, 'Checked', 'off'); % Corresponding menu item 
    % Update the status bar text:
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green
    set(handles.text39, 'String', 'NOAA Active Region marking off');
end

% Update preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end


% Custom Active Region detection treshold level edit box
function edit17_Callback(hObject, eventdata, handles)
% hObject    handle to edit17 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

key = double(get(gcbf, 'CurrentCharacter'));
% if 'enter' is pressed in the edit box, the preview function is called
if(key == 13)
    if(isempty(get(handles.listbox1, 'String')) == 0)       
        % Checking that the user has entered only numbers into the edit box
        cmp1 = strcmp(num2str(str2num(get(handles.edit17, 'String'))), '');
        if(cmp1 == 1)
            msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
            figure1_ResizeFcn(hObject, eventdata, handles); % Why is this here?
            return;
        end

        % Call preview if there are files in the listbox:
        if(isempty(get(handles.listbox1, 'Value')) == 0)
            plottingList = get(handles.listbox1,'String');
            indexes = get(handles.listbox1, 'Value');
            plot_map = plottingList(indexes(1));
            preview(plot_map, handles.figure1);
        end
    end
end

end

% --- Executes during object creation, after setting all properties.
function edit17_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit17 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
end


% --- Executes on button press in checkbox51.
function checkbox51_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox51 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(get(handles.checkbox51, 'Value') == 1.0)
    set(handles.text88, 'Enable', 'On');
    set(handles.edit17, 'Enable', 'On');
    % Set the status ber text:
    set(handles.text39, 'String', 'Custom Active Region treshold in use');
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)    
else
    set(handles.text88, 'Enable', 'Off');
    set(handles.edit17, 'Enable', 'Off');
    % Set the status ber text:
    set(handles.text39, 'String', 'Default Active Region treshold in use');
    set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)        
end

% Call preview if there are files in the listbox:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

end


% --- Executes during object creation, after setting all properties.
function checkbox20_CreateFcn(hObject, eventdata, handles)
% hObject    handle to checkbox20 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called
end


% --------------------------------------------------------------------
% Menu item for preset settings
function presets_Callback(hObject, eventdata, handles)
% hObject    handle to presets (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
% Fixed levels jet preset settings menu item
function Untitled_125_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_125 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set all the settings in the GUI and in the menus to the "Fixed levels Jet"
% mode:

% Data scale to linear:
% GUI:
set(handles.radiobutton10, 'Value', 1.0);
set(handles.radiobutton11, 'Value', 0.0);
% Menus:
set(handles.lin_scale, 'Checked', 'On');
set(handles.log_scale, 'Checked', 'Off');

% Data units to [% of QSL]:
% GUI:
set(handles.radiobutton12, 'Value', 1.0);
set(handles.radiobutton13, 'Value', 0.0);
% Menus:
set(handles.Untitled_106, 'Checked', 'On');
set(handles.Untitled_107, 'Checked', 'Off');

% Map type:
% GUI (!!!!) Remember: The popupmenu is still in the GUI and it's still
% used in getting the values even though it's been set invisible!!
set(handles.popupmenu9, 'Value', 1.0);
% Menu:
set(handles.filled_contours_no_contour_lies, 'Checked', 'on'); % SIC!!
set(handles.filled_contours_with_contour_lines, 'Checked', 'off');
set(handles.contour_lines_only, 'Checked', 'off');

% Custom number of levels to 64:
set(handles.checkbox35, 'Value', 1.0);
set(handles.edit12, 'Enable', 'On');
set(handles.edit12, 'String', '64');
set(handles.checkbox37, 'Value', 0.0); % Uncheck custom level separation
set(handles.edit13, 'Enable', 'Off'); % Disable custom level separation edit box
set(handles.text52, 'Enable', 'Off'); % Disable custom level separation unit text (%)

% Custom levels on: 90-130 % of QSL
set(handles.checkbox31, 'Value', 1.0);
set(handles.edit10, 'Enable', 'On');
set(handles.edit10, 'String', '90');
set(handles.edit11, 'Enable', 'On');
set(handles.edit11, 'String', '130');
set(handles.text46, 'Enable', 'On'); % Enable texts associated with the custom levels:
set(handles.text47, 'Enable', 'On');
set(handles.text53, 'Enable', 'On');
set(handles.text54, 'Enable', 'On');


% Colormap: "Jet - 512 colors (default)":
set(handles.popupmenu6, 'Value', 1);
set(handles.popupmenu6, 'Enable', 'on');

% Miscellaneous:

% Solar grid & radec offs. err: off:
% Checkbox:
set(handles.checkbox38, 'Value', 0.0);
set(handles.checkbox38, 'Enable', 'On');
% Toggle button:
set(handles.togglebutton7, 'Value', 0.0);
solar_area_lines_str = [pwd, '/img/solar_area_lines_icon.png'];
solar_area_lines_img = imread(solar_area_lines_str);
set(handles.togglebutton7, 'cdata', solar_area_lines_img); % Reset button icon
% Menu item:
set(handles.Untitled_57, 'Checked', 'off');

% Mark active areas on map: disabled, off:
set(handles.checkbox41, 'Value', 0.0);
set(handles.checkbox41, 'Enable', 'Off');
%Menu item:
set(handles.Untitled_55, 'Checked', 'off');
set(handles.Untitled_55, 'Enable', 'off');

% Mark NOAA AR's on map: disabled, off:
set(handles.checkbox50, 'Enable', 'Off');
set(handles.checkbox50, 'Value', 0.0);
% Menu item:
set(handles.Untitled_135, 'Checked', 'off');
set(handles.Untitled_135, 'Enable', 'off');

% AR detect. treshold checkbox, text & edit box disabled:
set(handles.checkbox51, 'Enable', 'off');
set(handles.text88, 'Enable', 'off');
set(handles.edit17, 'Enable', 'off');

% Quiet Sun Level contour: off
% Checkbox:
set(handles.checkbox27, 'Value', 0.0);
set(handles.checkbox27, 'Enable', 'On');
% Menu item:
set(handles.Untitled_122, 'Checked', 'off');
set(handles.Untitled_122, 'Enable', 'On');
% Toggle button:
set(handles.togglebutton8, 'Value', 0.0);

% Contour line annotations: disabled, off:
% Checkbox:
set(handles.checkbox36, 'Value', 0.0);
set(handles.checkbox36, 'Enable', 'Off');
% Menu item:
set(handles.Untitled_58, 'Enable', 'off');
set(handles.Untitled_58, 'Checked', 'off');

% Separate figure:

% Beam size: on:
%GUI
set(handles.checkbox20, 'Value', 1.0);
set(handles.checkbox20, 'Enable', 'On');
% Menu:
set(handles.Untitled_109, 'Checked', 'on');
% Handles variable:
handles.fig_beam_size = 1.0;

% Rot angl. & arrow: on:
% GUI:
set(handles.checkbox30, 'Value', 1.0);
set(handles.checkbox30, 'Enable', 'On');
% Menu:
set(handles.Untitled_110, 'Checked', 'on');
set(handles.Untitled_110, 'Enable', 'on');
% Handles variable:
handles.fig_rot_angle = 1.0;

% Grid: off:
% GUI:
set(handles.checkbox26, 'Value', 0.0);
set(handles.checkbox26, 'Enable', 'On');
% Menu:
set(handles.Untitled_111, 'Checked', 'off');
set(handles.Untitled_111, 'Enable', 'on');
% Handles variable:
handles.fig_grid = 0.0;

% Simple figure: off:
% GUI:
set(handles.checkbox39, 'Value', 0.0);
set(handles.checkbox39, 'Enable', 'On');
% Menu:
set(handles.Untitled_112, 'Checked', 'off');
% Handles variable:
handles.fig_simple = 0.0;

% Size:
% GUI (!!!)
%set(handles.popupmenu7, 'Value', 1.0);
% Menu:
set(handles.Untitled_59, 'Enable', 'on');
set(handles.Untitled_60, 'Checked', 'on');
set(handles.Untitled_61, 'Checked', 'off');
set(handles.Untitled_62, 'Checked', 'off');
set(handles.Untitled_63, 'Checked', 'off');
% Handles variable:
handles.fig_size = 1.0;

% Coordinate & observation tools:

% Enable tools & disable map rotation: off:
set(handles.checkbox40, 'Value', 0.0);
set(handles.checkbox40, 'Enable', 'On');
set(handles.checkbox40, 'BackgroundColor', [0.906 0.906 0.906]); % Background color
set(handles.checkbox40, 'ForegroundColor', [0 0 0]); % Text color

% Data cursor: disabled, off:
set(handles.checkbox28, 'Value', 0.0);
set(handles.checkbox28, 'Enable', 'Off');

% "Observe custom map" button disabled, up:
set(handles.togglebutton9, 'Enable', 'Off');
set(handles.togglebutton9, 'Value', 0.0);

% "Observe full disk map" button disabled:
set(handles.pushbutton37, 'Enable', 'Off');

% Command listbox disabled:
set(handles.listbox6, 'Enable', 'Off');

% "Clear" button disabled:
set(handles.pushbutton39, 'Enable', 'Off');

% "Continue" button disabled:
set(handles.pushbutton36, 'Enable', 'Off');

% Enable "Reset settings" button:
% GUI:
set(handles.pushbutton27, 'Enable', 'On');
% Menu:
set(handles.reset_settings, 'Enable', 'On');

% Call the "Use data matrix center" menu item:
Untitled_120_Callback(hObject, eventdata, handles);

% Notify user that the settings are in use:
set(handles.text39, 'String', '"Fixed levels ''Jet''" preset settings in use');
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

guidata(hObject, handles);


end

% --------------------------------------------------------------------
function Untitled_124_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_124 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set all the settings in the GUI and in the menus to the "Full Sun"
% mode:

% Data scale to linear:
% GUI:
set(handles.radiobutton10, 'Value', 1.0);
set(handles.radiobutton11, 'Value', 0.0);
% Menus:
set(handles.lin_scale, 'Checked', 'On');
set(handles.log_scale, 'Checked', 'Off');

% Data units to [% of QSL]:
% GUI:
set(handles.radiobutton12, 'Value', 1.0);
set(handles.radiobutton13, 'Value', 0.0);
% Menus:
set(handles.Untitled_106, 'Checked', 'On');
set(handles.Untitled_107, 'Checked', 'Off');

% Map type:
% GUI (!!!!) Remember: The popupmenu is still in the GUI and it's still
% used in getting the values even though it's been set invisible!!
set(handles.popupmenu9, 'Value', 1.0);
% Menu:
set(handles.filled_contours_no_contour_lies, 'Checked', 'on'); % SIC!!
set(handles.filled_contours_with_contour_lines, 'Checked', 'off');
set(handles.contour_lines_only, 'Checked', 'off');

% Custom number of levels to 128:
set(handles.checkbox35, 'Value', 1.0);
set(handles.edit12, 'Enable', 'On');
set(handles.edit12, 'String', '128');
set(handles.checkbox37, 'Value', 0.0); % Uncheck custom level separation
set(handles.edit13, 'Enable', 'Off'); % Disable custom level separation edit box
set(handles.text52, 'Enable', 'Off'); % Disable custom level separation unit text (%)

% Custom levels on: -20...130 % of QSL
set(handles.checkbox31, 'Value', 1.0);
set(handles.edit10, 'Enable', 'On');
set(handles.edit10, 'String', '-20');
set(handles.edit11, 'Enable', 'On');
set(handles.edit11, 'String', '130');
set(handles.text46, 'Enable', 'On'); % Enable texts associated with the custom levels:
set(handles.text47, 'Enable', 'On');
set(handles.text53, 'Enable', 'On');
set(handles.text54, 'Enable', 'On');


% Colormap: "Violetjet":
set(handles.popupmenu6, 'Value', 10);
set(handles.popupmenu6, 'Enable', 'on');

% Miscellaneous:

% Solar grid & radec offs. err: off:
% Checkbox:
set(handles.checkbox38, 'Value', 0.0);
set(handles.checkbox38, 'Enable', 'On');
% Toggle button:
set(handles.togglebutton7, 'Value', 0.0);
solar_area_lines_str = [pwd, '/img/solar_area_lines_icon.png'];
solar_area_lines_img = imread(solar_area_lines_str);
set(handles.togglebutton7, 'cdata', solar_area_lines_img); % Reset button icon
% Menu item:
set(handles.Untitled_57, 'Checked', 'off');

% Mark active areas on map: disabled, off:
set(handles.checkbox41, 'Value', 0.0);
set(handles.checkbox41, 'Enable', 'Off');
%Menu item:
set(handles.Untitled_55, 'Checked', 'off');
set(handles.Untitled_55, 'Enable', 'off');

% Mark NOAA AR's on map: disabled, off:
set(handles.checkbox50, 'Enable', 'Off');
set(handles.checkbox50, 'Value', 0.0);
% Menu item:
set(handles.Untitled_135, 'Checked', 'off');
set(handles.Untitled_135, 'Enable', 'off');

% AR detect. treshold checkbox, text & edit box disabled:
set(handles.checkbox51, 'Enable', 'off');
set(handles.text88, 'Enable', 'off');
set(handles.edit17, 'Enable', 'off');

% Quiet Sun Level contour: off
% Checkbox:
set(handles.checkbox27, 'Value', 0.0);
set(handles.checkbox27, 'Enable', 'On');
% Menu item:
set(handles.Untitled_122, 'Checked', 'off');
set(handles.Untitled_122, 'Enable', 'On');
% Toggle button:
set(handles.togglebutton8, 'Value', 0.0);

% Contour line annotations: disabled, off:
% Checkbox:
set(handles.checkbox36, 'Value', 0.0);
set(handles.checkbox36, 'Enable', 'Off');
% Menu item:
set(handles.Untitled_58, 'Enable', 'off');
set(handles.Untitled_58, 'Checked', 'off');

% Separate figure:

% Beam size: on:
%GUI
set(handles.checkbox20, 'Value', 1.0);
set(handles.checkbox20, 'Enable', 'On');
% Menu:
set(handles.Untitled_109, 'Checked', 'on');
% Handles variable:
handles.fig_beam_size = 1.0;

% Rot angl. & arrow: on:
% GUI:
set(handles.checkbox30, 'Value', 1.0);
set(handles.checkbox30, 'Enable', 'On');
% Menu:
set(handles.Untitled_110, 'Checked', 'on');
set(handles.Untitled_110, 'Enable', 'on');
% Handles variable:
handles.fig_rot_angle = 1.0;

% Grid: off:
% GUI:
set(handles.checkbox26, 'Value', 0.0);
set(handles.checkbox26, 'Enable', 'On');
% Menu:
set(handles.Untitled_111, 'Checked', 'off');
set(handles.Untitled_111, 'Enable', 'on');
% Handles variable:
handles.fig_grid = 0.0;

% Simple figure: off:
% GUI:
set(handles.checkbox39, 'Value', 0.0);
set(handles.checkbox39, 'Enable', 'On');
% Menu:
set(handles.Untitled_112, 'Checked', 'off');
% Handles variable:
handles.fig_simple = 0.0;

% Size:
% GUI (!!!)
%set(handles.popupmenu7, 'Value', 1.0);
% Menu:
set(handles.Untitled_59, 'Enable', 'on');
set(handles.Untitled_60, 'Checked', 'on');
set(handles.Untitled_61, 'Checked', 'off');
set(handles.Untitled_62, 'Checked', 'off');
set(handles.Untitled_63, 'Checked', 'off');
% Handles variable:
handles.fig_size = 1.0;

% Coordinate & observation tools:

% Enable tools & disable map rotation: off:
set(handles.checkbox40, 'Value', 0.0);
set(handles.checkbox40, 'Enable', 'On');
set(handles.checkbox40, 'BackgroundColor', [0.906 0.906 0.906]); % Background color
set(handles.checkbox40, 'ForegroundColor', [0 0 0]); % Text color

% Data cursor: disabled, off:
set(handles.checkbox28, 'Value', 0.0);
set(handles.checkbox28, 'Enable', 'Off');

% "Observe custom map" button disabled, up:
set(handles.togglebutton9, 'Enable', 'Off');
set(handles.togglebutton9, 'Value', 0.0);

% "Observe full disk map" button disabled:
set(handles.pushbutton37, 'Enable', 'Off');

% Command listbox disabled:
set(handles.listbox6, 'Enable', 'Off');

% "Clear" button disabled:
set(handles.pushbutton39, 'Enable', 'Off');

% "Continue" button disabled:
set(handles.pushbutton36, 'Enable', 'Off');

% Enable "Reset settings" button:
% GUI:
set(handles.pushbutton27, 'Enable', 'On');
% Menu:
set(handles.reset_settings, 'Enable', 'On');

% Call the "Use data matrix center" menu item:
Untitled_120_Callback(hObject, eventdata, handles);

% Notify user that the settings are in use:
set(handles.text39, 'String', '"Full Sun" preset settings in use');
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

guidata(hObject, handles);

end


% --------------------------------------------------------------------
function Untitled_128_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_128 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --- Executes on key press with focus on edit17 and none of its controls.
function edit17_KeyPressFcn(hObject, eventdata, handles)
% hObject    handle to edit17 (see GCBO)
% eventdata  structure with the following fields (see UICONTROL)
%	Key: name of the key that was pressed, in lower case
%	Character: character interpretation of the key(s) that was pressed
%	Modifier: name(s) of the modifier key(s) (i.e., control, shift) pressed
% handles    structure with handles and user data (see GUIDATA)
% guidata(hObject, handles);
% 
% fprintf('\nedit17 keypress!\n');
% get(handles.edit17, 'String')
% 
% % Check for invalid Active Area treshold values:
% if(get(handles.checkbox51, 'Value') == 1)   
%     
%     % Checking that the user has entered only numbers into the edit box:
%     cmp1 = strcmp(num2str(str2num(get(handles.edit17, 'String'))), '');
%     if(cmp1 == 1)
%         set(handles.edit17, 'BackgroundColor', [0.85, 0.16, 0]);
%         set(handles.edit17, 'ForegroundColor', [1, 1, 1]);
%         return;       
%     end
%     
%     % Checking that the edit box isn't empty:     
%     if(isempty(get(handles.edit17, 'String')))
%         set(handles.edit17, 'BackgroundColor', [0.85, 0.16, 0]);
%         set(handles.edit17, 'ForegroundColor', [1, 1, 1]);
%         return;
%     end
%     
%     set(handles.edit17, 'BackgroundColor', [1, 1, 1]);
%     set(handles.edit17, 'ForegroundColor', [0, 0, 0]);
%     
% end

end


% --------------------------------------------------------------------
function Untitled_129_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_129 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Set all the settings in the GUI and in the menus to the "Dynamic levels Jet"
% mode:

% Data scale to linear:
% GUI:
set(handles.radiobutton10, 'Value', 1.0);
set(handles.radiobutton11, 'Value', 0.0);
% Menus:
set(handles.lin_scale, 'Checked', 'On');
set(handles.log_scale, 'Checked', 'Off');

% Data units to [% of QSL]:
% GUI:
set(handles.radiobutton12, 'Value', 1.0);
set(handles.radiobutton13, 'Value', 0.0);
% Menus:
set(handles.Untitled_106, 'Checked', 'On');
set(handles.Untitled_107, 'Checked', 'Off');

% Map type:
% GUI (!!!!) Remember: The popupmenu is still in the GUI and it's still
% used in getting the values even though it's been set invisible!!
set(handles.popupmenu9, 'Value', 1.0);
% Menu:
set(handles.filled_contours_no_contour_lies, 'Checked', 'on'); % SIC!!
set(handles.filled_contours_with_contour_lines, 'Checked', 'off');
set(handles.contour_lines_only, 'Checked', 'off');

% Custom number of levels off:
set(handles.checkbox35, 'Value', 0.0);
set(handles.edit12, 'Enable', 'Off');
set(handles.edit12, 'String', '16');
set(handles.checkbox37, 'Value', 0.0); % Uncheck custom level separation
set(handles.edit13, 'Enable', 'Off'); % Disable custom level separation edit box
set(handles.text52, 'Enable', 'Off'); % Disable custom level separation unit text (%)

% Custom levels off:
set(handles.checkbox31, 'Value', 0.0);
set(handles.edit10, 'Enable', 'Off'); % Min level edit box disabled
%set(handles.edit10, 'String', '90');
set(handles.edit11, 'Enable', 'Off'); % Max level edit box disabled
%set(handles.edit11, 'String', '130');
set(handles.text46, 'Enable', 'Off'); % Disable texts associated with the custom levels:
set(handles.text47, 'Enable', 'Off');
set(handles.text53, 'Enable', 'Off');
set(handles.text54, 'Enable', 'Off');


% Colormap: "Jet - 512 colors (default)":
set(handles.popupmenu6, 'Value', 1);
set(handles.popupmenu6, 'Enable', 'on');

% Miscellaneous:

% Solar grid & radec offs. err: off:
% Checkbox:
set(handles.checkbox38, 'Value', 0.0);
set(handles.checkbox38, 'Enable', 'On');
% Toggle button:
set(handles.togglebutton7, 'Value', 0.0);
solar_area_lines_str = [pwd, '/img/solar_area_lines_icon.png'];
solar_area_lines_img = imread(solar_area_lines_str);
set(handles.togglebutton7, 'cdata', solar_area_lines_img); % Reset button icon
% Menu item:
set(handles.Untitled_57, 'Checked', 'off');

% Mark active areas on map: disabled, off:
set(handles.checkbox41, 'Value', 0.0);
set(handles.checkbox41, 'Enable', 'Off');
%Menu item:
set(handles.Untitled_55, 'Checked', 'off');
set(handles.Untitled_55, 'Enable', 'off');

% Mark NOAA AR's on map: disabled, off:
set(handles.checkbox50, 'Enable', 'Off');
set(handles.checkbox50, 'Value', 0.0);
% Menu item:
set(handles.Untitled_135, 'Checked', 'off');
set(handles.Untitled_135, 'Enable', 'off');

% AR detect. treshold checkbox, text & edit box disabled:
set(handles.checkbox51, 'Enable', 'off');
set(handles.text88, 'Enable', 'off');
set(handles.edit17, 'Enable', 'off');

% Quiet Sun Level contour: off
% Checkbox:
set(handles.checkbox27, 'Value', 0.0);
set(handles.checkbox27, 'Enable', 'On');
% Menu item:
set(handles.Untitled_122, 'Checked', 'off');
set(handles.Untitled_122, 'Enable', 'On');
% Toggle button:
set(handles.togglebutton8, 'Value', 0.0);

% Contour line annotations: disabled, off:
% Checkbox:
set(handles.checkbox36, 'Value', 0.0);
set(handles.checkbox36, 'Enable', 'Off');
% Menu item:
set(handles.Untitled_58, 'Enable', 'off');
set(handles.Untitled_58, 'Checked', 'off');

% Separate figure:

% Beam size: on:
%GUI
set(handles.checkbox20, 'Value', 1.0);
set(handles.checkbox20, 'Enable', 'On');
% Menu:
set(handles.Untitled_109, 'Checked', 'on');
% Handles variable:
handles.fig_beam_size = 1.0;

% Rot angl. & arrow: on:
% GUI:
set(handles.checkbox30, 'Value', 1.0);
set(handles.checkbox30, 'Enable', 'On');
% Menu:
set(handles.Untitled_110, 'Checked', 'on');
set(handles.Untitled_110, 'Enable', 'on');
% Handles variable:
handles.fig_rot_angle = 1.0;

% Grid: off:
% GUI:
set(handles.checkbox26, 'Value', 0.0);
set(handles.checkbox26, 'Enable', 'On');
% Menu:
set(handles.Untitled_111, 'Checked', 'off');
set(handles.Untitled_111, 'Enable', 'on');
% Handles variable:
handles.fig_grid = 0.0;

% Simple figure: off:
% GUI:
set(handles.checkbox39, 'Value', 0.0);
set(handles.checkbox39, 'Enable', 'On');
% Menu:
set(handles.Untitled_112, 'Checked', 'off');
% Handles variable:
handles.fig_simple = 0.0;

% Size:
% GUI (!!!)
%set(handles.popupmenu7, 'Value', 1.0);
% Menu:
set(handles.Untitled_59, 'Enable', 'on');
set(handles.Untitled_60, 'Checked', 'on');
set(handles.Untitled_61, 'Checked', 'off');
set(handles.Untitled_62, 'Checked', 'off');
set(handles.Untitled_63, 'Checked', 'off');
% Handles variable:
handles.fig_size = 1.0;

% Coordinate & observation tools:

% Enable tools & disable map rotation: off:
set(handles.checkbox40, 'Value', 0.0);
set(handles.checkbox40, 'Enable', 'On');
set(handles.checkbox40, 'BackgroundColor', [0.906 0.906 0.906]); % Background color
set(handles.checkbox40, 'ForegroundColor', [0 0 0]); % Text color

% Data cursor: disabled, off:
set(handles.checkbox28, 'Value', 0.0);
set(handles.checkbox28, 'Enable', 'Off');

% "Observe custom map" button disabled, up:
set(handles.togglebutton9, 'Enable', 'Off');
set(handles.togglebutton9, 'Value', 0.0);

% "Observe full disk map" button disabled:
set(handles.pushbutton37, 'Enable', 'Off');

% Command listbox disabled:
set(handles.listbox6, 'Enable', 'Off');

% "Clear" button disabled:
set(handles.pushbutton39, 'Enable', 'Off');

% "Continue" button disabled:
set(handles.pushbutton36, 'Enable', 'Off');

% Enable "Reset settings" button:
% GUI:
set(handles.pushbutton27, 'Enable', 'On');
% Menu:
set(handles.reset_settings, 'Enable', 'On');

% Call the "Use data matrix center" menu item:
Untitled_120_Callback(hObject, eventdata, handles);

% Notify user that the settings are in use:
set(handles.text39, 'String', '"Dynamic levels ''Jet''" preset settings in use');
set(handles.text39, 'ForegroundColor', [0 0.5 0]); % Set status bar text color to green (it may be left as red by a failed track generation)

% Update the preview:
if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
    plottingList = get(handles.listbox1,'String');
    indexes = get(handles.listbox1, 'Value');
    plot_map = plottingList(indexes(1));
    preview(plot_map, handles.figure1);
end

guidata(hObject, handles);


end


% --------------------------------------------------------------------
% "Plot (and save) solar radius data of selected maps" context menu item
function Untitled_134_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_134 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Call corresponding menu item:
radius_save_button_Callback(hObject, eventdata, handles);

end


% --------------------------------------------------------------------
% Mark NOAA AR's on map menu item
function Untitled_135_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_135 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if(strcmp(get(handles.Untitled_135, 'Checked'), 'off'))
    set(handles.Untitled_135, 'Checked', 'on');
    set(handles.checkbox50, 'Value', 1.0);
    checkbox50_Callback(hObject, eventdata, handles);
elseif(strcmp(get(handles.Untitled_135, 'Checked'), 'on'))
    set(handles.Untitled_135, 'Checked', 'off');
    set(handles.checkbox50, 'Value', 0.0);
    checkbox50_Callback(hObject, eventdata, handles);
end

end


% --- Executes on button press in togglebutton15.
function togglebutton15_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton15 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Button is set up --> call menu item for setting centering off:
if(get(handles.togglebutton15, 'Value') == 1.0)
   
    % Set a custom image as an icon for the solar disk centering toggle button:
    solar_center_tool_str = [pwd, '/img/center_tool_icon_down.png'];
    solar_center_tool_button_img = imread(solar_center_tool_str);
    set(handles.togglebutton15, 'cdata', solar_center_tool_button_img);
    
    % Call menu item
    Untitled_121_Callback(hObject, eventdata, handles);
    
else % BUtton is set down --> call menu item for setting centering on:
       
    % Set a custom image as an icon for the solar disk centering toggle button:
    solar_center_tool_str = [pwd, '/img/center_tool_icon.png'];
    solar_center_tool_button_img = imread(solar_center_tool_str);
    set(handles.togglebutton15, 'cdata', solar_center_tool_button_img);

    % Call menu item
    Untitled_120_Callback(hObject, eventdata, handles);    
    
end

end


% --------------------------------------------------------------------
% Polar region average calculation and save
function polar_avg_button_Callback(hObject, eventdata, handles)
% hObject    handle to polar_avg_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%get the current list of file names from the listbox
inputFileNames = get(handles.listbox1,'String');
    
%get the values for the selected file names
option = get(handles.listbox1,'Value');
 
%if there is nothing to plot, nothing happens
if (isempty(option) == 1 || option(1) == 0 || isempty(inputFileNames))
    return
end

inputFileNames = get(handles.listbox1,'String');
%get the values for the selected file names
option = get(handles.listbox1,'Value');

%only the selected file+path is written into the list:
filenames = inputFileNames(option);

opt.img_type = '.png'; % not needed anymore
opt.destFolder = 'foo'; % not needed anymore
opt.plottingList = filenames;
opt.beam_size = handles.fig_beam_size;
opt.solar_north = handles.fig_rot_angle;
opt.grid_toggle = handles.fig_grid; %get(handles.checkbox26, 'Value');
%opt.cmap = cmap;
opt.abs_scale = get(handles.checkbox31, 'Value');
opt.mapType = get(handles.popupmenu9, 'Value') ;
opt.customNumOfLevelsToggle = get(handles.checkbox35, 'Value') ;
opt.customNumOfLevels = get(handles.edit12, 'String') ;

if(ischar(get(handles.edit10, 'String')) == 1 || ischar(get(handles.edit11, 'String')) == 1)
    opt.min = str2num(get(handles.edit10, 'String'));
    opt.max = str2num(get(handles.edit11, 'String'));
else
    opt.min = get(handles.edit10, 'String');
    opt.max = get(handles.edit11, 'String');
end
opt.img_save = 0;
opt.fig_save = 0;
opt.close_prev = 0;
%opt.img_size = options.image_size;
opt.rot_disable = get(handles.checkbox40, 'Value');
opt.qsl_cont = get(handles.checkbox27, 'Value');

polar_saver(opt, handles.figure1);

end


% --------------------------------------------------------------------
% Menu item for removing partial maps from Sunmap's file list
function remove_partial_maps_Callback(hObject, eventdata, handles)
% hObject    handle to remove_partial_maps (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


    data_type_filter(handles.figure1, 'partial', 'remove');
    % Call preview:
    if(isempty(get(handles.listbox1, 'String')) == 0 && isempty(get(handles.listbox1, 'Value')) == 0)
        plottingList = get(handles.listbox1,'String');
        indexes = get(handles.listbox1, 'Value');
        plot_map = plottingList(indexes(1));

        preview(plot_map, handles.figure1);
    end

end


% --- Executes on button press in checkbox56.
% Dynamic f.o.v. checkbox for separate figure
function checkbox56_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox56 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.pushbutton27, 'Enable', 'on'); % Enable GUI Reset button
set(handles.reset_settings, 'Enable', 'On'); % Enable Menu Reset button

end


% Function for "map-meas." radio button
function radiobutton20_Callback(hObject, eventdata, handles)

    % Due to a lack of screen space in the GUI, the coupling of the radio
    % buttons is done manually:
    % Tie clicking of this radio button to the sun-meas. radio button:
    if(get(hObject, 'Value') == 1.0)
        set(handles.radiobutton21, 'Value', 0.0);
        % Set status bar text string & color (green):
        set(handles.text39, 'String', 'Map-measure command type selected (default)');
        set(handles.text39, 'ForegroundColor', [0 0.5 0]); 
    else
        set(handles.radiobutton21, 'Value', 1.0);
        % Set status bar text string & color (green):
        set(handles.text39, 'String', 'Sun-measure command type selected (legacy)');
        set(handles.text39, 'ForegroundColor', [0 0.5 0]); 
    end

end

% Function for "sun-meas." radio button
function radiobutton21_Callback(hObject, eventdata, handles)

    % Due to a lack of screen space in the GUI, the coupling of the radio
    % buttons is done manually:
    % Tie clicking of this radio button to the map-meas. radio button:
    if(get(hObject, 'Value') == 1.0)
        set(handles.radiobutton20, 'Value', 0.0);
        % Set status bar text string & color (green):
        set(handles.text39, 'String', 'Sun-measure command type selected (legacy)');
        set(handles.text39, 'ForegroundColor', [0 0.5 0]); 
    else
        set(handles.radiobutton20, 'Value', 1.0);
        % Set status bar text string & color (green):
        set(handles.text39, 'String', 'Map-measure command type selected (default)');
        set(handles.text39, 'ForegroundColor', [0 0.5 0]); 
    end

end
